(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 12.1' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[    450395,       8744]
NotebookOptionsPosition[    441980,       8612]
NotebookOutlinePosition[    442404,       8629]
CellTagsIndexPosition[    442361,       8626]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Compare Entropy Functions - New definition", "Section",
 CellChangeTimes->{{3.825360739046709*^9, 
  3.8253607552288427`*^9}},ExpressionUUID->"160571c2-5acf-4c9f-ad69-\
dee4855dae9c"],

Cell[TextData[{
 "A new definition of the Coupled Entropy was created on March 21, 2021 in \
order to have consistency with the definition of the coupled logarithm given \
by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["ln", "\[Kappa]"], 
      RowBox[{"p", "(", 
       StyleBox["x",
        FontWeight->"Bold"], ")"}]}], "\[Congruent]", 
     RowBox[{
      FractionBox["1", "\[Kappa]"], 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"p", "(", 
          StyleBox["x",
           FontWeight->"Bold"], ")"}], 
         FractionBox["\[Kappa]", 
          RowBox[{"1", "+", 
           RowBox[{"d", " ", "\[Kappa]"}]}]]], "-", "1"}], ")"}]}]}], ","}], 
   TraditionalForm]],
  FormatType->"TraditionalForm",ExpressionUUID->
  "3ee8979a-80aa-406d-b033-4825a8e7854d"],
 " where ",
 StyleBox["d",
  FontSlant->"Italic"],
 " is the dimensions of ",
 StyleBox["x",
  FontWeight->"Bold",
  FontSlant->"Italic"],
 ". The power term -\[Alpha]  for the entropy function only enters as a power \
of the variable ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"p", "(", "x", ")"}], 
    RowBox[{"-", "\[Alpha]"}]], TraditionalForm]],
  FormatType->"TraditionalForm",ExpressionUUID->
  "3dd1dbd9-183f-45ec-9b60-2cfc173da62e"],
 "."
}], "Text",
 CellChangeTimes->{{3.825360118249825*^9, 3.8253602353671627`*^9}, {
  3.825360407147523*^9, 3.8253606383949833`*^9}, {3.825360717348386*^9, 
  3.8253607231891937`*^9}, {3.8253607987473392`*^9, 
  3.825360820340345*^9}},ExpressionUUID->"37b1d44c-7184-414d-ab3a-\
23be0d24221a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Assumptions", "=", 
   RowBox[{
    RowBox[{"n", "\[Element]", "PositiveIntegers"}], "&&", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "\[Kappa]", ",", "\[Alpha]", ",", "\[Sigma]", ",", ",", "\[Mu]", ",", 
       "ZP", ",", "\[Lambda]", ",", "W"}], "}"}], "\[Element]", "Reals"}], "&&", 
    RowBox[{"0", "\[LessEqual]", "\[Kappa]", "<", "\[Infinity]"}], "&&", 
    RowBox[{"0", "<", "\[Alpha]", "<", "\[Infinity]"}], "&&", 
    RowBox[{"0", "<", "\[Sigma]", "<", "\[Infinity]"}], "&&", 
    RowBox[{"0", "<", "\[Mu]", "<", "\[Infinity]"}], "&&", 
    RowBox[{"0", "<", "ZP", "<", "\[Infinity]"}], "&&", 
    RowBox[{"0", "<", "\[Lambda]", "<", "\[Infinity]"}], "&&", 
    RowBox[{"0", "<", "W", "<", "\[Infinity]"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.825447441948601*^9, 3.82544745742377*^9}, {
  3.825447487660862*^9, 3.825447559364297*^9}, {3.8254478072286997`*^9, 
  3.825447807448332*^9}, {3.825450435630589*^9, 3.8254504621648827`*^9}, {
  3.825672457517437*^9, 3.825672492779811*^9}, {3.825737056384109*^9, 
  3.825737087180092*^9}, {3.825760486643572*^9, 3.825760491223114*^9}, {
  3.825763244862693*^9, 3.825763246816259*^9}},
 CellLabel->"In[72]:=",ExpressionUUID->"f1a7f510-09d9-453e-ab58-758cb7449e0c"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "PlotCoupledEntDist", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"PlotCoupledEntDist", "[", "]"}], ":=", "\[IndentingNewLine]", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "coupledDist", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"coupledDist", "=", 
      RowBox[{"CoupledNormalDistribution", "[", 
       RowBox[{"0.", ",", "1.", ",", "\[Kappa]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Plot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"NCoupledEntropy", "[", 
          RowBox[{
          "coupledDist", ",", "\[Kappa]", ",", "1", ",", "2", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}]}], 
          "]"}], ",", " ", 
         RowBox[{"(*", " ", 
          RowBox[{
           RowBox[{"\[Alpha]", " ", "=", " ", "1"}], ";", " ", 
           RowBox[{"ie", " ", "no", " ", "root"}]}], " ", "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"NCoupledEntropy", "[", 
          RowBox[{
          "coupledDist", ",", "0", ",", "1", ",", "2", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}]}], 
          "]"}]}], " ", 
        RowBox[{"(*", " ", "Shannon", " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"TsallisEntropy", "[", 
           RowBox[{
           "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
            "False"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{"TsallisEntropy", "[", 
           RowBox[{
           "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
            "True"}], "]"}], ","}], "*)"}], "\[IndentingNewLine]", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Kappa]", ",", "0", ",", "1"}], "}"}], ",", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"PlotLegends", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
         "\"\<Coupled Entropy\>\"", ",", " ", "\[IndentingNewLine]", 
          "\"\<Shannon Entropy\>\""}], " ", "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{
          ",", " ", "\"\<Tsallis Entropy\>\"", ",", " ", 
           "\"\<Normalized Tsallis Entropy\>\""}], "*)"}], 
         "\[IndentingNewLine]", "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotTheme", "\[Rule]", "\"\<Detailed\>\""}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<Coupling (\[Kappa])\>\"", ",", " ", "\"\<Entropy\>\""}],
          "}"}]}]}], "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.825360890434689*^9, 3.8253610526236277`*^9}, {
   3.8253611206635113`*^9, 3.825361137001877*^9}, {3.825361182352261*^9, 
   3.825361230850746*^9}, {3.825361341413456*^9, 3.825361351413355*^9}, {
   3.8256878742074833`*^9, 3.825687898782297*^9}, {3.825688338407009*^9, 
   3.825688344221237*^9}, {3.8256902660598793`*^9, 3.825690266166663*^9}, {
   3.825699292303846*^9, 3.825699315382307*^9}, 3.825699355884129*^9, {
   3.825699490634377*^9, 3.825699523827836*^9}, {3.825763214369049*^9, 
   3.825763229703871*^9}, {3.825763268509869*^9, 3.8257632998935833`*^9}, {
   3.8257635099823923`*^9, 
   3.825763510334773*^9}},ExpressionUUID->"2e347783-5b72-46ae-ae76-\
6cff4c31e00d"],

Cell[BoxData[
 RowBox[{"PlotCoupledEntDist", "[", "]"}]], "Input",
 CellLabel->"In[75]:=",ExpressionUUID->"7e75eb80-ba0d-45fc-a1cb-8cf6b73462cd"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwUV3c4lm8bJlGppCVKJZVUQhIRTsnoJ0k2RVYUFUnSFAkVJSQKyd7ZM3vv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          "]]}, Annotation[#, "Charting`Private`Tag$7695#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwUl3c8Vu8fxkvKSCVJlEolLUnSIFyyKhoSUUlIFJWSpGUlISQkOyvZe++9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          "]]}, 
        Annotation[#, "Charting`Private`Tag$7695#2"]& ]}}, {}}, {
    DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
     AxesOrigin -> {0, 1.3903133819526212`}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {False, False}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 1.3903133819526212`}, 
     DisplayFunction :> Identity, Frame -> {{True, True}, {True, True}}, 
     FrameLabel -> {{
        FormBox["\"Entropy\"", TraditionalForm], None}, {
        FormBox["\"Coupling (\[Kappa])\"", TraditionalForm], None}}, 
     FrameStyle -> Automatic, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, GridLinesStyle -> Directive[
       GrayLevel[0.4, 0.5], 
       AbsoluteThickness[1], 
       AbsoluteDashing[{1, 2}]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None,
        "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, 
     PlotRange -> {{0, 1}, {1.3903133819526212`, 2.7514551446069406`}}, 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{"\:f3b5Coupled Entropy", "\"Shannon Entropy\""}, 
      "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), Editable -> True, 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& )], 
     TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Input",
 CellChangeTimes->{{3.825763504350575*^9, 
  3.825763505022952*^9}},ExpressionUUID->"0ff8ce3e-7f07-4e60-a991-\
272829c0f262"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "PlotCoupledEntDist", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"PlotCoupledEntDist", "[", "]"}], ":=", "\[IndentingNewLine]", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "coupledDist", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"coupledDist", "=", 
      RowBox[{"CoupledNormalDistribution", "[", 
       RowBox[{"0.", ",", "1.", ",", "\[Kappa]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Show", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Plot", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"\[Kappa]", "-", 
            RowBox[{
             SuperscriptBox["\[Pi]", 
              FractionBox["\[Kappa]", 
               RowBox[{"1", "+", "\[Kappa]"}]]], " ", 
             SuperscriptBox["\[Kappa]", 
              FractionBox["1", 
               RowBox[{"1", "+", "\[Kappa]"}]]], " ", 
             RowBox[{"(", 
              RowBox[{"1", "+", "\[Kappa]"}], ")"}], " ", 
             SuperscriptBox[
              RowBox[{"(", 
               FractionBox[
                RowBox[{"Gamma", "[", 
                 FractionBox["1", 
                  RowBox[{"2", " ", "\[Kappa]"}]], "]"}], 
                RowBox[{"Gamma", "[", 
                 FractionBox[
                  RowBox[{"1", "+", "\[Kappa]"}], 
                  RowBox[{"2", " ", "\[Kappa]"}]], "]"}]], ")"}], 
              FractionBox[
               RowBox[{"2", " ", "\[Kappa]"}], 
               RowBox[{"1", "+", "\[Kappa]"}]]]}]}], 
           RowBox[{"2", " ", 
            SuperscriptBox["\[Kappa]", "2"]}]]}], ",", "\[IndentingNewLine]", 
         
         RowBox[{"{", 
          RowBox[{"\[Kappa]", ",", "0", ",", "1"}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"PlotTheme", "\[Rule]", "\"\<Detailed\>\""}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", "\[IndentingNewLine]", 
         RowBox[{"PlotLegends", "\[Rule]", 
          RowBox[{"{", "\"\<Coupled Entropy Symbolic\>\"", "}"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
         "PlotLabel", "\[Rule]", 
          "\"\<Entropy of Coupled Normal Distribution\>\""}]}], 
        "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"ListPlot", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Table", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"{", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"\[Kappa]", ",", 
                RowBox[{"NCoupledEntropy", "[", 
                 RowBox[{
                 "coupledDist", ",", "\[Kappa]", ",", "1", ",", "2", ",", "1",
                   ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], 
                   "}"}]}], "]"}]}], "}"}], ",", " ", 
              RowBox[{"(*", " ", 
               RowBox[{
                RowBox[{"\[Alpha]", " ", "=", " ", "1"}], ";", " ", 
                RowBox[{"ie", " ", "no", " ", "root"}]}], " ", "*)"}], 
              "\[IndentingNewLine]", 
              RowBox[{"{", 
               RowBox[{"\[Kappa]", ",", 
                RowBox[{"NCoupledEntropy", "[", 
                 RowBox[{
                 "coupledDist", ",", "0", ",", "1", ",", "2", ",", "1", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], 
                   "}"}]}], "]"}]}], "}"}]}], " ", 
             RowBox[{"(*", " ", "Shannon", " ", "*)"}], "\[IndentingNewLine]", 
             RowBox[{"(*", 
              RowBox[{
               RowBox[{"TsallisEntropy", "[", 
                RowBox[{
                "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", 
                 ",", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], 
                 ",", "False"}], "]"}], ",", "\[IndentingNewLine]", 
               RowBox[{"TsallisEntropy", "[", 
                RowBox[{
                "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", 
                 ",", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], 
                 ",", "True"}], "]"}], ","}], "*)"}], "\[IndentingNewLine]", 
             "}"}], ",", "\[IndentingNewLine]", 
            RowBox[{"{", 
             RowBox[{"\[Kappa]", ",", "0", ",", "1", ",", "0.02"}], "}"}]}], 
           "\[IndentingNewLine]", "]"}], "//", "Transpose"}], ",", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"PlotLegends", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
           "\"\<Coupled Entropy\>\"", ",", " ", "\n", " ", 
            "\"\<Shannon Entropy\>\""}], " ", 
           RowBox[{"(*", 
            RowBox[{
            ",", " ", "\"\<Tsallis Entropy\>\"", ",", " ", 
             "\"\<Normalized Tsallis Entropy\>\""}], "*)"}], "}"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"PlotTheme", "\[Rule]", "\"\<Detailed\>\""}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"FrameLabel", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
           "\"\<Coupling (\[Kappa])\>\"", ",", " ", "\"\<Entropy\>\""}], 
           "}"}]}]}], "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]}], "Input",
 CellChangeTimes->{{3.825360890434689*^9, 3.8253610526236277`*^9}, {
   3.8253611206635113`*^9, 3.825361137001877*^9}, {3.825361182352261*^9, 
   3.825361230850746*^9}, {3.825361341413456*^9, 3.825361351413355*^9}, {
   3.8256878742074833`*^9, 3.825687898782297*^9}, {3.825688338407009*^9, 
   3.825688344221237*^9}, {3.8256902660598793`*^9, 3.825690266166663*^9}, {
   3.825699292303846*^9, 3.825699315382307*^9}, 3.825699355884129*^9, {
   3.825699490634377*^9, 3.825699523827836*^9}, {3.8257052463739567`*^9, 
   3.825705334185957*^9}, {3.825705382818261*^9, 3.8257053857271214`*^9}, {
   3.825705463782173*^9, 3.825705479772291*^9}, {3.8257055723190727`*^9, 
   3.825705575993329*^9}, {3.825763120397818*^9, 3.825763180332958*^9}, {
   3.825763915811318*^9, 3.825763949322093*^9}, {3.825764238964526*^9, 
   3.825764278793641*^9}, {3.825764339982176*^9, 3.825764389043417*^9}, {
   3.8257648324220667`*^9, 3.825764911848778*^9}, {3.825765000920702*^9, 
   3.825765009712072*^9}, {3.8257651585380793`*^9, 3.825765229343678*^9}},
 CellLabel->"In[86]:=",ExpressionUUID->"232a5b2e-e16e-496d-a44b-258988428cd2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotCoupledEntDist", "[", "]"}]], "Input",
 CellLabel->"In[88]:=",ExpressionUUID->"da8233c6-bcd1-4cb6-80b0-bc3f7b1318aa"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{{}, {}, 
        TagBox[{
          Directive[
           Opacity[1.], 
           RGBColor[0.368417, 0.506779, 0.709798], 
           AbsoluteThickness[1.6]], 
          LineBox[CompressedData["
1:eJwdjnk41HkAh+f6oSVH2iR36BBqS3Soz7fLtalIYbckFTWpXTbr6BCLJKVD
JULJllK0MZYaaZFSpEIlmjRJxPjJkzkMMyvv83yez/P+95oF/Oa5g8VgMPxH
9/3ddnS9eNC9YWnHL20iaZEYKsY2nki2g1rrq1J/nhgD2lzTduZSrH41eZtK
iRjzstMuFumvRI5Qq6t01P1KjnD317lg4M/sg5v/FUMjPjKqdrwXfnpFV4WV
iaE5rs+pT7wVN1p8ljwrF8N7u6T8tP52ZG6dGyu7L8a72UdWHNffB9We2zXy
WjHiPjFrF3kfgipXd459sxhB1jenL8uKxV7V1NyHbWJc+nFtQpJ+HOoPKbyp
djEerD77xtz1NKwdDLbacSSIUSuaflQ/FaKcDwXWGhL41hy6mzCQDpdj4/O2
WElwPajIv+zaZTS6REUsWyuBj+/lwv6vueA66Oo5BUugrozZ5GxciJzOvHVh
NRJYevR1VR0rhpp5VYC7iRQyrw2j8KDSHxoWaCPFRu3N+16s+hcLVaexTddJ
cf7Cd8rQ0T8c+TRSivBnC5H2rQxWImf+jMNS+NeEGD4K4WNkoOXjwE0p9Ip1
BExmBYaUvCfMVikGlN+pgDJz8AQlkqJEuKKuwbESom2JMweXyJBk4XrawrUS
87XuxPIgQyU308C3txp/z/rmnpEtg3is7yF0fW6sPF8gw+C0JVq39jzC0/YD
CyzkMoSPBdZipDcspd1tCCxZvraB5RPsv/VXt9BvCPZK/5L1BXUYJ3/3OYw/
hLo4u2c1Vg0wS73TkDVZDvOxvgZ4GHqE5TrIYXKyR3a97zlsVVa9nXRQjh7J
e0HznJdQCUh2c26VI/fLvSeaJxph3ZNp07psGJpG1+iOjiYEVgS7/HBtGI/H
epuxpv7Oj/6Ph5G8ixd4yeIVxFZTNJqpEYhJfsfO6NdwInnTDPaMYN/c6uJJ
VW9QoTjbKaofwfuC6mMvzN4iv+Yoe62DAnJuU8bMqFZsq0vXcsxWYNYyVUXg
vTaYuKf4XWAo4ZxjFvSPngBMXU3vDK4S4UZbpn7Y9B5Bio0tinolGGO0o8y+
eZwKg0Gsezeov3Zvh6Utgrv0GERXq1+teqgd1Q/vLLR1ZxBORUah+cUPyPQ9
E6UXzSC77e61jPtZiJhH84wDeQyS7xpw6b5YiIaGiHDRFwYZ7Jmi3XnuI+Ly
PVTUjZiEt96ZnbqyA3k3Bjhp65kk2iU+u7izA4+EPduLE5jE7J3bmyvxn1AU
cfi5vJRJFjnEperP7YS54+LpfJpJNhk3yJiNnSja6JilnMoig0+OC3+P+Yyg
+Vvqq35hkTSO3z5Piy6UHytK46SwiK6Al8Oq7oIvX+vjrkoWoQ7GeC0M6cbk
uGlCnyEWYVz3KxRO+AKN6NCXpbZsEppeeHxC5Rek7jkluB7EJuWzEtV09vbA
9OvX8w+y2UTHLdQ6VrMXnSM77jo2sUmbzfPGSH4v7he+vuY5nkNuZ8Giw0+E
Q9VO2o+Xc8jmQnsdI4UIerVNzt7hHHI4/nJM240+XJ7XGpt4m0NudRllaK6m
cWpOW7pmJ4ckZsb7C9bQ+DQc0j8y6ts9RZYFHjS8khmuos8cYsjn33b3pmG2
Q5Vd180hSSm/1iQH0MjXLU0/KuKQnfYXvv4QSUM+McGHGuSQqXETXVSv0pBl
vdVQsimiWHBg/Os8GgeSVvzXx6FIi+jjy6v5NDIqL0UIKIqc9C7a7PQPjeAk
h/5yVYoorTz/iOfTsD77knVQnSJtL1Ky2I00EuOyefIJFClNkGxrbKahdsbx
Qo8uRVIXb5l55Q0Ny5Kn0a0TKfLz37bFywU00m5WbLg3iSJ3I+prY7ppPIsa
stg/hSLnbOanePTSaF1jYbrbgCIhwoteZjSNTYYw/tWQIjNWB79/8I2G5yRn
88XGFOEwm3JPSmhMGJ5nNcuEIu28xVz/IRof3HXsDEwpwudemT1nhEZol4Co
m1HkvIn6oFI56nkX18lH/X9jxOzV
           "]]}, 
         Annotation[#, "Charting`Private`Tag$313206#1"]& ]}}, {}}, {{}, {{{
         Directive[
          PointSize[0.009166666666666668], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         PointBox[CompressedData["
1:eJw1UXss1HEA/92L2E1LLM05qa15ZUlaefT5Yh7pnzx6cR5rQipRZo2FJrbK
pjV/lNLkbpTFEtWhoVyUZIymk3RCXoda87jzyu++vz7bd999X5/X1+7MpdCz
fIZh3DYGO/9Hx65t8qzeBWRaVAcOZxQhNlDS2Oa4yK0V4CtytXMli1BK86ZD
lNWoSYj0Efstced1SKsKfpxxaAnF91nU45iDUcub3iXufhNKF2QaBzMdtCHs
zlu42j5wlh7Xce9V8A1zipx5rYNpvzzF27Qd0ppKX3W6juP7iOynJXd27NHD
QVb4bt6+E0bXRittTPQcfxeGLhQHyNv1UGy87pd3Y2drY4xn1jKn14M+ifsB
dfgy3AzoRZulV3KOaIXT78OHv9uv5M4uo3XefkPhC5xOFU/9GV/h/PTD+dn4
uecFqwg1CH5FUUxhnBtZ5fyp8VKVJImqX8VwBks4AKvY5qM+yjXO7yCUk+ZF
2q3rSGXtpXyHV1VQV+uLNc7/ENxemf4Qu6yD/oYG+shEb2MBQ2geDco17h7x
1gwJogIQfep2qJYzhOYbRkPOwdrwMoawarLCn3jy3k4dWsoQmncENyQDVp5b
eITGGUFSUWrFeWseoflHcfOeuGAkjkdsWTvSMZSl+dtGpPMI7WMM5cGyq4Ed
PJIQz+IXGhJaBh8peYT2M47YpoATtxL5hGWzqB5HnaPRwv4oPqF9TeB2qV4h
VfHJAks3P4GefPMj/Xo+of1NwmXooclYBZ8cNhBOoaPZ86RzsoDQuFOIDljb
rLkrIAZ7edPQXc+c62OEhParRcS309E2KgH53MlCi6HpZI+4cCGhfc9gb2aY
ynW3kFgaDM7CbN+0olYhJLT/Wfhl++vrjEXEQCebQ8XipsviciGh//EbFxvy
RwQWIvIPQ31/Uw==
          "]]}, {
         Directive[
          PointSize[0.009166666666666668], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         PointBox[CompressedData["
1:eJw1UWtIU1EAvtskxcics1IC7QWpPRQcFL2+Y1JJmLRl9qMtIrGFQq0UCgXR
dJL4Q3yhRiAyowf5IENnidPUpTlT03SGisvHbFtqpm7ObebdvX5wOJzX9zr7
7zwQJ3ApigrfHPS8hSCrbU2gWEWab80lfWox+uua3vh4W9h1FabKPDKlRy1Q
BShMIlUNpit76kNkW+cfIN7uuNk0bsHzchpN8LrS/ym+0creb4GuVW0pyrfC
LKJ32hBTpexJXray7zswy5ddS4heg+eIUn7W8wt2RKhb6qrXWL5uHJmrKE1x
tyFYkv95JUgLuUZ4WvvWxvJ/w2DZtr+FfTZUbb4eUfZDFioMJDHrrN4AipVD
72Q6G8JdGESiNUN6fHSd1R/CCerlxe+adbSvBG0q/MBAXkJqRZqd9TMC/dfa
q43Zdohdgjr4vxDcqOU7WH+jWCqTE1uBA/pUmvAnFrJq44KjHazfMXhnPv54
+b4TD2l78nHYi0OMJ81O1v8EelS3O0+ZnGB+YxLDjZOvTWc22DyT0HRLKY8Q
ikQxAuhIjC3Jydhg8+lxTDyaXhRDEVpNkv8LisP7GgQSijB5pxC5c9r4vp4i
TJwpvI33ydL2UoTJP4176lf8dDtFAmk7ATMobE4r+GegCNPHDML4u5d0ERwi
u0tjFhJ0VWpyOITpxwDhhbz20GoOodl8awy4FZmYVGrkEKavOUQ1lws2vLhk
laZbmcOqm4+ozZ9LmP5+I2DvkwPuJVxyzkVohLEhbkD4iEuYuEac53DCni5y
icuewoTOXGfH4hiXMP2aET2RpNaH8UivloYZnst7ujxaeYTp+w9KpSq5KIlH
drkMzuOglym7Y55HmP7nYfDLTX42zCMuOskC+vzGD3VfdyPMfywiNiVvOMrE
I/8B8oKNtQ==
          "]]}}}, {{}, {}}}}, {
    DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
     AxesOrigin -> {0, 1.4189385692058574`}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {False, False}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 1.4189385692058574`}, 
     DisplayFunction :> Identity, Frame -> {{True, True}, {True, True}}, 
     FrameLabel -> {{None, None}, {None, None}}, FrameStyle -> Automatic, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, GridLinesStyle -> Directive[
       GrayLevel[0.4, 0.5], 
       AbsoluteThickness[1], 
       AbsoluteDashing[{1, 2}]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None,
        "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, PlotLabel -> 
     FormBox["\"Entropy of Coupled Normal Distribution\"", TraditionalForm], 
     PlotRange -> {{0, 1}, {1.4189385692058574`, 2.6415926263626313`}}, 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    TagBox[
     GridBox[{{
        FormBox[
         TemplateBox[{"\"Coupled Entropy Symbolic\""}, "LineLegend", 
          DisplayFunction -> (StyleBox[
            StyleBox[
             PaneBox[
              TagBox[
               GridBox[{{
                  TagBox[
                   GridBox[{{
                    GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}}, 
                    GridBoxAlignment -> {
                    "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                    AutoDelete -> False, 
                    GridBoxDividers -> {
                    "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                    GridBoxItemSize -> {
                    "Columns" -> {{All}}, "Rows" -> {{All}}}, 
                    GridBoxSpacings -> {
                    "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
                GridBoxAlignment -> {
                 "Columns" -> {{Left}}, "Rows" -> {{Top}}}, AutoDelete -> 
                False, GridBoxItemSize -> {
                 "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
                GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
               "Grid"], Alignment -> Left, AppearanceElements -> None, 
              ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
              "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
            FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
            False]& ), InterpretationFunction :> (RowBox[{"LineLegend", "[", 
             RowBox[{
               RowBox[{"{", 
                 RowBox[{"Directive", "[", 
                   RowBox[{
                    RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
                 "}"}], ",", 
               RowBox[{"{", #, "}"}], ",", 
               RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
               RowBox[{"LabelStyle", "\[Rule]", 
                 RowBox[{"{", "}"}]}], ",", 
               RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
          Editable -> True], TraditionalForm]}, {
        FormBox[
         TemplateBox[{"\"Coupled Entropy\"", "\"Shannon Entropy\""}, 
          "PointLegend", DisplayFunction -> (StyleBox[
            StyleBox[
             PaneBox[
              TagBox[
               GridBox[{{
                  TagBox[
                   GridBox[{{
                    GraphicsBox[{{}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                    ImageSize -> {10, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                    GraphicsBox[{{}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                    ImageSize -> {10, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
                    GridBoxAlignment -> {
                    "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                    AutoDelete -> False, 
                    GridBoxDividers -> {
                    "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                    GridBoxItemSize -> {
                    "Columns" -> {{All}}, "Rows" -> {{All}}}, 
                    GridBoxSpacings -> {
                    "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
                GridBoxAlignment -> {
                 "Columns" -> {{Left}}, "Rows" -> {{Top}}}, AutoDelete -> 
                False, GridBoxItemSize -> {
                 "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
                GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
               "Grid"], Alignment -> Left, AppearanceElements -> None, 
              ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
              "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
            FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
            False]& ), 
          InterpretationFunction :> (RowBox[{"PointLegend", "[", 
             RowBox[{
               RowBox[{"{", 
                 RowBox[{
                   RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"PointSize", "[", "0.009166666666666668`", "]"}], 
                    ",", 
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
                   ",", 
                   RowBox[{"Directive", "[", 
                    RowBox[{
                    RowBox[{"PointSize", "[", "0.009166666666666668`", "]"}], 
                    ",", 
                    InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                    RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], 
                    "]"}]}], "}"}], ",", 
               RowBox[{"{", 
                 RowBox[{#, ",", #2}], "}"}], ",", 
               RowBox[{"LegendMarkers", "\[Rule]", 
                 RowBox[{"{", 
                   RowBox[{
                    RowBox[{"{", 
                    RowBox[{"False", ",", "Automatic"}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{"False", ",", "Automatic"}], "}"}]}], "}"}]}], 
               ",", 
               RowBox[{"Joined", "\[Rule]", 
                 RowBox[{"{", 
                   RowBox[{"False", ",", "False"}], "}"}]}], ",", 
               RowBox[{"LabelStyle", "\[Rule]", 
                 RowBox[{"{", "}"}]}], ",", 
               RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
          Editable -> True], TraditionalForm]}}, 
      GridBoxAlignment -> {"Columns" -> {{Left}}}, AutoDelete -> False, 
      GridBoxItemSize -> {
       "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
      GridBoxSpacings -> {"Columns" -> {{0}}}], "Grid"], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.82570546069872*^9, 3.82570548760837*^9}, 
   3.82570558154597*^9, 3.8257630012203207`*^9, 3.825763104976214*^9, 
   3.825763138580124*^9, 3.8257643101187677`*^9, 3.825764846975481*^9, 
   3.8257649321648607`*^9, 3.825765023027112*^9, 3.825765168865561*^9, 
   3.825765239145857*^9},
 CellLabel->"Out[88]=",ExpressionUUID->"4b6ecd3d-1c6c-4098-bf0b-ba276a8908e2"]
}, Open  ]],

Cell["Saved Results", "Text",
 CellChangeTimes->{{3.8256983614173307`*^9, 
  3.825698363972776*^9}},ExpressionUUID->"42b177a6-1ba4-43ce-af1d-\
45836247adf6"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{}, {{{
        Directive[
         PointSize[0.009166666666666668], 
         RGBColor[0.368417, 0.506779, 0.709798], 
         AbsoluteThickness[1.6]], 
        PointBox[CompressedData["
1:eJw1UVtIk1EA3i0LxTRRLCszH8oSXDCQWep35qUkS90sLR2VK6el4oUyNOkG
EiloiQrpg5D6EOmgMl0E2tQwwUXemklacy6XLlvgnG5zy3//7weHw7l9t3NQ
VijJ5rBYLMHmoOYtTCR2B1WPruKOr+KUtrweeb589ZPlrXUb0l5GxYv3W6AM
rFwSKxX4d1P2YcRkYc67MCe7mtKitaDpGYV3EKrGTzRErjH3e/HWUWgbal+D
UUztqBCQJzozn7rOvB/E+dIr2WKBFe6a1qIo9yGUyVMNsT/WGb5hDKY+3R6h
t+KItLbfHDKCMKW5WRdnZfg/g+dXJ6pasaJt87Wm9Qt2dJc1Xyq2MXqjSBFJ
hnP4NghcGEfQ3bHVOB87oz+BBEWSUOltx4A5ZFNhEkkZmcL6h3bGjwYxQoWw
tdcOiUtwCjWGrMwV2wbj7xtK1BXXL15wQFtOEU7j1tALR0+ng/H7HbFzu/nh
EieKKXtFMzi0dyo8f8LB+J9FyjXUNJ10gv6Nn4jObbvh0+hk8vyE6s0rVYAX
iyTQAvDcU13RkM8idD4tIr0S/QuaWYRSk9bOwdsUPLZLyCZ0Xh2M98OcbmoW
oePo8D5z0piTxiZ0/nl8XZt98JHPJgcoO4F69IsOH2vuYBO6Dz0+hba3JBvY
JEdO4RdI7ii3spNN6H4WwGf1CTbSOYRi81UsoDQunTgbOYTuy4Bek8lTU8Uh
qxSd2QCP4Z05EUYOofv7jRD90eMJUi6JdhEu4nQVr0PuziV03EWc87DE6G5z
icte5RJCZ/w14iUuofs1IlFudZsc4BL1CAUjxvbFn309xSN0338QXHu5x5zB
I34ug8uQzWSH3kvmEbr/ZRQ8L1l/lMUjLjrpX/Q9NozXuW8j9H+Y0NHV0+2c
5pH/vXiEww==
         "]]}, {
        Directive[
         PointSize[0.009166666666666668], 
         RGBColor[0.880722, 0.611041, 0.142051], 
         AbsoluteThickness[1.6]], 
        PointBox[CompressedData["
1:eJw1UX0slGEAf++jNvqnzUe00ZUzbJaFP5Lye5okbZGPSXWkk9Ipzm1pKUo6
bmVZG0nNVvlqFdoy0dDIym1qKE7u8v15hLLj7pyT997Xb3v27Pn6fT27xakR
iVyKonw2Bj1vQnL5btbUq2XctK0JHskoREymwe1Owea6HO4p8dYHslfQ4Cyf
DW+owVDsqH+keoU9r0Nc1bmRL/Z6PC2h0YhPS49MLv569n4LrFLDpLYKPebC
6Z1WJD08LPQUG9j37cin/voVOxlgrSqTHrL+CsHxKHv9eQPLp4RYc6RaZjDA
Q1TQpnPvRJNuuzj9tJHl/w6xJExtvmdE+cZrVVkXIpv7FpP1RlavG//ak8ST
QavwseAHYoUKQ3L9Kqv/E9296Q19USZ81rlvKPSi0r50yHOHifWjgtA5OtRV
aUKERbAfPY3yd9e3rbH+fqEnIW08uGQNIxk04QCUHjbFL1zMrF8NfJsrOw5G
mZFG25P+hlapyXnpsM76H4R3oGyhUWsG8xvD2Cpoa5meXGfzDGPaW1PhIKTI
MUYAWQmuaTfy19l8I9C9Nd4vyqEIrSYqGIVQUt9fmU4RJu8Y5NqPte9vUYSJ
MwaJ14XyB10UYfKPoyfIJzquliK7aDvOE+jery6ZseIQpo8JCIo+qPgCDrl0
kcYkSkNTvQaqOITpZwrPX9tFxFdwCM1mWzMF39LCE3IDhzB9TaN1rzgoz4VL
lmk63TQCHEOyT8ZwCdPfDJ4tuZ/VZHJJgIVQi9K+K5GyJ1zCxNXiVNM1lbqZ
Syz25LNQXA10k61yCdPvHHJzQvZUO/LIt04ac+joEz1+I+IRpu8/aEnpbDiz
k0fsLAbnUZzgZ+OUyCNM//PYF16n4LfyiIVOtIDcruhBbOET5j8WcVSad5uK
5ZP/LdqHWQ==
         "]]}}}, {{}, {}}}, {
    DisplayFunction -> Identity, DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {False, False}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 1.3422494099579143`}, 
     DisplayFunction :> Identity, Frame -> {{True, True}, {True, True}}, 
     FrameLabel -> {{
        FormBox["\"Entropy\"", TraditionalForm], None}, {
        FormBox["\"Coupling (\[Kappa])\"", TraditionalForm], None}}, 
     FrameStyle -> Automatic, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, GridLinesStyle -> Directive[
       GrayLevel[0.4, 0.5], 
       AbsoluteThickness[1], 
       AbsoluteDashing[{1, 2}]], 
     Method -> {
      "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
       "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           Identity[
            Part[#, 1]], 
           Identity[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           Identity[
            Part[#, 1]], 
           Identity[
            Part[#, 2]]}& )}}, 
     PlotRange -> {{0, 1.}, {1.3422494099579143`, 2.630162864240645}}, 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{"\"Coupled Entropy with no root\"", "\"Shannon Entropy\""}, 
      "PointLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                    ImageSize -> {10, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], 
                    PointBox[
                    NCache[{
                    Scaled[{
                    Rational[1, 2], 
                    Rational[1, 2]}]}, {
                    Scaled[{0.5, 0.5}]}]]}}}, AspectRatio -> Full, 
                    ImageSize -> {10, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"PointLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"PointSize", "[", "0.009166666666666668`", "]"}], 
                   ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"PointSize", "[", "0.009166666666666668`", "]"}], 
                   ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", 
             RowBox[{"{", 
               RowBox[{
                 RowBox[{"{", 
                   RowBox[{"False", ",", "Automatic"}], "}"}], ",", 
                 RowBox[{"{", 
                   RowBox[{"False", ",", "Automatic"}], "}"}]}], "}"}]}], ",", 
           RowBox[{"Joined", "\[Rule]", 
             RowBox[{"{", 
               RowBox[{"False", ",", "False"}], "}"}]}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.82570546069872*^9, 3.82570548760837*^9}, 
   3.82570558154597*^9, 3.8257630012203207`*^9, 3.825763104976214*^9, 
   3.825763138580124*^9},
 CellLabel->"Out[71]=",ExpressionUUID->"f06a7e95-6a7e-4fda-8e63-743e5348ed89"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Compare Entropy Functions - Legacy Comparisons", "Section",
 CellChangeTimes->{{3.8154918348885117`*^9, 3.8154918442298193`*^9}, {
   3.825360097969391*^9, 3.8253601059942207`*^9}, 
   3.825362100665683*^9},ExpressionUUID->"ac19dfb8-be49-418a-809a-\
28f5cf7cb726"],

Cell[TextData[{
 "A new definition of the Coupled Entropy was created on March 25, 2021 in \
order to have consistency with the definition of the coupled logarithm given \
by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox["ln", "\[Kappa]"], 
      RowBox[{"p", "(", 
       StyleBox["x",
        FontWeight->"Bold"], ")"}]}], "\[Congruent]", 
     RowBox[{
      FractionBox["1", "\[Kappa]"], 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"p", "(", 
          StyleBox["x",
           FontWeight->"Bold"], ")"}], 
         FractionBox["\[Kappa]", 
          RowBox[{"1", "+", 
           RowBox[{"d", " ", "\[Kappa]"}]}]]], "-", "1"}], ")"}]}]}], ","}], 
   TraditionalForm]],ExpressionUUID->"b07b2a05-d20d-49b4-8927-fa6356da7356"],
 " where ",
 StyleBox["d",
  FontSlant->"Italic"],
 " is the dimensions of ",
 StyleBox["x",
  FontWeight->"Bold",
  FontSlant->"Italic"],
 ". The power term -\[Alpha] only enters as a power of the variable ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"p", "(", "x", ")"}], 
    RowBox[{"-", "\[Alpha]"}]], TraditionalForm]],ExpressionUUID->
  "e4505090-8413-48e3-8495-247531399041"],
 ". This section is maintained as legacy in which the power term was also a \
multiple of the division by \[Kappa]; however, the functions which call \
CoupledEntropy will not work."
}], "Text",
 CellChangeTimes->{{3.825360118249825*^9, 3.8253602353671627`*^9}, {
  3.825360407147523*^9, 3.8253606383949833`*^9}, {3.825360717348386*^9, 
  3.8253607231891937`*^9}, {3.8256879562220078`*^9, 3.8256879562556763`*^9}, {
  3.825688019087283*^9, 
  3.825688032865616*^9}},ExpressionUUID->"be2941ff-a13a-4c68-9e78-\
ee6fe13f9aa3"],

Cell[TextData[{
 "See Coupled Functions file for the CoupledEntropy Function.\nThe Tsallis \
and Normalized Tsallis entropy functions are related to the non-root form of \
the Coupled Entropy\n\tTsallisNormalizedEntropy = (1 + \[Kappa]) \
CoupledEntropy\n\tTsallisEntropy = (1 + \[Kappa]) ",
 Cell[BoxData[
  FormBox[
   RowBox[{"Sum", "(", 
    SubsuperscriptBox["p", "i", 
     RowBox[{"1", "+", 
      FractionBox[
       RowBox[{"\[Alpha]", " ", "\[Kappa]"}], 
       RowBox[{"1", "+", "\[Kappa]"}]]}]], ")"}], TraditionalForm]],
  ExpressionUUID->"ac9a89c7-89fb-4425-b0ee-2fe390a99a38"],
 "CoupledEntropy"
}], "Text",
 CellChangeTimes->{{3.815492049753841*^9, 
  3.8154922557274933`*^9}},ExpressionUUID->"5b8c9bf8-061e-4d5d-ac01-\
1c9866ad995f"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"TsallisEntropy", "[", 
     RowBox[{"dist_", ",", "\[Kappa]_", ",", 
      RowBox[{"\[Alpha]_", ":", "1"}], ",", 
      RowBox[{"d_", ":", "1"}], ",", 
      RowBox[{"limits_", ":", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}]}], ",", 
      RowBox[{"normalize_", ":", "False"}]}], "]"}], ":=", 
    "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{"normalize", ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "+", "\[Kappa]"}], ")"}], 
       RowBox[{"CoupledEntropy", "[", 
        RowBox[{
        "dist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "d", ",", "limits", 
         ",", "False"}], "]"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "+", "\[Kappa]"}], ")"}], 
       RowBox[{"CoupledEntropy", "[", 
        RowBox[{
        "dist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "d", ",", "limits", 
         ",", "False"}], "]"}], 
       RowBox[{
        SubsuperscriptBox["\[Integral]", 
         RowBox[{
         "limits", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
         RowBox[{
         "limits", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"PDF", "[", 
           RowBox[{"dist", ",", "x"}], "]"}], 
          RowBox[{"1", "+", 
           RowBox[{"\[Alpha]", 
            FractionBox[
             RowBox[{" ", "\[Kappa]"}], 
             RowBox[{"1", "+", "\[Kappa]"}]]}]}]], 
         RowBox[{"\[DifferentialD]", "x"}]}]}]}]}], " ", 
     "\[IndentingNewLine]", "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TsallisRootEntropy", "[", 
   RowBox[{"dist_", ",", "\[Kappa]_", ",", 
    RowBox[{"\[Alpha]_", ":", "1"}], ",", 
    RowBox[{"d_", ":", "1"}], ",", 
    RowBox[{"limits_", ":", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}]}], ",", 
    RowBox[{"normalize_", ":", "False"}]}], "]"}], ":=", 
  "\[IndentingNewLine]", 
  RowBox[{"If", "[", 
   RowBox[{"normalize", ",", "\[IndentingNewLine]", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "+", "\[Kappa]"}], ")"}], 
      FractionBox["1", "\[Alpha]"]], 
     RowBox[{"CoupledEntropy", "[", 
      RowBox[{
      "dist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "d", ",", "limits", ",", 
       "True"}], "]"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "+", "\[Kappa]"}], ")"}], 
      FractionBox["1", "\[Alpha]"]], 
     RowBox[{"CoupledEntropy", "[", 
      RowBox[{
      "dist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "d", ",", "limits", ",", 
       "True"}], "]"}], 
     RowBox[{
      SubsuperscriptBox["\[Integral]", 
       RowBox[{
       "limits", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
       RowBox[{
       "limits", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]], 
      RowBox[{
       SuperscriptBox[
        RowBox[{"PDF", "[", 
         RowBox[{"dist", ",", "x"}], "]"}], 
        RowBox[{"1", "+", 
         RowBox[{"\[Alpha]", 
          FractionBox[
           RowBox[{" ", "\[Kappa]"}], 
           RowBox[{"1", "+", "\[Kappa]"}]]}]}]], 
       RowBox[{"\[DifferentialD]", "x"}]}]}]}]}], " ", "\[IndentingNewLine]", 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.8154923244115343`*^9, 3.8154925592119513`*^9}, {
   3.8154926382284307`*^9, 3.815492679898773*^9}, 3.815492710989706*^9, {
   3.815492742313916*^9, 3.815492809265806*^9}, {3.81549284700355*^9, 
   3.815492903136022*^9}, {3.8154929527955103`*^9, 3.81549298655193*^9}, {
   3.815493157768717*^9, 3.8154931909345093`*^9}, {3.8155041481881*^9, 
   3.815504172234705*^9}, {3.815563392700075*^9, 3.815563434718917*^9}, {
   3.817212958503401*^9, 3.817212983466419*^9}},
 CellLabel->"In[48]:=",ExpressionUUID->"8d63fd49-254f-4bfe-89a7-219d53d9ab06"],

Cell[CellGroupData[{

Cell["Plot Comparison", "Subsubsection",
 CellChangeTimes->{{3.815502694642395*^9, 
  3.815502697607382*^9}},ExpressionUUID->"fe322833-1152-426c-b436-\
6b15341cac6c"],

Cell["Compare entropies when the coupling matches the distribution", "Text",
 CellChangeTimes->{{3.815502771603779*^9, 
  3.8155028159155807`*^9}},ExpressionUUID->"07bfb0cf-1109-404a-9c53-\
998f1f5b23c1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotCoupledEntDist", "[", "2", "]"}]], "Input",
 CellChangeTimes->{3.8155034153571367`*^9},
 CellLabel->"In[59]:=",ExpressionUUID->"eb39eb5a-0b49-4cc3-98d1-c1afea678255"],

Cell[BoxData["$Aborted"], "Output",
 CellChangeTimes->{
  3.815503416944875*^9, {3.815503539300544*^9, 3.815503567626999*^9}, 
   3.8155036018981943`*^9, 3.8155036409629507`*^9, 3.815503790943838*^9, 
   3.815504129125773*^9, 3.815504419478259*^9, 3.815504668304594*^9, 
   3.815505049786837*^9, 3.8155060696823263`*^9, 3.815507190473363*^9, {
   3.8172078037677517`*^9, 3.817207810137926*^9}, 3.8172121740575247`*^9, 
   3.817226969712369*^9, 3.819666818127804*^9, 3.819667175207226*^9},
 CellLabel->"Out[53]=",ExpressionUUID->"9ad4f1c6-5c46-482c-995c-eeb162f8fd22"]
}, Open  ]],

Cell["\<\
Aborted because seemed to be taking unusually long; however, next result did \
complete.\
\>", "Text",
 CellChangeTimes->{{3.8172269815109863`*^9, 
  3.8172270176352177`*^9}},ExpressionUUID->"6dd421a1-2300-4c33-ac6a-\
c14ff51f04fc"],

Cell["\<\
Saved Plot to compare with Dec 17, 2020 update to CoupledEntropy\
\>", "Text",
 CellChangeTimes->{{3.817207676656767*^9, 
  3.817207692339222*^9}},ExpressionUUID->"a3818579-dcee-4077-8112-\
347e96aea0a7"],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    TagBox[
     {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], Opacity[
      1.], LineBox[CompressedData["
1:eJwVkHk41HkAh8fZhPEz7vhZVzeVUoyhvh/qUSupsHK0KlJsNB1EKzFIUZjx
tMRank3rGNkUkVRTLZYOipTF2OlOrshZmG3/eJ/3ef99TQN47kHyDAZjyzf+
d7j/2TX9PycSButPhS7pOFl8zzaFP51E4sTVM77vxonE/E2P7slkwugeC+jq
GyfOvQ5nHPlphMHYy5OMjxM1bmTYw/XniWDdhox3ahNEUrf4rolLHjHBCjbL
boJgoP5akEUJ4TAqeKXCCXKYr23JzK0i57n1D6w2TBKTpmfGtpYNhOEb1rz6
8RTRXfUpXmnJE1L2Keh+4vqvJPQUtyLcrJ08svgS7lQzTc7tYItbT3cSjcet
n1W9ZonsGtNOTvgvCUyduD4ukZF8Z+eNVMIrMnjdQP9FAwOFx0QCqdVbklET
f29Bnhy8+r5j55S/J9IT8wKlIfIIzfeRuez7SD7Grh5o/l4BFhcuzX8z0E/e
mSXaZhgporVN+EEcPETulIiF2UOK4BWZpTapDRMVY01jvedKcDmd6LjIbYR0
jOS9jLuijI6kpstxos/kR0mTcv5vc9BZtGt/NGOMvBK7VN9PZsKqL2btT+Tb
5zedu5SC58LOLfaBZ/YEaTfTS7T1U0FWG/OodfckSS7OtjODKoqiIlpZ3C8k
NEXDexVXDRbEmp8s/EqUzaqeiVayoLj4+Oapp9PEyWbAnGOgDoHVxUd7l80S
gV//iLE+hcN+HiF1h2XkDCd3U5CCBl63iUSvdjCw2++vI/RxDbxPNprerSiH
zUn2ydlTGtDZ7unn84cclp31iLM9wgZdPP1Li4c8Zn6t4F0dY0OSviVKVyaP
wijTsrhITTSdCQh+kacAt7X6fjbDmkhj91rruiqCzYvK74nQQkypNatlQBEV
YdyurSNaWPh8YZr3OSXE1VY6dYVqo0Rw9uMujjIy1umtOjiiDa+dlZ4vO5SR
UCtfmcPTQXe1W440aQ4YThFT0gkdpM4vG/BZwoRS9lv54khdTJpWfihoZEIk
rh0IHtOFwxVL8YGIuajJusj3OqkH0ZaQgjJdFcj+NkzNndXDSLReYFi9Chw8
otKNU/Qxyj34YOaQKpJ0KOMsrXkwUU0mTLYahm4FPnQ8Pw8LDugIBHfUsLyE
POeaGOCH7qDBjN0s+G+/KOdZYADS4z8aJGPh+qiryH6+IeQPNbVXlapDc3/J
U365IYY0J+v4rhRcbJTqG61pLI++cXvrVgqG5FSDlg2Ng6+jqozcKazmcFr8
OTT6K6aKanZQ6HhpPT7qQOOD59eU4QAKoUGhOabONKSZs9v2HKdgvmSN/Qlv
Gq0GShIUUohRqOuwiaHBTqhvVy+hYPTZXBwfS2Nb36nm7lIKCxzzLjfzaTTX
Kt+NukqhcrAnd18SjYc7mQXltygobotuzUynUZenGmLSRuFC5jHh5O80FOY8
2jPYTsGPb9q74RINJ94531sdFGLLOzcKC2ncJSxX7x4Kjf+EGywtpXFbqr5C
0Put3ay1d1bSmN7Ussi/n0KWu/2l4ioaDlfTTSyHKBzN3mw/doPGTb6GZuMo
Bd4w/2TabRpTvU9UMicoZCQULu0S0+C4CxX2fqHQ6vtEsug+jcib26dXzlAY
rZ7NCq+jUWWmOSaTURjqsvK510DjP/CTJdI=
       "]]},
     Annotation[#, "Charting`Private`Tag$11256810#1"]& ], 
    TagBox[
     {RGBColor[0.880722, 0.611041, 0.142051], AbsoluteThickness[1.6], Opacity[
      1.], LineBox[CompressedData["
1:eJwVyns0lHkcx/HngobB0xTDWgpLONhG01Ky/b50ajPJrcklGnLJJYVSqVY1
iFxiVFq1VjatyVBb6djNVMrBSboZpqxcyya30JUxZvbZPz7nfV7nfCwiEv2j
CQzDNtP7vymCvB/GD2ei9xNt3J1JCmT7wCVXqMxCa9oLPg7tVaDe74b62Edz
UOXWS8zI/Qq0YcTtpLuwAO2NSFy//bAC6boe3N227izad7t4yDOT/jfZ3jfn
lSGO/pJZskSBYKL5ZrR9FeKl3HqgvKdAyUIDB0ZpHRoc3iRgMueQeWvnUheH
FqTTg4vqxXOIvWIqXdPuOZK8yqkv5ShRwgnX2hRLOSJaLsc8vqZE+YGsBll2
NzLX3nimcPk8Ut9krMaL+pFVgHRxZ/k8urhhw09UxmtkUJ5v02KgQpUHJKIB
zr/IWenEDzmuQgFjS1gXrg8j4+Azv5X3q1DCxWA1b+coWmt4OLzPXY3sSy5b
DU2MI6P26NXO59VI1lH0riF2EnWFfpRQY2qUKLY81ao7jYZVDU8USzHgZWe6
23h/QKbO1lpv1mPQldVac1zyEYkWVmS578agWxwWcwT7jHw3lel4nsaAM5b2
Yzz6gr4d8BCm1mKw2vvYI/75ryi2NEHMfoHBLx2MfdyeGTS9JS2C8xkDcep+
mZ6rAkmb+29WGuJgj7jCnKI59LIw70TQShw0bA9tmm1Xoox1BoPzPjiIOJce
RzmqULtZZtuHBBySQ7bENSWrkeOvUy0G2Ti86ZBIXgdicEEa6fL2Ig7DOWbK
cA0ckgRpMzO3cTD044cE/4EDX3zlXl47DqZXlMXPthCQ4iTN8xnFobdwcypb
TYBDEvdsH05A68mI2JdlJBSvMhnkGhFQwBrhsr00IKPxEbPGkYC0aq7eswkN
KDkXodT1IGDZi2UFQfmaEPSkvODoVgKqRHmjYau0wFsuSdCMJyAg9BZ/sEsL
Jk0z3l75mYCev7wvDGQtgNmwlHi+iIBTVlcngu0Y8DrHzdrlEgEzFrfeVTxk
QF4Fh+dXS4Dbnw4Nu/ZrQ+p1zDmmmQDJ5riKq2wdWHPgmGavnIAPR4widzfr
wFvx79+sGybgk+ueR/NJTCg1Ge4f/UqAOTMHMVi6EJseWcxeQIL1LkOR6J4u
fKk+GGNjRMLWnuj3p8P1IH4q2+WpNQmoT/ApWq0HZTL/uW4uCURSq7yuWh/O
fJGeavQgYXLRTJPQi4IVzdrxob4kfH/k77s+PhS8sjJOK6G9501qnZk/BWJW
Sm4n7fHaWfHtQAqS9I3PevmR8I4/lzsdQQExYRvp5k/CwDmV745DFBhGcaZN
+CTITDR7oZKCbY29so5AElgZzXL9Kgos3XlOVBAJvmMnnvZUU1DVvjSXR/up
VOt+6g0KovK17R/QbgtlVFy/Q4GN2aTdtWASmsqYceYdFGyUNphmh5BALni8
472cAj2ppWUjbY/E/G13uigI91RazNO+j/S8gvookMWzWftCSbg7oL9cNEJB
o7WqRrCdBOXGZzaCcQrSMces87TdbhSaO0xSsNbon+BO2vXChYsefqJAy6tm
ylNAwuzIc51zXyngLZbXZtJe5V9ERiko0Ik7mtxA+2C9n9JpnoK9FlV2Ctp1
los+q9UUCBcG9KwMI+E/EVUcog==
       "]]},
     Annotation[#, "Charting`Private`Tag$11256810#2"]& ], 
    TagBox[
     {RGBColor[0.560181, 0.691569, 0.194885], AbsoluteThickness[1.6], Opacity[
      1.], LineBox[CompressedData["
1:eJwV0Hk01HsABXCNbYR+KHopQillaXHaHJ377fVSpFBKKCGiKO1EqsnSzuhJ
pU5e2xTlMKVR5FExiDdZUo5jq98YM8aSmbEMXt68P+6553PO/etaBkdtC2Wo
qantVeX/PhFwdUVvbCL+Cv3qMHRqGDbvV11hTSRjx0fhsayoYbTNE7abnL0M
jhe/xzN8GC4S50vrWCmIElW6V/sOQ88p+lDN+nQo9f8ombZWtS+3KbNwu48b
S849ozWGQfoqXobaZsO+nf51/M4QjrJm2DHv8bCLsUFtxxcFCpqCy5UyHvYO
5fKYAgVGF3F3S10L4RLaEPWhUoGzDVuuC0YKwQzOqNpYrMCVeRf707e/BWMy
wPbOQwUeVyjzLXXfwUzs98DjmAK9iQOZZiVlOBefl9JkrIBF9Ze5q+z4yGuN
XvI+Qo5rI5FRAm8+jNalVxmEyTFqrVkaGs/H/PZ2y5BgOeovrAhIF/DxYz1R
N/GV44Jzxr3BI5Xwy4jX4rjIQef5zHrxugqWHRYex63k4GS0GFmiBhv4Z5w4
rTKYLP95QXNRHWbG1K+86ydDiD8XUuc6yGMXO33eKQM38dhEnWcdFid7KTS3
y+D+TXHyXkwdrA8ms+I2y8A6owx3rKpDasWXvGRnGXr5DI+gsHqY1ttLTefK
8N7feHbJkwY4Xq+ZbSMaRGSS06sTVk0YNmB/cIsbxDUfw9KGiy34zSa1YG3e
T0y+ZK6ZktaBdCfHjyK9n8hycdlIJfzAW/t9fbE9/eCcymF3Lu1C/LZO2WZx
H3ZKzQ0z87thb2Ntvn9GHyKzfCfd9veA5RAhqN7UC9vbj+cL+3oRWGORtydL
iobGNHFp+ADGCvznF+pIEfXU6nq13iDWlWa2mEX3wO1i4rqFW2UQU2G75oxL
0Jxc/eJ8jhzsgT2S9+claHm6NyxObQjadmOHNk2XYKk0fu1BDENg9k9w4msx
1mw998n7zgh0Sh8MNPmIcauRedyxdRTmhl9cjbXFeBpzskHfaQwJ9EEHA143
bOHIupw2jk05zzICI7uhYXN6s7J+AjqzspK87brBXvqwNsT+F3ouxfucl4pw
1H/7gfKjk0D/a42ulyLQjTk5P3zUiPklX4eSsyJ0XzabCNSYQsIPX6llu4pg
7OXt7/tkCrkvbXNvNxVhzrOJm5+3M4jC+89MW2kX2lK3xJhMMshqIgjTLe1C
9aXg8G/31UmYvdFd09tdSDGUOJq4axAtgx/apkdUvz931P/cp0H03gmjue5d
WPB1Qcqua5qE+ynNKNu2C9nsqz17V2uRD8b9Jak6Xdi5u8D7e7MWGYtrNt4g
FaK1cGtmZ7I22dnPaVPWCHF9fm6f7yImsR6/4z87X4hRywLxoyomUXS61o7c
FMI5z6404qQOqf71lW6JFSJny4FHuSZTyQptidnjYCFkcTP3HaqYSvr2M4oT
XIVQOB3+9O8RXbLkZYhe9zIhLHQvg2moR4o6luHjHCGsI4zZ7L/1SEdQ1mip
lhA7WkP7bwTqE29zH5manAbaAxShk/rEsbvgln8nDcaR6ibe82lkzvq7Od9r
aQwYjZaz3CnSmGUcF1hMwyHuTYmHB0VqRSnF01U+TMfwzLZRZPbplczKIhq9
r5RP3/pQpGGKJt9eZbH3+JXBYIpwgjwax9/Q6Mz45Rl0miLcNabLM3k0Gkw1
2wiHIgJuSW8Hl4ZhQkXTtGyKODPO56ar7ClNErQ+p0i4v0msq8qCYq2yGC5F
lpsWrHyVT6NmN/NR/juKTL21f+3FPBrl93UPWDRSxKiSnF2WS0Nduzaov4ki
nh+VY6IXNH6Puub3rpkibTK/hHsql0HffVc7Rcxm6BZqq1zSOW0JW0KRT5yv
2e05NCY2fV4Y0EuRA+eSEtJVduamWtgNUGTfID/cTeUiloFRlUL1Bz28g5dN
Qympm5oxQpEs7c49kSqv3pamHjJGkXRdr6NWKkcXeU0s+5ciEes3spuf0eBZ
GQ1NTlKkTL2sKEXl/wDVJvOa
       "]]},
     Annotation[#, "Charting`Private`Tag$11256810#3"]& ], 
    TagBox[
     {RGBColor[0.922526, 0.385626, 0.209179], AbsoluteThickness[1.6], Opacity[
      1.], LineBox[CompressedData["
1:eJwBMQLO/SFib1JlAgAAACIAAAACAAAAdF6LNex6hD9nlcWvbBv3PyjBN4qB
/IY/YNHxQico9z/dI+TeFn6JP/Gs0bTlNPc/Ruk8iEGBjj/kNZU2bk73Pww6
d23LQ5Q/2Cl7v62B9z/dxCjAIEqePxnoxVDn6Pc/QO3FsmUrqT/eMQS5R7r4
P3GBFCwInLY/Hu48UNdo+j8gydQfNyzGP/ZipohjRf4/FjLwggUp0D/F5ozG
7xIBQGyFOrN0ItU/XqJxh8EaA0CMVxG/0ofaP4x+fxqxdAVA/7IIOQGR3z8W
zXba1soHQJ9GRkcPg+I/JELHGGZ7CkCmdqqQ4DDlP2WV0GPXSA1AVuseEZqw
5z/OYOtRzAUQQGyfWf9KZuo/BfkNw7eZEUArmKQk5O3sP6q5Xkp+JhNA0tOR
6L9o7z9WEihMH74UQG+nIo3JDPE/k43GqYGNFkBKh4RBJ07yPxZaNo7gURhA
2IbJrICq8z8E5kBCOlIaQNqnX2d7APU/06K3xKdfHEAw6309aj/2P7O2zIXf
XR5AOU5/ylSZ9z9ZElPQuk4gQJfTCHMz3Pg/C0+8wddlIUCneHXSDTr6P4So
D+DLnyJAKz8zgYmR+z+RdzUHwt8jQAQoeUv50fw/6ImKeckUJUCQMKLMZC3+
PxBOLuZkbyZAcVtTacRx/z9TEs8J670nQOPTqqriVwBAkZM9lSYQKUDniR38
YAQBQAIR4zGliipAwggWreiAAUDYNgOoVKQrQLDx/dI=
       "]]},
     Annotation[#, "Charting`Private`Tag$11256810#4"]& ], 
    TagBox[
     {RGBColor[0.528488, 0.470624, 0.701351], AbsoluteThickness[1.6], Opacity[
      1.], LineBox[CompressedData["
1:eJwVkns0VAsbxmdv91y2Gcz+cskklUoSKvnofdNd45JUlISk6FQUCSki5ZIo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       "]]},
     Annotation[#, "Charting`Private`Tag$11256810#5"]& ]}, {}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{False, False},
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  DisplayFunction->Identity,
  Frame->{{True, True}, {True, True}},
  FrameLabel->{{
     FormBox["\"Entropy\"", TraditionalForm], None}, {
     FormBox["\"Coupling (\[Kappa])\"", TraditionalForm], None}},
  FrameStyle->Automatic,
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{Automatic, Automatic},
  GridLinesStyle->Directive[
    GrayLevel[0.4, 0.5], 
    AbsoluteThickness[1], 
    AbsoluteDashing[{1, 2}]],
  ImagePadding->All,
  Method->{
   "DefaultBoundaryStyle" -> Automatic, 
    "DefaultGraphicsInteraction" -> {
     "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
      "Effects" -> {
       "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
        "Droplines" -> {
         "freeformCursorMode" -> True, 
          "placement" -> {"x" -> "All", "y" -> "None"}}}}, "DefaultMeshStyle" -> 
    AbsolutePointSize[6], "ScalingFunctions" -> None, 
    "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
        (Identity[#]& )[
         Part[#, 1]], 
        (Identity[#]& )[
         Part[#, 2]]}& ), "CopiedValueFunction" -> ({
        (Identity[#]& )[
         Part[#, 1]], 
        (Identity[#]& )[
         Part[#, 2]]}& )}},
  PlotRange->{{0.01, 4}, {0., 13.820958376304972`}},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Ticks->{Automatic, Automatic}]], "Input",ExpressionUUID->"b535a6d9-3fc4-\
421c-87bf-f06697449221"],

Cell[BoxData[
 RowBox[{
  RowBox[{"PlotCoupledEntDist", "[", "\[Alpha]_", "]"}], ":=", 
  "\[IndentingNewLine]", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "coupledDist", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"coupledDist", "=", 
      RowBox[{"CoupledNormalDistribution", "[", 
       RowBox[{"0.", ",", "1.", ",", "\[Kappa]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Plot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"CoupledEntropy", "[", 
           RowBox[{
           "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
            "False"}], "]"}], ","}], "*)"}], "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"CoupledEntropy", "[", 
          RowBox[{
          "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
           "True"}], "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{
           RowBox[{"TsallisEntropy", "[", 
            RowBox[{
            "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",",
              "False"}], "]"}], ",", "\[IndentingNewLine]", 
           RowBox[{"TsallisEntropy", "[", 
            RowBox[{
            "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",",
              "True"}], "]"}], ","}], "*)"}], "\[IndentingNewLine]", 
         RowBox[{"CoupledEntropy", "[", 
          RowBox[{"coupledDist", ",", "0", ",", "1", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
           "False"}], "]"}]}], "\[IndentingNewLine]", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Kappa]", ",", "0.01", ",", "4"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotLegends", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
         "\"\<Coupled Entropy\>\"", ",", " ", "\"\<Coupled Entropy Root\>\"", 
          ",", " ", "\n", "\"\<Tsallis Entropy\>\"", ",", " ", 
          "\"\<Normalized Tsallis Entropy\>\"", ",", " ", 
          "\"\<Shannon Entropy\>\""}], "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotTheme", "\[Rule]", "\"\<Detailed\>\""}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<Coupling (\[Kappa])\>\"", ",", " ", "\"\<Entropy\>\""}],
          "}"}]}]}], "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.815502740004176*^9, 3.8155027661131372`*^9}, {
  3.815502821364442*^9, 3.8155031419003677`*^9}, {3.8155031802716312`*^9, 
  3.815503397030766*^9}, {3.815503463374557*^9, 3.815503496664483*^9}, {
  3.815503544690342*^9, 3.815503545687908*^9}, {3.815503586241334*^9, 
  3.815503650972384*^9}, {3.8155038216707163`*^9, 3.815503854605308*^9}, {
  3.815504219144484*^9, 3.815504225222538*^9}, {3.8155044807817783`*^9, 
  3.815504514883499*^9}, {3.815504887880929*^9, 3.815504891746828*^9}, {
  3.81550513661983*^9, 3.815505229223941*^9}, {3.815506359699359*^9, 
  3.81550635976447*^9}, {3.81550653122783*^9, 3.815506626535421*^9}, {
  3.8196669477429*^9, 3.8196669509147043`*^9}, {3.819667566088126*^9, 
  3.819667581810585*^9}},
 CellLabel->"In[58]:=",ExpressionUUID->"efb1ac77-5079-45ff-9367-9207616a8aa3"],

Cell["Plot with all entropies taking root", "Text",
 CellChangeTimes->{{3.815563577915373*^9, 
  3.815563587743211*^9}},ExpressionUUID->"e97a4ae0-f75b-400b-97c4-\
c662de408480"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotCoupledEntDist", "[", "2", "]"}]], "Input",
 CellLabel->"In[67]:=",ExpressionUUID->"9a599a6b-74d4-4a10-b1ee-c2ac2fa298e4"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVz3s0lHkcBvD3gjB4m2K0DsUs4VCNnTZx7Pl96dRmmnJpXGZplEsyKWwq
1VJTE7nE6KK0rWyqYaitdOxmKmVxkm6GKVtulc09lIoxZvbdP57znM85zz+P
Q3RScByBYdh6Ov93qij3+5H9UjQ52sLdmqxBLg88cyTaTOTdmv+p72cN6vq2
r5uVkY0uh1xkxOzWoDWDPsd8JfkoPTpp9ab9GmTmvXdHy6pTKOX26T5/Kb1v
cLlvzytBSy0WTpNnNQhGG2/GuVUgfuqtB9p7GpQisXQ3Pl+DuvvXiRiMGWTf
3L7I070JmXTislr5DGJ9N37Y0PU5UrzOrj3P0aLEo97VqWw1wpsuxT++pkV5
Ycw6VdYrxDZZe7Jg2SzS3zT2wgt7kGOocn576Sy6sGbNj9SRt8iqNM+5yVKH
ruxRyHo5/yIvrYcg4pAOhQ4vZJ673o9shCd/K+3RocQLQj1v6xACq/2bu331
yO3sJce+0RHEao3zWlGsR6q2woG6bWPoZeQnBTWsR0ly9vFmswk0oKt7olmE
AS9L6uu84SOyXeFk9G41Bh2ZzVWHFJ+QbG5Zpu8ODF7Jo+IPYJ9R4LoSU/8T
GHCG038Qoy/IrtdPklaNgdeGg48ExV+R+HyinPUCgzNtxru4nVNoYmN6NOcz
BvK03Spzbw2609hz84oVDm6IK8kunEEdBblHw5fjYOCyb910qxZJV1m+mQ3A
Qca5+Dh2iQ6p7KQtHxNxSInYmNCQQv/6dbzJMguHd20KxdswDIqVMZ7vL+DQ
n22n3WyAQ5IofWrqNg5WQYII4WUcQuTl93JbcbAt155+tpGAVA9lbsAQDl0F
69NYegLck7mnunECmo9Fb3tZQkLRSps3XGsC8pmDXBbfAKT1jxhVSwhIr+Sa
Pxs1gDNF0VozPwIWv1icH55nCOFPSvMzQgiokOUORa00ggC1ItFQTEBo5C3B
mw4j+GB75H35LwR0/rnhXG/mHNBEpYoFMgKOO14dFboaw9tsHyfPiwRMOdwa
KHtoDMfKOLygagJ8/nCv277bBNKuYyviGwlQrE8ou8oyBe89Bw271AR8PGAd
s6PRFN7Lf/9mVT8Bk947H80mM+CcTX/P0FcC7BnZyJhpBgmHY06z5pDgtN1K
JrtnBl8q98Y7W5MQ0hn34cRmcxCPZ3k+dSIBdYsm4/TmUKoKnnnFJYFIblbX
VFrAiS/K4/V+JIzNm2qQ8CngNJqIIwNJWHrgr7sBARS8dlyQfpb2zndpNXbB
FMiZqTnttEeqp+W3wyjYabHgFD+IhAHBTM5ENAX4qEuMTzAJvUW6wC37KGDF
ciZsBCSobAy74AoFwvouVVsYCcwjjWqLCgrYvjwPKpyEwOGjTzsrKShvXZTD
o/1UaXQ/7QYFsXkmbg9ot0Qal12/Q4GL3ZjrNSEJDSWMBPs2CnjKOtusCBLI
OY+3fFBTYKFks+tp+yXl/XSng4IYf63DLO37yJwf3k1Bq5jF3BVJwt1ei2Wy
QQr+dtJViTaRoF37zFk0QsFhbElmMW2fGwX27mMU+Fj/I2ynXSuZO+/hJAWG
/KpxfxEJ04PPTYu+UsCfr66W0l4ZXEjGaihgJGSk1NHeWxuk9ZilYJdDhauG
dg173me9noKMuaGdy6NI+A9mLRy1
          "]]}, 
        Annotation[#, "Charting`Private`Tag$2022641#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVynk81PkDx3FHk0n4xsZuWtZRaFXEtoTd96dfsW0pisiRK9qKiFQ2Xdpi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          "]]}, 
        Annotation[#, "Charting`Private`Tag$2022641#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVkHk41HkAh8c9jvEzw89W2MUKlZSmLTzZ76faVblSbFhHUSrJVXJEjhSN
xPCUpGKjHNOxI9amclRaTTrkaG0PrZIcmTTlZpq1f3ye95/3ed7n+RgFhG0N
lGcwGM5z+5+Rfid/GD58jHDdqn42z5km5vdWpyfPppI23wIrUd406f7+3Wvd
BB65uDE0L6hwmtgPrjmxNjmTLE29+bLs2jTRsI0OaV5/mmxyjqeNmub8RvMG
Q4cC4i22z5+cmSYQP7wZuKScEB3/2vWBMyQiWceCeaGaLG4am3RYNUsMRe3f
rbb4iyQqhFtHjEqJ7opPR5UWtZC+gxWpCRkysv+4bWWkcQfpt212D0lnIMOD
Xd+a9opUU/MNjLzlILvJtJHL/pcs80+0s1sgj0J7+w1Uyluy6bqoPKtPHiVR
An7P8j7yQlEs5ZUrYNuHb9n5wn6iPXRjaJ2vIvYXeskcdg+RuAs9XyX6SliS
d9nknXiY3Hc7o6fyVAmtbdkD9XtHSKz0gVgnWhlhpcanRBoSUqLryO20UIFD
2rG1Zi6fCYdu3z3VpoLOVNG1JMEX0iHxiMxMZeJV6fY9cYwx4uYab+THVcXy
D0fs9pFx0tRutiWgVRU2LomP3c9NEPlP1jbn4tVwto15kNs1SezmC3NC56uj
NOZQK8t2mgzUVVbZ3lPHEsJN5mXPkPfjJuve7NSAonms49SLWdLOO27ZMqUB
/vKiJ7uWfiWFA4yYmHwWIrzdghojZMQoiatSZ6mJ3jaB4K0HA1Njt2JP1Gmi
n2cwu0NRDuOHs/1aPSnQW9y9va7I4byHcxH6KeiXzZ557iaPTSZPL0qJFrqz
nGN0ZXO/gnw2S9eC6ETA3r8LFNAgSV9R9EwLmexBrq6TIlJ6cxLM5rFx5CqX
9VysiJrcsV49HzZMX5pmemYo4YBj7Su9AjbK+SeHtlsrwydkXjqrj41tPlXu
bzqVEeyq/9MlUw66/nTJ70lVQfGGlRKtEA5OmVwXey1iYua+mZ/dDQ4mjaoG
ih8xYdpo9SBXzMGa3y3qgw+pYtfI0e4qK20InIOKr+uqIURf+M+dcG18jvtm
Z8hDNcwbTaw5WKWNUdvQx9JwdTDq9+/2GdeGoTqPMNka2Oez0teDq4OFwTSf
X6eBgImCKHGkDn7pCvyYs4MFYUzOiKBCB+S132igjIV446joB8M6kA8XdVRf
1cRCoaQtwILGCGeyMdmJguU+6YTTHhqWcbdqN2+mkHP6C8ttL43Q3phqg60U
DNq363kF0RiunCqt8aCwo/KifmAwjQH3mXRJAAXHF4234sNo9OR+dfWPpWBt
EOYpiKbRukCpGyUUkg4U2crSaLBTHnZollOQDnZBkUfD9cPxZ11XKbSsL/lR
NZ3GszvKDTEVFFKEA4baGTSafZjFwrtz/b61SaZ8Go0F6kGGbRSW0hKBYx4N
BZUn/h87KIB/aLHrORrrwjJ+vds51zc/c8k9n0YDYTl5vqZQUlMW5XuBRm2P
5jL+IIX3Javuhv5GY3bjczO/YQoCl/LJA5dorKnIMrQYofB0Ya1FdBGN28la
nEejFFyCmuMSL9OYGmxRy52gMOTZlJtyhYb11myFXdMUwhP2lKeV0Ii+vWXW
SkrB5vy1ypOlNKqNOWMyGYUj4Wf/yCqj8R93DSaU
          "]]}, 
        Annotation[#, "Charting`Private`Tag$2022641#3"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.922526, 0.385626, 0.209179], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVkHs0lAkYxufu+4SvcTRtiia5VGyJdYoj75uNWqRccumCRFGkkcqlkRHG
pYStlESbENKS0mYpFdXQYTJrt+SeiiLVUoxh1v7xnOc85/z+eM5viX+YayCD
RqNtns3/HeGTZjEcnQDtmq3l0QI5LHu4JlWkSAL/+rwruhFy6Fo60M2LTYH0
YpuiR0flYD9knbxelA48t9FB1eNyULM6Ftr881lIjtW0y02a5RuW1fMd8sDW
YRX/0SU54EjjrUDjEhgOmcj74YkcBCItEyK3GuI3Rt/q154CvuSvxWtMnkDl
6qZXdW1TwDP7HM9eLgXraHujAqECQhKtqiL02kEaVTdSu3QaTnlyH7SJOyAl
wk52r30alLcIS3pmD+j2yQaCo2Yg395+I3WyHxxTk1sGjZRQdLQ0o9f0LcS8
u3bFUKoEj4+63JyK9yBOGB/St6BhSL630mHvB7AIv7vOVExD4wvX9AdGhmHu
c4PDu9po2CbLHHwQNAoGO3OZj7XpGFasd1qi9gW8fBcUDeyjo4M4Yb2R81co
eislsm7S8WWS5EZc6b/gs3D89YtJOnYU++6LoY2DwFLC3WXLQNOPwnX74Rv4
dl6e25vAQEvnE03uF79DeV2jl7SZgdky4rB55wTUuPBVv6gxsTjySJu6lRxy
jy3c4OfKRGMwF6VkTsHVvoKZQ+eYyFoW5Tj5QgHBm651aciYmGF69XnAjzNQ
Et5v163FQsEOt+AGgRJ43q+fBLuw8I2stLTfk4Yr/eInxtJY+D5FR+HHoqPW
59C47CYWznNx3+FdSMeeXLNmYLFx0XXFuVY3BrroVPW9X8/GrjObI3lKBt6s
qCvLF7JRkuwf9E8eEzN1437Nv83GdO6QOc+JhRY6XjfEo2wUlpmrt46w0Kmj
3P6VAQcN/zZM9zrFRo+mReeF2zlYkpH2wXctB8f2GtuMZXHQY+dt976XHKxM
zBGRTznYedc5pzdJBQv1JB1dMxw8rV8+4r2cQMY+u/gQMxWcWHJ7sOAZgTbE
xcLwABW0/t3kwYEjJAb179fXz1HB0s3BBeU8VVwqkUfWNqvg15j5e0IbVfHZ
Y6exTzQCx6wONk0fmoOyS4+y838ikD8nBQiuGjo7X2L7BRBocGBeRsZ9NeR5
ynZdzyZwW2fgpyw/dXTcYwfznhII3T5jgUp15F9d9Gb5+OyvQ5L26jINlOmd
EN4xJHFUc6JB5EShdg9xWexO4sqYP+q2bKHwytc4N7dtJB58E1mt40qhda0t
Y7EHicNVk8X3PCk07CnaVO1J4qD7VOoXfwoDPcmoge0k9p6f2bo7isIFZt+C
YDeJbdrsLiyiUL/CL200lETuycZ2jRIKU87ecao5SOLWj4ktnWUU2mwJ4ySG
kdjyJ6c+spLCmd+09mgLSGzeSRRU1FJYusLm4YYIEhvy5gTzZRRK7z/lXogm
kanyfPendgoFv7R4+8eQaBt2anvtSwpr6BtzTI6TWA/qTl7dFNY7ZrEeCkms
69VYlTFEYXl4cexQHImKTa1GPsMUDmR7pFaJSLSuPMM3GaXwcHJsujCexBrR
XM1nYxSaz7eO4yaQODkkVT3/nUKjHbLQ17N7rWsmM0A+6+vOO7fCRBKP1bgo
Vk9TuGLyqFlYEonVeprjSuWs34kE0lJM4n/75iM5
          "]]}, 
        Annotation[#, "Charting`Private`Tag$2022641#4"]& ]}}, {}}, {
    DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
     AxesOrigin -> {0, 0}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {False, False}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, DisplayFunction :> 
     Identity, Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox["\"Entropy\"", TraditionalForm], None}, {
        FormBox["\"Coupling (\[Kappa])\"", TraditionalForm], None}}, 
     FrameStyle -> Automatic, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, GridLinesStyle -> Directive[
       GrayLevel[0.4, 0.5], 
       AbsoluteThickness[1], 
       AbsoluteDashing[{1, 2}]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None,
        "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, 
     PlotRange -> {{0.01, 4}, {0., 5.414610707634094}}, PlotRangeClipping -> 
     True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{
      "\"Coupled Entropy Root\"", "\"Tsallis Root Entropy\"", 
       "\"Normalized Tsallis Root Entropy\"", "\"Shannon Root Entropy\""}, 
      "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.560181`", ",", "0.691569`", ",", "0.194885`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6150173333333333, 0.25708400000000003`, 
                    0.13945266666666667`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.922526`", ",", "0.385626`", ",", "0.209179`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.922526, 0.385626, 0.209179]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.922526, 0.385626, 0.209179], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3, ",", #4}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.815564405480465*^9, 3.817213159439478*^9},
 CellLabel->"Out[67]=",ExpressionUUID->"5d9ddeab-f152-46d6-9c3c-a5cb6bf8be85"]
}, Open  ]],

Cell["\<\
Saved Plot Dec 17, 2020 
\tnote that the Shannon Root Entropy mistakenly had \[Alpha] = 1; should have \
been 2
\tand Tsallis entropies did not have the (1+\[Kappa]) term raised to a power \
which is required\
\>", "Text",
 CellChangeTimes->{{3.817207716747178*^9, 3.817207723401729*^9}, {
  3.817208206251073*^9, 3.817208233829035*^9}, {3.817220877186275*^9, 
  3.817220915040614*^9}},ExpressionUUID->"228f4d5a-2b94-477b-bb93-\
b29f3429e8ea"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVyns0lHkcx/HngobB0xTDWgpLONhG01Ky/b50ajPJrcklGnLJJYVSqVY1
iFxiVFq1VjatyVBb6djNVMrBSboZpqxcyya30JUxZvbZPz7nfV7nfCwiEv2j
CQzDNtP7vymCvB/GD2ei9xNt3J1JCmT7wCVXqMxCa9oLPg7tVaDe74b62Edz
UOXWS8zI/Qq0YcTtpLuwAO2NSFy//bAC6boe3N227izad7t4yDOT/jfZ3jfn
lSGO/pJZskSBYKL5ZrR9FeKl3HqgvKdAyUIDB0ZpHRoc3iRgMueQeWvnUheH
FqTTg4vqxXOIvWIqXdPuOZK8yqkv5ShRwgnX2hRLOSJaLsc8vqZE+YGsBll2
NzLX3nimcPk8Ut9krMaL+pFVgHRxZ/k8urhhw09UxmtkUJ5v02KgQpUHJKIB
zr/IWenEDzmuQgFjS1gXrg8j4+Azv5X3q1DCxWA1b+coWmt4OLzPXY3sSy5b
DU2MI6P26NXO59VI1lH0riF2EnWFfpRQY2qUKLY81ao7jYZVDU8USzHgZWe6
23h/QKbO1lpv1mPQldVac1zyEYkWVmS578agWxwWcwT7jHw3lel4nsaAM5b2
Yzz6gr4d8BCm1mKw2vvYI/75ryi2NEHMfoHBLx2MfdyeGTS9JS2C8xkDcep+
mZ6rAkmb+29WGuJgj7jCnKI59LIw70TQShw0bA9tmm1Xoox1BoPzPjiIOJce
RzmqULtZZtuHBBySQ7bENSWrkeOvUy0G2Ti86ZBIXgdicEEa6fL2Ig7DOWbK
cA0ckgRpMzO3cTD044cE/4EDX3zlXl47DqZXlMXPthCQ4iTN8xnFobdwcypb
TYBDEvdsH05A68mI2JdlJBSvMhnkGhFQwBrhsr00IKPxEbPGkYC0aq7eswkN
KDkXodT1IGDZi2UFQfmaEPSkvODoVgKqRHmjYau0wFsuSdCMJyAg9BZ/sEsL
Jk0z3l75mYCev7wvDGQtgNmwlHi+iIBTVlcngu0Y8DrHzdrlEgEzFrfeVTxk
QF4Fh+dXS4Dbnw4Nu/ZrQ+p1zDmmmQDJ5riKq2wdWHPgmGavnIAPR4widzfr
wFvx79+sGybgk+ueR/NJTCg1Ge4f/UqAOTMHMVi6EJseWcxeQIL1LkOR6J4u
fKk+GGNjRMLWnuj3p8P1IH4q2+WpNQmoT/ApWq0HZTL/uW4uCURSq7yuWh/O
fJGeavQgYXLRTJPQi4IVzdrxob4kfH/k77s+PhS8sjJOK6G9501qnZk/BWJW
Sm4n7fHaWfHtQAqS9I3PevmR8I4/lzsdQQExYRvp5k/CwDmV745DFBhGcaZN
+CTITDR7oZKCbY29so5AElgZzXL9Kgos3XlOVBAJvmMnnvZUU1DVvjSXR/up
VOt+6g0KovK17R/QbgtlVFy/Q4GN2aTdtWASmsqYceYdFGyUNphmh5BALni8
472cAj2ppWUjbY/E/G13uigI91RazNO+j/S8gvookMWzWftCSbg7oL9cNEJB
o7WqRrCdBOXGZzaCcQrSMces87TdbhSaO0xSsNbon+BO2vXChYsefqJAy6tm
ylNAwuzIc51zXyngLZbXZtJe5V9ERiko0Ik7mtxA+2C9n9JpnoK9FlV2Ctp1
los+q9UUCBcG9KwMI+E/EVUcog==
          "]]}, 
        Annotation[#, "Charting`Private`Tag$19730943#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVj3s01HkAxSehSfiFpZNe6MWhUiq06n61LZtCRUlZpLQpMZUek01LCiHD
1mxZq7OpaLAeifJOw/ITMRKJTDMqDCYaxkTN2j/uuf98zrmfa+wXvMtfhcFg
eE7l/w7xjl03cD4S7IAMPbUxBUyfWl8Nn7yCsiLDp1V9CnQt7nlrEBaDaoWD
JLZLAYc+u2j78GvQnzY3hFQroLnh7PH6H67DqedMkuWNKZ5vWmnklIp9Afbf
L7JWgAxW5/ubP0B334E3DRHjOBH+nQUzpRCC6VzL8PVyFLT68RUjhahcUZDZ
YS7HuFmel2RrEULKaqs2GssRJnCOb5QXwfAkL8FIU46ri6OGrrs9gc37oHUe
ojHcrVbkGs8qRcS8J9EXOGMYiJQmLyirhEG+d47n51EY1b1cZG1RA7G0tnaw
SYY4eWBwo3sN3HisiZ5aGcaXqlX4X6hBjWWAtahShuaIdd7XG2swsf1izlCu
DBF23JRh1r+wcmUIXRJlEOd4zM16VIu/5Uq/ZW4y3Od26BqjHr5h7Blqbz7D
YM2nCDWzJhQUL8yz/jKCwMsbHoaYtGJrXHLnQMgw4jx0KgRRHTA1OyJx0PoE
ZT7TdlpiN/ayV2adaR/EbQcHR+qSCN1BKuyTRRLcP8PjCC3fw9bCdCxV3Ic9
koU6ybkf0ceSC3t9exF421PpdLgftn8pQ0f7P8D85t0lPYMDKCyed5aZ/x6C
lsTeiiNSBHPoXueUHgSnm8TXaQ4jRrqsQjtWDKeoSPvlLiO4u7CudEaaCO1X
6rJ+433GswBF2c6yd+hI9/kllDGKpcLg+6ZSISwlFzYexRhCt4TYGG8Swtbl
Iu1+Sw65m8i8ZG43/mhhnrLqHAf7UOTj6I9dSD93WqC14Qt+NDjU3fCwE+aw
Co9JnIBJeardpT/fQNWUvU3RPIk1vakibnoHOJZ3nh9a8Q2SbTXOwbmvcWK/
WwD/hBJ3yg19WO3tELfweCIPBuF1MblOOu34GLNg0ld1GnH0M9qzxKEN+jvd
93vem0aKeiyPpv/+CvMzJm+8cFMhHIbjYceuVnQlOJ8zUKqQ6FN+Jy6vbUVd
tN+RttTppE1D727S7Ze4ptNnZbBdldxqz+yPV3+JC5lWWi8GVYlLJtt8/sUW
LHu17NreODUSuKrH7hmjBQ84sf0+NuqkbCZLti9MgD1eBe7v2tVJSL6yP0pD
gM4il2ThlRmE+2y1z8ekZsQvyR70NGOS5IXqVodNmzFuXNCbVsskvJfD0hh+
E+xyLCqOnZ5JxK4Jld+8msBzDkjLNtAgqxotNicwmjASOufg8WoNoqjK2fSz
/QvINgTRX1mziElUQzWJb4TRrBgwdTTJzLbBar2BBiw9ps/hlGsS/Yy2Evct
Ddjd6T+U5KtF+De9RNLs58Bbb5m/UosItPMClIueQ4VV11qYqU2YcfMlzbH1
kOqO88O3U+Tg8AGLx+r1WBn6uMzVlSITvFE+S7UeQeJzhQt2UeTBipQSM5V6
DDxUpD/xoMij1tqqlK80et0nrg77UYRvsm59lIyGkPttxwE2RVQy9FKPi2gI
DNW6yH2KLFUavr5YTkPnUnWr9gOKBN25NnNLKY0dksuNnZkUeWfhdYNZTKOx
RL3yXB5FtG/llCc9olHvxUzLLaVIcIZ4d2YWDX7qrACjForoN8guDSbTmD7j
+YGh1qn9XXq+RTdpbA6O21faTpFjNrbccC6NSmht3/uWIk8POs6ek0SjTKi9
itM35eOzxNYhhsbkTy+Wew9QZKhzrVg3ioZdXoKRhZQiDsJ7H7ojaRSHz9at
lVFkJGKP4vxvNBR9TRpc+dT/okDm1jAaNrsSpx/6QhFpCoc151caZ4t3Tq7+
SpF58dmrP7BpFJrojiqVFPna+8+2R2dp/AdOI8bx
          "]]}, 
        Annotation[#, "Charting`Private`Tag$19730943#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV0fs31HkAh3F30xjzZTChaIzLfC1FqeOSPZ93tatdiWRKqq0IIZEQpTZK
Si7RWq224xI10Y2yQyLapVJyn7IdUy5dCBWrZIxmd394zusPeMwCItcHqSgp
Ka39r/+N2Za2bPRgMpHWMfYZVMoIfc/xVJI8hVgtza8evCsjUvNXL7g/p5Kc
kguvbzbLiNuw68kVSZkkZNBp3OeljLBc4vY8XpVDbK9G/VnInCHSRrqB555P
nheMjwb6zxCMNd0MsikluUW7onMoOYlK0rdlnBeTRC/hxwMRs4TX3L3A0fY+
CXEaMWQ7K4G75ONRdet2YuiZUhcmUkb4cZdbMXwJMW+o8G7RU0W6r25954nn
JG1/zjehJWpQ3GQ4K2e/JMuvb3o5ZqqBAje31dSxATKyQXq0u04Tl/aXZfXZ
vybqrmGjuX5zsHHEVPdc+VsyeS/eZ7uUifACP4V78Dsira3ii46zYPNbicWr
sVHyoCD5KdeAjc6u7KH6kA9kzkfXeUUFFCJF/Ixm1jgpH3BaEi7SgfuJ5BUC
zwkylBGYoojSRU9K89XEsn9IURjXUsOLg+ei7bsSlD6RpbaGWwOt9WA/cvjb
MPKZJLwV9jt+1oOz55FHwrwp4lI/0H1Woo+zXYxoh94vRCI+tHuhyACi+NhO
bRcZ8Y7khIWmcWFDHJJSs2fIZUc+Jzd8LtToA2umO+SkOvJNcC4xRJb9hZbA
hV9JjLjfZIOlEaK2+IQ2RilIOavQuEpuhMGusrIBXyXElsfGXm43xttUE/kO
NWXIC0s/RVTNg4G3cIvfRWVEiYWtmafnY/5l+a9tPirI42rZ5R00gfT02niu
QgXGbyzu8wJM0XwyIORZviqcDsucG1wWIFN32IHroYacV+0Fdno8HL7ioN02
poaJ1rYbB5N4sHpqlbkpXR0Xdx6JGxzjoTQr7d12Jw3I1Lgdkp1m2Li1Utjf
o4GegaJp604z9FZ5nutL0cTi61ExOu58ZFhcG/OzZqDi72i/NTV8fDGrHCp+
yIBnVXFbNm0O1xu29btj//t8Z1VQdKE5ytaGFl/jMrEiZiEnR8cCEwlzd+5p
YmI0+my+7ykLTLpEPJrdqwWVGi0tcxVL8LRSCUOXBd/g83rG+yxhudsgK+su
C1orj7FqRyyxoTfo/Zkd2mA3fm068pMVyIttk0EKbdjLh/Mcn1hBZW+zRHyF
DeM6YXeRmwAfOF8akzwofLc8s82kWoBFCdV1Xl4U/tILnjCsESBiMF5ssp7C
hYwmQ/1aAUZvTYtu+1KYU260l9kgwJBw5tR4AAVxHfPHqQcC9OV+Xed/gILZ
9fLijmcCdBqrS3GJQsceg+mTUwLoHmuSsEspnFGw05OnBVg3cry19woF56tH
+YkzArTe0WiIr6CQWPbUL04hwOOtjOLyWgol6ZYTQZo0GvO1QnldFDiVys2r
5tJQ1Wzxfy+hcEsW9gsxorEyMn1zbQ+F23+s9l8+j0YD0fbY9ILC/AE/TYcF
NOr62HZZwxTepEgO8QU05D+0CbaNUggebA0wtabhWnGaZ/uBwqIQJw9jGxo1
STqch5MU+r/fbM2xozE93M7MnaLwe4uuCXsxDaf12aqBMgpezxz1mQ404mq8
5YtnKejMPmZrLKMh5nM+KRQUNCVPWCqONP4F3nodPQ==
          "]]}, 
        Annotation[#, "Charting`Private`Tag$19730943#3"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.922526, 0.385626, 0.209179], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVjXk41HkAh8e4xvkzw8xvN4lcqShFB4/6fmQf7cooV2Y6hMWalCuVI9Vs
iEi0olpbbQhTKhI6pDYq6UlYGxZJWuRoKeNoarb9433e5/3rnR8Q7hHEZDAY
/K/872jftBUjcYnEZU7aI1a3lFg8WHVMLEsmkpK9pZw+Kek26e/hHUwlIqcn
AoMBKXEeckhxFGeQ4v68WdtxKdG037+70SmbHA4q27ZbeYp011ncN3I5R9xP
1Jkzl0wRjNaXBy0uIT/8/sqk+tAUiRTrWbLyKklpXtjxVWbTxKjhT8NVlo9I
y62oDlH2DOEt//dn5YUvyFhKc+G1D7NkV5L9jWjjNtLv+S5Sx0NG0n3YtS1H
O4lafECiSetnIi9n2SlkvSLzTF0/jK+Rk/POzuupI31EXPSquC+QgUv7JJm9
1m9JzfkAO30LBWwensc+e32ApN8TZtf0KGDXeaHcJfgd6YiRPjYqZGLx6QLT
/tER4hix3emgryJaWrMGa0Pek/Fr+aZr9ZQQXmR8vEFznNjF+Jr6dSjB5Wii
4wK3CcJvTr65JEsZ7ckNVw5LPpDyDVeerxSooLNox0/xjEmyydOOL9RXhfVw
wpqdREqiDlec5jSrws7t0FOvM1OkTcLo6DrFQm4ra49N1zS5mNeX4MBXQ1HM
3hYt+1kS9suvnFxddSwmNuLUrE/kWfRAx0ybOpQsYjfMNMtIdtO3bvJjGsi0
vvgs0OoL+UcW/HbQTRORWz1FdZFy8jBxjeYhZS28aZVI+nwYSCpsuWldrYWB
VAOZn5ICOIbciYpYbXDdvbYKCxUgW+qs/8mKwtxi2akmTyZCGm/0q4xT6D7B
j+HJmdjhPZvPE+igISUg5OU5RVwfnjbllekggz1kw3NVgszpQUEkm42EyzZa
TaNKSHGKSCWhbJj/ZZ4hSFdGatVYr/lDNkoy097tWK0Cs2hbl0BjDjZvq/B6
3a6C8rWOF+bFc9BV5Xa2N1kVoc1Vyxb9zcFx09JR4UIWtpCBLdW2upieXzGY
/4SFlPqCruB0XThcs6wN3auGzpyBNP9hXUj4ovxSnjoEj7/xG3TSw0Q8/ePu
enXMubB9UVW+Hj7ahz39HKGBCO+VpEiBCyONVMJia+LSxdGjlkIuzEK5mZn3
vnb6H99NVnHh3RU0dtJPC1JvqstHhwfS4/sxSK4FZi2LehvMAzOioa3ysjZW
vFmXW1bHw3vOdJ3YlcJEmck9w7k0lsRX12zcSOGg7evRQgMaYW9iKg08KMzv
fKFvaUhj5MZM0S0fCgc81ketNqYx6PXp2HgAheVuOWruFjR6c75s8o+lUKo3
8FpsS6NljnI3LlHQLiiy73OlwT5S36ZdQsHyoeqyEDcam4aTnnddpqDCvWA2
tpHG8zsq92PKKJxix6nNetBo3MbKv36XAv9q+x22kEbdOQ2RUSsFteCdjQii
oaj6zH+sjcJpq8DCR8E01oWnb7nbTsHZ/EyCawiN+0TLVdBDIVJ000IQSqOm
V3tp5hCFtl13ReGRNGTfNy3wHaGQMJG9YjKKhkPZCSPL9xS45hJ5XDSN22Id
zpOPFPZ5pWUk76cxM/RCPWeKgt1VH0/NWBqrPbIUA2cpJO/x552Mo7H/trts
2WcKza3FL+kDNCqNOZNy+df/F6vc3xJo/AcLnSAh
          "]]}, 
        Annotation[#, "Charting`Private`Tag$19730943#4"]& ]}}, {}}, {
    DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
     AxesOrigin -> {0, 0}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {False, False}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, DisplayFunction :> 
     Identity, Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox["\"Entropy\"", TraditionalForm], None}, {
        FormBox["\"Coupling (\[Kappa])\"", TraditionalForm], None}}, 
     FrameStyle -> Automatic, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, GridLinesStyle -> Directive[
       GrayLevel[0.4, 0.5], 
       AbsoluteThickness[1], 
       AbsoluteDashing[{1, 2}]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None,
        "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, 
     PlotRange -> {{0.01, 4}, {0., 12.107437515378946`}}, PlotRangeClipping -> 
     True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{
      "\"Coupled Entropy Root\"", "\"Tsallis Root Entropy\"", 
       "\"Normalized Tsallis Root Entropy\"", "\"Shannon Root Entropy\""}, 
      "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.560181`", ",", "0.691569`", ",", "0.194885`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6150173333333333, 0.25708400000000003`, 
                    0.13945266666666667`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.922526`", ",", "0.385626`", ",", "0.209179`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.922526, 0.385626, 0.209179]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.922526, 0.385626, 0.209179], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3, ",", #4}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.815564405480465*^9},
 CellLabel->
  "Out[133]=",ExpressionUUID->"1842b8d6-1265-4926-a0b8-51b2eca3d61d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"PlotCoupledEntDist", "[", "\[Alpha]_", "]"}], ":=", 
  "\[IndentingNewLine]", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "coupledDist", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"coupledDist", "=", 
      RowBox[{"CoupledNormalDistribution", "[", 
       RowBox[{"0.", ",", "1.", ",", "\[Kappa]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Plot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"CoupledEntropy", "[", 
          RowBox[{
          "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
           "True"}], "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"TsallisRootEntropy", "[", 
          RowBox[{
          "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
           "False"}], "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"TsallisRootEntropy", "[", 
          RowBox[{
          "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
           "True"}], "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"CoupledEntropy", "[", 
          RowBox[{"coupledDist", ",", "0", ",", "2", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
           "True"}], "]"}]}], "\[IndentingNewLine]", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Kappa]", ",", "0.01", ",", "4"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotLegends", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
         "\"\<Coupled Entropy Root\>\"", ",", " ", 
          "\"\<Tsallis Root Entropy\>\"", ",", " ", 
          "\"\<Normalized Tsallis Root Entropy\>\"", ",", " ", 
          "\"\<Shannon Root Entropy\>\""}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotTheme", "\[Rule]", "\"\<Detailed\>\""}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<Coupling (\[Kappa])\>\"", ",", " ", "\"\<Entropy\>\""}],
          "}"}]}]}], "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.815563595733975*^9, 3.815563630248687*^9}, {
  3.817208250292293*^9, 3.8172082503464937`*^9}},
 CellLabel->"In[66]:=",ExpressionUUID->"5ea7c959-99c7-4169-b08d-4ad685050bf5"],

Cell["Compare Entropies for a Cauchy Distribution", "Text",
 CellChangeTimes->{{3.8155077987021427`*^9, 
  3.8155078178859863`*^9}},ExpressionUUID->"e0ab9dd1-d052-4b67-acd5-\
77402e78c39b"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotCoupledEntDist", "[", 
  RowBox[{"2", ",", "1"}], "]"}]], "Input",
 CellChangeTimes->{{3.815508064079863*^9, 3.815508068935999*^9}},
 CellLabel->
  "In[118]:=",ExpressionUUID->"3a090cdf-446c-49b7-b911-5741df1f0c92"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV13k8lN0XAPDBPJJtopJEEaESSqixnCulhSyVVOTFa8lSyStNSiF71rLT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          "]]}, 
        Annotation[#, "Charting`Private`Tag$13744684#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVzHk4lAkcB/B3XE2YXqPYZ8tsQweFYukwa/v92GpLyNVhszlCihylUFQT
xq1hQ6lNTxINnnRpawvVaBNFjvK4UmozcpSk951zZ//4Pt/n8zzf52saFOUV
okEQhJs6/3fszqwVo4dT4IJ9ockpCwZaPFiVyZcLoMXt3wc2avcveDdgfDQD
igruvH5mzsD1Esd0J34unL1ZkThTbX1e3L7mX05By7UhTcEi9V5s0cB1OQ+9
kRfK8s0YiGON10Msr8DoPLuqcRMGxvDnWDHP1UKzVVFUmCEDuU2d81dZPQb8
ffPRNQoCjX/8dEJ7SRvsgy3PbkkIjEjl3Yg164K0x9Pfi7sIzN7Grm9P64ES
7ZQFrXUEqq4zHRh5r8HpQpySVUFgyfr1v5LJb6FzWmQ4I5fAy4dEwkGb95DQ
E+DifIDArR9/YBfXfIB1adpDAb7q/xJflUvoCCwsS3mU7Eig5elLC9+NjUKi
rsBKziWwvSNvuD5sAhx18guLtAmMKjfLadL/DEPHgzM4rSpwSUtxMnefBOut
63i3ClXQLWiqOi76AnZ/Gm0M8VVBT7n/7iPEVyhIqtvgwFGBzcekn/fCNPjN
7oxc+14JDu7Hnvqc+Qb4wuufNxVKKOpgHrDro4A1uYw5HK2E8viD7SyeFEpu
TASL7ZVgCXb8jDwZlPb2TnlKFaBlkbCJfiGHigZ9XpBYAUKbiy3B1krYFbv4
1OlMBcTs8N4jjlGB5ZSxp/cWBQx1iERvtxHov+j+YbGpAj5kcOQBWgwszQ78
qXhEDkaePjt8yxi4mqy51HtbDiYV8oJWbw0M8o/zqOLLof+kW7yxSgOpVO8o
dzc5NKUHhb06r4m8hrESe44cctkSO2NXLXQOtbHukcggqdKO1TqmhTsMEthV
d2Ww+OXi3O3Z2pjvfCacmyODK8KsEf/VOljdmEgm+clgq99NnzfdOljjtFzV
tVwGfbfdiwcFM9A2dU3ZSkIGOQurx3yXMPFeRcOJ+E4pUKY3h0ufMHFcsDvd
SSQFx6tW9eEHZ+JZdrCnfZIURG57SquNdTEnJnvwpbcUJo98t2tfoy4+cVjl
V2cuhSle5FNFtB4+aH1tba2ggauXAUy2PlamVybPfUnDonAjobBOH/P7/RP3
VtGwpS9kPD+AhZnuEbbzBTTAwM6pEBULB0/XV5oH0qAR3dRVWzkLV2BsQbUD
DROGlJjvSuKhtMyycCMalh356/7mzSQGXSwmmufQEDkUX8vxIvHc84LQpWqP
3qDL72wjMXr7Rm+JIQ3DPrLMz0Ek2izIuLrbgIbBQqVHYAKJeh5Z+0P1aGif
q92Pl0ksbiueF8GggZ3c2DXrCom2oybtLQQNHh9Tn/dVkhjB3fuHtdrP/9Zp
iL9GYu6rwJUTSgqa/ZilNfdInFsmUuyXUyA+r7eH20Hi2jmPhInfKNCc0RI4
3kUiL+zYl/5pCpyjsn+7103iw/HpAFC7AViu2wdI1D8320/jKwX3B2ctF0rU
f7c/tKdPUiDf0Gq+c5TE7IHArJHPFDheO8m1miBxw6aT7q5q3+UbGD6ZInGp
SjVFfqKAlrTpFn4jsXaTU3fMBAWrvfI0g6Ukhg3bPu4YpyDurqfcVkGiQXdb
3Qq1a80Mv6pUJF7mcB4WjVHwH8uISkE=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$13744684#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV1Hk41HsDBXBrI+EnN9pUJEUkrupGvOd7qytFURFFKjdFSbi60SbaqLHm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          "]]}, 
        Annotation[#, "Charting`Private`Tag$13744684#3"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.922526, 0.385626, 0.209179], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVyn881HcAx3Gdo5tf306cItzX+kEokYcfD+3z7qclKdeVlCGxToXa+qHM
7BLi/DhbIwpFElcLNZWRLDU/mprrGvIrrIhDjSin2+2P9+P1eD4ebzooghfC
UFNT26za/z3iL3IcPnmacBJsXb6LYcKy1ilJqIgn+2pLy7YlMNH5eX8X5/tE
crYyi2GTxsSGQbczq4WphPt6k3F3LhM6rsfDmtaeJR5hjjle91T/Osv7XI9c
sljEV3rNMAH5w/IQ62Ky/NrgujtRGjgsNLBhXaggjjq9foITmuA2PDN3snlE
SvKFLzVyWeDYj53SsHpKSK18jXuGFg7Gud48YiEj2/tojZFIHST7sGtaEtrJ
+LRhQM0RPSjLWS6z0rvJN4esc9zN5yBvwwZ3KraXnCu1G3HSY+PKsRJxj90/
RBBzh5wcZWPHkBk7u/Q1Eamnm+26q4+Deb5Kj6/fkLbinbbHRHNhfe7ywn75
MNF1TuFluRugRZo+UCMYJeJS7aVNUwaIKLJIadB5Sx7Ua/60tswQHgmnVy/x
ekduh049Dt7BQWt8w7UfSv4lUtl6hinDCO1FAfui1CbIi2iJ/o0CI9gNRa/a
T94TF8Z75/0r58HFK6aRnzVJ1Fwk6c5N85ApZX3r0DFFJjuS3kfy5qMo8miL
rutHkjMmm+Z3z4c1cRAmpk+Tlzl+IstgYzAtT2z68JeC5P9a1Rbxwhhiu/zH
wbafyBcnozsnfExwePe20LrDSpL1am7W2B8m6JOWlPT6qGGkjqd4tWIBXiea
KgKZsxA4OGrQeHEBDL35u30LZ6HF+9ndfKYpFlxV/PxkGwPdmW8rDcNM0Zm2
OZKjZGBlmFuh2lNTNJwJEvydq44/4zYaLbcyQyp70IHjyUTImyKt8ngzREsc
dJ/ImTh1L7rQ7IUZFj9fnLozWQOB4b+Mb7U2R7FY9CbAWROCuraZzBhz7PC7
xX/Zqon1LnHND5rM0XHbK7snfjaGKgX+fbpcpCy8Lve1YsFAPKE+z56LKfrW
QEE9CwOTX9Wm8rlwu2FTc+DoZwh6RF2QRXJRsjm04DpHC/b9FeHlWVy8izLa
G/ZQC6P9CsH6Ki7GXcMbZw5p49KIfZpjJxdc7UTCYuugNU0yrD/NxaIDhmLx
PR3M7Vm0KtWYxvaOkJEfA3UhLEqJsnCiQbr8x0OUukj1FVw7wKPBONQgq5Do
IeJUyk23CBqj+lN1Qk8KXbGcY/sTaSyLulO9ZQsFwdWVzztVDu+LrDDlUSAu
NlbeSTSGb34ouutDITSitMhJRGOAP530NohCY43HbGYKjZ6MT1v3nKDw+1gZ
L1tMo8VYoxNXKKwoS7SqzqTBjn0o0yumsEk+Z3DZORpbh+KaOyQU6pfePn9R
5ebfNO9HllGYvyq0PTaLRpMfq6C0ikKyJK9643kadbnaoVwphSViB1NpLg31
2Y/3jMgoZNtKqHV5NNZEJO+qaqVQ1bd34pbK94mu584uCo+oS9kZF2lU9+gt
Fw9SkMfFefrm01B8+WSJ/zAFmdbqtgaV3crSuDajFEzOePm6FtCoFM7Rrx+n
wJDuczC5TOPD4FOtjEkKHOnxFJHKzrx09eCPFGLtetunVT5e6a1YMUMhQL3Y
5GAhjQoL/QmlkgJ/b/OWDpX/A7ngIbs=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$13744684#4"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.528488, 0.470624, 0.701351], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJxTTMoPSmViYGAwAWIQXRLXbfqmqsX+2tsPizvtWRwc3h7dlKq9Es4vbBTR
4ZizDc5XOHlF3lznGJwvZvShiVXzApyf02q1uUTpKpzfEy64/1L7LTj//yYO
S8aJ9+H8+W5u7vzNj+D8ZWWrJjwweArnh72WE5y14TnC/PmR/73SXsH52jOW
qDx5+wbOv3R54ov9Ge/h/PzlSr0neT7C+V7tLY7qfp/g/BttJ9c0rPoM599a
Hp9ezfAVzjd4XWubZf8Nzrf0qz8VMvM7nD/9Mkex8Z0fcP7yitJLvFa/EO6z
N27snPgbzmfRqPT+efEPnD/BYNGZFN1/iPCODs48Uvgfzn98edWqR+EMDjD+
807ZPwksjHC+aGBIdORSBF9mxZ+p54OZ4Py7/b4VYv8R/JMdSRnX5zHD+X2C
L43FfFjg/NrVxrzn3yL4atfU+iJ6WOH8lRO6X8VbsMH5YTFbQh7eQPDvbPeb
9aCNHc7vVVn7NlKTA87/objlxeITCL7Nep392aWccP4q38zFa8W44PxP1eLJ
uUcR/C9Weaf+FnDD+QrcnfYcgjxwvmq26IQJ+xD80Dup7yYl8ML59vfivqT+
R/CZCk5e3baaD85/L/TjSKMPP5x/SYr1rsMyBH+bktDX//8RfAC4uqQP
          "]]}, 
        Annotation[#, "Charting`Private`Tag$13744684#5"]& ]}}, {}}, {
    DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
     AxesOrigin -> {0, 0.5836594145428733}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {False, False}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0.5836594145428733}, 
     DisplayFunction :> Identity, Frame -> {{True, True}, {True, True}}, 
     FrameLabel -> {{
        FormBox["\"Entropy\"", TraditionalForm], None}, {
        FormBox["\"Coupling (\[Kappa])\"", TraditionalForm], None}}, 
     FrameStyle -> Automatic, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, GridLinesStyle -> Directive[
       GrayLevel[0.4, 0.5], 
       AbsoluteThickness[1], 
       AbsoluteDashing[{1, 2}]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None,
        "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, PlotLabel -> 
     FormBox["\"Entropy of Coupled Gaussian with \[Kappa] = 1\"", 
       TraditionalForm], 
     PlotRange -> {{0.01, 4}, {0.5836594145428733, 8.82394647020707}}, 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{
      "\"Coupled Entropy\"", "\"Coupled Entropy Root\"", 
       "\"Tsallis Entropy\"", "\"Normalized Tsallis Entropy\"", 
       "\"Shannon Entropy\""}, "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #5}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.560181`", ",", "0.691569`", ",", "0.194885`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6150173333333333, 0.25708400000000003`, 
                    0.13945266666666667`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.922526`", ",", "0.385626`", ",", "0.209179`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.922526, 0.385626, 0.209179]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.922526, 0.385626, 0.209179], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.3523253333333333, 0.3137493333333333, 
                    0.46756733333333333`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.528488`", ",", "0.470624`", ",", "0.701351`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.528488, 0.470624, 0.701351]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.528488, 0.470624, 0.701351], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3, ",", #4, ",", #5}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.8155086621666517`*^9},
 CellLabel->
  "Out[118]=",ExpressionUUID->"46b6c723-fbec-44a2-bdf9-c6e8297aa4db"]
}, Open  ]],

Cell["Saved Plot Dec 17, 2020", "Text",
 CellChangeTimes->{{3.817207764745207*^9, 
  3.817207769562533*^9}},ExpressionUUID->"c57a1c3c-9f79-4654-8940-\
b481dcb7d3f4"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV13k8lN0XAPDBPJJtopJEEaESSqixnCulhSyVVOTFa8lSyStNSiF71rLT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          "]]}, 
        Annotation[#, "Charting`Private`Tag$13744684#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVzHk4lAkcB/B3XE2YXqPYZ8tsQweFYukwa/v92GpLyNVhszlCihylUFQT
xq1hQ6lNTxINnnRpawvVaBNFjvK4UmozcpSk951zZ//4Pt/n8zzf52saFOUV
okEQhJs6/3fszqwVo4dT4IJ9ockpCwZaPFiVyZcLoMXt3wc2avcveDdgfDQD
igruvH5mzsD1Esd0J34unL1ZkThTbX1e3L7mX05By7UhTcEi9V5s0cB1OQ+9
kRfK8s0YiGON10Msr8DoPLuqcRMGxvDnWDHP1UKzVVFUmCEDuU2d81dZPQb8
ffPRNQoCjX/8dEJ7SRvsgy3PbkkIjEjl3Yg164K0x9Pfi7sIzN7Grm9P64ES
7ZQFrXUEqq4zHRh5r8HpQpySVUFgyfr1v5LJb6FzWmQ4I5fAy4dEwkGb95DQ
E+DifIDArR9/YBfXfIB1adpDAb7q/xJflUvoCCwsS3mU7Eig5elLC9+NjUKi
rsBKziWwvSNvuD5sAhx18guLtAmMKjfLadL/DEPHgzM4rSpwSUtxMnefBOut
63i3ClXQLWiqOi76AnZ/Gm0M8VVBT7n/7iPEVyhIqtvgwFGBzcekn/fCNPjN
7oxc+14JDu7Hnvqc+Qb4wuufNxVKKOpgHrDro4A1uYw5HK2E8viD7SyeFEpu
TASL7ZVgCXb8jDwZlPb2TnlKFaBlkbCJfiGHigZ9XpBYAUKbiy3B1krYFbv4
1OlMBcTs8N4jjlGB5ZSxp/cWBQx1iERvtxHov+j+YbGpAj5kcOQBWgwszQ78
qXhEDkaePjt8yxi4mqy51HtbDiYV8oJWbw0M8o/zqOLLof+kW7yxSgOpVO8o
dzc5NKUHhb06r4m8hrESe44cctkSO2NXLXQOtbHukcggqdKO1TqmhTsMEthV
d2Ww+OXi3O3Z2pjvfCacmyODK8KsEf/VOljdmEgm+clgq99NnzfdOljjtFzV
tVwGfbfdiwcFM9A2dU3ZSkIGOQurx3yXMPFeRcOJ+E4pUKY3h0ufMHFcsDvd
SSQFx6tW9eEHZ+JZdrCnfZIURG57SquNdTEnJnvwpbcUJo98t2tfoy4+cVjl
V2cuhSle5FNFtB4+aH1tba2ggauXAUy2PlamVybPfUnDonAjobBOH/P7/RP3
VtGwpS9kPD+AhZnuEbbzBTTAwM6pEBULB0/XV5oH0qAR3dRVWzkLV2BsQbUD
DROGlJjvSuKhtMyycCMalh356/7mzSQGXSwmmufQEDkUX8vxIvHc84LQpWqP
3qDL72wjMXr7Rm+JIQ3DPrLMz0Ek2izIuLrbgIbBQqVHYAKJeh5Z+0P1aGif
q92Pl0ksbiueF8GggZ3c2DXrCom2oybtLQQNHh9Tn/dVkhjB3fuHtdrP/9Zp
iL9GYu6rwJUTSgqa/ZilNfdInFsmUuyXUyA+r7eH20Hi2jmPhInfKNCc0RI4
3kUiL+zYl/5pCpyjsn+7103iw/HpAFC7AViu2wdI1D8320/jKwX3B2ctF0rU
f7c/tKdPUiDf0Gq+c5TE7IHArJHPFDheO8m1miBxw6aT7q5q3+UbGD6ZInGp
SjVFfqKAlrTpFn4jsXaTU3fMBAWrvfI0g6Ukhg3bPu4YpyDurqfcVkGiQXdb
3Qq1a80Mv6pUJF7mcB4WjVHwH8uISkE=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$13744684#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV1Hk41HsDBXBrI+EnN9pUJEUkrupGvOd7qytFURFFKjdFSbi60SbaqLHm
arletJhCWUpKIRWyZ5ssWWfGOrZhLJMx3nn/OM95Ps9z/j6armf3uUlJSEjE
i/P/9nO5s3HgwnWcoE1aLAqWJjqffrsdJLyJ+5OBQ4s9pUmrFqdN7UooEv9g
rzx0QJpY9JmF/B4UDt35lyWj9aSJgun5M+XbYnDLUI7+pVGKtBbqFGjsisdD
32bLI+ZSZPPElVPc2wmgvbYPbFgrRUrvnpL/4ZaIP7gD/YGLpIhWUUFWEuMR
hAFSZ1z4kqRhzZm5W3Se4l7w0Lb/pEkSMlj0yk0vGf7f382Z0JUkg7I8Wnl5
MlJK5rpoLJEkD5erHzb0TEGnxvEX3vKShG/jS5t+kYreUNO5QVwJ8vzVcudI
gzSUJ9Bs96VLELncD8Gt9zNh+pW71cpcgvgELdCXi8vGtfbc0zvbRchiuhYK
RrPR+ctHFaciEaZ0M525O9+i13yBkJEqwpXa3WFVk29xKcQlqN9fhNtat4Zi
9udgwS/CkHlqIjwtEmRozstFlrVzyuJDM+hZbLnzF9dcMEQmTQHbZrDWK7ZT
JicXaxN3e8xdN4NMNWOV3hN52Mj77JAlMYP8k55/pX3OR/7F8UTdFCEGrg8/
XJZXgC43TC2RFUIpXe1kbG8Bagqv0wx501jfZG5MLfiE5/V7dTxbp+G7jl4u
cfoT2iK8ahyypyFg6gjZiz5DzjE3ZMRjGnPWuB5O8fuC3oDXv5Y3/IRGaf2K
3/SL0ZZWsSyyXAD6pOfZKrtivJG+k1SdK8CUtuxHt8vFKNc5rK+bJkBN8EaX
mKpidN64ZK0ZJUCwWWwcz/srzmiq/rPGUQB2usPiF29KsOi1ZZhz/xQYsc0q
mijHo3x5CYXlU1Ap8nV9d7IcEdImfj3zp3BlTP6VTWQ5fPVeVdbLTsHBdsve
K53liPfRjOYMTkJOPi7ix7UKDDLTtOvzJ+Fx+bBCbEklpv87Mbfuz0mo/ToS
LKtbjVVO6mbHciZw3CkTXLNqpBue9D2ZPoHM677CattqcJQffr+QNAHrBv65
OP9qfBCQgpKoCQRdErgbl1Rj3qrD1qJTExgolrI5drIG6/vv5r9fMYFPTqpL
85JqEfvOeFtT1DiUbnxveJxTi4nFUVYOoeNwTrsXE1JZC9n9Gq/br45jQmKJ
kt14LYy+z7WjvMehz1guyd1eB02tiPqePeOIHVnTs4hTh6ttNcu8lMbhecP0
td9KJmyWkLRDMXxkdXjoyW9mglN6fMVxOh8zWx48SdjNRIN994Hz1/kIH528
W+bPhCB9bc17Pz4yjrzx0/jGxFX6edksez74JoabKi5+h+XWE2FHF/NxaUg7
R4vZgIU7Nq42fjaGwp32Rjn9DbAsuHbPPnEMiknXk/dINEK9LOlH4IMxxDux
HwToNcLeiHrGvTOGgpLEC9+CGlEV/P7Hcp8xyDxZanbRoAlv+xUYvWZjoDvM
/1h7qxkL0pITsptG8XODwH/y32Zs0FF7rFk/CneVzl/VM5rhv33P+uiqUWyr
zGC4NTbjs6NJ6bUv4v1W2/ApnR9QDTZY9PzlKDwMIg4vL/uBc4xCDAWPwkJW
Ueih0IphN7VkM6NRzL6SM5GMakfTN3qb5788uH92SWMy2lEYItzxNJaH6pos
rZTcdqQG2/zREcXD45GjSvt722E16NfpFsKD1r0Wn9oVHaCXWHEZ53j4Wr9h
x6rzHSj7vLD0py0PlG0X76t2Jw4vva9ZIM9DgoXFDuoaCy3PpA/Eho7gaMNp
enEYCzHm/IaGGyPQcI+quXyfhcCV96+oB48gMbTFaeAlC0ozm9emXxC7wte7
tJGFsw60cMFpsfclPLhmwMbvHTKbVtiM4LHLFHeyiQ1a/6m3MQtHwPg7JbLD
sAseC26u6Dg6jADLF3+3m3ZBdHdgpY7TMKyXpDm3be9Cx8viDX72w+DlZeq0
OHbB3znZd9GuYZjJ5hQ0XO2CBXNbR6TxMGrvfh2pqu6C0bzZNnPaMEQZHNs8
726ERXf6eWYO4QB3+fyHGT04ZjXqsVt5CJ4JB2d3neiHEGyRV8sA9O4/XcUZ
HMCfLiZs1R4uauuiej+6D6NSv2uYUuPi7LOVYaUKPEyLnqgO2fRj163rv6/Z
MwojCyu7TS/70Hiz9MXVlDGYlwoONiv3ofnZkZMXJcbRpJ2cdSCkF4bcy+an
MAG/gnXjLaq9MNkTWGb3YBKWnHMLXiT14F6d3F/GLVOg3EXbki168Mz/XK2i
6U9ocbfqt/V2Qw/GQaFR06Bx3P9uiemGjE6AlaBGCNM7+ctqrbsRafi44vg6
EUKUBqYrZbrh47Tfo9BnFlmHTLQYxV1g16WksBwkyBilXzoZ1oWe0GXCozKS
pIYR/f6jXRdU99o5HUySJC3B9UETK7ug/lz4z7f9UkTVW/INfYSD1ojd/mqz
UkQj4Mm49WcOSkNc3RvipclSue7wsgcchM/vM1azliG1yYqn/L05uJxqrPht
UIaYzXkUoGXNwervq8Md6bKEWfnwse9aDpIj7/Qf2TyHJC7560Q7jYMDzll2
nY1ziHIUXdOuj42Wt3sedtykkTz5dR9GStkIW/Vy8KCuHOG+bHqskcbGlGZW
75MSOZL+botQJ4YNs3T9j6fPzSUBkZn7HQPYSNnt8eSlmjzJSVAuCjzGxujF
hX+eKZInjMqcrmZLNvimXmUz3vPIlgHbvlIjNjTmhUJuvgLZFMd33bSMDe3T
qpGR+QpktaO2ZwuNDfsWt6Hoo4qk5AxJKeKzgDYXvtusIsnecHNnJYsFKe9S
ZnaqEpHzeRSkVs3CsMpUYZA1RWYcRo33fmTB4OK7PBsbigRyjhbTxPZi+2cv
20cRg2IHt/x8FgZeC57lOFCkMqemRE/sXrvp2zxXiqiPxWnT8ljoiBXZHgug
SGphiXHBexZql8i2EgZFVrd50bdkszD/WhFTKZkiJvyvyWNvWLDl3qhqSaVI
9PD2ulSxqz7MKfDPpMiDPfG71MUud5Z7kpFLkckx/7KZ1ywUxs/z0KijyLlT
WgpfMlmQplUcG2JSxPGedf9FsbeepR/KbaRIs5RH/QaxC6Bo7dhGkXcRZysY
GSzkdSitj+yjSEX4Eas76SwILb+tcRmgSOLbqpDtYptlRmjoD1NEed3i2pk0
Ft4HKauU8CnSsHBDiLfYgr5q+dhJisTHifi6Ym/eFyV9/CdFftLvn2GLf+b8
+71CoxmKaI+JeHFiZ69UGZ+dpYh1s0HwAbH/Bwm3+JY=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$13744684#3"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.922526, 0.385626, 0.209179], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVyn881HcAx3Gdo5tf306cItzX+kEokYcfD+3z7qclKdeVlCGxToXa+qHM
7BLi/DhbIwpFElcLNZWRLDU/mprrGvIrrIhDjSin2+2P9+P1eD4ebzooghfC
UFNT26za/z3iL3IcPnmacBJsXb6LYcKy1ilJqIgn+2pLy7YlMNH5eX8X5/tE
crYyi2GTxsSGQbczq4WphPt6k3F3LhM6rsfDmtaeJR5hjjle91T/Osv7XI9c
sljEV3rNMAH5w/IQ62Ky/NrgujtRGjgsNLBhXaggjjq9foITmuA2PDN3snlE
SvKFLzVyWeDYj53SsHpKSK18jXuGFg7Gud48YiEj2/tojZFIHST7sGtaEtrJ
+LRhQM0RPSjLWS6z0rvJN4esc9zN5yBvwwZ3KraXnCu1G3HSY+PKsRJxj90/
RBBzh5wcZWPHkBk7u/Q1Eamnm+26q4+Deb5Kj6/fkLbinbbHRHNhfe7ywn75
MNF1TuFluRugRZo+UCMYJeJS7aVNUwaIKLJIadB5Sx7Ua/60tswQHgmnVy/x
ekduh049Dt7BQWt8w7UfSv4lUtl6hinDCO1FAfui1CbIi2iJ/o0CI9gNRa/a
T94TF8Z75/0r58HFK6aRnzVJ1Fwk6c5N85ApZX3r0DFFJjuS3kfy5qMo8miL
rutHkjMmm+Z3z4c1cRAmpk+Tlzl+IstgYzAtT2z68JeC5P9a1Rbxwhhiu/zH
wbafyBcnozsnfExwePe20LrDSpL1am7W2B8m6JOWlPT6qGGkjqd4tWIBXiea
KgKZsxA4OGrQeHEBDL35u30LZ6HF+9ndfKYpFlxV/PxkGwPdmW8rDcNM0Zm2
OZKjZGBlmFuh2lNTNJwJEvydq44/4zYaLbcyQyp70IHjyUTImyKt8ngzREsc
dJ/ImTh1L7rQ7IUZFj9fnLozWQOB4b+Mb7U2R7FY9CbAWROCuraZzBhz7PC7
xX/Zqon1LnHND5rM0XHbK7snfjaGKgX+fbpcpCy8Lve1YsFAPKE+z56LKfrW
QEE9CwOTX9Wm8rlwu2FTc+DoZwh6RF2QRXJRsjm04DpHC/b9FeHlWVy8izLa
G/ZQC6P9CsH6Ki7GXcMbZw5p49KIfZpjJxdc7UTCYuugNU0yrD/NxaIDhmLx
PR3M7Vm0KtWYxvaOkJEfA3UhLEqJsnCiQbr8x0OUukj1FVw7wKPBONQgq5Do
IeJUyk23CBqj+lN1Qk8KXbGcY/sTaSyLulO9ZQsFwdWVzztVDu+LrDDlUSAu
NlbeSTSGb34ouutDITSitMhJRGOAP530NohCY43HbGYKjZ6MT1v3nKDw+1gZ
L1tMo8VYoxNXKKwoS7SqzqTBjn0o0yumsEk+Z3DZORpbh+KaOyQU6pfePn9R
5ebfNO9HllGYvyq0PTaLRpMfq6C0ikKyJK9643kadbnaoVwphSViB1NpLg31
2Y/3jMgoZNtKqHV5NNZEJO+qaqVQ1bd34pbK94mu584uCo+oS9kZF2lU9+gt
Fw9SkMfFefrm01B8+WSJ/zAFmdbqtgaV3crSuDajFEzOePm6FtCoFM7Rrx+n
wJDuczC5TOPD4FOtjEkKHOnxFJHKzrx09eCPFGLtetunVT5e6a1YMUMhQL3Y
5GAhjQoL/QmlkgJ/b/OWDpX/A7ngIbs=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$13744684#4"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.528488, 0.470624, 0.701351], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJxTTMoPSmViYGAwAWIQXRLXbfqmqsX+2tsPizvtWRwc3h7dlKq9Es4vbBTR
4ZizDc5XOHlF3lznGJwvZvShiVXzApyf02q1uUTpKpzfEy64/1L7LTj//yYO
S8aJ9+H8+W5u7vzNj+D8ZWWrJjwweArnh72WE5y14TnC/PmR/73SXsH52jOW
qDx5+wbOv3R54ov9Ge/h/PzlSr0neT7C+V7tLY7qfp/g/BttJ9c0rPoM599a
Hp9ezfAVzjd4XWubZf8Nzrf0qz8VMvM7nD/9Mkex8Z0fcP7yitJLvFa/EO6z
N27snPgbzmfRqPT+efEPnD/BYNGZFN1/iPCODs48Uvgfzn98edWqR+EMDjD+
807ZPwksjHC+aGBIdORSBF9mxZ+p54OZ4Py7/b4VYv8R/JMdSRnX5zHD+X2C
L43FfFjg/NrVxrzn3yL4atfU+iJ6WOH8lRO6X8VbsMH5YTFbQh7eQPDvbPeb
9aCNHc7vVVn7NlKTA87/objlxeITCL7Nep392aWccP4q38zFa8W44PxP1eLJ
uUcR/C9Weaf+FnDD+QrcnfYcgjxwvmq26IQJ+xD80Dup7yYl8ML59vfivqT+
R/CZCk5e3baaD85/L/TjSKMPP5x/SYr1rsMyBH+bktDX//8RfAC4uqQP
          "]]}, 
        Annotation[#, "Charting`Private`Tag$13744684#5"]& ]}}, {}}, {
    DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
     AxesOrigin -> {0, 0.5836594145428733}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {False, False}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0.5836594145428733}, 
     DisplayFunction :> Identity, Frame -> {{True, True}, {True, True}}, 
     FrameLabel -> {{
        FormBox["\"Entropy\"", TraditionalForm], None}, {
        FormBox["\"Coupling (\[Kappa])\"", TraditionalForm], None}}, 
     FrameStyle -> Automatic, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, GridLinesStyle -> Directive[
       GrayLevel[0.4, 0.5], 
       AbsoluteThickness[1], 
       AbsoluteDashing[{1, 2}]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None,
        "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, PlotLabel -> 
     FormBox["\"Entropy of Coupled Gaussian with \[Kappa] = 1\"", 
       TraditionalForm], 
     PlotRange -> {{0.01, 4}, {0.5836594145428733, 8.82394647020707}}, 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{
      "\"Coupled Entropy\"", "\"Coupled Entropy Root\"", 
       "\"Tsallis Entropy\"", "\"Normalized Tsallis Entropy\"", 
       "\"Shannon Entropy\""}, "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #5}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.560181`", ",", "0.691569`", ",", "0.194885`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6150173333333333, 0.25708400000000003`, 
                    0.13945266666666667`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.922526`", ",", "0.385626`", ",", "0.209179`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.922526, 0.385626, 0.209179]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.922526, 0.385626, 0.209179], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.3523253333333333, 0.3137493333333333, 
                    0.46756733333333333`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.528488`", ",", "0.470624`", ",", "0.701351`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.528488, 0.470624, 0.701351]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.528488, 0.470624, 0.701351], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3, ",", #4, ",", #5}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.8155086621666517`*^9},
 CellLabel->
  "Out[118]=",ExpressionUUID->"2f29f9a9-5207-418c-a049-930b0d8e6115"],

Cell[BoxData[
 RowBox[{
  RowBox[{"PlotCoupledEntDist", "[", 
   RowBox[{"\[Alpha]_", ",", "\[Kappa]Dist_"}], "]"}], ":=", 
  "\[IndentingNewLine]", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "coupledDist", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"coupledDist", "=", 
      RowBox[{"CoupledNormalDistribution", "[", 
       RowBox[{"0.", ",", "1.", ",", "\[Kappa]Dist"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Plot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"CoupledEntropy", "[", 
          RowBox[{
          "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
           "False"}], "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"CoupledEntropy", "[", 
          RowBox[{
          "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
           "True"}], "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"TsallisEntropy", "[", 
          RowBox[{
          "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
           "False"}], "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"TsallisEntropy", "[", 
          RowBox[{
          "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
           "True"}], "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"CoupledEntropy", "[", 
          RowBox[{"coupledDist", ",", "0", ",", "1", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
           "False"}], "]"}]}], "\[IndentingNewLine]", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Kappa]", ",", "0.01", ",", "4"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotLegends", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
         "\"\<Coupled Entropy\>\"", ",", " ", "\"\<Coupled Entropy Root\>\"", 
          ",", " ", "\n", "\"\<Tsallis Entropy\>\"", ",", " ", 
          "\"\<Normalized Tsallis Entropy\>\"", ",", " ", 
          "\"\<Shannon Entropy\>\""}], "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotTheme", "\[Rule]", "\"\<Detailed\>\""}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<Coupling (\[Kappa])\>\"", ",", " ", "\"\<Entropy\>\""}],
          "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotLabel", "\[Rule]", 
        RowBox[{
        "\"\<Entropy of Coupled Gaussian with \[Kappa] = \>\"", "<>", 
         RowBox[{"ToString", "[", "\[Kappa]Dist", "]"}]}]}]}], 
      "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.815507856532708*^9, 3.8155078749313307`*^9}, {
  3.815507920600128*^9, 3.8155079991771803`*^9}, {3.815508052975773*^9, 
  3.8155080540454187`*^9}},
 CellLabel->
  "In[117]:=",ExpressionUUID->"39e2ea94-e57b-48d6-93ae-9d5cc48d76d3"],

Cell["Comparison when other entropies also have a root term", "Text",
 CellChangeTimes->{{3.81556329777697*^9, 
  3.815563314612989*^9}},ExpressionUUID->"bce2d6ca-9099-417d-bd7e-\
fffa1b163966"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotCoupledEntDist", "[", 
  RowBox[{"2", ",", "1"}], "]"}]], "Input",
 CellLabel->
  "In[131]:=",ExpressionUUID->"5496c6cf-fd44-47c8-8eea-384e590a70c0"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVzHk4lAkcB/B3XE2YXqPYZ8tsQweFYukwa/v92GpLyNVhszlCihylUFQT
xq1hQ6lNTxINnnRpawvVaBNFjvK4UmozcpSk951zZ//4Pt/n8zzf52saFOUV
okEQhJs6/3fszqwVo4dT4IJ9ockpCwZaPFiVyZcLoMXt3wc2avcveDdgfDQD
igruvH5mzsD1Esd0J34unL1ZkThTbX1e3L7mX05By7UhTcEi9V5s0cB1OQ+9
kRfK8s0YiGON10Msr8DoPLuqcRMGxvDnWDHP1UKzVVFUmCEDuU2d81dZPQb8
ffPRNQoCjX/8dEJ7SRvsgy3PbkkIjEjl3Yg164K0x9Pfi7sIzN7Grm9P64ES
7ZQFrXUEqq4zHRh5r8HpQpySVUFgyfr1v5LJb6FzWmQ4I5fAy4dEwkGb95DQ
E+DifIDArR9/YBfXfIB1adpDAb7q/xJflUvoCCwsS3mU7Eig5elLC9+NjUKi
rsBKziWwvSNvuD5sAhx18guLtAmMKjfLadL/DEPHgzM4rSpwSUtxMnefBOut
63i3ClXQLWiqOi76AnZ/Gm0M8VVBT7n/7iPEVyhIqtvgwFGBzcekn/fCNPjN
7oxc+14JDu7Hnvqc+Qb4wuufNxVKKOpgHrDro4A1uYw5HK2E8viD7SyeFEpu
TASL7ZVgCXb8jDwZlPb2TnlKFaBlkbCJfiGHigZ9XpBYAUKbiy3B1krYFbv4
1OlMBcTs8N4jjlGB5ZSxp/cWBQx1iERvtxHov+j+YbGpAj5kcOQBWgwszQ78
qXhEDkaePjt8yxi4mqy51HtbDiYV8oJWbw0M8o/zqOLLof+kW7yxSgOpVO8o
dzc5NKUHhb06r4m8hrESe44cctkSO2NXLXQOtbHukcggqdKO1TqmhTsMEthV
d2Ww+OXi3O3Z2pjvfCacmyODK8KsEf/VOljdmEgm+clgq99NnzfdOljjtFzV
tVwGfbfdiwcFM9A2dU3ZSkIGOQurx3yXMPFeRcOJ+E4pUKY3h0ufMHFcsDvd
SSQFx6tW9eEHZ+JZdrCnfZIURG57SquNdTEnJnvwpbcUJo98t2tfoy4+cVjl
V2cuhSle5FNFtB4+aH1tba2ggauXAUy2PlamVybPfUnDonAjobBOH/P7/RP3
VtGwpS9kPD+AhZnuEbbzBTTAwM6pEBULB0/XV5oH0qAR3dRVWzkLV2BsQbUD
DROGlJjvSuKhtMyycCMalh356/7mzSQGXSwmmufQEDkUX8vxIvHc84LQpWqP
3qDL72wjMXr7Rm+JIQ3DPrLMz0Ek2izIuLrbgIbBQqVHYAKJeh5Z+0P1aGif
q92Pl0ksbiueF8GggZ3c2DXrCom2oybtLQQNHh9Tn/dVkhjB3fuHtdrP/9Zp
iL9GYu6rwJUTSgqa/ZilNfdInFsmUuyXUyA+r7eH20Hi2jmPhInfKNCc0RI4
3kUiL+zYl/5pCpyjsn+7103iw/HpAFC7AViu2wdI1D8320/jKwX3B2ctF0rU
f7c/tKdPUiDf0Gq+c5TE7IHArJHPFDheO8m1miBxw6aT7q5q3+UbGD6ZInGp
SjVFfqKAlrTpFn4jsXaTU3fMBAWrvfI0g6Ukhg3bPu4YpyDurqfcVkGiQXdb
3Qq1a80Mv6pUJF7mcB4WjVHwH8uISkE=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$16989367#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV13k4lF0UAHDEzLxjmFAKLZQsEQml4js3LSQ7qYQiu5KlEJEslUgUQrKG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          "]]}, 
        Annotation[#, "Charting`Private`Tag$16989367#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVynk41HkAx3Ezrsn1M45xZOY3jhXLqqhNYr8f1doO9xE6FFEkOVKRq1nT
oUOjYjX1qCjWSIdKbSVa6nG0iGVlnWU3cnUQcsy2f7yf1z9vw6BIzxCmjIyM
y9f+NzbgxNKhg0JitNZ9icCbAbOny44LZo4Qpdd1DYn+DHQa93VxktOIebFS
QVwAA04D9sccBenk1p7vmdGhDKjYHYioW3WOrI0V/BCc+PWvMqvgr8shqf5b
k7yuMoDhZyUhFoUkq6mlYP0YA9ECLUvWxVLi9NnHMuQcE/yaP+llls+JXvtx
cd6ILDjW73+WN28k3YfmIr5YyWP3Ybs7sUYthCYr700GK+CkL7u86Wg7KbH7
/K7jpiKkJazljIxu8h7/ti7uZuGSk9NPVOprYut/JvCBoRLy90tEPYv+Idvd
6mkFP2VsGOSxxbfekg+STy/b9qpg9yV/6bod70jMvJ7KzCuqsMi+atI3PEQ0
31wxqLinhqbmjP7y0FEyvXCwJLacQmSB0akalQ+E7pbam0aoY91RoeMC14/k
QKWMd96IOtqO1Fw/JPlERImRPTvj2Ggv2LozQWacZD6NNzCcZWPRYJLDLvKZ
bJGcFZft18By15Ra7/MTxNO7d3TLpAZ+aWbttemYJF0xYRdcIjVRELevSdXu
C4ly/NFOcUQTFsRGkJYxTXTj0sUbQ7UgZxa/furlDBmfaa273qYF0aLcF8Hf
zZHfXz2/muGujehNXmFV0VISGejQ8ccTbbxplkhe+8rAWo8j9DHn4G0ad2ab
HANMofM+hwscaHt4b/K/xkAZV1ivKKcDg19nMhu8mBiwfbpaPlYHnadd4jhS
JjbuUswL79FBzbGg0L9yZOHodq998WpdpLMHbDjOclhVXFk0e0MXSUU2qg3D
cpiWvSYMZevBtNU03e+kPDIOtqo+itJDoejEu622CshmpBgMtOphw+a73r1t
CtBbrTCmv1QfHfddxT1HFGGVLPaTFevjlEnxsL85C1rGAcYmE/qYNLzbn1fN
Qp+H0bIYj/mwv2lZHr5vHork+UPuJfMhcQnLK+YooWSSWf9Y2QAfE3S2RzxT
wtqP2JAdboAxuz21s1HKyEVQfnG1AfjKaYTFVsHF+/NW5PC4+CZcWyR6ooIY
UX7K3we58OkIGTmzTRVrsusvz2/kgnQFjIVIVVETZjHaZ8QDM6qmpbRIDYmF
ufHtCTyMakxWCZwpXD5vXqtZz4NVwoMyNzcKDsIRB0EDD3vexJVyPSkUuY3n
jjTyMHRnquA3XwrJnAbUNPPQ7z19/EMQBWub3rqkVzz0ZM25B8ZTWLFkmN/X
x0OTvnwn8in49TOark3zwE591qJWSMHG4tto9iwP7oOH6zuKKGhbVzOS5nio
f6RQEXebgng8fMKLQaNuMyvv1mMKdmYpSxiKNKpylMP4zRSippZP+GnQkFV8
ETjSQsHINflGpSaNlZEnNz5uo9A07OFrpU2jgqg6+3VRWL+jOElWl0ZZj9pC
0QCFG5mB3GIujZk1DQsChijo5DgH6tA07G+f5luOUqgQN2cJ+DQeCtQ1qsco
uFU8avUxpjE10KiUNUHBUZ3fW25Cw9YzQzb4C4WLuaad5qY0Djz0mFk8S8FS
6WXt2QU0So00xqVSCude8SWzZjT+A85eFqY=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$16989367#3"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.922526, 0.385626, 0.209179], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJxTTMoPSmViYGAwAWIQXRLXbfqmqsW++92HxZ32LA4Ob49uStVeCecXNoro
cMzZBucrnLwib65zDM4XM/rQxKp5Ac7PabXaXKJ0Fc7vCRfcf6n9Fpz/fxOH
JePE+3D+fDc3d/7mR3D+srJVEx4YPIXzw17LCc7a8Bxh/vzI/15pr+B87RlL
VJ68fQPnX7o88cX+jPdwfv5ypd6TPB/hfK/2Fkd1v09w/o22k2saVn2G828t
j0+vZvgK5xu8rrXNsv8G51v61Z8Kmfkdzp9+maPY+M4POH95ReklXqtfCPfZ
Gzd2TvwN57NoVHr/vPgHzp9gsOhMiu4/RHhHB2ceKfwP5z++vGrVo3AGBxj/
eafsnwQWRjhfNDAkOnIpgi+z4s/U88FMcP7dft8Ksf8I/smOpIzr85jh/D7B
l8ZiPixwfu1qY97zbxF8tWtqfRE9rHD+ygndr+It2OD8sJgtIQ9vIPh3tvvN
etDGDuf3qqx9G6nJAef/UNzyYvEJBN9mvc7+7FJOOH+Vb+bitWJccP6navHk
3KMI/hervFN/C7jhfAXuTnsOQR44XzVbdMKEfQh+6J3Ud5MSeOF8+3txX1L/
I/hMBSevblvNB+e/F/pxpNGHH86/JMV612EZgr9NSejr//8IPgBKHpUH
          "]]}, 
        Annotation[#, "Charting`Private`Tag$16989367#4"]& ]}}, {}}, {
    DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
     AxesOrigin -> {0, 0}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {False, False}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, DisplayFunction :> 
     Identity, Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox["\"Entropy\"", TraditionalForm], None}, {
        FormBox["\"Coupling (\[Kappa])\"", TraditionalForm], None}}, 
     FrameStyle -> Automatic, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, GridLinesStyle -> Directive[
       GrayLevel[0.4, 0.5], 
       AbsoluteThickness[1], 
       AbsoluteDashing[{1, 2}]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None,
        "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, PlotLabel -> 
     FormBox["\"Entropy of Coupled Gaussian with \[Kappa] = 1\"", 
       TraditionalForm], PlotRange -> {{0.01, 4}, {0., 7.790030153517723}}, 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{
      "\"Coupled Entropy Root\"", "\"Tsallis Root Entropy\"", 
       "\"Normalized Tsallis Root Entropy\"", "\"Shannon Root Entropy\""}, 
      "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.560181`", ",", "0.691569`", ",", "0.194885`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6150173333333333, 0.25708400000000003`, 
                    0.13945266666666667`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.922526`", ",", "0.385626`", ",", "0.209179`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.922526, 0.385626, 0.209179]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.922526, 0.385626, 0.209179], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3, ",", #4}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.815564288971333*^9},
 CellLabel->
  "Out[131]=",ExpressionUUID->"e9a3d61a-ad98-46be-820b-6ae655cacfc1"]
}, Open  ]],

Cell["Saved Plot Dec 17, 2020", "Text",
 CellChangeTimes->{{3.817207764745207*^9, 
  3.817207769562533*^9}},ExpressionUUID->"627ef1f3-d256-40ed-978e-\
9acaab0affc1"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVzHk4lAkcB/B3XE2YXqPYZ8tsQweFYukwa/v92GpLyNVhszlCihylUFQT
xq1hQ6lNTxINnnRpawvVaBNFjvK4UmozcpSk951zZ//4Pt/n8zzf52saFOUV
okEQhJs6/3fszqwVo4dT4IJ9ockpCwZaPFiVyZcLoMXt3wc2avcveDdgfDQD
igruvH5mzsD1Esd0J34unL1ZkThTbX1e3L7mX05By7UhTcEi9V5s0cB1OQ+9
kRfK8s0YiGON10Msr8DoPLuqcRMGxvDnWDHP1UKzVVFUmCEDuU2d81dZPQb8
ffPRNQoCjX/8dEJ7SRvsgy3PbkkIjEjl3Yg164K0x9Pfi7sIzN7Grm9P64ES
7ZQFrXUEqq4zHRh5r8HpQpySVUFgyfr1v5LJb6FzWmQ4I5fAy4dEwkGb95DQ
E+DifIDArR9/YBfXfIB1adpDAb7q/xJflUvoCCwsS3mU7Eig5elLC9+NjUKi
rsBKziWwvSNvuD5sAhx18guLtAmMKjfLadL/DEPHgzM4rSpwSUtxMnefBOut
63i3ClXQLWiqOi76AnZ/Gm0M8VVBT7n/7iPEVyhIqtvgwFGBzcekn/fCNPjN
7oxc+14JDu7Hnvqc+Qb4wuufNxVKKOpgHrDro4A1uYw5HK2E8viD7SyeFEpu
TASL7ZVgCXb8jDwZlPb2TnlKFaBlkbCJfiGHigZ9XpBYAUKbiy3B1krYFbv4
1OlMBcTs8N4jjlGB5ZSxp/cWBQx1iERvtxHov+j+YbGpAj5kcOQBWgwszQ78
qXhEDkaePjt8yxi4mqy51HtbDiYV8oJWbw0M8o/zqOLLof+kW7yxSgOpVO8o
dzc5NKUHhb06r4m8hrESe44cctkSO2NXLXQOtbHukcggqdKO1TqmhTsMEthV
d2Ww+OXi3O3Z2pjvfCacmyODK8KsEf/VOljdmEgm+clgq99NnzfdOljjtFzV
tVwGfbfdiwcFM9A2dU3ZSkIGOQurx3yXMPFeRcOJ+E4pUKY3h0ufMHFcsDvd
SSQFx6tW9eEHZ+JZdrCnfZIURG57SquNdTEnJnvwpbcUJo98t2tfoy4+cVjl
V2cuhSle5FNFtB4+aH1tba2ggauXAUy2PlamVybPfUnDonAjobBOH/P7/RP3
VtGwpS9kPD+AhZnuEbbzBTTAwM6pEBULB0/XV5oH0qAR3dRVWzkLV2BsQbUD
DROGlJjvSuKhtMyycCMalh356/7mzSQGXSwmmufQEDkUX8vxIvHc84LQpWqP
3qDL72wjMXr7Rm+JIQ3DPrLMz0Ek2izIuLrbgIbBQqVHYAKJeh5Z+0P1aGif
q92Pl0ksbiueF8GggZ3c2DXrCom2oybtLQQNHh9Tn/dVkhjB3fuHtdrP/9Zp
iL9GYu6rwJUTSgqa/ZilNfdInFsmUuyXUyA+r7eH20Hi2jmPhInfKNCc0RI4
3kUiL+zYl/5pCpyjsn+7103iw/HpAFC7AViu2wdI1D8320/jKwX3B2ctF0rU
f7c/tKdPUiDf0Gq+c5TE7IHArJHPFDheO8m1miBxw6aT7q5q3+UbGD6ZInGp
SjVFfqKAlrTpFn4jsXaTU3fMBAWrvfI0g6Ukhg3bPu4YpyDurqfcVkGiQXdb
3Qq1a80Mv6pUJF7mcB4WjVHwH8uISkE=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$16989367#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV13k4lF0UAHDEzLxjmFAKLZQsEQml4js3LSQ7qYQiu5KlEJEslUgUQrKG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          "]]}, 
        Annotation[#, "Charting`Private`Tag$16989367#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVynk41HkAx3Ezrsn1M45xZOY3jhXLqqhNYr8f1doO9xE6FFEkOVKRq1nT
oUOjYjX1qCjWSIdKbSVa6nG0iGVlnWU3cnUQcsy2f7yf1z9vw6BIzxCmjIyM
y9f+NzbgxNKhg0JitNZ9icCbAbOny44LZo4Qpdd1DYn+DHQa93VxktOIebFS
QVwAA04D9sccBenk1p7vmdGhDKjYHYioW3WOrI0V/BCc+PWvMqvgr8shqf5b
k7yuMoDhZyUhFoUkq6mlYP0YA9ECLUvWxVLi9NnHMuQcE/yaP+llls+JXvtx
cd6ILDjW73+WN28k3YfmIr5YyWP3Ybs7sUYthCYr700GK+CkL7u86Wg7KbH7
/K7jpiKkJazljIxu8h7/ti7uZuGSk9NPVOprYut/JvCBoRLy90tEPYv+Idvd
6mkFP2VsGOSxxbfekg+STy/b9qpg9yV/6bod70jMvJ7KzCuqsMi+atI3PEQ0
31wxqLinhqbmjP7y0FEyvXCwJLacQmSB0akalQ+E7pbam0aoY91RoeMC14/k
QKWMd96IOtqO1Fw/JPlERImRPTvj2Ggv2LozQWacZD6NNzCcZWPRYJLDLvKZ
bJGcFZft18By15Ra7/MTxNO7d3TLpAZ+aWbttemYJF0xYRdcIjVRELevSdXu
C4ly/NFOcUQTFsRGkJYxTXTj0sUbQ7UgZxa/furlDBmfaa273qYF0aLcF8Hf
zZHfXz2/muGujehNXmFV0VISGejQ8ccTbbxplkhe+8rAWo8j9DHn4G0ad2ab
HANMofM+hwscaHt4b/K/xkAZV1ivKKcDg19nMhu8mBiwfbpaPlYHnadd4jhS
JjbuUswL79FBzbGg0L9yZOHodq998WpdpLMHbDjOclhVXFk0e0MXSUU2qg3D
cpiWvSYMZevBtNU03e+kPDIOtqo+itJDoejEu622CshmpBgMtOphw+a73r1t
CtBbrTCmv1QfHfddxT1HFGGVLPaTFevjlEnxsL85C1rGAcYmE/qYNLzbn1fN
Qp+H0bIYj/mwv2lZHr5vHork+UPuJfMhcQnLK+YooWSSWf9Y2QAfE3S2RzxT
wtqP2JAdboAxuz21s1HKyEVQfnG1AfjKaYTFVsHF+/NW5PC4+CZcWyR6ooIY
UX7K3we58OkIGTmzTRVrsusvz2/kgnQFjIVIVVETZjHaZ8QDM6qmpbRIDYmF
ufHtCTyMakxWCZwpXD5vXqtZz4NVwoMyNzcKDsIRB0EDD3vexJVyPSkUuY3n
jjTyMHRnquA3XwrJnAbUNPPQ7z19/EMQBWub3rqkVzz0ZM25B8ZTWLFkmN/X
x0OTvnwn8in49TOark3zwE591qJWSMHG4tto9iwP7oOH6zuKKGhbVzOS5nio
f6RQEXebgng8fMKLQaNuMyvv1mMKdmYpSxiKNKpylMP4zRSippZP+GnQkFV8
ETjSQsHINflGpSaNlZEnNz5uo9A07OFrpU2jgqg6+3VRWL+jOElWl0ZZj9pC
0QCFG5mB3GIujZk1DQsChijo5DgH6tA07G+f5luOUqgQN2cJ+DQeCtQ1qsco
uFU8avUxpjE10KiUNUHBUZ3fW25Cw9YzQzb4C4WLuaad5qY0Djz0mFk8S8FS
6WXt2QU0So00xqVSCude8SWzZjT+A85eFqY=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$16989367#3"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.922526, 0.385626, 0.209179], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJxTTMoPSmViYGAwAWIQXRLXbfqmqsW++92HxZ32LA4Ob49uStVeCecXNoro
cMzZBucrnLwib65zDM4XM/rQxKp5Ac7PabXaXKJ0Fc7vCRfcf6n9Fpz/fxOH
JePE+3D+fDc3d/7mR3D+srJVEx4YPIXzw17LCc7a8Bxh/vzI/15pr+B87RlL
VJ68fQPnX7o88cX+jPdwfv5ypd6TPB/hfK/2Fkd1v09w/o22k2saVn2G828t
j0+vZvgK5xu8rrXNsv8G51v61Z8Kmfkdzp9+maPY+M4POH95ReklXqtfCPfZ
Gzd2TvwN57NoVHr/vPgHzp9gsOhMiu4/RHhHB2ceKfwP5z++vGrVo3AGBxj/
eafsnwQWRjhfNDAkOnIpgi+z4s/U88FMcP7dft8Ksf8I/smOpIzr85jh/D7B
l8ZiPixwfu1qY97zbxF8tWtqfRE9rHD+ygndr+It2OD8sJgtIQ9vIPh3tvvN
etDGDuf3qqx9G6nJAef/UNzyYvEJBN9mvc7+7FJOOH+Vb+bitWJccP6navHk
3KMI/hervFN/C7jhfAXuTnsOQR44XzVbdMKEfQh+6J3Ud5MSeOF8+3txX1L/
I/hMBSevblvNB+e/F/pxpNGHH86/JMV612EZgr9NSejr//8IPgBKHpUH
          "]]}, 
        Annotation[#, "Charting`Private`Tag$16989367#4"]& ]}}, {}}, {
    DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
     AxesOrigin -> {0, 0}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {False, False}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, DisplayFunction :> 
     Identity, Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox["\"Entropy\"", TraditionalForm], None}, {
        FormBox["\"Coupling (\[Kappa])\"", TraditionalForm], None}}, 
     FrameStyle -> Automatic, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, GridLinesStyle -> Directive[
       GrayLevel[0.4, 0.5], 
       AbsoluteThickness[1], 
       AbsoluteDashing[{1, 2}]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None,
        "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, PlotLabel -> 
     FormBox["\"Entropy of Coupled Gaussian with \[Kappa] = 1\"", 
       TraditionalForm], PlotRange -> {{0.01, 4}, {0., 7.790030153517723}}, 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{
      "\"Coupled Entropy Root\"", "\"Tsallis Root Entropy\"", 
       "\"Normalized Tsallis Root Entropy\"", "\"Shannon Root Entropy\""}, 
      "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.560181`", ",", "0.691569`", ",", "0.194885`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6150173333333333, 0.25708400000000003`, 
                    0.13945266666666667`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.922526`", ",", "0.385626`", ",", "0.209179`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.922526, 0.385626, 0.209179]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.922526, 0.385626, 0.209179], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3, ",", #4}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.815564288971333*^9},
 CellLabel->
  "Out[131]=",ExpressionUUID->"479c3f34-eda3-46f5-a008-e43a8ea902a0"],

Cell[BoxData[
 RowBox[{
  RowBox[{"PlotCoupledEntDist", "[", 
   RowBox[{"\[Alpha]_", ",", "\[Kappa]Dist_"}], "]"}], ":=", 
  "\[IndentingNewLine]", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "coupledDist", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"coupledDist", "=", 
      RowBox[{"CoupledNormalDistribution", "[", 
       RowBox[{"0.", ",", "1.", ",", "\[Kappa]Dist"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Plot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"CoupledEntropy", "[", 
          RowBox[{
          "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
           "True"}], "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"TsallisRootEntropy", "[", 
          RowBox[{
          "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
           "False"}], "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"TsallisRootEntropy", "[", 
          RowBox[{
          "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
           "True"}], "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"CoupledEntropy", "[", 
          RowBox[{"coupledDist", ",", "0", ",", "2", ",", "1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
           "True"}], "]"}]}], "\[IndentingNewLine]", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Kappa]", ",", "0.01", ",", "4"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotLegends", "\[Rule]", 
        RowBox[{"{", " ", 
         RowBox[{
         "\"\<Coupled Entropy Root\>\"", ",", " ", 
          "\"\<Tsallis Root Entropy\>\"", ",", " ", 
          "\"\<Normalized Tsallis Root Entropy\>\"", ",", " ", 
          "\"\<Shannon Root Entropy\>\""}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotTheme", "\[Rule]", "\"\<Detailed\>\""}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<Coupling (\[Kappa])\>\"", ",", " ", "\"\<Entropy\>\""}],
          "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotLabel", "\[Rule]", 
        RowBox[{
        "\"\<Entropy of Coupled Gaussian with \[Kappa] = \>\"", "<>", 
         RowBox[{"ToString", "[", "\[Kappa]Dist", "]"}]}]}]}], 
      "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.815507856532708*^9, 3.8155078749313307`*^9}, {
  3.815507920600128*^9, 3.8155079991771803`*^9}, {3.815508052975773*^9, 
  3.8155080540454187`*^9}, {3.815563356018063*^9, 3.815563356873921*^9}, {
  3.815563453958754*^9, 3.8155634924795218`*^9}, {3.817208296746953*^9, 
  3.817208296855381*^9}},ExpressionUUID->"bcd002f5-ec51-4ac2-baf8-\
98846dc348b8"],

Cell["\<\
Derivative as a function of coupling for the Coupled Entropy, Coupled Root \
Entropy, and Tsallis Entropy\
\>", "Text",
 CellChangeTimes->{{3.8155088788721523`*^9, 
  3.815508908663927*^9}},ExpressionUUID->"79797356-18d0-4b9e-9880-\
97c2fe17bfce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotDerivativeCoupledEntDist", "[", 
  RowBox[{"2", ",", "1"}], "]"}]], "Input",
 CellChangeTimes->{{3.8155096422419033`*^9, 3.81550964254325*^9}},
 CellLabel->
  "In[127]:=",ExpressionUUID->"aab23c28-4ca2-4222-8865-f6f271b6f4b5"],

Cell[BoxData[
 TemplateBox[{
  "General", "ivar", 
   "\"\\!\\(\\*RowBox[{\\\"0.01008151`\\\"}]\\) is not a valid variable.\"", 
   2, 127, 125, 31225010404182441629, "Local"},
  "MessageTemplate"]], "Message", "MSG",
 CellChangeTimes->{{3.815509645240756*^9, 3.815509666531644*^9}, 
   3.8155097304001617`*^9},
 CellLabel->
  "During evaluation of \
In[127]:=",ExpressionUUID->"46a0c922-070e-407e-a893-7701721f33ac"],

Cell[BoxData[
 TemplateBox[{
  "General", "ivar", 
   "\"\\!\\(\\*RowBox[{\\\"0.09151008142857144`\\\"}]\\) is not a valid \
variable.\"", 2, 127, 126, 31225010404182441629, "Local"},
  "MessageTemplate"]], "Message", "MSG",
 CellChangeTimes->{{3.815509645240756*^9, 3.815509666531644*^9}, 
   3.815509730871421*^9},
 CellLabel->
  "During evaluation of \
In[127]:=",ExpressionUUID->"410e051b-7a18-4498-992a-6031ca311323"],

Cell[BoxData[
 TemplateBox[{
  "General", "ivar", 
   "\"\\!\\(\\*RowBox[{\\\"0.17293865285714288`\\\"}]\\) is not a valid \
variable.\"", 2, 127, 127, 31225010404182441629, "Local"},
  "MessageTemplate"]], "Message", "MSG",
 CellChangeTimes->{{3.815509645240756*^9, 3.815509666531644*^9}, 
   3.815509731347848*^9},
 CellLabel->
  "During evaluation of \
In[127]:=",ExpressionUUID->"a235089f-770b-4576-9053-0787b15ee90c"],

Cell[BoxData[
 TemplateBox[{
  "General", "stop", 
   "\"Further output of \\!\\(\\*StyleBox[RowBox[{\\\"General\\\", \
\\\"::\\\", \\\"ivar\\\"}], \\\"MessageName\\\"]\\) will be suppressed during \
this calculation.\"", 2, 127, 128, 31225010404182441629, "Local"},
  "MessageTemplate"]], "Message", "MSG",
 CellChangeTimes->{{3.815509645240756*^9, 3.815509666531644*^9}, 
   3.815509731352797*^9},
 CellLabel->
  "During evaluation of \
In[127]:=",ExpressionUUID->"67c56e26-0990-4d54-bc07-3f9af9fdb7ec"],

Cell[BoxData["$Aborted"], "Output",
 CellChangeTimes->{3.815509656103508*^9, 3.815509692312213*^9, 
  3.815509741831614*^9},
 CellLabel->
  "Out[127]=",ExpressionUUID->"2cbee2ff-e9b3-4c99-80f1-3bda487e99f4"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"PlotDerivativeCoupledEntDist", "[", 
   RowBox[{"\[Alpha]_", ",", "\[Kappa]Dist_"}], "]"}], ":=", 
  "\[IndentingNewLine]", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "coupledDist", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"coupledDist", "=", 
      RowBox[{"CoupledNormalDistribution", "[", 
       RowBox[{"0.", ",", "1.", ",", "\[Kappa]Dist"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Plot", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"DifferenceQuotient", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"CoupledEntropy", "[", 
            RowBox[{
            "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",",
              "False"}], "]"}], ",", "\[IndentingNewLine]", 
           RowBox[{"(*", 
            RowBox[{
             RowBox[{"CoupledEntropy", "[", 
              RowBox[{
              "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
               
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], 
               ",", "True"}], "]"}], ","}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{"TsallisEntropy", "[", 
            RowBox[{
            "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",",
              "False"}], "]"}], ",", "\[IndentingNewLine]", 
           RowBox[{"TsallisEntropy", "[", 
            RowBox[{
            "coupledDist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "1", ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",",
              "True"}], "]"}], ",", "\[IndentingNewLine]", 
           RowBox[{"CoupledEntropy", "[", 
            RowBox[{"coupledDist", ",", "0", ",", "1", ",", "1", ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",",
              "False"}], "]"}]}], "\[IndentingNewLine]", "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"\[Kappa]", ",", "0.01"}], "}"}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"\[Kappa]", ",", "0.01", ",", "4"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PlotLegends", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
         "\"\<Coupled Entropy\>\"", ",", " ", "\"\<Coupled Entropy Root\>\"", 
          ",", " ", "\n", "\"\<Tsallis Entropy\>\"", ",", " ", 
          "\"\<Normalized Tsallis Entropy\>\"", ",", " ", 
          "\"\<Shannon Entropy\>\""}], "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotTheme", "\[Rule]", "\"\<Detailed\>\""}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"FrameLabel", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<Coupling (\[Kappa])\>\"", ",", " ", "\"\<Entropy\>\""}],
          "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotLabel", "\[Rule]", 
        RowBox[{
        "\"\<Derivative of Entropy of Coupled Gaussian with \[Kappa] = \>\"", 
         "<>", 
         RowBox[{"ToString", "[", "\[Kappa]Dist", "]"}]}]}]}], 
      "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.81550893824189*^9, 3.815508940079618*^9}, {
   3.815509564067985*^9, 3.815509622572283*^9}, 3.815509659676433*^9, {
   3.815509718188531*^9, 3.81550972123829*^9}},
 CellLabel->
  "In[126]:=",ExpressionUUID->"abda6c95-f27a-497e-8aaa-6b753f5dd0a0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Limit of Coupled Root Entropy", "Subsubsection",
 CellChangeTimes->{{3.8155074648104143`*^9, 
  3.815507473760462*^9}},ExpressionUUID->"6e29c606-03b6-431a-a4a4-\
d97818734ab1"],

Cell["Need symbolic definition of Coupled Root Entropy", "Text",
 CellChangeTimes->{{3.815507753100246*^9, 
  3.8155077653029413`*^9}},ExpressionUUID->"510ae414-5b46-42fd-b1f3-\
4718d1c241ca"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Limit", "[", 
  RowBox[{
   RowBox[{"CoupledEntropy", "[", 
    RowBox[{
     RowBox[{"CoupledNormalDistribution", "[", 
      RowBox[{"0.", ",", "\[Sigma]", ",", "\[Kappa]"}], "]"}], ",", 
     "\[Kappa]", ",", "2", ",", "1", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
     "True"}], "]"}], ",", "\[IndentingNewLine]", 
   RowBox[{"\[Kappa]", "\[Rule]", "\[Infinity]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.815507481370454*^9, 3.81550748505613*^9}, {
  3.815507530111717*^9, 3.815507555751562*^9}, {3.815507613574238*^9, 
  3.815507639795665*^9}},
 CellLabel->
  "In[112]:=",ExpressionUUID->"4ea59560-5572-47f9-81d1-8fc5f9f5327d"],

Cell[BoxData[
 TemplateBox[{
  "NIntegrate", "inumr", 
   "\"The integrand \\!\\(\\*RowBox[{RowBox[{\\\"If\\\", \\\"[\\\", \
RowBox[{RowBox[{\\\"\[Kappa]\\\", \\\"\[Equal]\\\", \\\"0\\\"}], \\\",\\\", \
RowBox[{\\\"PDF\\\", \\\"[\\\", RowBox[{RowBox[{\\\"ProbabilityDistribution\\\
\", \\\"[\\\", RowBox[{FractionBox[\\\"1\\\", RowBox[{RowBox[{\\\"Piecewise\\\
\", \\\"[\\\", RowBox[{RowBox[{\\\"{\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \\\
\"2\\\", \\\"\[RightSkeleton]\\\"}], \\\"}\\\"}], \\\",\\\", \
RowBox[{\\\"Times\\\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \\\"5\\\", \
\\\"\[RightSkeleton]\\\"}], \\\"]\\\"}]}], \\\"]\\\"}], \\\" \\\", \
SqrtBox[RowBox[{\\\"Which\\\", \\\"[\\\", RowBox[{RowBox[{\\\"Greater\\\", \\\
\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \\\"2\\\", \
\\\"\[RightSkeleton]\\\"}], \\\"]\\\"}], \\\",\\\", RowBox[{\\\"\
\[LeftSkeleton]\\\", \\\"6\\\", \\\"\[RightSkeleton]\\\"}], \\\",\\\", \
RowBox[{\\\"Message\\\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \
\\\"2\\\", \\\"\[RightSkeleton]\\\"}], \\\"]\\\"}]}], \\\"]\\\"}]]}]], \
\\\",\\\", RowBox[{\\\"If\\\", \\\"[\\\", RowBox[{RowBox[{\\\"\[Kappa]\\\", \
\\\"\[GreaterEqual]\\\", \\\"0\\\"}], \\\",\\\", RowBox[{\\\"{\\\", \
RowBox[{\\\"x$13681176\\\", \\\",\\\", RowBox[{\\\"-\\\", \
RowBox[{\\\"DirectedInfinity\\\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \
\\\"1\\\", \\\"\[RightSkeleton]\\\"}], \\\"]\\\"}]}], \\\",\\\", \\\"\
\[Infinity]\\\"}], \\\"}\\\"}], \\\",\\\", RowBox[{\\\"{\\\", \
RowBox[{\\\"x$13681176\\\", \\\",\\\", RowBox[{RowBox[{\\\"0.`\\\", \\\"\
\[VeryThinSpace]\\\"}], \\\"+\\\", RowBox[{\\\"Times\\\", \\\"[\\\", RowBox[{\
\\\"\[LeftSkeleton]\\\", \\\"2\\\", \\\"\[RightSkeleton]\\\"}], \
\\\"]\\\"}]}], \\\",\\\", RowBox[{RowBox[{\\\"0.`\\\", \\\"\[VeryThinSpace]\\\
\"}], \\\"+\\\", SqrtBox[RowBox[{\\\"\[LeftSkeleton]\\\", \\\"1\\\", \\\"\
\[RightSkeleton]\\\"}]]}]}], \\\"}\\\"}]}], \\\"]\\\"}]}], \\\"]\\\"}], \\\",\
\\\", \\\"x\\\"}], \\\"]\\\"}], \\\",\\\", RowBox[{\\\"FullSimplify\\\", \
\\\"[\\\", FractionBox[SuperscriptBox[RowBox[{\\\"PDF\\\", \\\"[\\\", \
RowBox[{RowBox[{\\\"ProbabilityDistribution\\\", \\\"[\\\", \
RowBox[{RowBox[{\\\"Times\\\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \\\
\"2\\\", \\\"\[RightSkeleton]\\\"}], \\\"]\\\"}], \\\",\\\", RowBox[{\\\"If\\\
\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \\\"3\\\", \\\"\
\[RightSkeleton]\\\"}], \\\"]\\\"}]}], \\\"]\\\"}], \\\",\\\", \\\"x\\\"}], \
\\\"]\\\"}], RowBox[{\\\"1\\\", \\\"-\\\", RowBox[{\\\"Times\\\", \\\"[\\\", \
RowBox[{\\\"\[LeftSkeleton]\\\", \\\"3\\\", \\\"\[RightSkeleton]\\\"}], \\\"]\
\\\"}]}]], RowBox[{SubsuperscriptBox[\\\"\[Integral]\\\", \
RowBox[{\\\"DirectedInfinity\\\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \
\\\"1\\\", \\\"\[RightSkeleton]\\\"}], \\\"]\\\"}], \
RowBox[{\\\"DirectedInfinity\\\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \
\\\"1\\\", \\\"\[RightSkeleton]\\\"}], \\\"]\\\"}]], \
RowBox[{SuperscriptBox[RowBox[{\\\"PDF\\\", \\\"[\\\", RowBox[{\\\"\
\[LeftSkeleton]\\\", \\\"2\\\", \\\"\[RightSkeleton]\\\"}], \\\"]\\\"}], \
RowBox[{\\\"Plus\\\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \\\"2\\\", \
\\\"\[RightSkeleton]\\\"}], \\\"]\\\"}]], RowBox[{\\\"\[DifferentialD]\\\", \
\\\"y\\\"}]}]}]], \\\"]\\\"}]}], \\\"]\\\"}], \\\" \\\", \
SqrtBox[RowBox[{\\\"If\\\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \\\"1\
\\\", \\\"\[RightSkeleton]\\\"}], \\\"]\\\"}]]}]\\) has evaluated to \
non-numerical values for all sampling points in the region with boundaries \
\\!\\(\\*RowBox[{\\\"{\\\", RowBox[{\\\"{\\\", RowBox[{RowBox[{\\\"-\\\", \
\\\"\[Infinity]\\\"}], \\\",\\\", \\\"0.`\\\"}], \\\"}\\\"}], \\\"}\\\"}]\\).\
\"", 2, 112, 116, 31225010404182441629, "Local"},
  "MessageTemplate"]], "Message", "MSG",
 CellChangeTimes->{3.815507655430998*^9},
 CellLabel->
  "During evaluation of \
In[112]:=",ExpressionUUID->"2c125974-feb1-463f-a5b2-d1fefa79d8b7"],

Cell[BoxData[
 TemplateBox[{
  "NIntegrate", "inumr", 
   "\"The integrand \\!\\(\\*RowBox[{RowBox[{\\\"If\\\", \\\"[\\\", \
RowBox[{RowBox[{\\\"\[Kappa]\\\", \\\"\[Equal]\\\", \\\"0\\\"}], \\\",\\\", \
RowBox[{\\\"PDF\\\", \\\"[\\\", RowBox[{RowBox[{\\\"ProbabilityDistribution\\\
\", \\\"[\\\", RowBox[{FractionBox[\\\"1\\\", RowBox[{RowBox[{\\\"Piecewise\\\
\", \\\"[\\\", RowBox[{RowBox[{\\\"{\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \\\
\"2\\\", \\\"\[RightSkeleton]\\\"}], \\\"}\\\"}], \\\",\\\", \
RowBox[{\\\"Times\\\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \\\"5\\\", \
\\\"\[RightSkeleton]\\\"}], \\\"]\\\"}]}], \\\"]\\\"}], \\\" \\\", \
SqrtBox[RowBox[{\\\"Which\\\", \\\"[\\\", RowBox[{RowBox[{\\\"Greater\\\", \\\
\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \\\"2\\\", \
\\\"\[RightSkeleton]\\\"}], \\\"]\\\"}], \\\",\\\", RowBox[{\\\"\
\[LeftSkeleton]\\\", \\\"6\\\", \\\"\[RightSkeleton]\\\"}], \\\",\\\", \
RowBox[{\\\"Message\\\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \
\\\"2\\\", \\\"\[RightSkeleton]\\\"}], \\\"]\\\"}]}], \\\"]\\\"}]]}]], \
\\\",\\\", RowBox[{\\\"If\\\", \\\"[\\\", RowBox[{RowBox[{\\\"\[Kappa]\\\", \
\\\"\[GreaterEqual]\\\", \\\"0\\\"}], \\\",\\\", RowBox[{\\\"{\\\", \
RowBox[{\\\"x$13681176\\\", \\\",\\\", RowBox[{\\\"-\\\", \
RowBox[{\\\"DirectedInfinity\\\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \
\\\"1\\\", \\\"\[RightSkeleton]\\\"}], \\\"]\\\"}]}], \\\",\\\", \\\"\
\[Infinity]\\\"}], \\\"}\\\"}], \\\",\\\", RowBox[{\\\"{\\\", \
RowBox[{\\\"x$13681176\\\", \\\",\\\", RowBox[{RowBox[{\\\"0.`\\\", \\\"\
\[VeryThinSpace]\\\"}], \\\"+\\\", RowBox[{\\\"Times\\\", \\\"[\\\", RowBox[{\
\\\"\[LeftSkeleton]\\\", \\\"2\\\", \\\"\[RightSkeleton]\\\"}], \
\\\"]\\\"}]}], \\\",\\\", RowBox[{RowBox[{\\\"0.`\\\", \\\"\[VeryThinSpace]\\\
\"}], \\\"+\\\", SqrtBox[RowBox[{\\\"\[LeftSkeleton]\\\", \\\"1\\\", \\\"\
\[RightSkeleton]\\\"}]]}]}], \\\"}\\\"}]}], \\\"]\\\"}]}], \\\"]\\\"}], \\\",\
\\\", \\\"x\\\"}], \\\"]\\\"}], \\\",\\\", RowBox[{\\\"FullSimplify\\\", \
\\\"[\\\", FractionBox[SuperscriptBox[RowBox[{\\\"PDF\\\", \\\"[\\\", \
RowBox[{RowBox[{\\\"ProbabilityDistribution\\\", \\\"[\\\", \
RowBox[{RowBox[{\\\"Times\\\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \\\
\"2\\\", \\\"\[RightSkeleton]\\\"}], \\\"]\\\"}], \\\",\\\", RowBox[{\\\"If\\\
\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \\\"3\\\", \\\"\
\[RightSkeleton]\\\"}], \\\"]\\\"}]}], \\\"]\\\"}], \\\",\\\", \\\"x\\\"}], \
\\\"]\\\"}], RowBox[{\\\"1\\\", \\\"-\\\", RowBox[{\\\"Times\\\", \\\"[\\\", \
RowBox[{\\\"\[LeftSkeleton]\\\", \\\"3\\\", \\\"\[RightSkeleton]\\\"}], \\\"]\
\\\"}]}]], RowBox[{SubsuperscriptBox[\\\"\[Integral]\\\", \
RowBox[{\\\"DirectedInfinity\\\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \
\\\"1\\\", \\\"\[RightSkeleton]\\\"}], \\\"]\\\"}], \
RowBox[{\\\"DirectedInfinity\\\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \
\\\"1\\\", \\\"\[RightSkeleton]\\\"}], \\\"]\\\"}]], \
RowBox[{SuperscriptBox[RowBox[{\\\"PDF\\\", \\\"[\\\", RowBox[{\\\"\
\[LeftSkeleton]\\\", \\\"2\\\", \\\"\[RightSkeleton]\\\"}], \\\"]\\\"}], \
RowBox[{\\\"Plus\\\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \\\"2\\\", \
\\\"\[RightSkeleton]\\\"}], \\\"]\\\"}]], RowBox[{\\\"\[DifferentialD]\\\", \
\\\"y\\\"}]}]}]], \\\"]\\\"}]}], \\\"]\\\"}], \\\" \\\", \
SqrtBox[RowBox[{\\\"If\\\", \\\"[\\\", RowBox[{\\\"\[LeftSkeleton]\\\", \\\"1\
\\\", \\\"\[RightSkeleton]\\\"}], \\\"]\\\"}]]}]\\) has evaluated to \
non-numerical values for all sampling points in the region with boundaries \
\\!\\(\\*RowBox[{\\\"{\\\", RowBox[{\\\"{\\\", RowBox[{RowBox[{\\\"-\\\", \
\\\"\[Infinity]\\\"}], \\\",\\\", \\\"0.`\\\"}], \\\"}\\\"}], \\\"}\\\"}]\\).\
\"", 2, 112, 117, 31225010404182441629, "Local"},
  "MessageTemplate"]], "Message", "MSG",
 CellChangeTimes->{3.8155076609133863`*^9},
 CellLabel->
  "During evaluation of \
In[112]:=",ExpressionUUID->"a98e5552-3156-48a7-a359-ae5dcf390ea6"],

Cell[BoxData[
 TemplateBox[{
  "Limit", "alimv", 
   "\"Warning: Assumptions that involve the limit variable are ignored.\"", 2,
    112, 118, 31225010404182441629, "Local"},
  "MessageTemplate"]], "Message", "MSG",
 CellChangeTimes->{3.815507660919628*^9},
 CellLabel->
  "During evaluation of \
In[112]:=",ExpressionUUID->"14e398d9-2da7-42aa-9637-4b87a7080b25"],

Cell[BoxData["$Aborted"], "Output",
 CellChangeTimes->{3.815507677948488*^9},
 CellLabel->
  "Out[112]=",ExpressionUUID->"73a325e9-2e10-4b7e-a365-2cc970764a0c"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Test functionality of CoupledEntropy", "Subsubsection",
 CellChangeTimes->{{3.815493006470311*^9, 
  3.8154930127716827`*^9}},ExpressionUUID->"bfda7348-f0e1-4579-8533-\
2e31e5eb59c6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Assumptions", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "1"}], "<", "\[Kappa]", "<", "\[Infinity]"}], "&&", 
    RowBox[{"0", "<", "\[Sigma]", "<", "\[Infinity]"}], "&&", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", "\[Kappa]", ",", "\[Sigma]"}], "}"}], "\[Element]", 
     "Reals"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.8154934446253347`*^9, 3.815493448302814*^9}, {
  3.815493489953537*^9, 3.815493510150202*^9}, {3.815494896995405*^9, 
  3.8154949147737617`*^9}, {3.815507580535099*^9, 3.815507598916457*^9}},
 CellLabel->
  "In[111]:=",ExpressionUUID->"b3cab995-c90f-4d01-b55a-2c9209e686c5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CoupledEntropy", "[", 
  RowBox[{
   RowBox[{"CoupledNormalDistribution", "[", 
    RowBox[{"0", ",", "1", ",", "1"}], "]"}], ",", "\[Kappa]", ",", "2"}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.815493022085582*^9, 3.815493023911517*^9}, {
  3.815493262178259*^9, 3.815493362825776*^9}},
 CellLabel->"In[48]:=",ExpressionUUID->"72d68c70-6f95-4398-826e-8a31a0c132ae"],

Cell[BoxData["$Aborted"], "Output",
 CellChangeTimes->{3.815493440972163*^9},
 CellLabel->"Out[48]=",ExpressionUUID->"529583c3-f6c7-44d8-9f64-73045f85795d"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CoupledEntropy", "[", 
    RowBox[{
     RowBox[{"CoupledNormalDistribution", "[", 
      RowBox[{"0", ",", "1", ",", "1"}], "]"}], ",", "#", ",", "2"}], "]"}], 
   "&"}], "/@", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"-", "0.5"}], ",", "0.", ",", "0.5", ",", "1.", ",", "1.5"}], 
   "}"}]}]], "Input",
 CellChangeTimes->{{3.8154935220425863`*^9, 3.815493529571732*^9}, {
  3.8154935655378513`*^9, 3.8154935871211233`*^9}, {3.815493641129386*^9, 
  3.815493643953992*^9}},
 CellLabel->"In[51]:=",ExpressionUUID->"3e411c41-8182-4db6-96ab-173dd8e2a727"],

Cell[BoxData[
 TemplateBox[{
  "Integrate", "idiv", 
   "\"Integral of \\!\\(\\*RowBox[{\\\"3.141592653589793`\\\", \\\" \\\", \
SuperscriptBox[RowBox[{\\\"(\\\", RowBox[{\\\"1\\\", \\\"+\\\", \
SuperscriptBox[\\\"y\\\", \\\"2\\\"]}], \\\")\\\"}], \\\"1.`\\\"]}]\\) does \
not converge on \\!\\(\\*RowBox[{\\\"{\\\", RowBox[{RowBox[{\\\"-\\\", \\\"\
\[Infinity]\\\"}], \\\",\\\", \\\"\[Infinity]\\\"}], \\\"}\\\"}]\\).\"", 2, 
   51, 7, 31225010404182441629, "Local"},
  "MessageTemplate"]], "Message", "MSG",
 CellChangeTimes->{3.815493588061145*^9, 3.815493645060169*^9},
 CellLabel->
  "During evaluation of \
In[51]:=",ExpressionUUID->"60da094d-e4d8-48e7-a424-281d4ac4815c"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubsuperscriptBox["\[Integral]", 
      RowBox[{"-", "\[Infinity]"}], "\[Infinity]"], 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1.5707963267948966`", " ", 
           SuperscriptBox[
            RowBox[{"(", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"1", "+", 
                SuperscriptBox["x", "2"]}], ")"}], "2"], ")"}], "0.5`"], " ", 
           
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"1", "+", 
                 SuperscriptBox["x", "2"]}], ")"}], "2"], "\[GreaterEqual]", 
              "0"}], ",", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "0.5`"}], "\[NotEqual]", "0"}], ",", 
               RowBox[{"-", 
                FractionBox[
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["\[Pi]", "2"], " ", 
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{"1", "+", 
                    SuperscriptBox["x", "2"]}], ")"}], "2"]}], ")"}], 
                   RowBox[{"-", 
                    FractionBox["0.5`", 
                    RowBox[{"1", "+", 
                    RowBox[{"1", " ", 
                    RowBox[{"(", 
                    RowBox[{"-", "0.5`"}], ")"}]}]}]]}]], "-", "1"}], 
                 "0.5`"]}], ",", 
               RowBox[{"Log", "[", 
                RowBox[{
                 SuperscriptBox["\[Pi]", "2"], " ", 
                 SuperscriptBox[
                  RowBox[{"(", 
                   RowBox[{"1", "+", 
                    SuperscriptBox["x", "2"]}], ")"}], "2"]}], "]"}]}], "]"}],
              ",", "Undefined"}], "]"}]}], ")"}], "/", 
         RowBox[{"(", 
          RowBox[{
           SubsuperscriptBox["\[Integral]", 
            RowBox[{"-", "\[Infinity]"}], "\[Infinity]"], 
           RowBox[{
            RowBox[{"3.141592653589793`", " ", 
             SuperscriptBox[
              RowBox[{"(", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{"1", "+", 
                  SuperscriptBox["y", "2"]}], ")"}], "2"], ")"}], "0.5`"]}], 
            RowBox[{"\[DifferentialD]", "y"}]}]}], ")"}]}], ")"}]}], 
      RowBox[{"\[DifferentialD]", "x"}]}]}]}], ",", "2.5310242469692907`", 
   ",", "2.6996173720632335`", ",", "2.641592653589793`", ",", 
   "2.496403403498721`"}], "}"}]], "Output",
 CellChangeTimes->{3.815493605447853*^9, 3.815493654135233*^9},
 CellLabel->"Out[51]=",ExpressionUUID->"b9a83591-2b17-47c5-be0d-a7b130fc2302"]
}, Open  ]],

Cell[CellGroupData[{

Cell[, "Input",ExpressionUUID->"f780ec73-0c1b-4f76-9368-733014bbfcdd"],

Cell[, "Output",ExpressionUUID->"dbe471ef-3891-4daa-bb01-dbfab0de4349"]
}, Open  ]],

Cell[CellGroupData[{

Cell[, "Input",ExpressionUUID->"49de8bd7-5866-4397-9340-431da7f863ef"],

Cell[, "Output",ExpressionUUID->"2152fb3b-6735-4f74-96de-60a96679ebd5"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[, "Subsubsection",ExpressionUUID->"6625a561-f15a-47c1-9f7c-794dd16a48b4"],

Cell[CellGroupData[{

Cell[, "Input",ExpressionUUID->"17bcad04-db35-48ec-abec-c5f12837cf4c"],

Cell[, "Output",ExpressionUUID->"424c115f-9c29-4049-a155-877d3bcc0dde"]
}, Open  ]],

Cell[CellGroupData[{

Cell[, "Input",ExpressionUUID->"2a09473d-bfa1-4e5a-9018-81ec1fec4d9c"],

Cell[, "Output",ExpressionUUID->"66d531c2-124b-4465-9333-f1333a511a41"]
}, Open  ]],

Cell[CellGroupData[{

Cell[, "Input",ExpressionUUID->"cf51caf2-1017-4bb0-b32b-5ba5f72eb4cf"],

Cell[, "Output",ExpressionUUID->"2bea6ba9-9830-4713-824f-b941e093c60a"]
}, Open  ]],

Cell[CellGroupData[{

Cell[, "Input",ExpressionUUID->"634c7674-3b4e-4b86-bca1-af76ab859c48"],

Cell[, "Output",ExpressionUUID->"dacde6da-2824-41ee-995f-2cea5ebdaf1c"]
}, Closed]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1331, 855},
WindowMargins->{{14, Automatic}, {Automatic, 23}},
Magnification:>1.5 Inherited,
FrontEndVersion->"12.1 for Mac OS X x86 (64-bit) (June 19, 2020)",
StyleDefinitions->"Default.nb",
ExpressionUUID->"128fb970-3af8-4b35-ae2c-96b85a924897"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 189, 3, 101, "Section",ExpressionUUID->"160571c2-5acf-4c9f-ad69-dee4855dae9c"],
Cell[772, 27, 1596, 49, 135, "Text",ExpressionUUID->"37b1d44c-7184-414d-ab3a-23be0d24221a"],
Cell[2371, 78, 1269, 23, 78, "Input",ExpressionUUID->"f1a7f510-09d9-453e-ab58-758cb7449e0c"],
Cell[3643, 103, 3811, 84, 636, "Input",ExpressionUUID->"2e347783-5b72-46ae-ae76-6cff4c31e00d"],
Cell[7457, 189, 145, 2, 46, "Input",ExpressionUUID->"7e75eb80-ba0d-45fc-a1cb-8cf6b73462cd"],
Cell[7605, 193, 139027, 2342, 375, InheritFromParent,ExpressionUUID->"0ff8ce3e-7f07-4e60-a991-272829c0f262"],
Cell[146635, 2537, 6756, 146, 1092, "Input",ExpressionUUID->"232a5b2e-e16e-496d-a44b-258988428cd2"],
Cell[CellGroupData[{
Cell[153416, 2687, 145, 2, 46, "Input",ExpressionUUID->"da8233c6-bcd1-4cb6-80b0-bc3f7b1318aa"],
Cell[153564, 2691, 23328, 469, 390, "Output",ExpressionUUID->"4b6ecd3d-1c6c-4098-bf0b-ba276a8908e2"]
}, Open  ]],
Cell[176907, 3163, 157, 3, 53, "Text",ExpressionUUID->"42b177a6-1ba4-43ce-af1d-45836247adf6"],
Cell[177067, 3168, 14218, 297, 379, "Output",ExpressionUUID->"f06a7e95-6a7e-4fda-8e63-743e5348ed89"]
}, Open  ]],
Cell[CellGroupData[{
Cell[191322, 3470, 270, 4, 101, "Section",ExpressionUUID->"ac19dfb8-be49-418a-809a-28f5cf7cb726"],
Cell[191595, 3476, 1728, 48, 170, "Text",ExpressionUUID->"be2941ff-a13a-4c68-9e78-ee6fe13f9aa3"],
Cell[193326, 3526, 750, 18, 178, "Text",ExpressionUUID->"5b8c9bf8-061e-4d5d-ac01-1c9866ad995f"],
Cell[194079, 3546, 4022, 103, 472, "Input",ExpressionUUID->"8d63fd49-254f-4bfe-89a7-219d53d9ab06"],
Cell[CellGroupData[{
Cell[198126, 3653, 166, 3, 67, "Subsubsection",ExpressionUUID->"fe322833-1152-426c-b436-6b15341cac6c"],
Cell[198295, 3658, 204, 3, 53, "Text",ExpressionUUID->"07bfb0cf-1109-404a-9c53-998f1f5b23c1"],
Cell[CellGroupData[{
Cell[198524, 3665, 194, 3, 46, "Input",ExpressionUUID->"eb39eb5a-0b49-4cc3-98d1-c1afea678255"],
Cell[198721, 3670, 567, 8, 52, "Output",ExpressionUUID->"9ad4f1c6-5c46-482c-995c-eeb162f8fd22"]
}, Open  ]],
Cell[199303, 3681, 242, 6, 53, "Text",ExpressionUUID->"6dd421a1-2300-4c33-ac6a-c14ff51f04fc"],
Cell[199548, 3689, 214, 5, 53, "Text",ExpressionUUID->"a3818579-dcee-4077-8112-347e96aea0a7"],
Cell[199765, 3696, 11399, 216, 373, "Input",ExpressionUUID->"b535a6d9-3fc4-421c-87bf-f06697449221"],
Cell[211167, 3914, 3888, 82, 543, "Input",ExpressionUUID->"efb1ac77-5079-45ff-9367-9207616a8aa3"],
Cell[215058, 3998, 177, 3, 53, "Text",ExpressionUUID->"e97a4ae0-f75b-400b-97c4-c662de408480"],
Cell[CellGroupData[{
Cell[215260, 4005, 150, 2, 46, "Input",ExpressionUUID->"9a599a6b-74d4-4a10-b1ee-c2ac2fa298e4"],
Cell[215413, 4009, 27609, 549, 387, "Output",ExpressionUUID->"5d9ddeab-f152-46d6-9c3c-a5cb6bf8be85"]
}, Open  ]],
Cell[243037, 4561, 453, 10, 122, "Text",ExpressionUUID->"228f4d5a-2b94-477b-bb93-b29f3429e8ea"],
Cell[243493, 4573, 27347, 548, 381, "Output",ExpressionUUID->"1842b8d6-1265-4926-a0b8-51b2eca3d61d"],
Cell[270843, 5123, 2789, 63, 512, "Input",ExpressionUUID->"5ea7c959-99c7-4169-b08d-4ad685050bf5"],
Cell[273635, 5188, 189, 3, 53, "Text",ExpressionUUID->"e0ab9dd1-d052-4b67-acd5-77402e78c39b"],
Cell[CellGroupData[{
Cell[273849, 5195, 243, 5, 46, "Input",ExpressionUUID->"3a090cdf-446c-49b7-b911-5741df1f0c92"],
Cell[274095, 5202, 36863, 722, 409, "Output",ExpressionUUID->"46b6c723-fbec-44a2-bdf9-c6e8297aa4db"]
}, Open  ]],
Cell[310973, 5927, 165, 3, 53, "Text",ExpressionUUID->"c57a1c3c-9f79-4654-8940-b481dcb7d3f4"],
Cell[311141, 5932, 36863, 722, 409, "Output",ExpressionUUID->"2f29f9a9-5207-418c-a049-930b0d8e6115"],
Cell[348007, 6656, 3441, 77, 574, "Input",ExpressionUUID->"39e2ea94-e57b-48d6-93ae-9d5cc48d76d3"],
Cell[351451, 6735, 194, 3, 53, "Text",ExpressionUUID->"bce2d6ca-9099-417d-bd7e-fffa1b163966"],
Cell[CellGroupData[{
Cell[351670, 6742, 177, 4, 46, "Input",ExpressionUUID->"5496c6cf-fd44-47c8-8eea-384e590a70c0"],
Cell[351850, 6748, 31918, 622, 409, "Output",ExpressionUUID->"e9a3d61a-ad98-46be-820b-6ae655cacfc1"]
}, Open  ]],
Cell[383783, 7373, 165, 3, 53, "Text",ExpressionUUID->"627ef1f3-d256-40ed-978e-9acaab0affc1"],
Cell[383951, 7378, 31918, 622, 409, "Output",ExpressionUUID->"479c3f34-eda3-46f5-a008-e43a8ea902a0"],
Cell[415872, 8002, 3236, 72, 543, "Input",ExpressionUUID->"bcd002f5-ec51-4ac2-baf8-98846dc348b8"],
Cell[419111, 8076, 257, 6, 53, "Text",ExpressionUUID->"79797356-18d0-4b9e-9880-97c2fe17bfce"],
Cell[CellGroupData[{
Cell[419393, 8086, 254, 5, 46, "Input",ExpressionUUID->"aab23c28-4ca2-4222-8865-f6f271b6f4b5"],
Cell[419650, 8093, 417, 10, 34, "Message",ExpressionUUID->"46a0c922-070e-407e-a893-7701721f33ac"],
Cell[420070, 8105, 422, 10, 34, "Message",ExpressionUUID->"410e051b-7a18-4498-992a-6031ca311323"],
Cell[420495, 8117, 422, 10, 34, "Message",ExpressionUUID->"a235089f-770b-4576-9053-0787b15ee90c"],
Cell[420920, 8129, 505, 11, 34, "Message",ExpressionUUID->"67c56e26-0990-4d54-bc07-3f9af9fdb7ec"],
Cell[421428, 8142, 207, 4, 52, "Output",ExpressionUUID->"2cbee2ff-e9b3-4c99-80f1-3bda487e99f4"]
}, Open  ]],
Cell[421650, 8149, 3887, 87, 667, "Input",ExpressionUUID->"abda6c95-f27a-497e-8aaa-6b753f5dd0a0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[425574, 8241, 182, 3, 67, "Subsubsection",ExpressionUUID->"6e29c606-03b6-431a-a4a4-d97818734ab1"],
Cell[425759, 8246, 192, 3, 53, "Text",ExpressionUUID->"510ae414-5b46-42fd-b1f3-4718d1c241ca"],
Cell[CellGroupData[{
Cell[425976, 8253, 729, 17, 78, "Input",ExpressionUUID->"4ea59560-5572-47f9-81d1-8fc5f9f5327d"],
Cell[426708, 8272, 3921, 58, 234, "Message",ExpressionUUID->"2c125974-feb1-463f-a5b2-d1fefa79d8b7"],
Cell[430632, 8332, 3923, 58, 234, "Message",ExpressionUUID->"a98e5552-3156-48a7-a359-ae5dcf390ea6"],
Cell[434558, 8392, 363, 9, 34, "Message",ExpressionUUID->"14e398d9-2da7-42aa-9637-4b87a7080b25"],
Cell[434924, 8403, 160, 3, 52, "Output",ExpressionUUID->"73a325e9-2e10-4b7e-a365-2cc970764a0c"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[435133, 8412, 189, 3, 67, "Subsubsection",ExpressionUUID->"bfda7348-f0e1-4579-8533-2e31e5eb59c6"],
Cell[435325, 8417, 657, 15, 46, "Input",ExpressionUUID->"b3cab995-c90f-4d01-b55a-2c9209e686c5"],
Cell[CellGroupData[{
Cell[436007, 8436, 392, 8, 46, "Input",ExpressionUUID->"72d68c70-6f95-4398-826e-8a31a0c132ae"],
Cell[436402, 8446, 156, 2, 52, "Output",ExpressionUUID->"529583c3-f6c7-44d8-9f64-73045f85795d"]
}, Open  ]],
Cell[CellGroupData[{
Cell[436595, 8453, 599, 15, 46, "Input",ExpressionUUID->"3e411c41-8182-4db6-96ab-173dd8e2a727"],
Cell[437197, 8470, 678, 13, 44, "Message",ExpressionUUID->"60da094d-e4d8-48e7-a424-281d4ac4815c"],
Cell[437878, 8485, 2861, 75, 202, "Output",ExpressionUUID->"b9a83591-2b17-47c5-be0d-a7b130fc2302"]
}, Open  ]],
Cell[CellGroupData[{
Cell[440776, 8565, 70, 0, 46, "Input",ExpressionUUID->"f780ec73-0c1b-4f76-9368-733014bbfcdd"],
Cell[440849, 8567, 71, 0, 52, "Output",ExpressionUUID->"dbe471ef-3891-4daa-bb01-dbfab0de4349"]
}, Open  ]],
Cell[CellGroupData[{
Cell[440957, 8572, 70, 0, 46, "Input",ExpressionUUID->"49de8bd7-5866-4397-9340-431da7f863ef"],
Cell[441030, 8574, 71, 0, 52, "Output",ExpressionUUID->"2152fb3b-6735-4f74-96de-60a96679ebd5"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[441150, 8580, 78, 0, 67, "Subsubsection",ExpressionUUID->"6625a561-f15a-47c1-9f7c-794dd16a48b4"],
Cell[CellGroupData[{
Cell[441253, 8584, 70, 0, 46, "Input",ExpressionUUID->"17bcad04-db35-48ec-abec-c5f12837cf4c"],
Cell[441326, 8586, 71, 0, 52, "Output",ExpressionUUID->"424c115f-9c29-4049-a155-877d3bcc0dde"]
}, Open  ]],
Cell[CellGroupData[{
Cell[441434, 8591, 70, 0, 46, "Input",ExpressionUUID->"2a09473d-bfa1-4e5a-9018-81ec1fec4d9c"],
Cell[441507, 8593, 71, 0, 52, "Output",ExpressionUUID->"66d531c2-124b-4465-9333-f1333a511a41"]
}, Open  ]],
Cell[CellGroupData[{
Cell[441615, 8598, 70, 0, 46, "Input",ExpressionUUID->"cf51caf2-1017-4bb0-b32b-5ba5f72eb4cf"],
Cell[441688, 8600, 71, 0, 52, "Output",ExpressionUUID->"2bea6ba9-9830-4713-824f-b941e093c60a"]
}, Open  ]],
Cell[CellGroupData[{
Cell[441796, 8605, 70, 0, 46, "Input",ExpressionUUID->"634c7674-3b4e-4b86-bca1-af76ab859c48"],
Cell[441869, 8607, 71, 0, 29785, "Output",ExpressionUUID->"dacde6da-2824-41ee-995f-2cea5ebdaf1c"]
}, Closed]]
}, Open  ]]
}, Open  ]]
}
]
*)

