(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    134656,       3148]
NotebookOptionsPosition[    126484,       3020]
NotebookOutlinePosition[    126963,       3039]
CellTagsIndexPosition[    126920,       3036]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Coupled Functions", "Section",
 CellChangeTimes->{{3.646132431389743*^9, 3.646132435055749*^9}, {
  3.659521527764338*^9, 
  3.659521528666133*^9}},ExpressionUUID->"ccfb8823-5ee3-4dd1-a5ef-\
635366e928a6"],

Cell[TextData[StyleBox["  \[Copyright]  Copyright 2020 Kenric P. Nelson\n\n   \
Licensed under the Apache License, Version 2.0 (the \
\[OpenCurlyDoubleQuote]License\[CloseCurlyDoubleQuote]);\nyou may not use \
this file except in compliance with the License.\nYou may obtain a copy of \
the License at\n\n    http://www.apache.org/licenses/LICENSE-2.0\n\nUnless \
required by applicable law or agreed to in writing, software\ndistributed \
under the License is distributed on an \[OpenCurlyDoubleQuote]AS IS\
\[CloseCurlyDoubleQuote] BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY \
KIND, either express or implied.\nSee the License for the specific language \
governing permissions and\nlimitations under the License.",
 FontSlant->"Italic"]], "Text",
 CellChangeTimes->{{3.8046130577546253`*^9, 3.804613061773901*^9}, {
  3.804613116770612*^9, 3.804613161352187*^9}, {3.804613193767625*^9, 
  3.804613240144112*^9}, {3.804613481342988*^9, 3.804613484230505*^9}, {
  3.80581126678088*^9, 3.805811299166359*^9}, {3.809668480043771*^9, 
  3.809668480957067*^9}},ExpressionUUID->"7c49f273-00ef-49a0-8d02-\
e2387935c131"],

Cell[TextData[{
 "Rev. 12 Kenric Nelson, April 21, 2021\nRevised CoupledExponential Function \
to an If statement based on whether (1 + \[Kappa] x) is positive rather than \
on whether \[Kappa] is positive. This corrects how negative values of x are \
handled and simplifies the function.\n\nRev. 11 Kenric Nelson\nAdded Coupled \
Box-M\[UDoubleDot]ller Method\n\nRev. 10 Kenric Nelson\nThe compact-support \
domain of the CoupledNormalDistribution was producing a Power error message \
due 1/",
 Cell[BoxData[
  FormBox[
   SqrtBox["0"], TraditionalForm]],ExpressionUUID->
  "bab95704-95c4-4a45-b543-6ab5139bedce"],
 ".  This was because CoupledExponential was defaulting to zero even for the \
reciprocal of zero. Updated CoupledNormalDistribution to be defined over the \
proper range for the compact-support domain. Updated CoupledExponential to \
evaluate to infinity for 1/0 situations. Also updated the \
CoupledExponentialDistribution functions.\n\nThe \
CoupledMultivariateDistribution function still needs attention.\n\nRev. 9\n\
Add translators between the coupling (\[Kappa]), risk (",
 StyleBox["r",
  FontSlant->"Italic"],
 "), and Tsallis (",
 StyleBox["q",
  FontSlant->"Italic"],
 ")\n\nRev. 8 \na) Added clear and memory to each function.  When the \
notebook is run it will clear the memory of that function. Each time the \
function is executed the result will be stored in memory, so that computation \
with the same inputs does not need to be repeated.\nb) reordered coupling \
input to be consistent with StudentsTDistribution and related Wolfram \
functions\n\nRev. 7 Removed the parameter alpha from the definition of the \
coupled exponential and logarithm.  This is preparation for experimenting on \
the proper two-parameter definitions for the functions.\n\nRev. 6 Added \
Weighted Generalized Mean\n\nRev. 5 Modified Coupled Product so that \
dimension used on outer exponent is d times Length[xList]\n\nRev. 4  Added \
Coupled Sine and Cosine Functions\n\tPrototype initiated of Coupled Product \
and Coupled Sum with structure of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Product and Sum functions\n"
}], "Text",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQBWIQvaP+7sHfzq8dT2TxnwHRUsapzbIerx2bNL60g+gP
LbelZ3u+dpTQ/KIEouP4Gt+8iHrtKKiZ/QFE33hXKvE/77Xj1mXXJUE0+5Fy
SZ+S147c/zyVQLSEeeE9lqevHbcIN4HpuAyW3woBbxwZmk+C6TSr7wdfB75x
zCoXPw6iWRKvZKaHvXGc9No5C0QbirfkgWgmOfUKEP38ytpTIDrE6vR5EF04
UdqmJ+KN4/ajzY4gOmlTl9lOIP1K7rI5iL6yYH4oiM46sDQaRP947Rv8IPKN
Y8y501Eg2mqBehZzFFC9o0AniA58OVeQBUjfawsVAdGvTvFJgeib/gHKIPrL
Ge2FT5LeOO7b7LoERF+bZHUuvhVoLq/NLRANAGE1qi0=
  "],ExpressionUUID->"3483005b-8452-4ec6-83c3-8e0bbc08b62e"],

Cell[CellGroupData[{

Cell["Coupled Exponential", "Subsection",
 CellChangeTimes->{{3.646132643363847*^9, 
  3.6461326462342515`*^9}},ExpressionUUID->"81a76452-688a-45bd-98be-\
669c9ee70463"],

Cell[TextData[{
 "The variable ",
 StyleBox["x ",
  FontSlant->"Italic"],
 "is applied to a coupled exponential function.  ",
 StyleBox["\[Kappa]",
  FontSlant->"Italic"],
 " controls the degree of nonlinear statistical coupling, where ",
 StyleBox["\[Kappa] ",
  FontSlant->"Italic"],
 "= 0 is linear domain of an exponential function.  ",
 StyleBox["d ",
  FontSlant->"Italic"],
 "and ",
 StyleBox["\[Alpha]",
  FontSlant->"Italic"],
 " should match the dimension and power (",
 StyleBox["x = ",
  FontSlant->"Italic"],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["y", "\[Alpha]"], TraditionalForm]],
  FontSlant->"Italic",ExpressionUUID->"6bb497f7-4b61-4038-8498-039633248b44"],
 ") of the variable ",
 StyleBox["x.\n\n",
  FontSlant->"Italic"],
 "Inputs\nx - variable to which coupled exponential is applied\n\[Kappa] - \
coupling parameter which modifies the coupled exponential function\nd - \
dimension, should be equal to the dimension of x\n"
}], "Text",
 CellChangeTimes->{{3.654415927952281*^9, 3.6544161904666653`*^9}, {
  3.6546922522633457`*^9, 3.654692365382756*^9}, {3.654692672840823*^9, 
  3.6546926737351418`*^9}, {3.661687908399137*^9, 3.6616882576557426`*^9}, {
  3.684851995912294*^9, 
  3.684852001272346*^9}},ExpressionUUID->"762dba98-5065-487c-b54b-\
83b044c6f343"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "CoupledExponential", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CoupledExponential", "[", 
   RowBox[{"x_", ",", 
    RowBox[{"\[Kappa]_", ":", "0"}], ",", 
    RowBox[{"d_", ":", "1"}]}], "]"}], ":=", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"CoupledExponential", "[", 
    RowBox[{"x", ",", "\[Kappa]", ",", "d"}], "]"}], "=", 
   "\[IndentingNewLine]", 
   RowBox[{"If", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"\[Kappa]", " ", "x"}]}], ")"}], ">", "0"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"\[Kappa]", "\[NotEqual]", "0"}], ",", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"1", "+", 
          RowBox[{"\[Kappa]", " ", "x"}]}], ")"}], 
        FractionBox[
         RowBox[{"1", "+", 
          RowBox[{"d", " ", "\[Kappa]"}]}], 
         RowBox[{" ", "\[Kappa]"}]]], ",", 
       RowBox[{"Exp", "[", "x", "]"}]}], "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        FractionBox[
         RowBox[{"1", "+", 
          RowBox[{"d", " ", "\[Kappa]"}]}], "\[Kappa]"], ">", "0"}], ",", "0",
        ",", "\[Infinity]"}], "]"}]}], "\[IndentingNewLine]", 
    "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.828022668229239*^9, 3.8280227023479843`*^9}, {
  3.8280228240523577`*^9, 3.828022878276782*^9}, {3.828023008695324*^9, 
  3.828023027328477*^9}},ExpressionUUID->"b7abe739-d553-4e6f-95e7-\
79ac5a3e2431"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "CoupledExponentialOld", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CoupledExponentialOld", "[", 
   RowBox[{"x_", ",", 
    RowBox[{"\[Kappa]_", ":", "0"}], ",", 
    RowBox[{"d_", ":", "1"}]}], "]"}], ":=", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"CoupledExponentialOld", "[", 
    RowBox[{"x", ",", "\[Kappa]", ",", "d"}], "]"}], "=", 
   "\[IndentingNewLine]", 
   RowBox[{"Which", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"\[Kappa]", ">", "0"}], ",", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "+", " ", 
        RowBox[{"\[Kappa]", " ", "x"}]}], ")"}], 
      FractionBox[
       RowBox[{"1", "+", 
        RowBox[{"d", " ", "\[Kappa]"}]}], 
       RowBox[{" ", "\[Kappa]"}]]], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "1"}], "/", "d"}], "\[LessSlantEqual]", "\[Kappa]", "<", 
      "0"}], ",", " ", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Simplify", "[", 
         RowBox[{"1", "+", 
          RowBox[{"\[Kappa]", " ", "x"}]}], "]"}], "\[GreaterEqual]", "0"}], 
       ",", "\[IndentingNewLine]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"1", "+", " ", 
          RowBox[{"\[Kappa]", " ", "x"}]}], ")"}], 
        FractionBox[
         RowBox[{"1", "+", 
          RowBox[{"d", " ", "\[Kappa]"}]}], "\[Kappa]"]], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          FractionBox[
           RowBox[{"1", "+", 
            RowBox[{"d", " ", "\[Kappa]"}]}], "\[Kappa]"], ">", "0"}], ",", 
         "0", ",", "\[Infinity]"}], "]"}]}], "\[IndentingNewLine]", "]"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"\[Kappa]", "\[Equal]", "0"}], ",", 
     RowBox[{"Exp", "[", "x", "]"}], ",", "\[IndentingNewLine]", "True", ",", 
     " ", 
     RowBox[{"Message", "[", 
      RowBox[{
       RowBox[{"CoupledExponentialOld", "::", "nnarg"}], ",", "\[Kappa]"}], 
      "]"}]}], "\[IndentingNewLine]", "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.6461326503526587`*^9, 3.6461329477203813`*^9}, {
   3.648723320496894*^9, 3.6487234070458455`*^9}, {3.6487234435343094`*^9, 
   3.648723478478371*^9}, {3.648723534435669*^9, 3.6487235821249533`*^9}, {
   3.6487236222482233`*^9, 3.6487236229190245`*^9}, 3.6487237079391737`*^9, {
   3.6487260579847016`*^9, 3.6487260720559263`*^9}, {3.6546923724163313`*^9, 
   3.654692381838135*^9}, {3.654692667993305*^9, 3.654692818726687*^9}, {
   3.65469293747476*^9, 3.6546929652401123`*^9}, {3.654693493743395*^9, 
   3.654693514998253*^9}, {3.684852007696492*^9, 3.684852058955173*^9}, {
   3.8033975461238537`*^9, 3.803397580959036*^9}, {3.805288146348254*^9, 
   3.805288149097271*^9}, {3.80528901884754*^9, 3.805289020500948*^9}, {
   3.805289127775028*^9, 3.8052891758653507`*^9}, {3.805289206066114*^9, 
   3.80528920778804*^9}, {3.8105873952924232`*^9, 3.8105873956850443`*^9}, {
   3.828022471423973*^9, 3.828022477386764*^9}, {3.828022632499872*^9, 
   3.828022643774858*^9}},ExpressionUUID->"d08c6e66-4bf1-46c3-8911-\
ba0a436bccc6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CoupledExponentialOld", "::", "nnarg"}], " ", "=", " ", 
   "\"\<Error: \[Kappa] = `1/d` is not greater than -1.\>\""}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6546935399322157`*^9, 3.6546935986816177`*^9}, {
  3.6546937822283907`*^9, 3.654693800241928*^9}, {3.810587404062738*^9, 
  3.8105874041365433`*^9}, {3.8280226460230093`*^9, 
  3.828022646395455*^9}},ExpressionUUID->"38dda050-2638-4b48-a179-\
17895d521415"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coupled Logarithm", "Subsection",
 CellChangeTimes->{{3.6461331034242544`*^9, 
  3.646133107605062*^9}},ExpressionUUID->"ab563a47-0e25-473b-b61a-\
d8d924bb7bbe"],

Cell["\<\
Generalization of the logarithm function, which defines smooth transition to \
power functions.\
\>", "Text",
 CellChangeTimes->{{3.6616884023527117`*^9, 
  3.6616884375340853`*^9}},ExpressionUUID->"e3c25309-77e2-4d04-85d4-\
66096212348e"],

Cell["\<\
Inputs
x - variable to which coupled exponential is applied
\[Kappa] - coupling parameter which modifies the coupled exponential function
d - dimension, should be equal to the dimension of x\
\>", "Text",
 CellChangeTimes->{{3.6616883024697313`*^9, 3.661688325148985*^9}, {
  3.684852076251569*^9, 3.684852076719949*^9}, {3.815471363766946*^9, 
  3.815471365165929*^9}},ExpressionUUID->"47a8764e-ea62-489d-9821-\
5795550c582b"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "CoupledLogarithm", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CoupledLogarithm", "[", 
   RowBox[{"x_", ",", 
    RowBox[{"\[Kappa]_", ":", "0"}], ",", 
    RowBox[{"d_", ":", "1"}]}], "]"}], ":=", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"CoupledLogarithm", "[", 
    RowBox[{"x", ",", "\[Kappa]", ",", "d"}], "]"}], "=", 
   "\[IndentingNewLine]", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"x", "\[GreaterEqual]", "0"}], ",", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"\[Kappa]", "\[NotEqual]", "0"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        FractionBox["1", "\[Kappa]"], 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", 
           FractionBox["\[Kappa]", 
            RowBox[{"1", "+", 
             RowBox[{"d", " ", "\[Kappa]"}]}]]], "-", "1"}], ")"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Log", "[", "x", "]"}]}], "\[IndentingNewLine]", "]"}], ",", 
     "\[IndentingNewLine]", "Undefined"}], "\[IndentingNewLine]", 
    "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.6461331535627427`*^9, 3.6461332333256826`*^9}, {
  3.6461332741041546`*^9, 3.646133400122376*^9}, {3.648723647988269*^9, 
  3.6487237693720818`*^9}, {3.684852083015833*^9, 3.68485209848703*^9}, {
  3.803397611621414*^9, 3.803397641622694*^9}},
 CellLabel->"In[4]:=",ExpressionUUID->"d35506b4-a030-4b74-b3b7-261dac74a4d0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coupled Gaussian - One Dimension", "Subsection",
 CellChangeTimes->{{3.646132445226967*^9, 3.646132447598171*^9}, {
  3.6544155271950903`*^9, 
  3.6544155320489798`*^9}},ExpressionUUID->"d85a8cb4-825c-4ccf-b02b-\
a3f67211e406"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "CoupledNormalDistribution", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CoupledNormalDistribution", "[", 
   RowBox[{
    RowBox[{"\[Mu]_", ":", "0"}], ",", "\[Sigma]_", ",", "\[Kappa]_"}], "]"}],
   ":=", " ", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"CoupledNormalDistribution", "[", 
    RowBox[{"\[Mu]", ",", "\[Sigma]", ",", "\[Kappa]"}], "]"}], "=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "x", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"ProbabilityDistribution", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        FractionBox["1", 
         RowBox[{"NormCG", "[", 
          RowBox[{"\[Sigma]", ",", "\[Kappa]"}], "]"}]], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"CoupledExponential", "[", 
           RowBox[{
            FractionBox[
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"x", "-", "\[Mu]"}], ")"}], "2"], 
             SuperscriptBox["\[Sigma]", "2"]], ",", "\[Kappa]"}], "]"}], 
          ")"}], 
         FractionBox[
          RowBox[{"-", "1"}], "2"]]}], ",", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"\[Kappa]", "\[GreaterEqual]", "0"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"x", ",", 
           RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"x", ",", " ", 
           RowBox[{"\[Mu]", "-", 
            SqrtBox[
             RowBox[{"-", 
              FractionBox[
               SuperscriptBox["\[Sigma]", "2"], "\[Kappa]"]}]]}], ",", " ", 
           RowBox[{"\[Mu]", "+", 
            SqrtBox[
             RowBox[{"-", 
              FractionBox[
               SuperscriptBox["\[Sigma]", "2"], "\[Kappa]"]}]]}]}], "}"}]}], 
        "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}]}], 
    "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.6461324589549913`*^9, 3.646132624721814*^9}, {
   3.646133117355079*^9, 3.646133133345107*^9}, {3.64872504373332*^9, 
   3.6487250471029263`*^9}, {3.6487251433082952`*^9, 
   3.6487251456170993`*^9}, {3.6487251798747597`*^9, 
   3.6487251825111637`*^9}, {3.6487252131028175`*^9, 
   3.6487252209340315`*^9}, {3.6487252585456977`*^9, 
   3.6487253041445775`*^9}, {3.648725743456949*^9, 3.648725745016952*^9}, {
   3.6487273010116854`*^9, 3.6487273229141245`*^9}, {3.649948071263131*^9, 
   3.6499480834623528`*^9}, {3.6499481450824604`*^9, 3.649948150074469*^9}, {
   3.684852114344809*^9, 3.684852152354267*^9}, 3.684852910330243*^9, 
   3.684856926721586*^9, {3.803397674043252*^9, 3.803397701244432*^9}, {
   3.803397891596146*^9, 3.8033979288886843`*^9}, {3.8033979721857986`*^9, 
   3.8033980143235617`*^9}, {3.803398048324604*^9, 3.803398056860504*^9}, {
   3.803398098144631*^9, 3.80339810187938*^9}, {3.805217509552724*^9, 
   3.805217540691843*^9}, {3.805217574201539*^9, 3.805217622241715*^9}, {
   3.8052177472887993`*^9, 3.805217757412714*^9}, {3.80521788112598*^9, 
   3.8052178902484207`*^9}},
 CellLabel->"In[6]:=",ExpressionUUID->"9ee0c0bd-5fb8-4431-bc94-d7134679b664"],

Cell[CellGroupData[{

Cell["Normalization of 1-D Coupled Gaussian", "Subsubsection",
 CellChangeTimes->{{3.646136837054983*^9, 
  3.646136848973404*^9}},ExpressionUUID->"6e0767ec-c365-4754-9597-\
3f2c5735ddba"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NormCG", "[", 
   RowBox[{"\[Sigma]_", ",", "\[Kappa]_"}], "]"}], ":=", 
  TagBox[GridBox[{
     {"\[Piecewise]", GridBox[{
        {
         RowBox[{
          SqrtBox[
           RowBox[{"2", " ", "\[Pi]"}]], " ", "\[Sigma]"}], 
         RowBox[{"\[Kappa]", "\[Equal]", "0"}]},
        {
         FractionBox[
          RowBox[{
           SqrtBox["\[Pi]"], " ", "\[Sigma]", " ", 
           RowBox[{"Gamma", "[", 
            FractionBox[
             RowBox[{
              RowBox[{"-", "1"}], "+", "\[Kappa]"}], 
             RowBox[{"2", " ", "\[Kappa]"}]], "]"}]}], 
          RowBox[{
           SqrtBox[
            RowBox[{"-", "\[Kappa]"}]], " ", 
           RowBox[{"Gamma", "[", 
            RowBox[{"1", "-", 
             FractionBox["1", 
              RowBox[{"2", " ", "\[Kappa]"}]]}], "]"}]}]], 
         RowBox[{"\[Kappa]", "<", "0"}]},
        {
         FractionBox[
          RowBox[{
           SqrtBox["\[Pi]"], " ", "\[Sigma]", " ", 
           RowBox[{"Gamma", "[", 
            FractionBox["1", 
             RowBox[{"2", " ", "\[Kappa]"}]], "]"}]}], 
          RowBox[{
           SqrtBox["\[Kappa]"], " ", 
           RowBox[{"Gamma", "[", 
            FractionBox[
             RowBox[{"1", "+", "\[Kappa]"}], 
             RowBox[{"2", " ", "\[Kappa]"}]], "]"}]}]], 
         TagBox["True",
          "PiecewiseDefault",
          AutoDelete->True]}
       },
       AllowedDimensions->{2, Automatic},
       Editable->True,
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxItemSize->{
        "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
         "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.84]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.4]}, 
           Offset[0.2]}, "RowsIndexed" -> {}},
       Selectable->True]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxItemSize->{
     "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.35]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   "Piecewise",
   DeleteWithContents->True,
   Editable->False,
   SelectWithContents->True,
   Selectable->False]}]], "Input",
 CellChangeTimes->{{3.6461368583334208`*^9, 3.646136923385535*^9}, {
   3.64613695480399*^9, 3.6461370486849546`*^9}, {3.646137201721224*^9, 
   3.6461372067444324`*^9}, 3.6487251935559835`*^9, {3.6487257287617235`*^9, 
   3.6487257306805267`*^9}, {3.6487262070585637`*^9, 
   3.6487262382742186`*^9}, {3.648726505627488*^9, 3.6487265097458954`*^9}, {
   3.6499481234608226`*^9, 3.6499481278912306`*^9}, {3.803398022582658*^9, 
   3.803398026438119*^9}},
 CellLabel->"In[8]:=",ExpressionUUID->"5eb84abf-0c0f-479f-a442-af94720a7842"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coupled Exponential Distribution - One Dimension", "Subsection",
 CellChangeTimes->{{3.6487198994656854`*^9, 3.6487199097773037`*^9}, {
  3.654415535761125*^9, 
  3.6544155394191113`*^9}},ExpressionUUID->"0717bcc1-739c-46c7-b7a7-\
1938deecae14"],

Cell["\<\
The coupled exponential distribution is also known as the generalized Pareto \
distribution\
\>", "Text",
 CellChangeTimes->{{3.810587802810905*^9, 
  3.810587824553398*^9}},ExpressionUUID->"8f12ac37-5cdf-480c-99e7-\
0f871ab636ab"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "CoupledExponentialDistribution", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CoupledExponentialDistribution", "[", 
   RowBox[{
    RowBox[{"\[Mu]_", ":", "0"}], ",", "\[Sigma]_", ",", "\[Kappa]_"}], "]"}],
   ":=", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"CoupledExponentialDistribution", "[", 
    RowBox[{"\[Mu]", ",", "\[Sigma]", ",", "\[Kappa]"}], "]"}], "=", 
   "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "x", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"ProbabilityDistribution", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        FractionBox["1", "\[Sigma]"], 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"CoupledExponential", "[", 
           RowBox[{
            FractionBox[
             RowBox[{"x", "-", "\[Mu]"}], "\[Sigma]"], ",", "\[Kappa]", ",", 
            "1"}], "]"}], ")"}], 
         RowBox[{"-", "1"}]]}], ",", "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"x", ",", "\[Mu]", ",", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"\[Kappa]", "\[GreaterEqual]", "0"}], ",", "\[Infinity]", 
           ",", 
           RowBox[{
            FractionBox[
             RowBox[{"-", "\[Sigma]"}], "\[Kappa]"], "+", "\[Mu]"}]}], 
          "]"}]}], "}"}]}], "\[IndentingNewLine]", "]"}]}], 
    "\[IndentingNewLine]", "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.648720168441358*^9, 3.648720504294348*^9}, {
   3.648721030046471*^9, 3.6487210304052715`*^9}, {3.64872112041743*^9, 
   3.648721139901864*^9}, {3.64872139939392*^9, 3.648721409346738*^9}, {
   3.648721456958021*^9, 3.6487214700308437`*^9}, {3.648721542967372*^9, 
   3.648721570064619*^9}, {3.6487216663947887`*^9, 3.648721680918414*^9}, {
   3.6487217343641076`*^9, 3.648721744878526*^9}, {3.6487221378120165`*^9, 
   3.64872214553403*^9}, {3.6487222141585503`*^9, 3.6487222261081715`*^9}, {
   3.648722258369028*^9, 3.648722285138675*^9}, {3.6487223582092037`*^9, 
   3.6487223838712482`*^9}, {3.648722433479336*^9, 3.648722438705345*^9}, 
   3.6487225052862616`*^9, 3.6487229509322443`*^9, {3.648723001959934*^9, 
   3.648723006343542*^9}, 3.6487232717156076`*^9, {3.648723999129286*^9, 
   3.6487241419631367`*^9}, {3.648724450282078*^9, 3.648724485085739*^9}, {
   3.6499480947723722`*^9, 3.6499481005287824`*^9}, {3.654435141063149*^9, 
   3.654435162005294*^9}, {3.654435193701748*^9, 3.6544351947484503`*^9}, {
   3.684852204942642*^9, 3.684852228879825*^9}, 3.684852891392569*^9, {
   3.803398146091093*^9, 3.803398215674343*^9}, 3.803398287976729*^9, {
   3.805289860583312*^9, 3.805289881529201*^9}, {3.8052899123999166`*^9, 
   3.805289924420513*^9}, {3.8052899847646427`*^9, 3.8052900025692463`*^9}, {
   3.8052900947567873`*^9, 3.805290180270391*^9}},
 CellLabel->"In[9]:=",ExpressionUUID->"f5403d12-bf81-40dd-9bf3-674539af6199"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Multivariate Coupled Stretched Exponential Distribution", "Subsection",
 CellChangeTimes->{{3.654435044121716*^9, 
  3.654435078247295*^9}},ExpressionUUID->"0b940d3e-f6af-42d9-9ba2-\
2bf8c398d37f"],

Cell["\<\
Status:  Currently defined for \[Alpha] = 1,  \[Kappa] > 0 and or \[Alpha]=2, \
\[Kappa]>-1/d;
The compact support domain (-1/d < \[Kappa] < 0) for \[Alpha]=2 is not fully \
verified due to the integration not completing.\
\>", "Text",
 CellChangeTimes->{{3.654435528000255*^9, 3.654435548063262*^9}, {
  3.6544360749125957`*^9, 3.654436080855084*^9}, {3.654436218333522*^9, 
  3.654436220500187*^9}, {3.654437744017771*^9, 3.654437748488207*^9}, {
  3.808637821785707*^9, 3.8086378432611628`*^9}, {3.8216338639782343`*^9, 
  3.82163386626105*^9}, {3.821654433068281*^9, 
  3.821654584588933*^9}},ExpressionUUID->"9afd2663-16bd-4567-81ac-\
874a6041877b"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "MultivariateCoupledDistribution", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MultivariateCoupledDistribution", "[", 
   RowBox[{
   "\[Mu]_", ",", "\[CapitalSigma]_", ",", "\[Kappa]_", ",", "\[Alpha]_"}], 
   "]"}], ":=", " ", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"MultivariateCoupledDistribution", "[", 
    RowBox[{
    "\[Mu]", ",", "\[CapitalSigma]", ",", "\[Kappa]", ",", "\[Alpha]"}], 
    "]"}], "=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"dimMean", ",", "dimCor", ",", "x"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
      "Check", " ", "that", " ", "\[CapitalSigma]", " ", "is", " ", "a", " ", 
       "positive", " ", "definite", " ", "matrix", " ", "with", " ", 
       "dimensions", " ", "equal", " ", "to", " ", "length", " ", "of", " ", 
       "\[Mu]"}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"dimMean", "=", 
       RowBox[{"Length", "[", "\[Mu]", "]"}]}], ";", 
      RowBox[{"dimCor", "=", 
       RowBox[{"Dimensions", "[", "\[CapitalSigma]", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"dimCor", "\[NotEqual]", "dimMean"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Message", "[", 
         RowBox[{
          RowBox[{"MultivariateCoupledDistribution", "::", "argx"}], ",", 
          "dimCor", ",", "dimMean"}], "]"}]}], "\[IndentingNewLine]", "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"Not", "@", 
           RowBox[{"PositiveDefiniteMatrixQ", "[", "\[CapitalSigma]", "]"}]}],
           ",", "\[IndentingNewLine]", 
          RowBox[{"Message", "[", 
           RowBox[{"MultivariateCoupledDistribution", "::", "corr"}], "]"}]}],
          "\[IndentingNewLine]", "]"}], ";"}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"\[Alpha]", "\[NotEqual]", 
         RowBox[{"(", 
          RowBox[{"1", "||", "2"}], ")"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{"Message", "[", 
         RowBox[{
          RowBox[{"MultivariateCoupledDistribution", "::", "alpha"}], ",", 
          "\[Alpha]"}], "]"}]}], "\[IndentingNewLine]", "]"}], ";", 
      "\[IndentingNewLine]", " ", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"Define", " ", "the", " ", "distribution"}], " ", "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Evaluation", " ", "of", " ", "the", " ", "input", " ", "x", " ", "to",
         " ", "CoupledExponential", " ", "is", " ", "completed", " ", "to", 
        " ", "insure", " ", "that", " ", "the", " ", "definition", " ", "of", 
        " ", "CoupledExponential", " ", "can", " ", "stay", " ", "one", " ", 
        "dimensional"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"x", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"Symbol", "[", 
          RowBox[{"\"\<$x\>\"", "<>", 
           RowBox[{"ToString", "@", "i"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "dimMean"}], "}"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"ProbabilityDistribution", "[", "\[IndentingNewLine]", 
         RowBox[{
          FractionBox[
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"CoupledExponential", "[", 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{"x", "-", "\[Mu]"}], ")"}], ".", 
                  RowBox[{"Inverse", "[", "\[CapitalSigma]", "]"}], ".", 
                  RowBox[{"(", 
                   RowBox[{"x", "-", "\[Mu]"}], ")"}]}], ")"}], 
                FractionBox["\[Alpha]", "2"]], ",", "\[Kappa]", ",", 
               "dimMean"}], "]"}], ")"}], 
            FractionBox[
             RowBox[{"-", "1"}], "\[Alpha]"]], 
           RowBox[{"NormMultiCoupled", "[", 
            RowBox[{
            "\[CapitalSigma]", ",", "\[Kappa]", ",", "\[Alpha]", ",", 
             "dimMean"}], "]"}]], ",", "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{"Define", " ", "the", " ", "domain"}], " ", "*)"}], 
          "\[IndentingNewLine]", "##"}], "\[IndentingNewLine]", "]"}], "&"}], 
       "@@", "\[IndentingNewLine]", 
       RowBox[{"Which", "[", 
        RowBox[{
         RowBox[{"\[Alpha]", "\[Equal]", "1"}], ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{
             "x", "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}], ",", 
             RowBox[{
             "\[Mu]", "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}], 
             ",", "\[Infinity]"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "dimMean"}], "}"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"\[Alpha]", "\[Equal]", "2"}], ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{
             "x", "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}], ",", 
             RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "dimMean"}], "}"}]}], "]"}]}], 
        "\[IndentingNewLine]", "]"}]}]}]}], " ", "\[IndentingNewLine]", 
    "]"}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"MultivariateCoupledDistribution", "::", "argx"}], "=", 
    "\"\<Length of the first dimension of the matrix `1` does not match the \
length of the mean `2`.\>\""}], ";"}], " "}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"MultivariateCoupledDistribution", "::", "corr"}], "=", 
   "\"\<The correlation matrix \[CapitalSigma] is not positive \
definite.\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"MultivariateCoupledDistribution", "::", "alpha"}], "=", 
   "\"\<Alpha `1` is required to be 1 or 2\>\""}], ";"}]}], "Input",
 CellChangeTimes->CompressedData["
1:eJwdzVkowwEAx/G1xOaotZUk10KOB01TMld/5IhGrpy5tjnnHDFHm6PZg+1F
y8Ki4UGMckdTJMWLuaKkSDm2GmGKHP+fh2+fxy+7vCFTSKVQKGwyeMbf80vm
mYkQ1ZU/9BSecKDRmx4NZSlf6WkSM2EL2smC+69lhbAqwKcIOok1pbBXbKqE
NOZSLZT+roih62BEMwz/GfuXmRelgPUna2pIMYVNwlDp/QwMdqrbgC3qrX/j
6F7sqlwLYTvt5ULniRuJUWAhlvO6lfD4uWEI8g4l05CvdpmDMwylAdKGw/bh
iKjJCrf0EbHbpBdWPQETv21J8IrXlQPduJvF0PklQQgP4nfroP1quwR+9Im6
oJdCMQDPYqgqGMxo1EKNMl8Px89ZBvi0LV2Ds9ZII2TYGXZhgda2Bw3VOtkD
vmXvcqjza5yH8tDPBWh07A58JPVQ1obA2ztNxhNpzw41F7YH+WqhaPFtFHam
yiYgS3Y+BdsczBVmUvdLTg0UtPbTuR0W4jr7yAWWiOVJl50WYj1qORn+AaE5
EPQ=
  "],
 CellLabel->"In[11]:=",ExpressionUUID->"4f1d6f34-6fd4-4aad-8018-ed8e3ed20179"],

Cell[CellGroupData[{

Cell["Normalization of Multivariate Coupled Stretched Exponential", \
"Subsubsection",
 CellChangeTimes->{{3.654435331180427*^9, 
  3.654435342970766*^9}},ExpressionUUID->"90912a30-441e-48ab-8f81-\
2d559c243f1c"],

Cell["\<\
Normalization has only been completed for positive values of coupling\
\>", "Text",
 CellChangeTimes->{{3.6848582978916397`*^9, 
  3.684858316282687*^9}},ExpressionUUID->"3a201432-a434-435f-bfb3-\
be089adbc95c"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "NormMultiCoupled", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"NormMultiCoupled", "[", 
   RowBox[{
   "\[CapitalSigma]_", ",", "\[Kappa]_", ",", "\[Alpha]_", ",", "d_"}], "]"}],
   ":=", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"NormMultiCoupled", "[", 
    RowBox[{"\[CapitalSigma]", ",", "\[Kappa]", ",", "\[Alpha]", ",", "d"}], 
    "]"}], "=", "\[IndentingNewLine]", 
   RowBox[{"Which", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"\[Alpha]", "\[Equal]", "1"}], ",", 
     FractionBox[
      SuperscriptBox[
       RowBox[{"Det", "[", "\[CapitalSigma]", " ", "]"}], 
       RowBox[{"1", "/", "2"}]], 
      RowBox[{"1", "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", "d"}], ")"}], " ", "\[Kappa]"}]}]], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"\[Alpha]", "\[Equal]", "2"}], ",", 
       FractionBox[
        RowBox[{
         SqrtBox["\[Pi]"], " ", 
         SuperscriptBox[
          RowBox[{"Det", "[", "\[CapitalSigma]", " ", "]"}], 
          RowBox[{"1", "/", "2"}]], 
         RowBox[{"Gamma", "[", 
          FractionBox[
           RowBox[{"1", "+", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"-", "1"}], "+", "d"}], ")"}], " ", "\[Kappa]"}]}], 
           RowBox[{"2", " ", "\[Kappa]"}]], "]"}]}], 
        RowBox[{
         SqrtBox["\[Kappa]"], " ", 
         RowBox[{"Gamma", "[", 
          FractionBox[
           RowBox[{"1", "+", 
            RowBox[{"d", " ", "\[Kappa]"}]}], 
           RowBox[{"2", " ", "\[Kappa]"}]], "]"}]}]]}], "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"\[Alpha]", "==", "2"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Which", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Kappa]", ">", "0"}], ",", 
       RowBox[{
        SuperscriptBox["\[Pi]", 
         RowBox[{"d", "/", "2"}]], " ", 
        SuperscriptBox[
         RowBox[{"Det", "[", "\[CapitalSigma]", " ", "]"}], 
         RowBox[{"1", "/", "2"}]], 
        FractionBox[
         RowBox[{"(", 
          RowBox[{"1", "+", 
           RowBox[{"d", " ", "\[Kappa]"}]}], ")"}], 
         SuperscriptBox["\[Kappa]", 
          RowBox[{"d", "/", "2"}]]], 
        FractionBox[
         RowBox[{"Beta", "[", 
          RowBox[{
           RowBox[{
            FractionBox["1", 
             RowBox[{"2", "\[Kappa]"}]], "+", "1"}], ",", 
           FractionBox["d", "2"]}], "]"}], 
         RowBox[{"Gamma", "[", 
          FractionBox["d", "2"], "]"}]]}], ",", "\[IndentingNewLine]", 
       RowBox[{"\[Kappa]", "==", "0"}], ",", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"2", "\[Pi]"}], ")"}], 
         RowBox[{"d", "/", "2"}]], " ", 
        SuperscriptBox[
         RowBox[{"Det", "[", "\[CapitalSigma]", " ", "]"}], 
         RowBox[{"1", "/", "2"}]]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "1"}], "/", "d"}], "<", "\[Kappa]", "<", "0"}], ",", 
       " ", 
       RowBox[{
        SuperscriptBox["\[Pi]", 
         RowBox[{"d", "/", "2"}]], " ", 
        SuperscriptBox[
         RowBox[{"Det", "[", "\[CapitalSigma]", " ", "]"}], 
         RowBox[{"1", "/", "2"}]], 
        FractionBox["1", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "\[Kappa]"}], ")"}], 
          RowBox[{"d", "/", "2"}]]], 
        FractionBox[
         RowBox[{"Beta", "[", 
          RowBox[{
           RowBox[{
            FractionBox[
             RowBox[{"1", "+", 
              RowBox[{"d", " ", "\[Kappa]"}]}], 
             RowBox[{
              RowBox[{"-", "2"}], "\[Kappa]"}]], "+", "1"}], ",", 
           FractionBox["d", "2"]}], "]"}], 
         RowBox[{"Gamma", "[", 
          FractionBox["d", "2"], "]"}]]}]}], "\[IndentingNewLine]", "]"}]}], 
    "\[IndentingNewLine]", "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.654435356283599*^9, 3.65443538942196*^9}, {
   3.654435429150065*^9, 3.654435475931428*^9}, {3.654435564846726*^9, 
   3.6544356298691063`*^9}, {3.6544357156725197`*^9, 3.654435725340942*^9}, {
   3.654435883835155*^9, 3.654435951184559*^9}, {3.6544363665924377`*^9, 
   3.654436369983029*^9}, {3.654436990016789*^9, 3.654436997218802*^9}, {
   3.654437070149404*^9, 3.654437108586254*^9}, {3.65443727541739*^9, 
   3.6544372774157057`*^9}, 3.6544374547943573`*^9, {3.6544374850014057`*^9, 
   3.654437485407421*^9}, {3.654437570567185*^9, 3.654437585476795*^9}, {
   3.8033985700306387`*^9, 3.8033986493039722`*^9}, {3.8110015903644876`*^9, 
   3.811001700983467*^9}, {3.811002083430031*^9, 3.811002142728054*^9}, {
   3.821634975934084*^9, 3.8216349877370653`*^9}, {3.821635022072443*^9, 
   3.8216350449008617`*^9}, {3.821635081525106*^9, 3.821635129529698*^9}, {
   3.8216351611554213`*^9, 3.821635238073427*^9}, {3.821635329728446*^9, 
   3.821635331126581*^9}, {3.82163624030807*^9, 3.821636365099741*^9}, {
   3.821636499052621*^9, 3.821636500108556*^9}, 3.821638955974105*^9, 
   3.8216393305573378`*^9},
 CellLabel->"In[16]:=",ExpressionUUID->"16f9a46b-7ab5-4777-946d-4c70022e4dc6"],

Cell[BoxData[
 RowBox[{"(*", " ", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
   "The", " ", "integral", " ", "for", " ", "arbitrary", " ", "\[Alpha]", " ",
     "does", " ", "not", " ", "simplify"}], ";", " ", 
   RowBox[{
   "will", " ", "need", " ", "to", " ", "define", " ", "an", " ", "explicit", 
    " ", "solution", " ", "or", " ", "approach", " ", 
    RowBox[{"Assuming", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Infinity]", ">", "\[Kappa]", ">", "0"}], "&&", 
       RowBox[{"\[Infinity]", ">", "\[Alpha]", ">", "0"}]}], ",", 
      RowBox[{
       SubsuperscriptBox["\[Integral]", "0", "\[Infinity]"], 
       RowBox[{
        RowBox[{"Refine", "[", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"CoupledExponential", "[", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"(", 
               FractionBox["x", "\[Sigma]"], ")"}], "\[Alpha]"], ",", 
             "\[Kappa]", ",", "d"}], "]"}], ")"}], 
          FractionBox[
           RowBox[{"-", "1"}], "\[Alpha]"]], "]"}], 
        RowBox[{"\[DifferentialD]", "x"}]}]}]}], "]"}]}]}], 
  "\[IndentingNewLine]", "*)"}]], "Input",
 CellChangeTimes->{{3.65443702430345*^9, 3.65443703859688*^9}, {
  3.684853143571435*^9, 3.684853164150138*^9}, {3.684853221496646*^9, 
  3.6848532227839527`*^9}, {3.692817846011331*^9, 3.692817859485376*^9}, {
  3.6928185458527946`*^9, 3.692818600316616*^9}, {3.692818944198844*^9, 
  3.692818962076159*^9}, {3.6928262794859962`*^9, 3.692826340153672*^9}},
 CellLabel->"In[18]:=",ExpressionUUID->"5b5bc67f-e221-4b47-b2f2-9c9ecc94524a"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coupled Probability", "Subsection",
 CellChangeTimes->{{3.6490593829476566`*^9, 
  3.6490593858024616`*^9}},ExpressionUUID->"3e656e5f-d42d-48f2-ba1f-\
c05bb1602876"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  StyleBox[
   RowBox[{
    RowBox[{
    "The", " ", "Coupled", " ", "Probability", " ", "raises", " ", "a", " ", 
     "distribution", " ", "to", " ", "the", " ", "power"}], 
    "\[IndentingNewLine]", 
    RowBox[{"1", " ", "-", " ", 
     RowBox[{
     "riskBias", " ", "and", " ", "then", " ", "renormalizes", " ", "the", 
      " ", 
      RowBox[{"distribution", ".", "  ", "The"}], " ", "input"}]}], 
    "\[IndentingNewLine]", 
    RowBox[{
    "The", " ", "coupling", " ", "term", " ", "M", " ", "is", " ", "the", " ",
      "multiplicative", " ", 
     RowBox[{"coupling", ".", "  ", "It"}], " ", "is", " ", "related", " ", 
     "to", " ", "the", " ", "source", " ", "of", " ", "coupling", " ", 
     "\[Kappa]", " ", 
     RowBox[{"(", 
      RowBox[{"small", " ", "kappa"}], ")"}], " ", "by", " ", "the", " ", 
     "relationship"}], "\[IndentingNewLine]", "\t\t", 
    RowBox[{"riskBias", " ", "=", " ", 
     FractionBox[
      RowBox[{
       RowBox[{"-", "\[Alpha]"}], " ", "\[Kappa]"}], 
      RowBox[{"1", "+", 
       RowBox[{
        StyleBox["d",
         FontSlant->"Italic"], 
        StyleBox[" ",
         FontSlant->"Italic"], "\[Kappa]"}]}]]}], "\[IndentingNewLine]", 
    RowBox[{
    "where", " ", "\[Alpha]", " ", "is", " ", "the", " ", "power", " ", "and",
      " ", 
     StyleBox["d",
      FontSlant->"Italic"], 
     StyleBox[" ",
      FontSlant->"Italic"], "is", " ", "the", " ", "dimension", " ", "of", 
     " ", "the", " ", "random", " ", "variable", " ", "of", " ", "the", " ", 
     "distribution"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
     "The", " ", "function", " ", "is", " ", "currently", " ", "written", " ",
       "for", " ", "continuous", " ", "distributions"}], ",", " ", 
     RowBox[{
     "though", " ", "discrete", " ", "distributions", " ", "are", " ", "also",
       " ", 
      RowBox[{"possible", ".", "  ", "f"}], " ", "is", " ", "required", " ", 
      "to", " ", "be", " ", "distribution"}], ",", " ", 
     RowBox[{"such", " ", "that", " ", 
      RowBox[{"PDF", "[", 
       RowBox[{
        StyleBox["f",
         FontSlant->"Italic"], ",", 
        RowBox[{"{", 
         StyleBox[
          RowBox[{"x", ",", 
           SubscriptBox["x", "min"], ",", 
           SubscriptBox["x", "max"]}],
          FontSlant->"Italic"], "}"}]}], "]"}], " ", "will", " ", "produce", 
      " ", "a", " ", "continuous", " ", 
      RowBox[{"distribution", ".", "  ", "The"}], " ", "output", " ", "is", 
      " ", "also", " ", "a", " ", "new", " ", 
      RowBox[{"distribution", "."}]}]}]}],
   FontFamily->"Source Sans Pro",
   FontWeight->"Regular"]}]], "Text",
 CellChangeTimes->{{3.803398818344379*^9, 3.803398830417376*^9}, 
   3.803398901123645*^9, {3.822581935180739*^9, 
   3.822581942652763*^9}},ExpressionUUID->"55225ce6-d41c-4d0e-af97-\
deef655d0531"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "CoupledProbability", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"CoupledProbability", "[", 
    RowBox[{"dist_", ",", "riskBias_", ",", "x_", ",", 
     RowBox[{"xmin_", ":", 
      RowBox[{"(", 
       RowBox[{"-", "\[Infinity]"}], ")"}]}], ",", 
     RowBox[{"xmax_", ":", "\[Infinity]"}]}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"CoupledProbability", "[", 
     RowBox[{"dist", ",", "riskBias", ",", "x", ",", "xmin", ",", "xmax"}], 
     "]"}], "=", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"riskBias", "\[Equal]", "0"}], ",", 
      RowBox[{"PDF", "[", 
       RowBox[{"dist", ",", "x"}], "]"}], ",", "\[IndentingNewLine]", 
      RowBox[{"FullSimplify", "[", "\[IndentingNewLine]", 
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"PDF", "[", 
           RowBox[{"dist", ",", "x"}], "]"}], ")"}], 
         RowBox[{"1", "-", "riskBias"}]], 
        RowBox[{
         SubsuperscriptBox["\[Integral]", "xmin", "xmax"], 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"PDF", "[", 
             RowBox[{"dist", ",", "y"}], "]"}], ")"}], 
           RowBox[{"1", "-", "riskBias"}]], 
          RowBox[{"\[DifferentialD]", "y"}]}]}]], "\[IndentingNewLine]", 
       "]"}]}], "\[IndentingNewLine]", "]"}]}]}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ClearAll", "[", "NCoupledProbability", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"NCoupledProbability", "[", 
   RowBox[{"dist_", ",", "riskBias_", ",", "x_", ",", 
    RowBox[{"xmin_", ":", 
     RowBox[{"(", 
      RowBox[{"-", "\[Infinity]"}], ")"}]}], ",", 
    RowBox[{"xmax_", ":", "\[Infinity]"}]}], "]"}], ":=", 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"NCoupledProbability", "[", 
    RowBox[{"dist", ",", "riskBias", ",", "x", ",", "xmin", ",", "xmax"}], 
    "]"}], "=", "\[IndentingNewLine]", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"riskBias", "\[Equal]", "0"}], ",", 
     RowBox[{"PDF", "[", 
      RowBox[{"dist", ",", "x"}], "]"}], ",", "\[IndentingNewLine]", 
     FractionBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"PDF", "[", 
         RowBox[{"dist", ",", "x"}], "]"}], ")"}], 
       RowBox[{"1", "-", "riskBias"}]], 
      RowBox[{"NIntegrate", "[", 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"PDF", "[", 
           RowBox[{"dist", ",", "y"}], "]"}], ")"}], 
         RowBox[{"1", "-", "riskBias"}]], ",", 
        RowBox[{"{", 
         RowBox[{"y", ",", "xmin", ",", "xmax"}], "}"}], ",", 
        RowBox[{"Method", "\[Rule]", "\"\<AdaptiveMonteCarlo\>\""}]}], 
       "]"}]]}], "\[IndentingNewLine]", "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.6490593922452726`*^9, 3.6490596887081933`*^9}, {
   3.6490597269438605`*^9, 3.6490597882675686`*^9}, {3.6490600115663605`*^9, 
   3.6490600347792015`*^9}, {3.6490601162113447`*^9, 
   3.6490601266477623`*^9}, {3.6490601667710333`*^9, 3.649060224350734*^9}, {
   3.6490609704064445`*^9, 3.649061014351722*^9}, {3.649061140306343*^9, 
   3.6490611910688324`*^9}, {3.649061295230215*^9, 3.6490613109082427`*^9}, {
   3.6490616403184214`*^9, 3.6490616421436243`*^9}, {3.6490618108579206`*^9, 
   3.649062000008253*^9}, {3.649062166335745*^9, 3.64906221482063*^9}, {
   3.649062290652364*^9, 3.649062348434865*^9}, {3.6490627689492035`*^9, 
   3.6490628237677*^9}, {3.649068515954899*^9, 3.649068545688551*^9}, {
   3.6490685843142185`*^9, 3.64906860186545*^9}, {3.651061610290411*^9, 
   3.6510616195568275`*^9}, {3.651062707925139*^9, 3.6510627397959948`*^9}, {
   3.654415640069536*^9, 3.654415697712833*^9}, {3.654415731199668*^9, 
   3.654415733399055*^9}, {3.654417117012582*^9, 3.6544171535948544`*^9}, 
   3.6928263629715548`*^9, {3.803398741094001*^9, 3.803398801003497*^9}, {
   3.81547142479214*^9, 3.8154714275317574`*^9}, {3.8155063688678637`*^9, 
   3.815506380589901*^9}, {3.8155064437387047`*^9, 3.815506455221773*^9}, {
   3.8225818976735067`*^9, 3.822581926348982*^9}, {3.825761979384658*^9, 
   3.825762063258136*^9}},
 CellLabel->"In[19]:=",ExpressionUUID->"dc186a1f-396b-4d46-99cb-cc457ed4fa5e"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Biased Coupled Distribution", "Subsection",
 CellChangeTimes->{{3.822581091046672*^9, 3.8225810935156403`*^9}, {
  3.8225834125820227`*^9, 
  3.822583439115898*^9}},ExpressionUUID->"18fbf3bb-8224-4a63-a177-\
76d5b9207ab2"],

Cell[TextData[{
 "The Biased Coupled Distribution function provides an analytic solution for \
the Coupled Probability assuming the distribution is a member of the Coupled \
Exponential Family. The inputs are the parameters of the Coupled Exponential \
Family distribution and the coupling for the riskBias power. The output is a \
distribution from the Coupled Exponential Family with the coupling and scale \
modified by the riskBias power.\nThe first implementation focuses on \
multivariate coupled exponential distribution.\n\nThe modified coupling \
parameter for the output distribution is determined from the following \
relationship\[LineSeparator]\n",
 Cell[BoxData[
  RowBox[{"\t", 
   RowBox[{
    RowBox[{"Solve", "[", 
     RowBox[{
      RowBox[{
       FractionBox[
        RowBox[{"1", " ", "+", " ", 
         RowBox[{"d", " ", "\[Kappa]Mod"}]}], 
        RowBox[{
         RowBox[{"-", "\[Alpha]"}], " ", "\[Kappa]Mod"}]], "\[Equal]", 
       RowBox[{
        RowBox[{"(", 
         FractionBox[
          RowBox[{"1", "+", 
           RowBox[{"d", " ", "\[Kappa]Dist"}]}], 
          RowBox[{
           RowBox[{"-", "\[Alpha]"}], " ", "\[Kappa]Dist"}]], ")"}], 
        RowBox[{"(", 
         RowBox[{"1", "-", 
          FractionBox[
           RowBox[{
            RowBox[{"-", "\[Alpha]"}], " ", "\[Kappa]Bias"}], 
           RowBox[{"1", "+", " ", 
            RowBox[{"d", " ", "\[Kappa]Bias"}]}]]}], ")"}]}]}], ",", 
      "\[Kappa]Mod", ",", "Reals"}], "]"}], "//", "FullSimplify"}]}]], "Input",
  CellChangeTimes->{{3.8225824617158003`*^9, 3.8225824718099537`*^9}, {
   3.822582513550024*^9, 3.8225825602881107`*^9}, {3.822582651292385*^9, 
   3.822582732623412*^9}},ExpressionUUID->
  "e455db5c-4b9b-4c05-8125-6a00dd228ca2"],
 "\n\t",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{"{", 
    RowBox[{"\[Kappa]Mod", "\[Rule]", 
     FractionBox[
      RowBox[{"\[Kappa]Dist", "+", 
       RowBox[{"d", " ", "\[Kappa]Bias", " ", "\[Kappa]Dist"}]}], 
      RowBox[{"1", "+", 
       RowBox[{"\[Kappa]Bias", " ", 
        RowBox[{"(", 
         RowBox[{"d", "+", "\[Alpha]", "+", 
          RowBox[{"d", " ", "\[Alpha]", " ", "\[Kappa]Org"}]}], ")"}]}]}]]}], 
    "}"}], "}"}]], "Output",
  GeneratedCell->False,
  CellAutoOverwrite->False,
  CellChangeTimes->{{3.8225827217670403`*^9, 3.822582733561227*^9}},
  ExpressionUUID->"dd47261c-3cc0-4056-bdeb-547b06c8433a"],
 "\nThe correlation matrix is multiplied by this terms divided by \
\[Kappa]Dist since \[Kappa]Dist ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[CapitalSigma]", 
    RowBox[{"-", "1"}]], TraditionalForm]],ExpressionUUID->
  "a6f18654-7651-4585-91b7-bcff47a09491"],
 "is unchanged.\n"
}], "Text",
 CellChangeTimes->{{3.8225810995205183`*^9, 3.822581187785493*^9}, {
   3.8225819524876833`*^9, 3.8225820190689917`*^9}, {3.822582241549618*^9, 
   3.822582317624029*^9}, {3.822582775328066*^9, 3.822582852371616*^9}, {
   3.8225829845762157`*^9, 3.82258306565296*^9}, {3.822583127572399*^9, 
   3.8225831447655563`*^9}, {3.822583189582069*^9, 3.8225832158901033`*^9}, 
   3.822583287526347*^9, {3.8225834161086683`*^9, 3.822583417540853*^9}, {
   3.82258347505818*^9, 3.822583477413059*^9}, 
   3.824138184668495*^9},ExpressionUUID->"6531acc4-6650-4879-9d2f-\
e32794415adc"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "BiasedCoupledDistribution", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"BiasedCoupledDistribution", "[", 
   RowBox[{
   "\[Mu]_", ",", "\[CapitalSigma]_", ",", "\[Kappa]Dist_", ",", "\[Alpha]_", 
    ",", "\[Kappa]Bias_"}], "]"}], ":=", " ", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"BiasedCoupledDistribution", "[", 
    RowBox[{
    "\[Mu]", ",", "\[CapitalSigma]", ",", "\[Kappa]Dist", ",", "\[Alpha]", 
     ",", " ", "\[Kappa]Bias"}], "]"}], "=", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "dim", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"dim", "=", 
       RowBox[{"Length", "[", "\[Mu]", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"dim", "\[Equal]", "0"}], ",", " ", 
        RowBox[{"(*", " ", 
         RowBox[{
          RowBox[{
          "then", " ", "the", " ", "scaler", " ", "is", " ", "a", " ", 
           "one"}], "-", 
          RowBox[{"dimensional", " ", "distribution"}]}], " ", "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"\[Alpha]", "\[Equal]", "2"}], ",", "\[IndentingNewLine]", 
          RowBox[{"CoupledNormalDistribution", "[", 
           RowBox[{"\[Mu]", ",", 
            RowBox[{
             SqrtBox[
              FractionBox[
               RowBox[{"1", "+", "\[Kappa]Bias"}], 
               RowBox[{"1", "+", 
                RowBox[{"\[Kappa]Bias", " ", 
                 RowBox[{"(", 
                  RowBox[{"3", "+", 
                   RowBox[{"2", " ", "\[Kappa]Dist"}]}], ")"}]}]}]]], 
             "\[CapitalSigma]"}], ",", 
            FractionBox[
             RowBox[{"\[Kappa]Dist", "+", " ", 
              RowBox[{"\[Kappa]Bias", " ", "\[Kappa]Dist"}]}], 
             RowBox[{"1", "+", 
              RowBox[{"\[Kappa]Bias", " ", 
               RowBox[{"(", 
                RowBox[{"3", "+", 
                 RowBox[{"2", " ", "\[Kappa]Dist"}]}], ")"}]}]}]]}], "]"}], 
          ",", "\[IndentingNewLine]", 
          RowBox[{"CoupledExponentialDistribution", "[", 
           RowBox[{"\[Mu]", ",", 
            RowBox[{
             FractionBox[
              RowBox[{"1", "+", " ", "\[Kappa]Bias"}], 
              RowBox[{"1", "+", 
               RowBox[{"\[Kappa]Bias", " ", 
                RowBox[{"(", 
                 RowBox[{"2", "+", "\[Kappa]Dist"}], ")"}]}]}]], 
             "\[CapitalSigma]"}], ",", 
            FractionBox[
             RowBox[{"\[Kappa]Dist", "+", " ", 
              RowBox[{"\[Kappa]Bias", " ", "\[Kappa]Dist"}]}], 
             RowBox[{"1", "+", 
              RowBox[{"\[Kappa]Bias", " ", 
               RowBox[{"(", 
                RowBox[{"2", "+", "\[Kappa]Dist"}], ")"}]}]}]]}], "]"}]}], 
         "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{"Else", ",", " ", 
          RowBox[{
          "assume", " ", "dimension", " ", "is", " ", "greater", " ", "than", 
           " ", "2"}]}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"MultivariateCoupledDistribution", "[", 
         RowBox[{"\[Mu]", ",", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             FractionBox[
              RowBox[{"1", "+", 
               RowBox[{"dim", " ", "\[Kappa]Bias"}]}], 
              RowBox[{"1", "+", 
               RowBox[{"\[Kappa]Bias", " ", 
                RowBox[{"(", 
                 RowBox[{"dim", "+", "\[Alpha]", "+", 
                  RowBox[{"dim", " ", "\[Alpha]", " ", "\[Kappa]Dist"}]}], 
                 ")"}]}]}]], ")"}], 
            FractionBox["1", "\[Alpha]"]], "\[CapitalSigma]"}], ",", 
          FractionBox[
           RowBox[{"\[Kappa]Dist", "+", 
            RowBox[{"dim", " ", "\[Kappa]Bias", " ", "\[Kappa]Dist"}]}], 
           RowBox[{"1", "+", 
            RowBox[{"\[Kappa]Bias", " ", 
             RowBox[{"(", 
              RowBox[{"dim", "+", "\[Alpha]", "+", 
               RowBox[{"dim", " ", "\[Alpha]", " ", "\[Kappa]Dist"}]}], 
              ")"}]}]}]], ",", "\[Alpha]"}], "]"}]}], "\[IndentingNewLine]", 
       "]"}]}]}], "\[IndentingNewLine]", "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.822582118360045*^9, 3.82258213761425*^9}, {
   3.822582323236273*^9, 3.822582403283279*^9}, {3.822582857828364*^9, 
   3.822582963858697*^9}, {3.822583222683032*^9, 3.822583252679778*^9}, {
   3.822583445892209*^9, 3.822583470487311*^9}, {3.823636936328762*^9, 
   3.823636947478776*^9}, {3.823637008606123*^9, 3.823637025143964*^9}, {
   3.823637055833372*^9, 3.823637260695195*^9}, {3.823637599001193*^9, 
   3.823637640986619*^9}, 3.824674573446184*^9, {3.824674926631298*^9, 
   3.8246749465580263`*^9}},
 CellLabel->"In[23]:=",ExpressionUUID->"c1a0aeea-7d79-4ede-afb8-0d6d3d23ca3f"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coupled Entropy", "Subsection",
 CellChangeTimes->{{3.6510633275426273`*^9, 
  3.651063329648631*^9}},ExpressionUUID->"ce1512d8-343d-466d-9637-\
377e32d05b40"],

Cell["\<\
Computes the coupled entropy using the coupled logarithm and coupled \
probability functions.
Currently limited to one-dimensional distributions
Updated Dec 17, 2020 to call the Coupled Cross-Entropy function

Inputs
p - probability distribution, limited to one-dimension
\[Kappa] - coupling parameter
d - dimension of the random variable, default is 1
\[Alpha] - highest power of the constraints, default is 2 since this is \
associated with the Coupled Gaussian
\[Delta] - root of the coupled logarithm, default is 1; provides for \
two-parameter entropy experiments similar to c,d-Entropy
limits - domain of the variable; currently only one dimension is implemented

Output is the coupled entropy for a one-dimensional distribution\
\>", "Text",
 CellChangeTimes->{{3.6521944498370776`*^9, 3.652194478309618*^9}, {
  3.661687521781364*^9, 3.661687667603043*^9}, {3.661687720232087*^9, 
  3.661687745586032*^9}, {3.817207540601306*^9, 3.817207565237617*^9}, {
  3.8256871487050047`*^9, 3.825687286700712*^9}, {3.825687711446754*^9, 
  3.825687716276146*^9}},ExpressionUUID->"8f9c0919-0f86-4960-8fde-\
cd67499a95b6"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "CoupledEntropy", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CoupledEntropy", "[", 
   RowBox[{"dist_", ",", "\[Kappa]_", ",", 
    RowBox[{"d_", ":", "1"}], ",", 
    RowBox[{"\[Alpha]_", ":", "2"}], ",", 
    RowBox[{"\[Delta]_", ":", "1"}], ",", 
    RowBox[{"limits_", ":", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}]}]}], "]"}], ":=",
   "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"CoupledEntropy", "[", 
    RowBox[{
    "dist", ",", "\[Kappa]", ",", "d", ",", "\[Alpha]", ",", "\[Delta]", ",", 
     "limits"}], "]"}], "=", "\[IndentingNewLine]", 
   RowBox[{"CoupledCrossEntropy", "[", 
    RowBox[{
    "dist", ",", "dist", ",", "\[Kappa]", ",", "\[Alpha]", ",", "d", ",", 
     "\[Delta]", ",", "limits"}], "]"}]}]}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.6510633380102453`*^9, 3.651063351083069*^9}, {
   3.652194484007126*^9, 3.652194495859643*^9}, {3.652194542855711*^9, 
   3.6521945761932583`*^9}, {3.661687296991143*^9, 3.661687422228334*^9}, {
   3.6616876849942217`*^9, 3.661687685514419*^9}, {3.8033989395604*^9, 
   3.8033989753217173`*^9}, {3.8154720770081673`*^9, 3.815472081022881*^9}, {
   3.815472111926487*^9, 3.815472152648778*^9}, {3.815472314176469*^9, 
   3.815472322447019*^9}, {3.815472364421796*^9, 3.8154724572822113`*^9}, {
   3.8154726769109077`*^9, 3.815472715495721*^9}, 3.815473090263226*^9, {
   3.815492921823751*^9, 3.815492944828333*^9}, {3.815493084643812*^9, 
   3.815493151675417*^9}, {3.815494540908784*^9, 3.815494565370162*^9}, {
   3.81549528711161*^9, 3.815495344161553*^9}, {3.817207583017809*^9, 
   3.817207612019148*^9}, {3.8253615245100393`*^9, 3.825361537979047*^9}, {
   3.8253645014596786`*^9, 3.8253645060432158`*^9}, {3.8256873688820877`*^9, 
   3.825687417695279*^9}, {3.8256887657694263`*^9, 3.825688767690439*^9}},
 CellLabel->"In[25]:=",ExpressionUUID->"b037b0fd-c1db-4a66-a215-76eb107c050b"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "NCoupledEntropy", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"NCoupledEntropy", "[", 
   RowBox[{"dist_", ",", "\[Kappa]_", ",", 
    RowBox[{"d_", ":", "1"}], ",", 
    RowBox[{"\[Alpha]_", ":", "2"}], ",", 
    RowBox[{"\[Delta]_", ":", "1"}], ",", 
    RowBox[{"limits_", ":", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}]}]}], "]"}], ":=",
   "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"NCoupledEntropy", "[", 
    RowBox[{
    "dist", ",", "\[Kappa]", ",", "d", ",", "\[Alpha]", ",", "\[Delta]", ",", 
     "limits"}], "]"}], "=", "\[IndentingNewLine]", 
   RowBox[{"NCoupledCrossEntropy", "[", 
    RowBox[{
    "dist", ",", "dist", ",", "\[Kappa]", ",", "d", ",", "\[Alpha]", ",", 
     "\[Delta]", ",", "limits"}], "]"}]}]}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.825691323237154*^9, 3.825691330502615*^9}, {
  3.825762689490119*^9, 3.8257626938765306`*^9}},
 CellLabel->"In[27]:=",ExpressionUUID->"8fafd6e7-8e5d-49c6-987d-f5ecdb1297bc"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coupled Cross-Entropy", "Subsection",
 CellChangeTimes->{{3.6510633275426273`*^9, 3.651063329648631*^9}, {
  3.817051568587042*^9, 
  3.81705157144628*^9}},ExpressionUUID->"b75b8941-2946-4cd9-987a-\
42ed41a0df40"],

Cell["\<\
Computes the coupled cross-entropy using the coupled logarithm and coupled \
probability functions.
Currently limited to one-dimensional distributions

Inputs
p - probability distribution, limited to one-dimension
\[Kappa] - coupling parameter
d - dimension of the random variable, default is 1
\[Alpha] - highest power of the constraints, default is 2 since this is \
associated with the Coupled Gaussian
\[Delta] - root of the coupled logarithm, default is 1; provides for \
two-parameter entropy experiments similar to c,d-Entropy
limits - domain of the variable; currently only one dimension is implemented

Output is the coupled entropy for a one-dimensional distribution\
\>", "Text",
 CellChangeTimes->{{3.6521944498370776`*^9, 3.652194478309618*^9}, {
   3.661687521781364*^9, 3.661687667603043*^9}, {3.661687720232087*^9, 
   3.661687745586032*^9}, {3.81705171249304*^9, 3.81705171529816*^9}, 
   3.825687432545754*^9, {3.825687598882972*^9, 3.825687609276208*^9}, {
   3.8256876486197023`*^9, 
   3.825687671812688*^9}},ExpressionUUID->"5abc315b-145c-4e81-957a-\
34d53ccb6a8f"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "CoupledCrossEntropy", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CoupledCrossEntropy", "[", 
   RowBox[{"distP_", ",", "distQ_", ",", "\[Kappa]_", ",", 
    RowBox[{"d_", ":", "1"}], ",", 
    RowBox[{"\[Alpha]_", ":", "2"}], ",", " ", 
    RowBox[{"\[Delta]_", ":", "1"}], ",", 
    RowBox[{"limits_", ":", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}]}]}], "]"}], ":=", 
  RowBox[{
   RowBox[{"CoupledCrossEntropy", "[", 
    RowBox[{
    "distP", ",", "distQ", ",", "\[Kappa]", ",", "d", ",", "\[Alpha]", ",", 
     "\[Delta]", ",", "limits"}], "]"}], "=", 
   RowBox[{"Integrate", "[", 
    RowBox[{
     RowBox[{"FullSimplify", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"CoupledProbability", "[", 
        RowBox[{"distP", ",", 
         FractionBox[
          RowBox[{
           RowBox[{"-", "\[Alpha]"}], " ", "\[Kappa]"}], 
          RowBox[{"1", "+", 
           RowBox[{"d", " ", "\[Kappa]"}]}]], ",", "x"}], "]"}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FractionBox["1", "\[Alpha]"], 
          RowBox[{"CoupledLogarithm", "[", 
           RowBox[{
            SuperscriptBox[
             RowBox[{"PDF", "[", 
              RowBox[{"distQ", ",", "x"}], "]"}], 
             RowBox[{"-", "\[Alpha]"}]], ",", "\[Kappa]", ",", "d"}], "]"}]}],
          ")"}], 
        FractionBox["1", "\[Delta]"]]}], "\[IndentingNewLine]", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{
       "limits", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], ",", 
       RowBox[{
       "limits", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
      "}"}]}], "\[IndentingNewLine]", 
    "]"}]}]}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.6510633380102453`*^9, 3.651063351083069*^9}, {
   3.652194484007126*^9, 3.652194495859643*^9}, {3.652194542855711*^9, 
   3.6521945761932583`*^9}, {3.661687296991143*^9, 3.661687422228334*^9}, {
   3.6616876849942217`*^9, 3.661687685514419*^9}, {3.8033989395604*^9, 
   3.8033989753217173`*^9}, {3.8154720770081673`*^9, 3.815472081022881*^9}, {
   3.815472111926487*^9, 3.815472152648778*^9}, {3.815472314176469*^9, 
   3.815472322447019*^9}, {3.815472364421796*^9, 3.8154724572822113`*^9}, {
   3.8154726769109077`*^9, 3.815472715495721*^9}, 3.815473090263226*^9, {
   3.815492921823751*^9, 3.815492944828333*^9}, {3.815493084643812*^9, 
   3.815493151675417*^9}, {3.815494540908784*^9, 3.815494565370162*^9}, {
   3.81549528711161*^9, 3.815495344161553*^9}, {3.81705157708295*^9, 
   3.817051625475925*^9}, {3.817052600934704*^9, 3.817052604621415*^9}, 
   3.822837406379397*^9, 3.82285381736583*^9, {3.825359669196291*^9, 
   3.825359746267242*^9}, 3.825359796843709*^9, {3.8253598659025717`*^9, 
   3.825359934566908*^9}, 3.825361984195492*^9, {3.82536363032752*^9, 
   3.825363668109943*^9}, {3.825363698694439*^9, 3.825363700835299*^9}, {
   3.82536385833998*^9, 3.825363859276197*^9}, {3.825363893458907*^9, 
   3.825363895111163*^9}, {3.825363991687675*^9, 3.825364010975072*^9}, {
   3.8253642870387897`*^9, 3.825364495942276*^9}, {3.825687439091296*^9, 
   3.825687517009376*^9}, {3.8256894987067966`*^9, 3.825689507391066*^9}, 
   3.825690228205082*^9, 3.8257621341483192`*^9, {3.8257633619988813`*^9, 
   3.825763413161022*^9}},
 CellLabel->"In[29]:=",ExpressionUUID->"f9544062-c5df-48e4-94e8-fbfe4a775110"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "NCoupledCrossEntropy", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"NCoupledCrossEntropy", "[", 
   RowBox[{"distP_", ",", "distQ_", ",", "\[Kappa]_", ",", 
    RowBox[{"d_", ":", "1"}], ",", 
    RowBox[{"\[Alpha]_", ":", "2"}], ",", " ", 
    RowBox[{"\[Delta]_", ":", "1"}], ",", 
    RowBox[{"limits_", ":", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}]}]}], "]"}], ":=", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"NCoupledCrossEntropy", "[", 
     RowBox[{
     "distP", ",", "distQ", ",", "\[Kappa]", ",", "d", ",", "\[Alpha]", ",", 
      "\[Delta]", ",", "limits"}], "]"}], "="}], "*)"}], 
  RowBox[{"NIntegrate", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"NCoupledProbability", "[", 
      RowBox[{"distP", ",", 
       FractionBox[
        RowBox[{
         RowBox[{"-", "\[Alpha]"}], " ", "\[Kappa]"}], 
        RowBox[{"1", "+", 
         RowBox[{"d", " ", "\[Kappa]"}]}]], ",", "x"}], "]"}], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FractionBox["1", "\[Alpha]"], 
        RowBox[{"CoupledLogarithm", "[", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"PDF", "[", 
            RowBox[{"distQ", ",", "x"}], "]"}], 
           RowBox[{"-", "\[Alpha]"}]], ",", "\[Kappa]", ",", "d"}], "]"}]}], 
       ")"}], 
      FractionBox["1", "\[Delta]"]]}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", 
      RowBox[{
      "limits", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], ",", 
      RowBox[{
      "limits", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Method", "\[Rule]", "\"\<AdaptiveMonteCarlo\>\""}]}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.82569134941895*^9, 3.8256913572512913`*^9}, {
   3.8256985000185328`*^9, 3.825698502919244*^9}, {3.825705556983551*^9, 
   3.825705560100257*^9}, 3.825759377890703*^9, 3.8257599596815367`*^9, 
   3.825762148709173*^9, {3.8257621870410957`*^9, 3.825762222412712*^9}, {
   3.8257625148216543`*^9, 3.8257625299879*^9}, 3.8257628956917543`*^9, 
   3.8257630868391733`*^9},
 CellLabel->"In[31]:=",ExpressionUUID->"7ced798f-0812-42c2-ae7d-00272667bc97"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coupled Divergence", "Subsection",
 CellChangeTimes->{{3.6510633275426273`*^9, 3.651063329648631*^9}, {
  3.817051568587042*^9, 3.81705157144628*^9}, {3.817051659773526*^9, 
  3.817051661611739*^9}},ExpressionUUID->"cf0fcfbb-6853-48d9-a469-\
c2af898a7ae4"],

Cell["\<\
Computes the coupled divergence using its relationship with Coupled Entropy \
and Coupled Cross-Entropy
Currently limited to one-dimensional distributions

The definition of the Coupled Divergence is notionally:
\tDivergence = Cross-Entropy - Entropy
\t- CoupledProbability[distP, ...] (CoupledLogarithm[distQ, ...] - \
CoupledLogarithm[distP, ...])
This is equivalent to:
\t CoupledCrossEntropy[distP, distQ, ...] - CoupledEntropy[distP,...]
\tnote that for the first case the functional relationship for the \
generalization is the same as for \[Kappa] = 0.
\t\tAnd there is an alternative form in which the division of the densities \
is preserved but the sum is generalized
\t\t
For the second def

Inputs
p - probability distribution, limited to one-dimension
\[Kappa] - coupling parameter
d - dimension of the random variable, default is 1
\[Alpha] - highest power of the constraints, default is 2 since this is \
associated with the Coupled Gaussian
\[Delta] - root of the coupled logarithm, default is 1; provides for \
two-parameter entropy experiments similar to c,d-Entropy
limits - domain of the variable; currently only one dimension is implemented

Output is the coupled entropy for a one-dimensional distribution\
\>", "Text",
 CellChangeTimes->{{3.6521944498370776`*^9, 3.652194478309618*^9}, {
   3.661687521781364*^9, 3.661687667603043*^9}, {3.661687720232087*^9, 
   3.661687745586032*^9}, {3.817051780434779*^9, 3.817051869512435*^9}, {
   3.817051902615905*^9, 3.817052040955762*^9}, {3.817052076421652*^9, 
   3.817052098053524*^9}, {3.817052262032077*^9, 3.81705238067523*^9}, {
   3.817052413632213*^9, 3.817052484857831*^9}, {3.817052516668498*^9, 
   3.81705253246139*^9}, {3.817052646794004*^9, 3.817052662561796*^9}, 
   3.825687590240746*^9, {3.825687688502366*^9, 
   3.825687699868866*^9}},ExpressionUUID->"ec0d2064-6b4a-4c80-afc1-\
db7fb0f32afa"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "CoupledDivergence", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CoupledDivergence", "[", 
   RowBox[{"distP_", ",", "distQ_", ",", "\[Kappa]_", ",", 
    RowBox[{"d_", ":", "1"}], ",", 
    RowBox[{"\[Alpha]_", ":", "2"}], ",", 
    RowBox[{"\[Delta]_", ":", "1"}], ",", 
    RowBox[{"limits_", ":", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}]}]}], "]"}], ":=",
   "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"CoupledDivergence", "[", 
    RowBox[{
    "distP", ",", "distQ", ",", "\[Kappa]", ",", "d", ",", "\[Alpha]", ",", 
     "\[Delta]", ",", "limits"}], "]"}], "=", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"CoupledCrossEntropy", "[", 
     RowBox[{
     "distP", ",", "distQ", ",", "\[Kappa]", ",", "d", ",", "\[Alpha]", ",", 
      "\[Delta]", ",", "limits"}], "]"}], "-", "\[IndentingNewLine]", 
    RowBox[{"CoupledEntropy", "[", 
     RowBox[{
     "distP", ",", "\[Kappa]", ",", "d", ",", "\[Alpha]", ",", "\[Delta]", 
      ",", "limits"}], "]"}]}]}]}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.6510633380102453`*^9, 3.651063351083069*^9}, {
   3.652194484007126*^9, 3.652194495859643*^9}, {3.652194542855711*^9, 
   3.6521945761932583`*^9}, {3.661687296991143*^9, 3.661687422228334*^9}, {
   3.6616876849942217`*^9, 3.661687685514419*^9}, {3.8033989395604*^9, 
   3.8033989753217173`*^9}, {3.8154720770081673`*^9, 3.815472081022881*^9}, {
   3.815472111926487*^9, 3.815472152648778*^9}, {3.815472314176469*^9, 
   3.815472322447019*^9}, {3.815472364421796*^9, 3.8154724572822113`*^9}, {
   3.8154726769109077`*^9, 3.815472715495721*^9}, 3.815473090263226*^9, {
   3.815492921823751*^9, 3.815492944828333*^9}, {3.815493084643812*^9, 
   3.815493151675417*^9}, {3.815494540908784*^9, 3.815494565370162*^9}, {
   3.81549528711161*^9, 3.815495344161553*^9}, {3.81705157708295*^9, 
   3.817051625475925*^9}, {3.8170525720755663`*^9, 3.8170526253408813`*^9}, {
   3.817052674913907*^9, 3.817052710390066*^9}, {3.8253615479507713`*^9, 
   3.8253615739454308`*^9}, {3.82536451408698*^9, 3.825364527557592*^9}, {
   3.825687731446088*^9, 3.825687792712769*^9}},
 CellLabel->"In[33]:=",ExpressionUUID->"9ab3cca5-b43c-494c-8f1f-1425eecd59a4"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "NCoupledDivergence", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"NCoupledDivergence", "[", 
   RowBox[{"distP_", ",", "distQ_", ",", "\[Kappa]_", ",", 
    RowBox[{"d_", ":", "1"}], ",", 
    RowBox[{"\[Alpha]_", ":", "2"}], ",", 
    RowBox[{"\[Delta]_", ":", "1"}], ",", 
    RowBox[{"limits_", ":", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}]}]}], "]"}], ":=",
   "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"NCoupledDivergence", "[", 
    RowBox[{
    "distP", ",", "distQ", ",", "\[Kappa]", ",", "d", ",", "\[Alpha]", ",", 
     "\[Delta]", ",", "limits"}], "]"}], "=", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"NCoupledCrossEntropy", "[", 
     RowBox[{
     "distP", ",", "distQ", ",", "\[Kappa]", ",", "d", ",", "\[Alpha]", ",", 
      "\[Delta]", ",", "limits"}], "]"}], "-", "\[IndentingNewLine]", 
    RowBox[{"NCoupledEntropy", "[", 
     RowBox[{
     "distP", ",", "\[Kappa]", ",", "d", ",", "\[Alpha]", ",", "\[Delta]", 
      ",", "limits"}], "]"}]}]}]}]}], "Input",
 CellChangeTimes->{{3.8256913688046494`*^9, 3.825691378584091*^9}},
 CellLabel->"In[35]:=",ExpressionUUID->"7279d7fd-bc51-4f12-b53b-41fb687a0edf"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coupled Product", "Subsection",
 CellChangeTimes->{{3.654416393387126*^9, 
  3.654416395785495*^9}},ExpressionUUID->"f44e42f8-8535-4fca-8e3d-\
c9fd8b9d88e5"],

Cell[TextData[{
 "The Coupled Product is a generalization of the product function, which \
raises each input to a power and after combining the inputs, takes the root.  \
It is defined based on the sum of arguments of the coupled exponential:\n\t",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      SubscriptBox[
       UnderoverscriptBox["\[Product]", 
        RowBox[{"i", "=", "1"}], "N"], 
       SubscriptBox["\[CircleTimes]", 
        RowBox[{"\[Alpha]", ",", "\[Kappa]", ",", 
         SubscriptBox["d", "i"]}]]], 
      SubscriptBox["x", "i"]}], "=", 
     RowBox[{
      SubscriptBox["Exp", 
       RowBox[{"\[Alpha]", ",", "\[Kappa]", ",", "D"}]], "[", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "1"}], "N"], 
       RowBox[{
        SubscriptBox["Log", 
         RowBox[{"\[Alpha]", ",", "\[Kappa]", ",", 
          SubscriptBox["d", "i"]}]], "[", 
        SubscriptBox["x", "i"], "]"}]}], "]"}]}], ";", "   ", 
    RowBox[{"D", " ", "=", " ", 
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"i", "=", "1"}], "N"], 
      SubscriptBox["d", "i"]}]}]}], TraditionalForm]],ExpressionUUID->
  "8e1b6678-86ff-429f-ad9a-388d0cbe12d8"],
 "\n\t                   = (",
 Cell[BoxData[
  FormBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "1"}], "N"], 
    SubsuperscriptBox["x", "i", 
     FractionBox[
      RowBox[{"\[Alpha]", " ", "\[Kappa]"}], 
      RowBox[{"1", "+", 
       RowBox[{
        SubscriptBox["d", "i"], "\[Kappa]"}]}]]]}], TraditionalForm]],
  ExpressionUUID->"2c13fc4c-754a-4135-8657-cf4601835312"],
 "- ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"N", "-", "1"}], ")"}], ")"}], 
      FractionBox[
       RowBox[{"1", " ", "+", " ", 
        RowBox[{"D", " ", "\[Kappa]"}]}], 
       RowBox[{"\[Alpha]", " ", "\[Kappa]"}]]], ";"}], "   "}], 
   TraditionalForm]],ExpressionUUID->"589cf7d8-2985-4715-8ce3-3b39893aac88"],
 Cell[BoxData[
  FormBox[
   RowBox[{"D", " ", "=", " ", 
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"i", "=", "1"}], "N"], 
     SubscriptBox["d", "i"]}]}], TraditionalForm]],ExpressionUUID->
  "3321a210-1087-4afc-9853-9db83dbd1375"],
 "\n\t                   \nInputs\nxList  -  list of arguments to be combined \
by the coupled product\n\[Kappa]  -   coupling parameter\n\[Alpha]  -  \
multiplicative term, default is 2 which is associated with coupled Gaussian \
distributions\nd -  dimension, currently defined as a constant, but planned \
to also except of list the same dimensions as xList; version 5 modified the \
dimensions of the outer exponent to be d times Length[xList].  Thus the \
inputs are assumed to be the same dimension and the output dimension is sum \
of the input dimensions.\n\nOutput - coupled product of xList\n\nImprovements \
Planned:  \nThe dimension is defined as a constant.  The dimension of each \
coupled logarithm should be controllable.  The dimension of the coupled \
exponential should be the sum of the dimensions used for each coupled \
logarithm.\n\nThe CoupledProductPrototype is intended to allow the input of \
arguments to be combined match the structure of the built in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Product function.  However; more extensive pattern matching is required to \
process the variety of inputs possible using this structure.  \n\nOld \
version:  CoupledProductOld works but does not build upon the coupled \
exponential and coupled logarithm functions.  It does compute the output \
dimension as a sum of the input dimensions.\n"
}], "Text",
 CellChangeTimes->{{3.6544168939667273`*^9, 3.6544170643736763`*^9}, {
   3.654417179028306*^9, 3.654417243802367*^9}, {3.654417493973538*^9, 
   3.654417500858365*^9}, {3.654422422878655*^9, 3.65442284619258*^9}, {
   3.654429898725807*^9, 3.6544299149218607`*^9}, {3.659525407112253*^9, 
   3.659525811986944*^9}, {3.661685155280973*^9, 3.6616855128788033`*^9}, {
   3.661685620866274*^9, 3.661686265170774*^9}, {3.661686306241045*^9, 
   3.661686480563225*^9}, 3.661691831344734*^9, {3.662469298073042*^9, 
   3.662469336037122*^9}, {3.662469454500024*^9, 3.662469511690284*^9}},
 TextAlignment->
  AlignmentMarker,ExpressionUUID->"0d93f37a-5794-4851-8816-9124fbda03af"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "CoupledProduct", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"CoupledProduct", "[", 
    RowBox[{"xList_", ",", "\[Kappa]_", ",", 
     RowBox[{"\[Alpha]_", ":", "2"}], ",", 
     RowBox[{"d_", ":", "1"}]}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"CoupledProduct", "[", 
     RowBox[{"xList", ",", "\[Kappa]", ",", "\[Alpha]", ",", "d"}], "]"}], 
    "=", "\[IndentingNewLine]", 
    RowBox[{"CoupledExponential", "[", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"Total", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"CoupledLogarithm", "[", 
           RowBox[{
            SuperscriptBox["#", 
             RowBox[{"-", "\[Alpha]"}]], ",", "\[Kappa]", ",", "d"}], "]"}], 
          "&"}], "/@", "xList"}], "]"}], 
       FractionBox[
        RowBox[{"-", "1"}], "\[Alpha]"]], ",", "\[IndentingNewLine]", 
      "\[Kappa]", ",", 
      RowBox[{"d", " ", 
       RowBox[{"Length", "[", "xList", "]"}]}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"Not", " ", "used", "\[IndentingNewLine]", 
      RowBox[{"CoupledProductPrototype", "[", 
       RowBox[{"f_", ",", "index_", ",", "\[Kappa]_", ",", 
        RowBox[{"\[Alpha]_", ":", "2"}], ",", 
        RowBox[{"d_", ":", "1"}]}], "]"}]}], ":=", "\[IndentingNewLine]", 
     RowBox[{"CoupledExponential", "[", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"Sum", "[", 
         RowBox[{
          RowBox[{"CoupledLogarithm", "[", 
           RowBox[{
            SuperscriptBox["f", 
             RowBox[{"-", "\[Alpha]"}]], ",", "\[Kappa]", ",", "d"}], "]"}], 
          ",", "index"}], "]"}], 
        FractionBox[
         RowBox[{"-", "1"}], 
         RowBox[{" ", "\[Alpha]"}]]], ",", "\[IndentingNewLine]", "\[Kappa]", 
       ",", "\[Alpha]", ",", "d"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   "*)"}]}]}], "Input",
 CellChangeTimes->{{3.65954054626924*^9, 3.659540582889831*^9}, {
  3.659540647631373*^9, 3.6595406550539618`*^9}, {3.6616850071608267`*^9, 
  3.661685036557926*^9}, {3.662469170145957*^9, 3.662469185224148*^9}, {
  3.6848533681874647`*^9, 3.6848534796701736`*^9}, {3.80339900003829*^9, 
  3.803399035446292*^9}, {3.803399079550015*^9, 3.803399087080861*^9}},
 CellLabel->"In[37]:=",ExpressionUUID->"95a5b6d3-6977-4786-aff3-3d3964c92d48"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Not", " ", "used", "\[IndentingNewLine]", 
    RowBox[{"CoupledProductOld", "[", 
     RowBox[{"xList_", ",", "\[Kappa]_", ",", "\[Alpha]_", ",", "dList_"}], 
     "]"}]}], ":=", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"\[Kappa]MultList", " ", "=", " ", 
         RowBox[{"Table", "[", 
          RowBox[{
           FractionBox[
            RowBox[{
             RowBox[{"-", "\[Alpha]"}], " ", "\[Kappa]"}], 
            RowBox[{"1", "+", 
             RowBox[{
              RowBox[{
              "dList", "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}],
               " ", "\[Kappa]"}]}]], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", 
             RowBox[{"Length", "[", "dList", "]"}]}], "}"}]}], "]"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"\[Kappa]MultOutput", " ", "=", 
         FractionBox[
          RowBox[{
           RowBox[{"-", "\[Alpha]"}], " ", "\[Kappa]"}], 
          RowBox[{"1", "+", 
           RowBox[{
            RowBox[{"Sum", "[", 
             RowBox[{
              RowBox[{
              "dList", "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}],
               ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", 
                RowBox[{"Length", "[", "dList", "]"}]}], "}"}]}], "]"}], " ", 
            "\[Kappa]"}]}]]}]}], "\[IndentingNewLine]", "}"}], ",", " ", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "dList", "]"}], "\[NotEqual]", "1"}], ",", 
        "\[IndentingNewLine]", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Sum", "[", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
               "xList", "\[LeftDoubleBracket]", "i", 
                "\[RightDoubleBracket]"}], ")"}], 
              RowBox[{
              "\[Kappa]MultList", "\[LeftDoubleBracket]", "i", 
               "\[RightDoubleBracket]"}]], ",", 
             RowBox[{"{", 
              RowBox[{"i", ",", 
               RowBox[{"Length", "[", "xList", "]"}]}], "}"}]}], "]"}], 
           "\[IndentingNewLine]", "-", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Length", "[", "xList", "]"}], "-", "1"}], ")"}]}], 
          ")"}], 
         FractionBox["1", "\[Kappa]MultOutput"]], ",", "\[IndentingNewLine]", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Sum", "[", 
            RowBox[{
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{
               "xList", "\[LeftDoubleBracket]", "i", 
                "\[RightDoubleBracket]"}], ")"}], 
              RowBox[{
              "\[Kappa]MultList", "\[LeftDoubleBracket]", "1", 
               "\[RightDoubleBracket]"}]], ",", 
             RowBox[{"{", 
              RowBox[{"i", ",", 
               RowBox[{"Length", "[", "xList", "]"}]}], "}"}]}], "]"}], 
           "\[IndentingNewLine]", "-", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Length", "[", "xList", "]"}], "-", "1"}], ")"}]}], 
          ")"}], 
         FractionBox["1", 
          RowBox[{
          "\[Kappa]MultList", "\[LeftDoubleBracket]", "1", 
           "\[RightDoubleBracket]"}]]]}], "\[IndentingNewLine]", "]"}]}], 
     "\[IndentingNewLine]", "]"}], "/;", 
    RowBox[{
     RowBox[{
      RowBox[{"Length", "[", "xList", "]"}], ">", "1"}], "&&", 
     RowBox[{
      RowBox[{"Length", "[", "xList", "]"}], "==", 
      RowBox[{"Length", "[", "dList", "]"}]}]}]}]}], "\[IndentingNewLine]", 
  "*)"}]], "Input",
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGCQAGIQvWE3D4u91WvHti9+YDovSYkDRE99vAhMc635YgCi
7cpuGIHoWj9ucxCdwJBkB6KXcDgkgWh2tadg+tyKv9NB9DOxyzNA9IHlnvNB
9CnO0mUg+n+UgoIjkK5LUNUA0R2WuYYgelvjNhMQncD2OgxEK22PjQXRKeLV
JSB6wuSuahAtMye5GUQXRD2eDqKf7fZaCqI1Ds7fA6LNvBSOg+hPjw0vgmg/
86P3QfSZvrjHIDpkzuQXIJo9IeU9iFbYoVjqCqSLG60aQHS5z8E37kC654DF
WxDtOsHkG4jmXb7uO4hW3Xvuw29nYDjsu/gRRAud2nBmtudrx9itLmdBtJyT
TP0cIG2Wc7MTRAelyMwF0XJz2eeBaAN7u7KMsDeONw1/lINoAAPKsN4=
  "],
 CellLabel->"In[39]:=",ExpressionUUID->"ece73d97-070f-410e-9a67-74ef68e6d087"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coupled Sum", "Subsection",
 CellChangeTimes->{{3.654422872924858*^9, 
  3.654422897591241*^9}},ExpressionUUID->"0510a17d-c063-4252-b6e7-\
945be4e6c701"],

Cell[TextData[{
 "For two inputs the coupled sum is x + y + (-\[Alpha] \[Kappa]) x y\nFor \
more than two inputs, the solution is determined using recursion.  The last \
element of the list is extracted as one of the variables, and the other \
variable calls the coupled sum without the last element of the list.\nThe \
input is a list of variables to be operated on, the nonlinear coupling \
\[Kappa], and the power \[Alpha] affecting the variable ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "\[Alpha]"], TraditionalForm]],ExpressionUUID->
  "75d66eac-8fe0-4e0c-8c41-867aeb93799f"],
 ".  The operator currently assumes each variable in the list is \
one-dimensional."
}], "Text",
 CellChangeTimes->{{3.65443776909524*^9, 3.654437885352631*^9}, {
  3.654437926008856*^9, 
  3.654437996053404*^9}},ExpressionUUID->"ca149bd8-6cac-4796-a680-\
7ec6921a70fe"],

Cell[TextData[{
 "Improvement Planned\nA simpler specification of the function uses the fold \
function.  Use of alpha not equal to 1 still in evaluation.  Earlier plan no \
longer necessary:\n\tCoupledSumPrototype will define the coupled sum in terms \
of the product of arguments of the coupled logarithm.  The input \tis \
intended to follow the structure of the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Sum function, but requires additional pattern matching of input types \tto \
be fully functional."
}], "Text",
 CellChangeTimes->{
  3.654429928532255*^9, {3.6616865033031282`*^9, 3.661686617955531*^9}, {
   3.805840916510181*^9, 
   3.805840985245698*^9}},ExpressionUUID->"4c988f34-0a91-4b99-97d0-\
dcb036e0b13f"],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.659525828902882*^9, 3.659525876461569*^9}, {
   3.65952610154865*^9, 3.659526239574691*^9}, 3.661685086344303*^9},
 CellLabel->"In[40]:=",ExpressionUUID->"d73cc257-4767-4e60-8674-37a5f3c34176"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "CoupledSum", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"CoupledSum", "[", 
    RowBox[{"xList_", ",", "\[Kappa]_", ",", "\[Alpha]_"}], "]"}], ":=", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"CoupledSum", "[", 
     RowBox[{"xList", ",", "\[Kappa]", ",", "\[Alpha]"}], "]"}], "=", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "xList", "]"}], "\[Equal]", "2"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"First", "[", "xList", "]"}], "+", 
        RowBox[{"Last", "[", "xList", "]"}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "\[Alpha]"}], " ", "\[Kappa]"}], ")"}], 
         RowBox[{"First", "[", "xList", "]"}], 
         RowBox[{"Last", "[", "xList", "]"}]}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"Else", " ", "when", " ", "xList"}], " ", ">", " ", "2"}], 
        "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"CoupledSum", "[", 
         RowBox[{
          RowBox[{"Drop", "[", 
           RowBox[{"xList", ",", 
            RowBox[{"-", "1"}]}], "]"}], ",", "\[Kappa]", ",", "\[Alpha]"}], 
         "]"}], "+", "\[IndentingNewLine]", 
        RowBox[{"Last", "[", "xList", "]"}], "+", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "\[Alpha]"}], " ", "\[Kappa]"}], ")"}], 
         RowBox[{"CoupledSum", "[", 
          RowBox[{
           RowBox[{"Drop", "[", 
            RowBox[{"xList", ",", 
             RowBox[{"-", "1"}]}], "]"}], ",", "\[Kappa]", ",", "\[Alpha]"}], 
          "]"}], 
         RowBox[{"Last", "[", "xList", "]"}]}]}]}], "\[IndentingNewLine]", 
      "]"}], "/;", 
     RowBox[{
      RowBox[{"Length", "[", "xList", "]"}], ">", "1"}]}]}]}], "  ", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"only", " ", "execute", " ", "if", " ", "length"}], " ", ">", " ",
     "1"}], " ", "*)"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"CoupledSumPrototype", "[", 
    RowBox[{"list_", ",", "\[Kappa]_", ",", 
     RowBox[{"\[Alpha]_", ":", "1"}]}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"Fold", "[", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["#1", "\[Alpha]"], "+", 
         SuperscriptBox["#2", "\[Alpha]"], "+", 
         RowBox[{"\[Kappa]", 
          SuperscriptBox["#1", "\[Alpha]"], 
          SuperscriptBox["#2", "\[Alpha]"]}]}], ")"}], 
       FractionBox["1", "\[Alpha]"]], "&"}], ",", "list"}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.654422930390855*^9, 3.654422987187644*^9}, 
   3.65442312602667*^9, {3.6544275968873167`*^9, 3.654427688240391*^9}, {
   3.654427720959729*^9, 3.6544277992294493`*^9}, 3.654427836394781*^9, 
   3.654427866499819*^9, {3.65442792467592*^9, 3.6544280714460707`*^9}, {
   3.6544286709149303`*^9, 3.654428703040045*^9}, {3.654428756362895*^9, 
   3.654428759201425*^9}, {3.654429010964983*^9, 3.654429040963229*^9}, {
   3.654429113758327*^9, 3.654429152646302*^9}, {3.6595258239667892`*^9, 
   3.6595258243267612`*^9}, {3.661685080207913*^9, 3.661685102606616*^9}, {
   3.6848535344385767`*^9, 3.684853569957509*^9}, {3.803399148514113*^9, 
   3.8033991596620493`*^9}, {3.803399209070619*^9, 3.803399216328233*^9}, {
   3.8058342125215197`*^9, 3.8058342126966553`*^9}, {3.805840847084099*^9, 
   3.805840873337446*^9}},
 CellLabel->"In[41]:=",ExpressionUUID->"f22e041a-e479-43ac-b8ff-891e16779537"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coupled Sine and Cosine", "Subsection",
 CellChangeTimes->{{3.6564442703201733`*^9, 3.6564442722547703`*^9}, {
  3.656444968214149*^9, 
  3.65644497042837*^9}},ExpressionUUID->"26b68407-76c4-4bee-81bd-\
425209381ff5"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "CoupledSin", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CoupledSin", "[", 
   RowBox[{"x_", ",", "\[Kappa]_", ",", "\[Alpha]_", ",", "d_"}], "]"}], ":=",
   "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"CoupledSin", "[", 
    RowBox[{"x", ",", "\[Kappa]", ",", "\[Alpha]", ",", "d"}], "]"}], "=", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"CoupledExponential", "[", 
         RowBox[{
          RowBox[{"\[ImaginaryI]", " ", "x"}], ",", "\[Kappa]", ",", "d"}], 
         "]"}], 
        FractionBox[
         RowBox[{"-", "1"}], "\[Alpha]"]], "-", "\[IndentingNewLine]", 
       SuperscriptBox[
        RowBox[{"CoupledExponential", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "\[ImaginaryI]"}], " ", "x"}], ",", "\[Kappa]", ",", 
          "d"}], "]"}], 
        FractionBox[
         RowBox[{"-", "1"}], "\[Alpha]"]]}], ")"}], "/", "2"}], 
    "\[ImaginaryI]"}]}]}]}], "Input",
 CellChangeTimes->{{3.6564442845376873`*^9, 3.656444454303549*^9}, {
   3.684853582270767*^9, 3.684853626676385*^9}, 3.684853690689808*^9, {
   3.803400875499466*^9, 3.803400904390463*^9}, {3.803400945457807*^9, 
   3.8034009458148518`*^9}},
 CellLabel->"In[44]:=",ExpressionUUID->"31cf1cb8-4bc4-4203-952f-cb8f5f12a976"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ClearAll", "[", "CoupledCos", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CoupledCos", "[", 
   RowBox[{"x_", ",", "\[Kappa]_", ",", "\[Alpha]_", ",", "d_"}], "]"}], ":=",
   "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"CoupledCos", "[", 
    RowBox[{"x", ",", "\[Kappa]", ",", "\[Alpha]", ",", "d"}], "]"}], "=", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"CoupledExponential", "[", 
        RowBox[{
         RowBox[{"\[ImaginaryI]", " ", "x"}], ",", "\[Kappa]", ",", "d"}], 
        "]"}], 
       FractionBox[
        RowBox[{"-", "1"}], "\[Alpha]"]], "+", "\[IndentingNewLine]", 
      SuperscriptBox[
       RowBox[{"CoupledExponential", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "\[ImaginaryI]"}], " ", "x"}], ",", "\[Kappa]", ",", 
         "\[Alpha]", ",", "d"}], "]"}], 
       FractionBox[
        RowBox[{"-", "1"}], "\[Alpha]"]]}], ")"}], "/", "2"}]}]}]}], "Input",
 CellChangeTimes->{{3.656444981326223*^9, 3.656444997679358*^9}, {
  3.684853634939649*^9, 3.684853679695858*^9}, {3.8034009133808813`*^9, 
  3.8034009396098957`*^9}, {3.803400975082654*^9, 3.803400996544475*^9}},
 CellLabel->"In[46]:=",ExpressionUUID->"c72b9c84-5348-432a-a6fd-669df697aa3a"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Weighted Generalized Mean", "Subsection",
 CellChangeTimes->{{3.6820729359859333`*^9, 
  3.682072939728271*^9}},ExpressionUUID->"e09c2e2c-fa85-41ee-9c08-\
d97a81ec1b76"],

Cell[TextData[{
 "Computes the weighted generalized mean of a list.  For a 2-D list the \
weighted generalized mean of each column is computed.  The weights are \
normalized, thus weights equal to one correspond to the generalized mean\nY = \
",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"i", "=", "1"}], "N"], 
      RowBox[{
       FractionBox[
        SubscriptBox["W", "i"], 
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"i", "=", "1"}], "N"], 
         SubscriptBox["W", "i"]}]], 
       SubsuperscriptBox["X", "i", "p"]}]}], ")"}], 
    FractionBox["1", "p"]], TraditionalForm]],ExpressionUUID->
  "65dbf926-eb96-413e-b9fa-f5047ce2c68c"],
 "\n\nInputs:\nX - a one or two dimensional list of variables\nW - scalar, \
vector, or matrix of weights which modify the contribution of each input ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["X", "i"], TraditionalForm]],ExpressionUUID->
  "19e0d88f-6258-4171-8841-8502f3fc7a33"],
 "\np - scalar, power for the generalized mean\n    p = -1 is the harmonic \
mean\n    p =  0  is the geometric mean\n    p =  1 is the arithmetic mean\n  \
  p =  2 is the root mean square\n\nOutputs:\nY - scalar or vector of \
weighted generalized mean; for 2-D input mean of the first dimension is \
computed\n"
}], "Text",
 CellChangeTimes->{{3.682072954735381*^9, 3.682073060095602*^9}, {
  3.682073245071039*^9, 3.682073350081856*^9}, {3.6820733952455053`*^9, 
  3.6820734145527163`*^9}, {3.682073449809185*^9, 3.6820735181495333`*^9}, {
  3.682073670387249*^9, 3.682073853261612*^9}, {3.6820739092288303`*^9, 
  3.682073935322846*^9}, {3.798454217527864*^9, 
  3.79845423761854*^9}},ExpressionUUID->"1c76966d-6c46-41ac-8a26-\
4aa07aef9bfb"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ClearAll", "[", "WeightedGeneralizedMean", "]"}], ";", 
  RowBox[{
   RowBox[{"WeightedGeneralizedMean", "[", 
    RowBox[{"X_", ",", "p_", ",", 
     RowBox[{"W_", ":", "1"}]}], "]"}], " ", ":=", " ", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"WeightedGeneralizedMean", "[", 
     RowBox[{"X", ",", "p", ",", "W"}], "]"}], "=", "\[IndentingNewLine]", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "n", ",", "m", ",", "i", ",", "j", ",", "normW", ",", "sumW", ",", 
        "XArrayDepth", ",", "WArrayDepth", ",", "Y"}], "}"}], ",", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"XArrayDepth", " ", "=", " ", 
        RowBox[{"ArrayDepth", "[", "X", "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Which", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"XArrayDepth", " ", "\[Equal]", "1"}], ",", 
         RowBox[{
          RowBox[{"{", "n", "}"}], " ", "=", " ", 
          RowBox[{"Dimensions", "[", "X", "]"}]}], ",", "\[IndentingNewLine]", 
         RowBox[{"XArrayDepth", "\[Equal]", "2"}], ",", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"n", ",", "m"}], "}"}], " ", "=", " ", 
          RowBox[{"Dimensions", "[", "X", "]"}]}], ",", "\[IndentingNewLine]", 
         RowBox[{"XArrayDepth", ">", "2"}], ",", 
         RowBox[{
          RowBox[{"WeightedGeneralizedMean", "::", "argx"}], " ", "=", " ", 
          "\[IndentingNewLine]", 
          "\"\<Warning: Inputs X expected to have ArrayDepth of 2\>\""}]}], 
        "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
       "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Length", "[", "p", "]"}], ">", "0"}], ",", 
         RowBox[{
          RowBox[{"WeightedGeneralizedMean", "::", "argx"}], " ", "=", " ", 
          "\[IndentingNewLine]", 
          "\"\<Warning: Input p expected to be a scalar\>\""}]}], 
        "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Assert", "[", "True", "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "Check", " ", "dimensions", " ", "of", " ", "W", " ", "and", " ", 
         "initialize", " ", "normW", "\[IndentingNewLine]", "expand", " ", 
         "scalar", " ", "to", " ", "matrix", " ", "size", " ", "of", " ", "X",
          "\[IndentingNewLine]", "expand", " ", "column", " ", "vector", " ", 
         "into", " ", "n", " ", "rows", "\[IndentingNewLine]", "or", " ", 
         "set", " ", "message", " ", "warning", " ", "about", " ", "mismatch",
          " ", "in", " ", "size"}], "*)"}], "\[IndentingNewLine]", 
       RowBox[{"WArrayDepth", " ", "=", " ", 
        RowBox[{"ArrayDepth", "[", "W", "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"XArrayDepth", "\[Equal]", "1"}], ",", "\[IndentingNewLine]", 
         RowBox[{"Which", "[", 
          RowBox[{
           RowBox[{"WArrayDepth", "\[Equal]", "0"}], ",", 
           RowBox[{"normW", "=", 
            RowBox[{"Table", "[", 
             RowBox[{"W", ",", "n"}], "]"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{"WArrayDepth", "\[Equal]", "1"}], ",", 
           RowBox[{"normW", " ", "=", " ", "W"}], ",", "\[IndentingNewLine]", 
           RowBox[{"WArrayDepth", ">", "1"}], ",", 
           RowBox[{
            RowBox[{"WeightedGeneralizedMean", "::", "argx"}], " ", "=", " ", 
            "\[IndentingNewLine]", 
            "\"\<Warning: Input W expected to have same or less depth than \
X\>\""}]}], "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{
           RowBox[{"Else", " ", "if", " ", "XArrayDepth"}], " ", "\[Equal]", 
           " ", "2"}], " ", "*)"}], "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Which", "[", 
           RowBox[{
            RowBox[{"WArrayDepth", "\[Equal]", "0"}], ",", 
            RowBox[{"normW", "=", 
             RowBox[{"Table", "[", 
              RowBox[{"W", ",", 
               RowBox[{"{", "n", "}"}], ",", 
               RowBox[{"{", "m", "}"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
            RowBox[{"WArrayDepth", "\[Equal]", "1"}], ",", 
            RowBox[{"normW", " ", "=", " ", 
             RowBox[{
              RowBox[{"Table", "[", 
               RowBox[{"W", ",", "m"}], "]"}], "//", "Transpose"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"WArrayDepth", "\[Equal]", "2"}], ",", 
            RowBox[{"normW", "=", "W"}], ",", "\[IndentingNewLine]", 
            RowBox[{"WArrayDepth", ">", "2"}], ",", 
            RowBox[{
             RowBox[{"WeightedGeneralizedMean", "::", "argx"}], " ", "=", " ",
              "\[IndentingNewLine]", 
             "\"\<Warning: Input W expected to have ArrayDepth of 1 or \
2\>\""}]}], "\[IndentingNewLine]", "]"}], ";"}]}], "\[IndentingNewLine]", 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Dimensions", "[", "normW", "]"}], "\[NotEqual]", 
          RowBox[{"Dimensions", "[", "X", "]"}]}], ",", 
         RowBox[{
          RowBox[{"WeightedGeneralizedMean", "::", "argx"}], " ", "=", 
          "\[IndentingNewLine]", 
          "\"\<Dimensions of X and resized W do not match\>\""}]}], "]"}], 
       ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"XArrayDepth", "\[Equal]", "1"}], ",", 
         RowBox[{"Module", "[", 
          RowBox[{
           RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Assert", "[", "True", "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"sumW", " ", "=", " ", 
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "1"}], "n"], 
              SubscriptBox["normW", 
               RowBox[{
               "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}]]}]}], 
            ";", "\[IndentingNewLine]", 
            RowBox[{"normW", " ", "=", " ", 
             FractionBox["normW", "sumW"]}], ";", "\[IndentingNewLine]", 
            RowBox[{"Quiet", "[", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"p", "\[NotEqual]", "0"}], ",", "\[IndentingNewLine]", 
               RowBox[{"Check", "[", 
                RowBox[{
                 RowBox[{"Y", "=", 
                  RowBox[{
                   SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "1"}], "n"], 
                    RowBox[{
                    SubscriptBox["normW", 
                    RowBox[{
                    "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}]], 
                    SubsuperscriptBox["X", 
                    RowBox[{
                    "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}], 
                    "p"]}]}], ")"}], 
                    FractionBox["1", "p"]], "//", "N"}]}], ",", 
                 "\[IndentingNewLine]", 
                 RowBox[{"Y", "=", 
                  RowBox[{"If", "[", 
                   RowBox[{
                    RowBox[{"p", ">", "0"}], ",", 
                    RowBox[{"Max", "[", "X", "]"}], ",", 
                    RowBox[{"Min", "[", "X", "]"}]}], "]"}]}]}], 
                "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
               "\[IndentingNewLine]", 
               RowBox[{"Y", "=", 
                RowBox[{
                 RowBox[{
                  UnderoverscriptBox["\[Product]", 
                   RowBox[{"i", "=", "1"}], "n"], 
                  SubsuperscriptBox["X", 
                   RowBox[{
                   "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}], 
                   SubscriptBox["normW", 
                    RowBox[{
                    "\[LeftDoubleBracket]", "i", 
                    "\[RightDoubleBracket]"}]]]}], "//", "N"}]}]}], 
              "\[IndentingNewLine]", "]"}], "]"}]}]}], "\[IndentingNewLine]", 
          "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"Module", "[", 
          RowBox[{
           RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Y", " ", "=", " ", 
             RowBox[{"Table", "[", 
              RowBox[{"0", ",", "m"}], "]"}]}], ";", "\[IndentingNewLine]", 
            "\[IndentingNewLine]", 
            RowBox[{"(*", " ", 
             RowBox[{
             "Loop", " ", "column", " ", "and", " ", "rows", " ", "to", " ", 
              "compute", " ", "weighted", " ", "generalized", " ", "mean"}], 
             " ", "*)"}], "\[IndentingNewLine]", 
            RowBox[{"For", "[", 
             RowBox[{
              RowBox[{"j", " ", "=", " ", "1"}], ",", 
              RowBox[{"j", "\[LessEqual]", "m"}], ",", 
              RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"sumW", " ", "=", " ", 
                RowBox[{
                 UnderoverscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "1"}], "n"], 
                 SubscriptBox["normW", 
                  RowBox[{"\[LeftDoubleBracket]", 
                   RowBox[{"i", ",", "j"}], "\[RightDoubleBracket]"}]]}]}], 
               ";", "\[IndentingNewLine]", 
               RowBox[{
                SubscriptBox["normW", 
                 RowBox[{"\[LeftDoubleBracket]", 
                  RowBox[{"All", ",", "j"}], "\[RightDoubleBracket]"}]], " ", 
                "=", " ", 
                FractionBox[
                 SubscriptBox["normW", 
                  RowBox[{"\[LeftDoubleBracket]", 
                   RowBox[{"All", ",", "j"}], "\[RightDoubleBracket]"}]], 
                 "sumW"]}], ";", "\[IndentingNewLine]", 
               RowBox[{"Quiet", "[", 
                RowBox[{"If", "[", 
                 RowBox[{
                  RowBox[{"p", "\[NotEqual]", "0"}], ",", 
                  "\[IndentingNewLine]", 
                  RowBox[{"Check", "[", 
                   RowBox[{
                    RowBox[{
                    SubscriptBox["Y", 
                    RowBox[{
                    "\[LeftDoubleBracket]", "j", "\[RightDoubleBracket]"}]], 
                    "=", 
                    RowBox[{
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{
                    UnderoverscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "1"}], "n"], 
                    RowBox[{
                    SubscriptBox["normW", 
                    RowBox[{"\[LeftDoubleBracket]", 
                    RowBox[{"i", ",", "j"}], "\[RightDoubleBracket]"}]], 
                    SubsuperscriptBox["X", 
                    RowBox[{"\[LeftDoubleBracket]", 
                    RowBox[{"i", ",", "j"}], "\[RightDoubleBracket]"}], 
                    "p"]}]}], ")"}], 
                    FractionBox["1", "p"]], "//", "N"}]}], ",", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    SubscriptBox["Y", 
                    RowBox[{
                    "\[LeftDoubleBracket]", "j", "\[RightDoubleBracket]"}]], 
                    "=", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"p", ">", "0"}], ",", 
                    RowBox[{"Max", "[", 
                    SubscriptBox["X", 
                    RowBox[{"\[LeftDoubleBracket]", 
                    RowBox[{"All", ",", "j"}], "\[RightDoubleBracket]"}]], 
                    "]"}], ",", 
                    RowBox[{"Min", "[", 
                    SubscriptBox["X", 
                    RowBox[{"\[LeftDoubleBracket]", 
                    RowBox[{"All", ",", "j"}], "\[RightDoubleBracket]"}]], 
                    "]"}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}], ",", 
                  "\[IndentingNewLine]", 
                  RowBox[{
                   SubscriptBox["Y", 
                    RowBox[{
                    "\[LeftDoubleBracket]", "j", "\[RightDoubleBracket]"}]], 
                   "=", 
                   RowBox[{
                    RowBox[{
                    UnderoverscriptBox["\[Product]", 
                    RowBox[{"i", "=", "1"}], "n"], 
                    SubsuperscriptBox["X", 
                    RowBox[{"\[LeftDoubleBracket]", 
                    RowBox[{"i", ",", "j"}], "\[RightDoubleBracket]"}], 
                    SubscriptBox["normW", 
                    RowBox[{"\[LeftDoubleBracket]", 
                    RowBox[{"i", ",", "j"}], "\[RightDoubleBracket]"}]]]}], "//",
                     "N"}]}]}], "\[IndentingNewLine]", "]"}], "]"}]}]}], 
             "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", "]"}]}],
         "\[IndentingNewLine]", "]"}], ";", " ", "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"Print", "[", "sumW", "]"}], ";"}], "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"Assert", "[", "True", "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"N", "[", 
        RowBox[{"Y", ",", "5"}], "]"}]}]}], 
     RowBox[{"(*", " ", 
      RowBox[{"Set", " ", "output", " ", "of", " ", "module"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{"{", 
       RowBox[{"normW", ",", "XArrayDepth", ",", "WArrayDepth"}], "}"}], " ", 
      "*)"}], " ", 
     RowBox[{"(*", " ", 
      RowBox[{"Monitor", " ", "Variables"}], " ", "*)"}], 
     "\[IndentingNewLine]", "]"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.682073969782063*^9, 3.6820740188096647`*^9}, {
   3.682074050624607*^9, 3.682074084637199*^9}, {3.682074236724169*^9, 
   3.6820742403466*^9}, {3.682074299505542*^9, 3.682074380161516*^9}, {
   3.6820744293057117`*^9, 3.6820744483348618`*^9}, {3.682074679414274*^9, 
   3.6820747712744637`*^9}, {3.682074808242174*^9, 3.682074975179433*^9}, {
   3.682075051499032*^9, 3.682075209147509*^9}, {3.682075241073283*^9, 
   3.682075269380268*^9}, {3.682075421393964*^9, 3.6820754368641376`*^9}, {
   3.682075590025811*^9, 3.6820756000239058`*^9}, {3.6820757233141403`*^9, 
   3.6820758083729*^9}, {3.682075845215329*^9, 3.6820759155352287`*^9}, {
   3.682075965496529*^9, 3.6820760430532827`*^9}, {3.68207609355928*^9, 
   3.6820761725515947`*^9}, {3.6820762427563467`*^9, 3.682076323828973*^9}, {
   3.682076363171975*^9, 3.682076524942505*^9}, 3.682076567058272*^9, 
   3.682076785792582*^9, {3.6820768754085827`*^9, 3.682076891768167*^9}, {
   3.682076977122875*^9, 3.682077124903376*^9}, {3.682077259739594*^9, 
   3.682077317811062*^9}, {3.6820773906412067`*^9, 3.682077414958764*^9}, {
   3.682077497280077*^9, 3.6820775019056387`*^9}, {3.682077652306575*^9, 
   3.682077739206706*^9}, {3.6820779074166737`*^9, 3.682077954548904*^9}, {
   3.6820780681972027`*^9, 3.682078358857163*^9}, {3.6820917745120287`*^9, 
   3.68209183585417*^9}, {3.682091922662019*^9, 3.682091922978825*^9}, {
   3.682091971893577*^9, 3.682092029455851*^9}, {3.682092247940547*^9, 
   3.682092264020836*^9}, {3.682092616009431*^9, 3.682092668919636*^9}, {
   3.682092716958112*^9, 3.682092731132291*^9}, 3.682092844369768*^9, {
   3.682092984308284*^9, 3.682093035478931*^9}, {3.682093068965735*^9, 
   3.682093074250224*^9}, {3.682093119292329*^9, 3.682093120150098*^9}, {
   3.682093587822365*^9, 3.68209365369145*^9}, {3.682093683852354*^9, 
   3.6820936935753107`*^9}, {3.6820937359261923`*^9, 3.682093740457008*^9}, {
   3.682093815563542*^9, 3.682093819420195*^9}, {3.682093868763854*^9, 
   3.682093933946783*^9}, {3.682094802764517*^9, 3.682094803207081*^9}, {
   3.682094989208946*^9, 3.682095168506809*^9}, {3.682095213382718*^9, 
   3.682095310253001*^9}, {3.68209534352903*^9, 3.6820953448549433`*^9}, {
   3.6820953751690073`*^9, 3.682095376401803*^9}, {3.682095822897888*^9, 
   3.682095840277897*^9}, {3.68209587138122*^9, 3.682096062736731*^9}, {
   3.682096115872328*^9, 3.682096118531159*^9}, {3.682096232195483*^9, 
   3.682096354055832*^9}, {3.682096532511376*^9, 3.6820965857854853`*^9}, {
   3.682096785722357*^9, 3.682096790156042*^9}, {3.682096981259261*^9, 
   3.682096993284409*^9}, {3.682097218138431*^9, 3.682097222183494*^9}, {
   3.682097347378223*^9, 3.682097360107771*^9}, {3.682097394483705*^9, 
   3.6820973973799257`*^9}, {3.68209744090023*^9, 3.6820974487493553`*^9}, {
   3.682098102653763*^9, 3.682098140372251*^9}, {3.682098620329143*^9, 
   3.682098762583171*^9}, {3.682098822584041*^9, 3.6820988235122232`*^9}, {
   3.6820990875986767`*^9, 3.682099164962697*^9}, {3.682099349878027*^9, 
   3.682099370720602*^9}, {3.682099539343853*^9, 3.682099573331731*^9}, {
   3.682104912486621*^9, 3.682104966828895*^9}, {3.682105003223568*^9, 
   3.6821050052016907`*^9}, {3.682105104789999*^9, 3.682105112709539*^9}, {
   3.682105146289076*^9, 3.682105303312682*^9}, {3.682105377654875*^9, 
   3.6821054097185163`*^9}, {3.682105466314789*^9, 3.682105489242716*^9}, {
   3.6821055286667128`*^9, 3.682105550014287*^9}, {3.682105710136674*^9, 
   3.6821057528250933`*^9}, {3.682105813242679*^9, 3.682105816393845*^9}, {
   3.682106091155594*^9, 3.682106105717037*^9}, {3.68210630054673*^9, 
   3.6821063064551687`*^9}, {3.6821063406262293`*^9, 3.682106512724091*^9}, {
   3.682106601201786*^9, 3.682106601795141*^9}, {3.682106737107316*^9, 
   3.682106741107991*^9}, {3.682123324783671*^9, 3.682123407287929*^9}, {
   3.798466767721733*^9, 3.798466774439288*^9}, {3.7984669938397913`*^9, 
   3.7984670201428003`*^9}, {3.79846705258939*^9, 3.798467054262175*^9}, {
   3.7984671113217154`*^9, 3.798467112394794*^9}, {3.798472033333507*^9, 
   3.798472038597888*^9}, {3.798550332791583*^9, 3.7985503523296747`*^9}, {
   3.798552659854097*^9, 3.798552718257082*^9}, {3.7985529191274633`*^9, 
   3.798552919769569*^9}, {3.798553227192499*^9, 3.798553229354535*^9}, {
   3.7985533836502666`*^9, 3.79855338424641*^9}, {3.7985536608384323`*^9, 
   3.798553661890189*^9}, {3.798553922467361*^9, 3.798553924445911*^9}, {
   3.798554143297104*^9, 3.798554143940723*^9}, {3.798554248766634*^9, 
   3.798554257682262*^9}, {3.79855432216401*^9, 3.798554322652602*^9}, {
   3.798554353582111*^9, 3.798554383952227*^9}, {3.798554441133493*^9, 
   3.7985544509016943`*^9}, {3.7985547417044*^9, 3.798554742289309*^9}, {
   3.7985658560652943`*^9, 3.7985658565444937`*^9}, {3.7988377359493637`*^9, 
   3.7988377367351627`*^9}, {3.798876643710417*^9, 3.798876644174086*^9}, {
   3.798877165040564*^9, 3.798877186294641*^9}, {3.7988778283601418`*^9, 
   3.798877829126081*^9}, {3.7988809029204884`*^9, 3.798880911091716*^9}, {
   3.803401009353981*^9, 3.803401032473302*^9}},
 CellLabel->"In[48]:=",ExpressionUUID->"55ca3df7-2fed-4bf7-98b9-19885f0de6db"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Variable transformations", "Subsection",
 CellChangeTimes->{{3.804521001504775*^9, 
  3.804521007187131*^9}},ExpressionUUID->"e3a15860-d15d-4145-b873-\
f1ddc96e6fab"],

Cell["risk and coupling Domain Dual", "Text",
 CellChangeTimes->{{3.804521035273664*^9, 3.8045210420371017`*^9}, {
  3.804521079986062*^9, 3.804521094989695*^9}, {3.80452394076175*^9, 
  3.80452394258786*^9}},ExpressionUUID->"9879f4fc-392c-4fe8-b03c-\
f061ce040247"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"riskDomainDual", "[", 
    RowBox[{"r_", ",", 
     RowBox[{"\[Alpha]_", ":", "2"}], ",", 
     RowBox[{"d_", ":", "1"}]}], "]"}], ":=", 
   RowBox[{
    RowBox[{"riskDomainDual", "[", 
     RowBox[{"r", ",", "\[Alpha]", ",", "d"}], "]"}], "=", 
    "\[IndentingNewLine]", 
    FractionBox[
     RowBox[{
      RowBox[{"-", "\[Alpha]"}], " ", "r"}], 
     RowBox[{"\[Alpha]", "+", 
      RowBox[{"d", " ", "r"}]}]]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.804521106844378*^9, 3.80452114481575*^9}, {
   3.804521240637103*^9, 3.804521319882708*^9}, 3.804521437063973*^9},
 CellLabel->"In[49]:=",ExpressionUUID->"116b2078-36fd-4529-8669-1473540dd284"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"couplingDomainDual", "[", 
    RowBox[{"\[Kappa]_", ",", 
     RowBox[{"\[Alpha]_", ":", "2"}], ",", 
     RowBox[{"d_", ":", "1"}]}], "]"}], ":=", 
   RowBox[{
    RowBox[{"couplingDomainDual", "[", 
     RowBox[{"\[Kappa]", ",", "\[Alpha]", ",", "d"}], "]"}], "=", 
    "\[IndentingNewLine]", 
    FractionBox[
     RowBox[{"-", " ", "\[Kappa]"}], 
     RowBox[{"1", "+", 
      RowBox[{"d", " ", "\[Kappa]"}]}]]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.8045239531503563`*^9, 3.804523999107959*^9}, {
  3.8045243208251762`*^9, 3.8045243409457893`*^9}},
 CellLabel->"In[50]:=",ExpressionUUID->"bb387de4-96a7-4e6b-a1c8-6a46d51c1eb5"],

Cell["Coupling to Risk and Risk to Coupling", "Text",
 CellChangeTimes->{{3.8045216109065228`*^9, 3.8045216152359753`*^9}, {
  3.804521773288179*^9, 
  3.804521777655532*^9}},ExpressionUUID->"45937e69-274d-4a48-b114-\
dac0e622b99c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"couplingToRisk", "[", 
    RowBox[{"\[Kappa]_", ",", 
     RowBox[{"\[Alpha]_", ":", "2"}], ",", 
     RowBox[{"d_", ":", "1"}]}], "]"}], ":=", 
   RowBox[{
    RowBox[{"couplingToRisk", "[", 
     RowBox[{"\[Kappa]", ",", "\[Alpha]", ",", "d"}], "]"}], "=", 
    "\[IndentingNewLine]", 
    FractionBox[
     RowBox[{
      RowBox[{"-", "\[Alpha]"}], " ", "\[Kappa]"}], 
     RowBox[{"1", "+", 
      RowBox[{"d", " ", "\[Kappa]"}]}]]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.804521622308154*^9, 3.804521759667238*^9}, 
   3.8045218733979197`*^9},
 CellLabel->"In[51]:=",ExpressionUUID->"987dbcbd-0e6a-4d29-8779-e5d4ba452b51"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"riskToCoupling", "[", 
    RowBox[{"r_", ",", 
     RowBox[{"\[Alpha]_", ":", "2"}], ",", 
     RowBox[{"d_", ":", "1"}]}], "]"}], ":=", 
   RowBox[{
    RowBox[{"riskToCoupling", "[", 
     RowBox[{"r", ",", "\[Alpha]", ",", "d"}], "]"}], "=", 
    "\[IndentingNewLine]", 
    FractionBox[
     RowBox[{"-", "r"}], 
     RowBox[{
      RowBox[{"d", " ", "r"}], " ", "+", "\[Alpha]"}]]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.804521842728209*^9, 3.8045219189067183`*^9}},
 CellLabel->"In[52]:=",ExpressionUUID->"3cb7be96-60ac-4993-9dcc-1da19eacb641"],

Cell["RiskQDual", "Text",
 CellChangeTimes->{{3.804521981571062*^9, 
  3.804521992297504*^9}},ExpressionUUID->"71f7a60b-9f51-4359-ad8f-\
ebbd4deb0d2c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"riskQDual", "[", "rORq_", "]"}], ":=", 
   RowBox[{"1", "-", "rORq"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.804521995604246*^9, 3.804522033295027*^9}},
 CellLabel->"In[53]:=",ExpressionUUID->"f06e7f0b-78ef-4a83-9344-75a78fba327b"],

Cell["qToCoupling and couplingToq", "Text",
 CellChangeTimes->{{3.804522085908654*^9, 
  3.804522104900959*^9}},ExpressionUUID->"b85a4c4c-a6be-47df-9d34-\
523186868a57"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"couplingToq", "[", 
    RowBox[{"\[Kappa]_", ",", 
     RowBox[{"\[Alpha]_", ":", "2"}], ",", 
     RowBox[{"d_", ":", "1"}]}], "]"}], ":=", 
   RowBox[{
    RowBox[{"couplingToq", "[", 
     RowBox[{"\[Kappa]", ",", "\[Alpha]", ",", "d"}], "]"}], "=", 
    "\[IndentingNewLine]", 
    RowBox[{"1", "-", 
     RowBox[{"couplingToRisk", "[", 
      RowBox[{"\[Kappa]", ",", "\[Alpha]", ",", "d"}], "]"}]}]}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.804522126900489*^9, 3.804522196018564*^9}},
 CellLabel->"In[54]:=",ExpressionUUID->"d571b50f-69e9-4612-9763-438bea8a9459"],

Cell[BoxData[
 RowBox[{
  RowBox[{"qToCoupling", "[", 
   RowBox[{"q_", ",", 
    RowBox[{"\[Alpha]_", ":", "2"}], ",", 
    RowBox[{"d_", ":", "1"}]}], "]"}], ":=", 
  RowBox[{
   RowBox[{"qToCoupling", "[", 
    RowBox[{"q", ",", "\[Alpha]", ",", "d"}], "]"}], "=", 
   "\[IndentingNewLine]", 
   FractionBox[
    RowBox[{"-", 
     RowBox[{"(", 
      RowBox[{"1", "-", "q"}], ")"}]}], 
    RowBox[{"\[Alpha]", "+", 
     RowBox[{"d", 
      RowBox[{"(", 
       RowBox[{"1", "-", "q"}], ")"}]}]}]]}]}]], "Input",
 CellChangeTimes->{{3.8045221076240463`*^9, 3.804522111841148*^9}, {
  3.804522278302205*^9, 3.8045223273055553`*^9}, {3.8045223599705276`*^9, 
  3.804522383758575*^9}},
 CellLabel->"In[55]:=",ExpressionUUID->"a8abc6d5-ce6b-47c5-86af-dc3a0ed428f0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coupled Box-M\[UDoubleDot]ller Method", "Subsection",
 CellChangeTimes->{{3.805204185380233*^9, 3.805204195829665*^9}, {
  3.805204226499259*^9, 
  3.805204229978675*^9}},ExpressionUUID->"d5e17781-555a-4211-8c14-\
19b7d62f26ae"],

Cell[TextData[{
 "The coupled Box-Muller algorithm than has the following procedure.\n\n1)\t\
Draw two uniform random variables U1 and U2 over the range 0 to 1.\n2)\tApply \
the generalized transformation  ",
 Cell[BoxData[
  FormBox[
   TagBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["Z", "1"], ",", 
       SubscriptBox["Z", "2"]}], ")"}], "=", 
     RowBox[{"T", "(", 
      RowBox[{
       SubscriptBox["U", "1"], ",", 
       SubscriptBox["U", "2"]}], ")"}]}],
    "MathMLPresentationTag",
    AutoDelete->True], TraditionalForm]],ExpressionUUID->
  "e2e3d88e-c4a3-4970-a6f3-cae409a92cd4"],
 " where T is\n\t \t",
 Cell[BoxData[
  FormBox[GridBox[{
     {
      RowBox[{
       SubscriptBox["Z", "1"], "=", 
       RowBox[{
        SqrtBox[
         RowBox[{
          SubscriptBox["ln", "\[Kappa]"], "(", 
          SubsuperscriptBox["U", "1", 
           RowBox[{"-", "2"}]], ")"}]], 
        RowBox[{"cos", "(", 
         RowBox[{"2", "\[Pi]", 
          SubscriptBox["U", "2"]}], ")"}]}]}]},
     {
      RowBox[{
       SubscriptBox["Z", "2"], "=", 
       RowBox[{
        SqrtBox[
         RowBox[{
          SubscriptBox["ln", "\[Kappa]"], "(", 
          SubsuperscriptBox["U", "2", 
           RowBox[{"-", "2"}]], ")"}]], 
        RowBox[{
         RowBox[{"sin", "(", 
          RowBox[{"2", "\[Pi]", 
           SubscriptBox["U", "1"]}], ")"}], "."}]}]}]}
    },
    ColumnsEqual->True,
    RowsEqual->True], TraditionalForm]],ExpressionUUID->
  "e458df13-4334-4d71-a468-97538438a825"],
 "\t\nIn this case the dimension of the logarithm is zero and thus the \
variable U does not need to be raised to a power.\n3)\t",
 Cell[BoxData[
  FormBox[
   TagBox[
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["Z", "1"], ",", 
      SubscriptBox["Z", "2"]}], ")"}],
    "MathMLPresentationTag",
    AutoDelete->True], TraditionalForm]],ExpressionUUID->
  "7b9a6c59-2a8f-4bee-833b-3fb3392cf91b"],
 "  forms a two-dimensional coupled Gaussian  distribution with  ",
 Cell[BoxData[
  FormBox[
   TagBox[
    RowBox[{
     SubscriptBox["\[Mu]", "1"], "=", 
     RowBox[{
      SubscriptBox["\[Mu]", "2"], "=", "0"}]}],
    "MathMLPresentationTag",
    AutoDelete->True], TraditionalForm]],ExpressionUUID->
  "e39ee523-5685-44d4-b2c1-e8ed539e19ac"],
 " and ",
 Cell[BoxData[
  FormBox[
   TagBox[
    RowBox[{
     SuperscriptBox["\[CapitalSigma]", 
      RowBox[{"\[Minus]", "1"}]], "=", 
     RowBox[{"(", GridBox[{
        {"1", "0"},
        {"0", "1"}
       },
       ColumnsEqual->True,
       RowsEqual->True], ")"}]}],
    "MathMLPresentationTag",
    AutoDelete->True], TraditionalForm]],ExpressionUUID->
  "b5de4733-df46-41e9-bab8-1fc5f2adeefc"],
 ". For a set of independent random variables only one of the two variates \
should be selected, since although uncorrelated (no cross terms in the \
correlation matrix) the variates are dependent since the joint distribution \
cannot be factored. \n4)\tThe location and scale for the coupled Gaussian \
variates are added to Z by  ",
 Cell[BoxData[
  FormBox[
   TagBox[
    RowBox[{"X", "=", 
     RowBox[{"\[Mu]", "+", 
      RowBox[{"\[Sigma]", 
       RowBox[{"Z", "."}]}]}]}],
    "MathMLPresentationTag",
    AutoDelete->True], TraditionalForm]],ExpressionUUID->
  "91b70629-188f-46a4-bdcd-a326576b69e8"],
 "\n"
}], "Text",
 CellChangeTimes->{
  3.80520435338699*^9, 3.805204480093606*^9, {3.805204567794052*^9, 
   3.805204631486518*^9}, {3.805204670652411*^9, 3.8052047029977293`*^9}, {
   3.805222164552549*^9, 3.805222176309454*^9}, {3.8053972228630733`*^9, 
   3.805397226962572*^9}},ExpressionUUID->"4b2382db-ee22-4c7a-b87c-\
3f9bdcb20bec"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"Clear", "[", "CoupledVariate", "]"}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"CoupledVariate", "[", 
      RowBox[{
       RowBox[{"\[Mu]_", ":", "0"}], ",", 
       RowBox[{"\[Sigma]_", ":", "1"}], ",", "\[Kappa]_", ",", 
       RowBox[{"n_", ":", "1"}]}], "]"}], " ", ":=", 
     RowBox[{"Module", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"UniformVariates", "=", 
          RowBox[{"RandomReal", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"0", ",", "1"}], "}"}], ",", 
            RowBox[{"2", "n"}]}], "]"}]}], ",", "CoupledNormalVariates"}], 
        "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"CoupledNormalVariates", " ", "=", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"n", "\[Equal]", "1"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            SqrtBox[
             RowBox[{"CoupledLogarithm", "[", 
              RowBox[{
               SuperscriptBox[
                RowBox[{
                "UniformVariates", "\[LeftDoubleBracket]", "1", 
                 "\[RightDoubleBracket]"}], 
                RowBox[{"-", "2"}]], ",", "\[Kappa]", ",", "0"}], "]"}]], 
            RowBox[{"Cos", "[", 
             RowBox[{"2", "\[Pi]", " ", 
              RowBox[{"UniformVariates", "\[LeftDoubleBracket]", 
               RowBox[{"-", "1"}], "\[RightDoubleBracket]"}]}], "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"Table", "[", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{
              SqrtBox[
               RowBox[{"CoupledLogarithm", "[", 
                RowBox[{
                 SuperscriptBox[
                  RowBox[{
                  "UniformVariates", "\[LeftDoubleBracket]", "i", 
                   "\[RightDoubleBracket]"}], 
                  RowBox[{"-", "2"}]], ",", "\[Kappa]", ",", "0"}], "]"}]], 
              RowBox[{"Cos", "[", 
               RowBox[{"2", "\[Pi]", " ", 
                RowBox[{"UniformVariates", "\[LeftDoubleBracket]", 
                 RowBox[{"-", "i"}], "\[RightDoubleBracket]"}]}], "]"}]}], 
             ",", "\[IndentingNewLine]", 
             RowBox[{"{", 
              RowBox[{"i", ",", "n"}], "}"}]}], "\[IndentingNewLine]", 
            "]"}]}], "\[IndentingNewLine]", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"\[Mu]", "+", 
         RowBox[{"\[Sigma]", " ", "CoupledNormalVariates"}]}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.8052043823937263`*^9, 3.8052044574038477`*^9}, {
   3.805204729324924*^9, 3.8052052600398006`*^9}, {3.805205418220512*^9, 
   3.805205423430567*^9}, {3.8052074282124453`*^9, 3.8052074318611307`*^9}, {
   3.805207537678488*^9, 3.805207558315261*^9}, {3.805207713264778*^9, 
   3.805207722491312*^9}, {3.805208899308436*^9, 3.805208932857809*^9}, {
   3.8053972153781233`*^9, 3.8053972172538*^9}, {3.80544478466187*^9, 
   3.805444896392007*^9}, {3.805444940648394*^9, 3.805444964526825*^9}, {
   3.8054450043935957`*^9, 3.80544505802874*^9}, {3.805445396677753*^9, 
   3.805445430605041*^9}, {3.8054454634858217`*^9, 3.805445471197647*^9}, {
   3.80544552737258*^9, 3.8054455382247753`*^9}, {3.805445725738645*^9, 
   3.805445729509446*^9}, {3.8054457742150917`*^9, 3.805445788382329*^9}, {
   3.805445842267514*^9, 3.805445849874639*^9}, 3.805448261816383*^9, {
   3.805448403453761*^9, 3.8054484074685926`*^9}, 3.8054485309941874`*^9, {
   3.805448714040681*^9, 3.8054487403101587`*^9}, {3.805449755998056*^9, 
   3.805449793464038*^9}, 3.8054498754383163`*^9, {3.810239058085785*^9, 
   3.810239077512582*^9}, {3.8110993120899*^9, 3.81109931894904*^9}, {
   3.811099491731909*^9, 3.8110994936315928`*^9}},
 CellLabel->"In[56]:=",ExpressionUUID->"cf345c3e-cadc-4de8-a72d-425e5e2ddbab"]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1092, 855},
WindowMargins->{{-1, Automatic}, {Automatic, 38}},
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
Magnification:>1.25 Inherited,
FrontEndVersion->"12.1 for Mac OS X x86 (64-bit) (June 19, 2020)",
StyleDefinitions->"Default.nb",
ExpressionUUID->"3533444c-8fbf-416f-b34f-c60027422832"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 211, 4, 84, "Section",ExpressionUUID->"ccfb8823-5ee3-4dd1-a5ef-635366e928a6"],
Cell[793, 28, 1119, 16, 389, "Text",ExpressionUUID->"7c49f273-00ef-49a0-8d02-e2387935c131"],
Cell[1915, 46, 2666, 48, 995, "Text",ExpressionUUID->"3483005b-8452-4ec6-83c3-8e0bbc08b62e"],
Cell[CellGroupData[{
Cell[4606, 98, 169, 3, 67, "Subsection",ExpressionUUID->"81a76452-688a-45bd-98be-669c9ee70463"],
Cell[4778, 103, 1290, 35, 246, "Text",ExpressionUUID->"762dba98-5065-487c-b54b-83b044c6f343"],
Cell[6071, 140, 1581, 43, 276, "Input",ExpressionUUID->"b7abe739-d553-4e6f-95e7-79ac5a3e2431"],
Cell[7655, 185, 3137, 72, 400, "Input",ExpressionUUID->"d08c6e66-4bf1-46c3-8911-ba0a436bccc6"],
Cell[10795, 259, 467, 10, 37, "Input",ExpressionUUID->"38dda050-2638-4b48-a179-17895d521415"]
}, Open  ]],
Cell[CellGroupData[{
Cell[11299, 274, 167, 3, 67, "Subsection",ExpressionUUID->"ab563a47-0e25-473b-b61a-d8d924bb7bbe"],
Cell[11469, 279, 249, 6, 44, "Text",ExpressionUUID->"e3c25309-77e2-4d04-85d4-66096212348e"],
Cell[11721, 287, 437, 9, 131, "Text",ExpressionUUID->"47a8764e-ea62-489d-9821-5795550c582b"],
Cell[12161, 298, 1468, 35, 328, "Input",ExpressionUUID->"d35506b4-a030-4b74-b3b7-261dac74a4d0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[13666, 338, 233, 4, 67, "Subsection",ExpressionUUID->"d85a8cb4-825c-4ccf-b02b-a3f67211e406"],
Cell[13902, 344, 3291, 73, 483, "Input",ExpressionUUID->"9ee0c0bd-5fb8-4431-bc94-d7134679b664"],
Cell[CellGroupData[{
Cell[17218, 421, 188, 3, 56, "Subsubsection",ExpressionUUID->"6e0767ec-c365-4754-9597-3f2c5735ddba"],
Cell[17409, 426, 3275, 87, 148, "Input",ExpressionUUID->"5eb84abf-0c0f-479f-a442-af94720a7842"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[20733, 519, 251, 4, 67, "Subsection",ExpressionUUID->"0717bcc1-739c-46c7-b7a7-1938deecae14"],
Cell[20987, 525, 241, 6, 44, "Text",ExpressionUUID->"8f12ac37-5cdf-480c-99e7-0f871ab636ab"],
Cell[21231, 533, 2917, 58, 321, "Input",ExpressionUUID->"f5403d12-bf81-40dd-9bf3-674539af6199"]
}, Open  ]],
Cell[CellGroupData[{
Cell[24185, 596, 203, 3, 67, "Subsection",ExpressionUUID->"0b940d3e-f6af-42d9-9ba2-2bf8c398d37f"],
Cell[24391, 601, 664, 12, 73, "Text",ExpressionUUID->"9afd2663-16bd-4567-81ac-874a6041877b"],
Cell[25058, 615, 6931, 162, 966, "Input",ExpressionUUID->"4f1d6f34-6fd4-4aad-8018-ed8e3ed20179"],
Cell[CellGroupData[{
Cell[32014, 781, 212, 4, 56, "Subsubsection",ExpressionUUID->"90912a30-441e-48ab-8f81-2d559c243f1c"],
Cell[32229, 787, 221, 5, 44, "Text",ExpressionUUID->"3a201432-a434-435f-bfb3-be089adbc95c"],
Cell[32453, 794, 5182, 129, 546, "Input",ExpressionUUID->"16f9a46b-7ab5-4777-946d-4c70022e4dc6"],
Cell[37638, 925, 1591, 35, 162, "Input",ExpressionUUID->"5b5bc67f-e221-4b47-b2f2-9c9ecc94524a"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[39278, 966, 171, 3, 67, "Subsection",ExpressionUUID->"3e656e5f-d42d-48f2-ba1f-c05bb1602876"],
Cell[39452, 971, 2904, 71, 365, "Text",ExpressionUUID->"55225ce6-d41c-4d0e-af97-deef655d0531"],
Cell[42359, 1044, 4286, 96, 536, "Input",ExpressionUUID->"dc186a1f-396b-4d46-99cb-cc457ed4fa5e"]
}, Open  ]],
Cell[CellGroupData[{
Cell[46682, 1145, 228, 4, 67, "Subsection",ExpressionUUID->"18fbf3bb-8224-4a63-a177-76d5b9207ab2"],
Cell[46913, 1151, 3265, 75, 371, "Text",ExpressionUUID->"6531acc4-6650-4879-9d2f-e32794415adc"],
Cell[50181, 1228, 4872, 110, 587, "Input",ExpressionUUID->"c1a0aeea-7d79-4ede-afb8-0d6d3d23ca3f"]
}, Open  ]],
Cell[CellGroupData[{
Cell[55090, 1343, 165, 3, 67, "Subsection",ExpressionUUID->"ce1512d8-343d-466d-9637-377e32d05b40"],
Cell[55258, 1348, 1127, 23, 389, "Text",ExpressionUUID->"8f9c0919-0f86-4960-8fde-cd67499a95b6"],
Cell[56388, 1373, 2005, 38, 142, "Input",ExpressionUUID->"b037b0fd-c1db-4a66-a215-76eb107c050b"],
Cell[58396, 1413, 1091, 26, 142, "Input",ExpressionUUID->"8fafd6e7-8e5d-49c6-987d-f5ecdb1297bc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[59524, 1444, 219, 4, 67, "Subsection",ExpressionUUID->"b75b8941-2946-4cd9-987a-42ed41a0df40"],
Cell[59746, 1450, 1097, 23, 361, "Text",ExpressionUUID->"5abc315b-145c-4e81-957a-34d53ccb6a8f"],
Cell[60846, 1475, 3486, 72, 253, "Input",ExpressionUUID->"f9544062-c5df-48e4-94e8-fbfe4a775110"],
Cell[64335, 1549, 2259, 57, 208, "Input",ExpressionUUID->"7ced798f-0812-42c2-ae7d-00272667bc97"]
}, Open  ]],
Cell[CellGroupData[{
Cell[66631, 1611, 262, 4, 67, "Subsection",ExpressionUUID->"cf0fcfbb-6853-48d9-a469-c2af898a7ae4"],
Cell[66896, 1617, 1888, 39, 648, "Text",ExpressionUUID->"ec0d2064-6b4a-4c80-afc1-db7fb0f32afa"],
Cell[68787, 1658, 2296, 44, 168, "Input",ExpressionUUID->"9ab3cca5-b43c-494c-8f1f-1425eecd59a4"],
Cell[71086, 1704, 1265, 30, 142, "Input",ExpressionUUID->"7279d7fd-bc51-4f12-b53b-41fb687a0edf"]
}, Open  ]],
Cell[CellGroupData[{
Cell[72388, 1739, 163, 3, 67, "Subsection",ExpressionUUID->"f44e42f8-8535-4fca-8e3d-c9fd8b9d88e5"],
Cell[72554, 1744, 4347, 98, 785, "Text",ExpressionUUID->"0d93f37a-5794-4851-8816-9124fbda03af"],
Cell[76904, 1844, 2417, 59, 326, "Input",ExpressionUUID->"95a5b6d3-6977-4786-aff3-3d3964c92d48"],
Cell[79324, 1905, 4337, 112, 477, "Input",ExpressionUUID->"ece73d97-070f-410e-9a67-74ef68e6d087"]
}, Open  ]],
Cell[CellGroupData[{
Cell[83698, 2022, 159, 3, 67, "Subsection",ExpressionUUID->"0510a17d-c063-4252-b6e7-945be4e6c701"],
Cell[83860, 2027, 862, 17, 159, "Text",ExpressionUUID->"ca149bd8-6cac-4796-a680-7ec6921a70fe"],
Cell[84725, 2046, 735, 15, 188, "Text",ExpressionUUID->"4c988f34-0a91-4b99-97d0-dcb036e0b13f"],
Cell[85463, 2063, 242, 3, 37, "Input",ExpressionUUID->"d73cc257-4767-4e60-8674-37a5f3c34176"],
Cell[85708, 2068, 3670, 87, 366, "Input",ExpressionUUID->"f22e041a-e479-43ac-b8ff-891e16779537"]
}, Open  ]],
Cell[CellGroupData[{
Cell[89415, 2160, 223, 4, 67, "Subsection",ExpressionUUID->"26b68407-76c4-4bee-81bd-425209381ff5"],
Cell[89641, 2166, 1377, 35, 183, "Input",ExpressionUUID->"31cf1cb8-4bc4-4203-952f-cb8f5f12a976"],
Cell[91021, 2203, 1312, 32, 183, "Input",ExpressionUUID->"c72b9c84-5348-432a-a6fd-669df697aa3a"]
}, Open  ]],
Cell[CellGroupData[{
Cell[92370, 2240, 175, 3, 67, "Subsection",ExpressionUUID->"e09c2e2c-fa85-41ee-9c08-d97a81ec1b76"],
Cell[92548, 2245, 1791, 40, 501, "Text",ExpressionUUID->"1c76966d-6c46-41ac-8a26-4aa07aef9bfb"],
Cell[94342, 2287, 18821, 365, 2284, "Input",ExpressionUUID->"55ca3df7-2fed-4bf7-98b9-19885f0de6db"]
}, Open  ]],
Cell[CellGroupData[{
Cell[113200, 2657, 172, 3, 67, "Subsection",ExpressionUUID->"e3a15860-d15d-4145-b873-f1ddc96e6fab"],
Cell[113375, 2662, 266, 4, 44, "Text",ExpressionUUID->"9879f4fc-392c-4fe8-b03c-f061ce040247"],
Cell[113644, 2668, 699, 18, 83, "Input",ExpressionUUID->"116b2078-36fd-4529-8669-1473540dd284"],
Cell[114346, 2688, 683, 17, 80, "Input",ExpressionUUID->"bb387de4-96a7-4e6b-a1c8-6a46d51c1eb5"],
Cell[115032, 2707, 232, 4, 44, "Text",ExpressionUUID->"45937e69-274d-4a48-b114-dac0e622b99c"],
Cell[115267, 2713, 677, 18, 82, "Input",ExpressionUUID->"987dbcbd-0e6a-4d29-8779-e5d4ba452b51"],
Cell[115947, 2733, 601, 16, 83, "Input",ExpressionUUID->"3cb7be96-60ac-4993-9dcc-1da19eacb641"],
Cell[116551, 2751, 151, 3, 44, "Text",ExpressionUUID->"71f7a60b-9f51-4359-ad8f-ebbd4deb0d2c"],
Cell[116705, 2756, 281, 6, 37, "Input",ExpressionUUID->"f06e7f0b-78ef-4a83-9344-75a78fba327b"],
Cell[116989, 2764, 169, 3, 44, "Text",ExpressionUUID->"b85a4c4c-a6be-47df-9d34-523186868a57"],
Cell[117161, 2769, 621, 16, 64, "Input",ExpressionUUID->"d571b50f-69e9-4612-9763-438bea8a9459"],
Cell[117785, 2787, 765, 21, 88, "Input",ExpressionUUID->"a8abc6d5-ce6b-47c5-86af-dc3a0ed428f0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[118587, 2813, 234, 4, 67, "Subsection",ExpressionUUID->"d5e17781-555a-4211-8c14-19b7d62f26ae"],
Cell[118824, 2819, 3648, 112, 402, "Text",ExpressionUUID->"4b2382db-ee22-4c7a-b87c-3f9bdcb20bec"],
Cell[122475, 2933, 3981, 83, 453, "Input",ExpressionUUID->"cf345c3e-cadc-4de8-a72d-425e5e2ddbab"]
}, Open  ]]
}, Open  ]]
}
]
*)

