(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 10.2' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[    965663,      17896]
NotebookOptionsPosition[    959837,      17786]
NotebookOutlinePosition[    960332,      17804]
CellTagsIndexPosition[    960289,      17801]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Coupled Exponentials & Logarithms", "Title",
 CellChangeTimes->{{3.6608478800698853`*^9, 
  3.660847887053892*^9}},ExpressionUUID->"a118a5e0-a5c9-4e86-a00f-\
f7d4c46520e7"],

Cell[TextData[StyleBox["\[Copyright]  Copyright 2020 Kenric Nelson\n\n   \
Licensed under the Apache License, Version 2.0 (the \
\[OpenCurlyDoubleQuote]License\[CloseCurlyDoubleQuote]);\nyou may not use \
this file except in compliance with the License.\nYou may obtain a copy of \
the License at\n\n    http://www.apache.org/licenses/LICENSE-2.0\n\nUnless \
required by applicable law or agreed to in writing, software\ndistributed \
under the License is distributed on an \[OpenCurlyDoubleQuote]AS IS\
\[CloseCurlyDoubleQuote] BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY \
KIND, either express or implied.\nSee the License for the specific language \
governing permissions and\nlimitations under the License.",
 FontSlant->"Italic"]], "Text",
 CellChangeTimes->{
  3.805811481661235*^9},ExpressionUUID->"2df1e6b4-9101-433a-b5b1-\
aeba2eb8a8d9"],

Cell[CellGroupData[{

Cell["Graphic of Coupled Exponential", "Section",
 CellChangeTimes->{{3.660847907598814*^9, 
  3.6608479159407578`*^9}},ExpressionUUID->"c4ec21ec-5129-4d93-8bf7-\
20c01238b74b"],

Cell["\<\
Graph shows Coupled Exponential decay using the inverse of the \
CoupledExponential Function with coupling \[Kappa] values with \
compact-support (-0.5 to -0.1), exponential (0), and heavy-tail (0.1 to 0.5).\
\
\>", "Text",
 CellChangeTimes->{{3.6608488536133842`*^9, 3.660848904489455*^9}, {
  3.828023155426556*^9, 
  3.828023301563216*^9}},ExpressionUUID->"19e09968-b236-472a-becb-\
a9a83dd0169f"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"CouplingValues", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.5"}], ",", 
     RowBox[{"-", "0.4"}], ",", 
     RowBox[{"-", "0.3"}], ",", 
     RowBox[{"-", "0.2"}], ",", 
     RowBox[{"-", "0.1"}], ",", "0", ",", " ", "0.1", ",", "0.2", ",", "0.3", 
     ",", "0.4", ",", "0.5"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"CoupledExponential", "[", 
        RowBox[{"x", ",", "#"}], "]"}], 
       RowBox[{"-", "1"}]], "&"}], "/@", "CouplingValues"}], "//", 
    "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "1"}], ",", "10"}], "}"}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", "CouplingValues"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "Automatic"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.660848202137882*^9, 3.660848336304208*^9}, {
   3.6608484932843*^9, 3.660848503163884*^9}, {3.66084853360492*^9, 
   3.6608485750391417`*^9}, {3.660848653015645*^9, 3.660848654413473*^9}, {
   3.660848702363208*^9, 3.660848721178146*^9}, {3.660851721290873*^9, 
   3.660851758123169*^9}, {3.660851846593731*^9, 3.6608518562892523`*^9}, {
   3.6848555593077507`*^9, 3.684855567999827*^9}, {3.6848557165987997`*^9, 
   3.684855728986269*^9}, {3.6848559685199127`*^9, 3.684855986909089*^9}, 
   3.8211073175305862`*^9, {3.821107367457425*^9, 3.821107390383051*^9}, {
   3.821107430301713*^9, 3.821107445858255*^9}, {3.821107506129792*^9, 
   3.821107540365074*^9}},
 CellLabel->"In[60]:=",ExpressionUUID->"f464a9ac-8fef-4c2f-86fe-a490d2ae34f6"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwBYQKe/SFib1JlAgAAACUAAAACAAAAfH6jSAGLk7/9RpECFifwPyXfe7eK
gr0/DkKIVNcn7j9Cr7AixULVPzDUU7dOr+o/dK+6cTsn4j9GqCJHYuzmP5bl
8Av1iek/NY0HegU74z+SFfT0zTbwP9zUFxZkkt8/ZHbsNgXz8z84EyeS9RnY
P+xexRqzb/c/KEJ1ypkg0T+bNohb0dr6P5Ql35G6lMQ/8jTXY6fp+j84LKNw
YlnEP0gzJmx9+Po/4DJnTwoexD/2L8R8KRb7PyhA7wxap8M/UikAnoFR+z+4
Wv+H+bnCPwoceOAxyPs/2I8ffjjfwD94AWhlkrX8P0H0v9RsU7o/z/+2bWjE
/D+IAUiSvNy5Pyb+BXY+0/w/0Q7QTwxmuT/U+qOG6vD8P2Ap4MqreLg/L/Tf
p0Is/T+IXgDB6p22P+bmV+ryov0/0MhArWjosj9VzEdvU5D+P7A6gwvJ+qY/
ykweUiue/j9gMxveSh2mP0DN9DQDrP4/ACyzsMw/pT8szqH6ssf+P0Ad41XQ
hKM/AtD7hRL//j/g/0Kg1w6gP3hQ0mjqDP8/APG15bJinj/u0KhLwhr/P0Di
5Yq2p5w/2dFVEXI2/z/gxEXVvTGZP7DTr5zRbf8/AIoFasxFkj8mVIZ/qXv/
P0B7NQ/QipA/nNRcYoGJ/z8A2cpop5+NP4fVCSgxpf8/QJ6K/bWzhj/8VeAK
CbP/PwCB6ke9PYM/cta27eDA/z8Ax5QkiY9/P17XY7OQ3P8/AFEUTqa3cT/U
VzqWaOr/PwAsqMVpl2U/rA+N9aHx/z8AqODlFLxcPyNHMf8=
          "]], 
         LineBox[CompressedData["
1:eJxdz30s1AEcx/G7i+iO3N2PRHTn6jB3RJ6tkjysO21iooaG5WhSyKJFRefU
OrSuFEJ1lY0lZtbtt8n9rLpktlqxuGo6Wq3Wecg8dFfKat/v1nv77LvXn1+3
zGMJWQwajRb7ZyvXqyTUcosVbSftX8Pi5+xJNpqdF8h44YhWfGN9bBWiKe/i
EZUYbc4z6M74owuNmrbkCHTaVFa+VRo6YKbPJLuCNswd57qvooPrfSoH7zHR
e3Pq5B5c9CO9Zt7TDd3Q/3NMFI5OuFSl9juFpkSNAdu/o29m9iclTjDAf7MA
z+gqB0gS7VoQWp1auBocdzI3zDfOGpz6pDok3IYJbm9bCX0034d/1sgC2xNR
QUs6W/DVJUnT/XI7cA3v07ykkw3eceiWbqyWA9ZsrntglcsF99hLlUP7if/+
sQe3uJ/gdAWhbXIyLnp7OIAjhVUuMcQ6cLaFMKl0Cr2WqJDHvncEqyOTU0J6
14Pp0zIJdc0JbBgMFE/IncFdA6UPBZkbwIyFimBTvAs4IoaePuPnCpZyyJZz
thvBI6ytrztM6DhPQdfCKA8sJHOoI1l8sHXzu552Pdpwueb0ksQNXMZroVQU
2q7365ttfgKw4vqwqbsZ3cfi1Rc7bAIba+8wpXJ09NsUQ4gCfcOTID3Oo6VU
+WFLJbp19uAzrQqdvs9JEaxGv3KuoQv70V7Z0aPEY3RFt7mT8RTtuyc3fXwA
rSzb3df4Eh0zzijjfkA3iclE+gR6rqRAPD2Jvs0Z1w99Ri/v6g27MI1Oqi3i
lsyiO/SiL7I5dGpRQ33UIrpbG1/g/wPNtF0jEZjRGQe0fM4vtOZu8eLyMvo3
/UnxoQ==
          "]]}, Annotation[#, "Charting`Private`Tag$3780#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVxXs0lHkYAOD5vt86arS7paVWrSOFOiGhUsr76pyQLicqmUKFie5o0qSo
3MZMpmjlEIZSU7lEI1Qa1zQMMhLbDdHOllwzROO2u38851nkfdKVTTMYDOf/
/L/7LaaA2yAsm3SIp3VNBsGo41yxIFwK42HFHycb+2BjXr5h9cYWqHVuZuW2
dgMn72mgrUkX+M+R9rDLlZCoHNQa4XfD0avbOvb86IRrOsPDDasHgYyWcQbd
28A3uHe1hkoFihPWffeKWqHXq1qv4OQoREbq3mr8uQlSVsfsa3qnhvzuGt/1
f9TAkPVfPne71WCTqUOfHpaBk5XxsdAxNfSe1Sh6XieD7yuen1+mOw62aUcS
74fIwHUZQ3TJZRxyevgPBG9fAHMh9+PKmnHwq4m0KkqpBi7lz75WOAGPWLMv
zrOsgt31jid3xk2BjcTry7zdUpBv1XddLpqCM7Wqw5m/SQEbhq1JzhTkJ4Of
1+tnYPryploimwJ9noa3z65nQCvGo7Snp6CzRXNTrlsJ5DfnpTYdnwaIjVl7
0PcJRPdpKe8aMfD6JTPBgTuFUGF2pvVPUwautee5Gx8thInjXbILVgxcUDK/
xGBlIQT1P87aY8/AyLVZEwXSR+A5wA7Q9GTg7RnQwHpbANbfysYPJTAwI0S4
s1FPAl3Dp7SNCYXMF7U8xdVcSDaPqhMzKTRrupxfYpALO/wTI020KbR7Yuhh
JcmB0vePvy9dRGHst+uz6lqz4Ubl5LvlQGGcj3pxvnEWuMbxMleeozCJmXZv
slMMM+VJHpJwCo08okVa58VQTrJ0rAQULrHVq5DoisH8TD3POplCvw2UtWLb
HWB6zTm2pphCjjKC5V2VCRXLU6w3qCjcxe/MkNVnAJed0ydVU5hcrSztCMgA
83Sp2I6m8R8jRddNnQxI1f44H+fQmLgJL7pWioD7Y8nkxhU0msT5z33FSAUL
2YNqp6M0pim47A8VifCZUR5WG0Rjyy/+JqkGiSBa17TGOYTGh6O2Ix0Xr8Os
B6r7W/g0lmen59EOCfAlwebK9rs0/u0Y8nnxp3jI8K502/WJRh8fC07EWwGs
ChAOPvxKY8Fp/bGzCj7IQ90Fvw7R+HpRoGRAHgMjSf1SOYMgVjS95sujwblR
z8hen+ANki3uHwmHjg/KUpERQXn2TJkLMxw4X/PdJ0wJrgvcWXzH8BKkazjG
FtsS1Igwy2goDYNh21MqMxbBMkeLE/26XOBvthNePkBw6uWYDzMiGPT3zDTp
9iPY8JMoe/8YBzYHpe+9HUzwzd6cpNTpQGi/cGSYCiPowR3Kpa8FAEe46sr+
KILqLAEbPU+A6F5dhV4CwVftfWGhvx4Gq6LEfdwUgt9/P/fmE/8Q1FQdHGm5
RVAzXWzJW+ALqvbRpfEPCVoONAvPz94PMb2Vlf2PCQYp5k5PyPeCvlrosbWc
oG+HqWXtMzdw0lkSp9lI0I1Vvz5PvB3aDAeWsVsJTqY51EgYThBk8bSqso2g
eXCbKKDHHmbYRXkaKAm6CIlc2mMDaVt2jIb2EtzNaQ+IbTYHS9aC+PcqgtEL
XYY4DvrQynuhVowR7N9xs6pZqAX/ArVWNDw=
          "]], 
         LineBox[CompressedData["
1:eJxNz2tIkwEUBuBtaF6m2TY1NW1zuTZslnbROcwuzumWsNZFC6WmNLUWlSJp
oJVraeHcQG1lbY5aUSQ5TaQxEP0ib8R+RGaZFVOLosCmSLo2sgQ5pxcOh4f3
x+HEFp09oKSQSKTsf7O846tSfbf4+ewmraRen7Uop6GjTfQmawQ6y/Zo4BQX
bZwZ5U+K0Ele82B6DnowQKUwHkTPxpFb8orQ4vwEj+PSf/3Q1RG7HW0wtOxv
JdBCpeVt5TBaQyG+bB9Dh+/0kq2z6LSn5UILzxf/Mx5vbzCgY8pSGwvKV4Fl
F1TCRJk/uGCgUbArKBDc/ng56DPnNrMuz1DBoQxRsnsoGHzDLTE9qQ0B65hf
f0k614DTT9wdeq+ngW1xBqufig7uCZVqHUcY4JUrYPPG87SuZHRQaWFDAjcM
nMGpjxYzwsElPpzc6p/o1Qy1Zt+ntWBLRl6+oDcCTHYVS4ibkeCplzv405oo
cNdI9TN20TowZUGd4pFHg/eIyYrZpBiwlGY3XwleDx6jbh3t8KBlPHbXwjgT
zLGXEqeVLLB/28ee9gn0VJPuolsSC65hmolmAh3S++NdWhIbXHfrjae7Dd1H
ZbZWhm0Az+jvB0o16MwP+VOCOrSRx7Bzr6GlRO1JXy364dyx4f5mtOJwZF2K
Bf06SkfmPEfHl2SOM16g1d3eTsogOjFHpXCOoLU12X13XqHFTkoNfRJt4tsP
kafR81VlfNdn9D2ac8LxDb20t1d43YXO1VfQq+bQHRObvhfPowsqbreKFtHd
/fKybb/RgcEBErYXXXi0n0X7g7Y9qFxcWkL/BSBa6sg=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$3780#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVkHk41Ikfx+cwZua7IkSt7Y5yFrIkmc8nXSjryrmOopEoQoSiFBLZJcdG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          "]], 
         LineBox[CompressedData["
1:eJxFz2tI02EUBvBtTLJd0jltOrXN5VJriXbxUhJ52XAzCKWsMGyKU8uoBpZ+
0Mgpy3Bs4GhpNldZDLKcY0g2GO5foS4RikxcK11z2SjQKZazzVaBvOfAw8Pv
08NJqLxcIiHgcDjBv/zvKY7bYDeSjuI2b1o20aF8AZaFXFAxMHBqe6j6/jjY
QdJrUt6CD4S7tYfnwF+Z5U/PbYA7tQGDh0hGPsLuMUkpYA13xtwaCy5MK7Hp
s8FP+IKFpavgS1dS2TcWwb5ROXHhJ7g1fs5TtAHueqMyMigU5FeJy3mGFHC0
3SSZrQJjudn9OQ5wJL0gY32Minx7Xah91hKGrGR9+yUcDIf/qh6MfVTRkIcT
NYYtdRHIQ5EixeRpOvLmCrJu1zWaMQNMqa3o2JsUhZzPvRknoG9HriFyS5uW
wNvosraiWQZyX/6psixLNDLeWy3E7sQguyYO8ubbmMhGW9NzTmUsMmFNlukv
jkPOFeDFy+nxyCKaWddK3YE8Td43NeAHH0/mGNfsLGSuuRa7KGEjh/Z+Hup3
gF2dyuvrwgTkZpYOU2PgMMuPmZx0DrK864Pf1AseIbO6G6J2Ii+qHpFEbWD+
pzJXlhx8L5luTmoHi7CW8yEKsH6lfNyqBotPxsgz+8DvmUo89yV4dw3fTn8N
lpkCg4RRcNqxOrHTBlY0F470vAMLnITmiC9gLc98Aj8PXm2U8rxu8EOa0zHp
AQfzLIduecGlqvqIxhXwgGPP9+pV8Nn6u90FPrDJWizd/xtMom4VcgLgijNW
Nu0PePhxgy8YBP8FfK3qrw==
          "]]}, 
        Annotation[#, "Charting`Private`Tag$3780#3"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.922526, 0.385626, 0.209179], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVk3k01Q8Th+3Ltd17v0ikVRKyRDfrd4aEUkgqFWkhsmaNECmSkCIJWUtC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          "]], 
         LineBox[CompressedData["
1:eJxTTMoPSmViYGCwBmIQLbpnTtN0ZhEHBij4Y5R5ax4ngu/8uDZDjBfB75o8
6VuvAIJ/yXl5C6sIgp+05MKCL1II/qqQp3o5cgj+R5Zfex4rIvgNqco3L2kg
+PPVygQ3miH4Sy7HMhdbIZnX4PrVxA7B33ZL5OYOVwT/TN/mBQdCEPxf3z7o
XyhC8MOOZ/u9X4fg82QkduuqiyL8q9ou4yYsBuens6iG1bxH8PmEm1q874nD
+Yudw6Mt9krA+Ywf0jwPTpeE8x+dNtV53CIF5288WbNdKUkazmf63mT+O1AG
znd0Y0z4aCgL53sJ7prfzCsH51/jNrqy7jeC76+htPH7TXk4X3VXxsGcVAU4
n2Pe3a2rbyP4jyb11f30VITza+XnH5x8EMHn3/v6ho2hEpzfNuPq783zEPz9
3PIzy0WV4fx3/Uu4vFoQfNc70Y8s2hD8ORrCu9Q7EHyvg42ZrD0I/vJPcScO
TEbwE0Il28wXI/iXpfoYVQ8h+FrprjeFjyD4TZv/bGA6huAb+GQnPDiJ4PfU
euyffRHBd3vAVCv0EMGfq7MrhPExgv+lolDnwxMEf5Hgg9tnXyD4/532WnV+
QPDD+kuEKj4h+Otua79K+4Lgx5TMmunyA8HffCCw0PgXgs/Fy+mp9AfBT4w8
oCD4D8HfsbT8x///CD4AQkjIOQ==
          "]]}, 
        Annotation[#, "Charting`Private`Tag$3780#4"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.528488, 0.470624, 0.701351], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVV3c4V+8btvf82PszjJIUJetzzvskuxQqI6RkRVFCZCYk2SIksxQyopDs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          "]]}, 
        Annotation[#, "Charting`Private`Tag$3780#5"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.772079, 0.431554, 0.102387], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVjXc4Fgwbxe09H3s/wyYje9R9p8hOwkspOyIjURSSJEoR5VVWUfFqWJWS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          "]]}, Annotation[#, "Charting`Private`Tag$3780#6"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.363898, 0.618501, 0.782349], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVknc4FQ4Xx+10bS5SitxQiKRE6BxCKJWRUqGMyKiIIpSdGW6SZEUo/IxQ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          "]]}, 
        Annotation[#, "Charting`Private`Tag$3780#7"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[1, 0.75, 0], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVxXc8FPwDB3B3hM4qh3bGoaxHJJXw+aYI1RMK1yDKKr+WUhSFiKRy9FRK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          "]]}, 
        Annotation[#, "Charting`Private`Tag$3780#8"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.647624, 0.37816, 0.614037], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVk2k4FHoDxY29Ia61chMaW5a0qJuk80+R6EaLNoVqLKkUoRKFkJKEUvZu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          "]]}, 
        Annotation[#, "Charting`Private`Tag$3780#9"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.571589, 0.586483, 0.], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVknc4FfoDh51jj4ZZVJccOxWFUvL5UlxUyJVRUslqICpxjXDtllGnpERC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          "]]}, 
        Annotation[#, "Charting`Private`Tag$3780#10"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.915, 0.3325, 0.2125], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVkmk4FGoDhs2QtZL1yMmSCSmOkhTR86Y4cYpTp0QLKlvqJAZRtIhUUqhk
i6RSqWaGZCkx09hTyL5lCdl3Y+jw9f14rvvHfV33r2flMY+9zlQBAYGCX/s/
41N+qJa998v/zzySKq85igLJ55GRVOv8g92636sWjWL/kL7vwIIz/nmexpb/
NoLdku7PTNr9YWabpdWSPgKLh8Fsl8owKBxosTp0aATFP6SszpckIKibmSDP
HIZwuO3dkB2vEFoSpdntOgT1b+ezbgTlIZTS+N1h8QASZkvaXjp+QL3Y0Y/r
B/ohKy8v/GVrPoKECioNy/oRpGH74npgAR4tss/vv9YPzyPjMxWGHBypGVzO
Fe2HdblGtE16ITzveVzkS/VBIvV2pWvKJxRLzBoV/dmLIn2D737SFZi6pNsR
p92LIE4L7/rlCnRlTeTFSvVitlVL5aXDZ1xt9a2RbemBj3inwcySSlw8lc3g
efXAaVtAiHFZFQwvs821n3XDlMFUKzSthcaOn+v3//EdApyMvokLtchKrgxf
kPuO/Jq3TLV3v7zHC37lf10wnn2/9dKGOrCWqZX2VnRhk1npwS2r6hF4jdnd
4tEF7eaOKNaiRtxRj7F5nNsJOREZwcSSFvzlrxDFPdGBmuXyZZ+EWsHXbfcr
2t+BKO3lkXOkFQylieGebR1YtldFxS6nFT6OFg+DFDsg/mDtFtkXbQhxy6lX
rmjHxeWPDAvN2iGh426cu6kd3oxczy2anYgXHRJWN23Drms026IdneClLKu9
v7IN6kdvbt1zrBMB3322G1LaUCdtL+H6oBMXPLa9EWa3YrMv5UmUTBfWjelN
cExb8dN4Z0PfQhdmouuvnLdqQUhZ7daYhm5s4vw+3xncBPuUreq06W6sfh86
dPdEEwwCUiVey/RANdFh1ymrJvzQ8W/4aNWDA9m3bW8oNMEycoXXCLcHwktl
SwNeN2Kp7fEn5qxe+EqfjFvU0YDo7lGJqet96F2Zy5NxqMdLjlvqlSd9OMJ8
sCTcoh6cpHZTaXYfnBNOhKnq12PErtJfd6YPrtdMlaLF6rHzM6P3hEs/ljaW
ISuzDrNvT39sMx3AvnBeLEO6DkeuDZ4rmRuE2Uum29m2Gng7O8kf+G0I3Q5L
/p0sr8EN0xZWt94QpDPqrENzavD2Z/kPqvsQQjM7dIfv1mCpZ5qNScMQXkbF
tb/fVYMCO3e99DfD4Ny/NWJa8BVRcpOTFQajMBWU+EZ7Ww053lzVB7NRfNs9
6VH6uBoxDYIMxr5RxOiFegTfqUZSvMyJSK9R6PMyDYw8q/Fy5YbWfa9HEbbr
k02yTjWKdLwKm9XHYBQ2t9b0RRXmzEbv9cmOg7O697efrEpc0JzxaqKNo/6i
QZVMSiWoYgJ/l+uNQ2+OucPkbiVEP0mKv/p7HHZnlo5xfSshv1f3wpmb41BO
67n+2LgS6+xPu/AEJ6B3b2Sn0v0vcPIdNFg0MYHc9T46Ff0VEG46P8wTmIRx
sO4SdnUFnplIPO1fOolS8+y0z7kVGBRaI1e5ZhLhhgXxBmEV8LnjOhF3bBJB
856RG3UqcJXZyVhXPYmECM2qap9PeNbfsPowawobHDWWvlIoh6WVW7tV/hQ0
tiySjBMsxyCLd39bxRQC4uu+JwyXYZ2/vIhm3xSsk4MTu7hlyBHZ1z2mOv3r
H3MmXl5lKFv1JTk0YhoRn01tA6pKMWhfqJjhwUMdMzMtPLEEf7mX3HEI5IEa
PTZuf7MEL3zKFy8O4yE6xFXJ9FwJXG9WzTs95cEhx4hrYlOCzpzWTvlWHoRt
f5sVkSxBg+z0i3OWM5B0zvCqvVIMbpnGFlN1PvIiz2oV+ReBVqf1ZliPD0Zi
a2zaiSIEdWjrxBM+jGvm3R7aFQEzeiqTh/j4V8GsKcuwCLnqEHwaxcfDNXuX
JswWgnnpQLmYwCwO1d7asuZiIeINrh2qaprFtsaPz3qiuBjXrz+e2jeL487j
Pg5XuNi5QeNU4Mwsnt66+62XzsW0LjdAS34OF2sHDDX3c7FXSyDx8p45XIo5
MHREgQvxFX7t60vm0DP8riAu+SP8KG7OUZk/8UrljMXXDxzs//Snxz8R80Dv
yKkEVTbKdinvXZs4j8Y2h1FTOTZIxaS+4Mt5MIre+k2IsaH9OXk2vXgelPCZ
k14TBaBWzoVIL8xj5pN0Wk9RAZhfGQlV/y6gWSxSO8mjAFeHJLpT1QXIZJau
1rKIfHRO0qU1BCnk4MZTOjvU8xD7R0j5U3EK2f3jn4l/pfLwt1t0sKY0hRz/
EZ75/L/3+NCcPb16JYUsz3nQalP3HnGc/5rWgkJSylnXQq69x96I0JT15ynk
YoCxCG30Hdhr4/VNJihE2dGkpbE8Fw+PcWz2dVHJqypmjAMrGxvPhI+y+qnk
peIxL05yNsoCbW9IjlOJsYAlXS8qG1Mxw3llAoKkZY6eYUjPhuUXRfVtyoLk
I+G/6tqYjckt9AkdO0Gie4ZfGpuXhZ1yqyJEvgiSbE+FkQ11bzFWHFKamytE
zMW21wZoZSI6+u7fsWwhYiDb8DpZKRNGzin1Z0uESKGF6LEvUpkIprK79euE
iHjWNn2z2TeQN/lJYYwJkavrrFNiy9/AON3LKGX1IuK0f21Km8cbhCY4pIVF
LyI2/auYjz5kQMnTMPywlzAx59qtP0dPx1bD4eYv/sIkNLNnD9ctHfaUlDXb
LwuTpwv8XHn7dDyMXFyiFSFMaFrvzGt2pmNV+jcq75UwiRhod2Epp2PNRIhf
ZJ8w+fOAKX+wnAUDn+rjRY4iRGbMlaGvzYK1/0mjddai5Glx16vzVCYuUZ18
V9mIknMFgTnukwwwww6nKxwRJfTe1oC+LgaWJe5eQzkpStxtTy8R+8hAJUdX
sfqqKMnd/9Hsy2UG9i6e5NM/iBJzjlOGB4UBm6SAnCwdMTJdfCcwX/Q1DheG
b8ZicTIT/DyIufElTp/5Q/XSsARZtu+p/8S7Z5CV2WHAL15CUqdO/6Xa/AT3
+BYPXl2WJPUnbc4cuZ2CWyq90xbMZeSojdqJ0JPJ2OqUXNx0W4pcHDQIKz+e
hOxV0QyRk9JkyQDNMcojAZmyljcrbGUIx9rDhf0oFkkavlIsA1mikbjDfrft
fSx2OxqmoylHSjtpMfOr72G7eugKcxl5kvQiOb3U7A5chdRtAkbkicpA4lR4
ZiSWygQF/9X2GxldKzzN14tAyvYDhzbnKZASNZ5PdEc4KKMuFuz7y4nG8wze
iudh6CzfqN0VrEhONdq93XzhOlilAVlqx34nEvdYe++nhYLKC9o0t2cFSbU7
YpXaGYJt5hTHsfVKRDXeujbMMRiWUrlJV5Yok80ZqXnpIldQJ6FX83pOmexq
DDuolXEZ1qvVWLxGFXLgkKHh8chLUM91Y59yViUm+7i1G7ovQDSxNTOtWZUY
lRfrW3cFoDPq1gW+xUqyacGt1GrteQSqJLHvsFcSlydPmoq9/SGZN9BgvF6N
1Aw6aYv9PIurMbVzGYlq5A0nL8n+qS/yJVRiz8rRyIsfV5TY1j4Yvv1Y3DKY
RjxvbJvia3rDrOVQ5+arNJLxOOE4leaNhNUyuZrXaOTg10f0xcresGRfPrHo
Jo38LiLoqy7rjdRx+5KCOzSS4+w65i7gDcf9y69uSqGRJMuzf2xvpOOr4i2K
OodGlDIPrFt1k441rmaNMlwaiZvYdMshlI6gjJ9MatGvnsOUX0IQHet2nXRs
L6WRefkr44r+dNwM3JkfX0Uj2U8OOq1xocO8nRoo3UEjeVEXsqIIHQ+0c/dR
umik5sGfMT1b6Jj089Qe/U4jcn7zc1s20fFIqr254scvv51rMaBDx4JpntH1
URrxck8dOvQ7HTa3vaX9xmmkTKKm5p08Ha+b1/a7TNKI+zkZIyVpOg57x8Xu
mKGRw5XHAr6L0pFRsMdzwyyN6J+DraUQHeJLxCzUftKIrG8Jg7XghaN2BapS
8zTyuM0pSHHOC9lPzs4sLPzqKQR8CZ72wv8AoBbSAA==
          "]]}, 
        Annotation[#, "Charting`Private`Tag$3780#11"]& ], {}}}, {}}, {
    DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
     AxesOrigin -> {0, 0}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> {FontFamily -> "Arial"}, DisplayFunction :> Identity, 
     Frame -> {{False, False}, {False, False}}, 
     FrameLabel -> {{None, None}, {None, None}}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None,
        "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, 
     PlotRange -> {{-0.9999997755102041, 9.999999775510204}, {0., 
      1.0095424747746045`}}, PlotRangeClipping -> True, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{
       RowBox[{"-", "0.5`"}], 
       RowBox[{"-", "0.4`"}], 
       RowBox[{"-", "0.3`"}], 
       RowBox[{"-", "0.2`"}], 
       RowBox[{"-", "0.1`"}], "0", "0.1`", "0.2`", "0.3`", "0.4`", "0.5`"}, 
      "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #5}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #6}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.363898, 0.618501, 0.782349], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.363898, 0.618501, 0.782349], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #7}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[1, 0.75, 0], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[1, 0.75, 0], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #8}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"], 
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.647624, 0.37816, 0.614037], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.647624, 0.37816, 0.614037], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #9}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.571589, 0.586483, 0.], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.571589, 0.586483, 0.], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #10}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.915, 0.3325, 0.2125], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.915, 0.3325, 0.2125], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #11}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.560181`", ",", "0.691569`", ",", "0.194885`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6150173333333333, 0.25708400000000003`, 
                    0.13945266666666667`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.922526`", ",", "0.385626`", ",", "0.209179`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.922526, 0.385626, 0.209179]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.922526, 0.385626, 0.209179], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.3523253333333333, 0.3137493333333333, 
                    0.46756733333333333`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.528488`", ",", "0.470624`", ",", "0.701351`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.528488, 0.470624, 0.701351]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.528488, 0.470624, 0.701351], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.5147193333333333, 0.28770266666666666`, 
                    0.06825800000000001], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.772079`", ",", "0.431554`", ",", "0.102387`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.772079, 0.431554, 0.102387]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.772079, 0.431554, 0.102387], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.363898, 0.618501, 0.782349], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[0.24259866666666668`, 0.412334, 0.521566], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.363898`", ",", "0.618501`", ",", "0.782349`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.363898, 0.618501, 0.782349]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.363898, 0.618501, 0.782349], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[1, 0.75, 0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> RGBColor[0.6666666666666666, 0.5, 0.], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"1", ",", "0.75`", ",", "0"}], "]"}], NumberMarks -> 
                    False]], Appearance -> None, BaseStyle -> {}, 
                    BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                    ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[1, 0.75, 0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[1, 0.75, 0], Editable -> False, Selectable -> 
                    False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.647624, 0.37816, 0.614037], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.4317493333333333, 0.2521066666666667, 
                    0.40935800000000006`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.647624`", ",", "0.37816`", ",", "0.614037`"}], 
                    "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.647624, 0.37816, 0.614037]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.647624, 0.37816, 0.614037], Editable -> False, 
                    Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.571589, 0.586483, 0.], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[0.38105933333333336`, 0.39098866666666665`, 0.], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.571589`", ",", "0.586483`", ",", "0.`"}], 
                    "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.571589, 0.586483, 0.]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.571589, 0.586483, 0.], Editable -> False, 
                    Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.915, 0.3325, 0.2125], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6100000000000001, 0.22166666666666668`, 
                    0.14166666666666666`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.915`", ",", "0.3325`", ",", "0.2125`"}], "]"}],
                     NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.915, 0.3325, 0.2125]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.915, 0.3325, 0.2125], Editable -> False, 
                    Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             
             RowBox[{#, ",", #2, ",", #3, ",", #4, ",", #5, ",", #6, ",", #7, 
               ",", #8, ",", #9, ",", #10, ",", #11}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.684854244983327*^9, 3.684855570385227*^9, 3.684855730529784*^9, {
   3.6848559727069387`*^9, 3.684855988067931*^9}, 3.800107820745634*^9, 
   3.821107326156857*^9, {3.821107370228712*^9, 3.821107391758911*^9}, {
   3.821107435790167*^9, 3.8211074472046556`*^9}, {3.82110751229922*^9, 
   3.821107543687853*^9}, 3.82802331874142*^9},
 CellLabel->"Out[61]=",ExpressionUUID->"8c42d79d-7e69-4608-a328-b6f5bd4aeb57"]
}, Open  ]],

Cell[TextData[{
 "Graph shows Coupled Exponential over a broad range of coupling \[Kappa] and \
variable ",
 StyleBox["x ",
  FontSlant->"Italic"],
 "values."
}], "Text",
 CellChangeTimes->{{3.6608488536133842`*^9, 3.660848904489455*^9}, {
  3.828023155426556*^9, 3.828023301563216*^9}, {3.828023341673676*^9, 
  3.8280233682937117`*^9}},ExpressionUUID->"ba0f4fb5-ab15-4edb-b55f-\
92c2211dc171"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"CouplingValues", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "2.5"}], ",", 
     RowBox[{"-", "1.5"}], ",", 
     RowBox[{"-", "0.5"}], ",", "0", ",", " ", "0.5", ",", "1.5", ",", 
     "2.5"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"CoupledExponential", "[", 
       RowBox[{"x", ",", "#"}], "]"}], "&"}], "/@", "CouplingValues"}], "//", 
    "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "10"}], ",", "10"}], "}"}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", "CouplingValues"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "Automatic"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.660848202137882*^9, 3.660848336304208*^9}, {
   3.6608484932843*^9, 3.660848503163884*^9}, {3.66084853360492*^9, 
   3.6608485750391417`*^9}, {3.660848653015645*^9, 3.660848654413473*^9}, {
   3.660848702363208*^9, 3.660848721178146*^9}, {3.660851721290873*^9, 
   3.660851758123169*^9}, {3.660851846593731*^9, 3.6608518562892523`*^9}, {
   3.6848555593077507`*^9, 3.684855567999827*^9}, {3.6848557165987997`*^9, 
   3.684855728986269*^9}, {3.6848559685199127`*^9, 3.684855986909089*^9}, 
   3.8211073175305862`*^9, {3.821107367457425*^9, 3.821107390383051*^9}, {
   3.821107430301713*^9, 3.821107445858255*^9}, {3.821107506129792*^9, 
   3.821107540365074*^9}, {3.828023379954266*^9, 3.828023416528722*^9}, {
   3.828023466031395*^9, 3.828023516289062*^9}},
 CellLabel->"In[64]:=",ExpressionUUID->"836c154e-d64c-4d99-9bfe-fce529c935df"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwlxQkwlH0cwPFFaksl11rsK1mlIhUiVL+/KypLiIpCLHllXIsO6u3OyLyV
tqZdo/d1dKxCx7uU4nlyrVthsVZuuY9d12Yd7/WZ+c53k3+4W6AsgUBw/ad/
f3mAJl5epuIypeUdoYiCfsSRZoQLVNyKlsr8vp+CnDd0zuX/pOJ1sgJdZysK
0rGIWAyfouLEYJ8KIzMKKk5MJnb9oOI6GVyzcQMKIhq1aOO1VDxh7S5WoDoF
MaP9jlxNoeLJ8jcszMa10CsZRibBnIpn5z9dOpWiheQDHYw1Tuvic9zgwNh+
TaTSWhaa/mATLqgoa3+9VxO5lbADtlXp4K66dvSTSRoobmJ/6P3ejXjafUe1
sVoy+nCeHZLroo1P6JvJ5ZHJyDnZQ2eJRcHzwmqsLI+qI4fUQmfHUU08ecfl
6vHbJPScd7GZ4a2BV1oyWIJyNeRzk6lW/1Idz3I2lo8SqaIrfnI5CioknOJr
fHVeTxURdmzEbG+p4vj7TLvNx1VQ213WnVd8ZVxK5+XpRyojyppgzrSNEu6W
KPnEfqiEzu3ZLi+kKOJsrrR8hL0BBZh3vCwpUcAtWiqVxywU0fYLlsthmUQ8
8gT/EdFpHUr77l560VoeH0qfLUrSVEAD19T2OZBl8cmuX49mioiI8D/89Rbr
R+xzq1DBo632g56LWA1/yHSUsRLtGhJM70ydxfpD63RJX+WQXYFw9aLnJHa8
/a935toyyOM//djOGXOXZHwBIji5EiWVXixLQ4dG4khh1/qwGzZdPdiLaOvt
rN+lIIoyXM/I7sHUVESyCtFSiNrHoTY59GDiQZ+1nw5IIaY+3flxfDcWfp1J
dWych6lhg0N7ajoxKy6cFUl+Apv1LwHWsZPmoo0kcMcpJu/FWgHGyHpy2Yos
ASJJXE180Yr9VH0bbCSdgzWckdkqYQtW9Eeuby0+B4r1HU4u9s2Y/SZlh8HD
c0DWKpMcJzdiOQNlcq5es+AZRE1riK/E+m51vE4KmIbESEZCMrcCO3NPe/d7
22kojC8JdxvnYfo5IuMP1GnQexhwoMG3HPOuiv3o3TsFYixD+M2mBONfKfSf
95sCbRNHw3SlAozXO5NB8hDD5BMWXcSKwrqONfCatSYhXpZZdnDvSejv01NU
HpsAI3YAT9XsN1jttZwikz4B7iatn29bMyFH/Ozx+TUTEBl0bqiU+RxcJW+C
I/ljQCBwlwOK8qGt5E/fiqMjwBpS3d3n9BFEbR1vQxaHwbgx2p8uLIDGgVqx
L2cY6M9Ny+iSQniQYCkyWh4C+havMmONYrgb1nXqS9og8GoEiSbuFeBqyWFl
fO6H4AttPG+DSgjRLqYpu/cDUU+44qZcFZxNIC1vHOyDw3HtV5reVwNOizRt
WtcHNds6o2JU62H5XdzKEPseCOV35qaO1kPIE5s9aXXdcBtFzPr7f4WbC36v
ijy6QX80wuSu5TeYJm5d/8vpLgi2jcoWDjfAgdr9wtbH7bCXJa6e8GyE4mtL
rLrLQiBORA3LlTTChSNMf5XTbcBhM/QNU5pA87qpuoVMK1yanLJHq/jQZ2f9
Rr60GQ4fjKYfY/DBsEKF9v0SH4ZF0WnxR5phSWkwy6ywAQocZrD7+c3wpcg4
O8n2GySmxnRkUlug/g0thhBUD15TMwsf7rUA1/fMrWcKNWBwKFarVtoCHitG
Y+OCKmDh6axF99lWEJCVL648WQo107EnZhpbYSHX9ti8GwZXczzMKLsFsFku
cSTVhwt/A7OgiZo=
          "]], 
         LineBox[CompressedData["
1:eJxTTMoPSmViYGCwBmIQvaXl7h8xplv2DFBg8+37ZkMJBJ/Hp+KSawuCv/Z7
BX9ezG04/51fVec+nrtw/o66ffvWmz2A8610D24KmP4Yzje9cm3avpMIvkH1
myqdPwi+2klxZ47EJ3C+cGre5f1aT+H8UDBA8N/Mkf6it+8ZnD+fp8yU9+lL
OJ+pTPROiuQHOJ+dJ6Ax6d03OL/5wTXnosV/4XwocIAxNsxuC6j7xAjn5y9K
EXgSxgrnx2XNkm1v5IDzOwPLLI3ncMP5LCJTn9X/4oPzr59u/dDCLwjnh1Yn
mc06LATn53C/VjoXJQLny68+No3tlSicX73DOZh/tjicX7AsZ5uZviScz8Nk
nqT3VArOd80Nmao1WQbOl9gpm++qIgfnTyrqmGhwXR7OD2rz1527TQHO14pu
CV1SqwjnX3vGuWhyqBKcfy7pqaOQvDKc/5Bl+Y+pSgj+l2UZ6yXVEHzJ169l
FHUR/JTiT9/1rRH8X83/1/qGI/hqSyWlOvsRfEv32xd4JiP4Pi/ntE+YhuAX
6ip8nT4Xwd+zVe3CstUIftAx47YjxxH81IyvNh6nEfwKru2fT59D8Of6WSVe
vorgv7jmYPPoMZJ7K5g+pz5H8Hmlj6x8+QrBN4p3F//4EcF3ZeQ8V/IVwY9Y
fKrlxw8EP8u1x7rmD4Jf+9z30///CD4AB/XP9g==
          "]]}, 
        Annotation[#, "Charting`Private`Tag$4912#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwl0ns01AkfBvAxhjMui1KM1n2kXMImqU37/WUqVtHSGrUVIlayGbd1Gax2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          "]], 
         LineBox[CompressedData["
1:eJxN0GtMEgAQB3BgaCGY5XSGjxBofLChlWWZbtqIKJeGGFa09SDYzNkoasQG
5pyEudyyDNoQbKlh5dJmZbaxtkzJsjHWA3t9KIl3NVR8zHJWq+76b7e733Yf
bseUKkRyEoFA4P2q391dNBN9qcFTQPibKElPDWUATSAOHT4VQX/vfCfwcLzg
6ZLwqrK96C/mlEjmQ7SPt+aNOYweC261xbB94LcblTpvPdr+ajihtdQPvkJT
rY/1BMCmO+fomqQg2Ci5Ou/fjm68/nxosBut3sIu16pDYJHWqQpRv4JJqsQP
MnoYLDl480X/OnRvUcFTmhAtZRzpu6tHDwzbmsgRdF2KjN/hGAeTH/V2u+sm
wYtowlrpt2nc/+jiKdvn8d9/UvhvuN2iF56eIIIVbbKln8ujwPsrTWn1tYvB
DaWq3GwzFUxOMHhr5paAR0fOhHVxy8BijTTH9DgeXEUNsRySBDCjy26MDiaC
Nf28sriWJPAxa1VfThYdTCNtkGZ6ksH8o7sMGc2p4OUP0hT8lSvAF5VnL6we
ZYBF+p1cS186OGOfTtxRzQS7vJS2ZjEL7JB6Nscz2OBP5M5ZAwsdsVb00Dlo
eiiUyuSiZScmZrLy0HN1C7eKd6M51+jJDefRuYL3TlozekfAXN9kRB/npk9d
tqBt9zhOaxdaZM/WDz5Byyum8reNoNUx9ydHHGhLyaZDL1+j/a7C/DH3f/eq
SZNyHzo2ZfBGIIhee0CQND6O5hMpjpNT6D3tz3Szs+hKfmOe9ge62lc8sbCA
/gl82fXt
          "]]}, Annotation[#, "Charting`Private`Tag$4912#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwl04c/1f3/x3HbCWUmZEYpJGVk1eudkZIkO5UGyoriIisR1aWhkpkkDSvj
CkVy+XyMQ0YZUY49D47ROdZxcM75Xd/b7367PW/Pv+ChdNnf1oOLg4Nj7r/9
729Nnlxgs5Xxuf7oSw4ORCCHSy73bSjjFw5bXFV0JoK1yBC9nKGMP425sHvW
hQiKBteZ/ovKeFpY8cDdi0SofZBAGCYr4/sGx2VrfYhA0Pwtj39XxvVLxzRd
7xAh8a+LJ6LSlXFLr6Olu/4hwgfOwHccB5XxZd4qe4EtDcDrYXFA+vwOvJC4
J/jPeAOI9xB93zxTwmcev5gnVzWCbd0Ltz3NivjrxKSHea+/QfifQ75PxxTw
E1fqokgfmqDi5gvv4lPy+NNz9dphlc1gneCgyEqTxSmpNoUyay1gkfGv9bFZ
GVwtn3fE2fY7ZDeG/go8K42XRCWZFpf8ANfYxK1tudvwN8YRd0U02iHyIneR
oLgk7uhrqWvC1wEcexUw07sSeAgnV3zpYgf0Pky7/6FbDO/R6PH3k/gJsgKe
eUsmovij52X8aWZd4KOrxtsnK4wHn8kU0qrqBreDg7l1dYL4ZGt1LMnjN6iF
GLL93hFw2b2WiXwHSIAPWKuNJRPwm5dtDvgbksDR1M3BKY6AX4gl+A+YkODO
lkcfwI+AR1B39TXbkYD0bsBJRJ+A61nS5ChBJPi7I+qfjy38+OynMQ/4SgKy
WuOlpQU+PNyl3MvfsheyBuzqQ4/w4quCmbcdIvrgwPiBtS4dXvyY3s2sgrt9
UE8R1dLazYt3q72UF3jSB2R6Wzp5Cy8eyeg068nqAzUxq0C7fh58pnEro6ax
D0qOmu3Ye5MH32ZOGWqX6If6Yp2o4QJuvKKMI2FzWT9MRm81tpDiwj9aKU58
IAyCwK3iP88FuHAdNexkrcQgaIYcfzu8wYkn1AsHjSoOQrDfLYGwEU78c7zw
PmQwCHxnJ3oK8jnxkHw8M8B7EFR0PgWJHeLEP7CuaFb/GITLZPvigUsceK2v
mKRl5hBw/D+cVFw+I8U1DJVJu82nHJnYQeng5NyPw0A4O+nGaczEshz3ybMq
hsFRMfuOjBITm++9LuOCD8NCvjJuNbOBabbXJOxsG4Y9uLzRx9sb2Oz45SDB
uWFIpUjsD8tdx85Wa7bKqI1AEHDKCa0xMPbnR/5xeSNQy4Mb7RxiYGp3bS2x
jyMg0hzpcriegRW01DZvfBmBAvuNlOvxDGyCcm8yvnkExr3oYt07GNhGuHmN
yOwI2CbOEV6dWMX042WwRK1R0JomLe3LWMGKnBU/UbFR+Iu6d8+22BUs6WCL
R8K3UaigR59nea9gItcyhYw7RuEIn1pDq/4KdsVhwzNndBRslcNSvLqWsWFG
UvIQ7xgEnZcxfCu4jLlj5/uGTo5BZYfLbcmwRSyiVdI5aHwMWD1FpcyLi9ir
9+RfGvNjYDLMNTVusYjJJob2zNHHoGU+36Z06yI2HLiW/khwHPoF15VsPi5g
tsL0ksPa48A2T6+Pm6JhctfNFqNjx8Gssm8T05GKbdouHU/TnICi/PcGjUZU
7LXmvz/HDSdAKt3f65kiFSvi/+flxNEJ+EXnSzjR9Qe7tDVVQuTCBOwMdxTJ
WJvHHLKTNQSfTkBt9LIQspjDDBrI+4uXJmAjXpvv3vA0dlJLE7k3kaHdv9FD
Jn8ay5ASr9L/TYa3NmeJRYHT2FBHv6D8BBksxWJif/NNYzm18W+2c05CWlIn
1569U5iPmqoc2WAS9F7eYLeEkrEoie/tXUWT4OjwPxNYMuMEW4drCq7nFa+K
io9hcebFvMI5U6C1xS/GZHgUG5gNN9X4PAW0AI0tgYWjWP3NzVvPEacgwDhP
uctiFDOh7WYujk1BUNsb6+SIEWwtpKCcsGMaFinqx3Vbh7BZD6miiaxpeJH2
PySs4Z343fl8Cty3CvqcI0TC+IpHvQ3qKECQXGgh5PRgQ657LQS6KSCQN7PS
3PcbA1fm/cVVCgi3DVqdMv+F4bGsv6aPzIDUduKqk9RPLD6ll+9Jzww4XlHO
6oxowm75GGmAzBw8uBH4d8Knb1hzZpr/pOYc/BtR528734iZFKpvLzSdA5Xn
boc7LzRgjc5ROc+uzcEC9ravw6QOS2hmvmuonQN57WMab0QrMTO/ONXtgfNA
TU1zp6UFYEGK3j3X5/5ABFci8aj+Gbidtpajyk0FzRdujRJ6t+FjUv/pfGEq
2Gn3VN07kgj2kZBD2U2FG1d8pusTs2FQ7lr4uXPU/3r+xHarLoewAz2/9zVS
obGV9EDb7htocX59KFJEA8+Q3saz6k3g3V018aqaBgSVPp5Y7mb4Y1XtatVG
A8vw/siu0hZwU3AW5qPRoHXPUECQRBtsnne/yNJdAE/TgMI+Sic8kxrb7NSw
APppCy1/HH/CnrPVudykBSD8CaBw1/2EnNBP8d0zC5D3IlBVI70Lvm52YlFE
F4FC+ysr4sQv6NlTozR9YRF8X99MkisgQeBUmd8r7iUwXqGX7pfqBVOvs749
UksgZBXSaR7bC5scjr8GzSUopIcI+53rA9r+N7SUM0swbx0WVy00AF8mN5dF
lSxBRWR1dbHeMCR7Eo9g15bBcG9NiU3KGIg7HVJZ3kIH3a5fydVNY7AtP1PH
ZA8dtMJnwzQ2xoDb9klciykddjVtMyVcGodn/JrmyaF0EPfw+4mpTcD725fK
lafoMPty+5JmNRmUnspUTXeswqTp/p6XVDKc9dkX6UFdhVHK0SoB5UmosIi7
pSHMAJJ+QCz5/iSU3VBKe32SAQ1d3yRenZ6CzipAR34wIFMoWHfzxDT06dUS
QobW4EXpQ+nwbRTw0xclHuReh2SXLObUcQpMxmfIOamuw6PcVmJ9EQX0hkc+
Vl5fhxAzZceIkBlQlw57fYSwAbYR7cEzgnMQ/Sny+I8TTDipTHY5c3gOyo7V
s08FM+FY8/rhxutz0Pf7eN3JLCYcllLlf9s9B+rp9vwNq0xQL7uV4pI5D0ee
nPGbKmQBV/DWfndpKnxLkfd+LM2BXC7md1boUEG73bTdU5cDlVhCk5ANFazP
/zJ7f5oDXVbw+lx2jwqhAxqXjz/kQLXfqp7yLFHhZ5f2QCg3J5IpPX3fWZgG
WbdVWlJ3cKKADPKtAjUa5CvluquZcKIdASI+9pdo0MB+ml18hxPFbHc3f/eD
Bm2K3iuPNnGhfl6G0eo0DQQ3fM481OBCOtTHB6x4F6BXtuKxmA0XGq8vV1gy
WgD7ltid31K5kJmf4Jpp3gKwFtkTmvu5EU9NSdFYzCIYiJ1LMYnmQec+WLw/
mLkIMytx3+4X8aCypP70h5WLoBz5ND25nwe5e/PFadMWgXCEmrDLiBcRxV3c
YlyXYP0Fb9U/XHxInjV/5nfoEiRoZNeM6vGh4KkYG/WkJSAR2t5t8eVDu/4t
PPSzZQnKn1kmXSfxoXtXOKWUDZbBraRhQaeGHw3aJG+5ab8Mv97GbhJi8SM9
I3XeFv9luOLwe0bOmIDIwg4LAdnLYFCXuEm6ioCOfsltrRVfgQv+1MHE1k3o
1dtDdZL7VuDGT0NxTwkBtPK484u35QpcfHExre28AMq+vJEtFr0C7uecdiUs
CyB+IZvoy/MroMLby3PfUAg5q7rgepvocFCuvp/7iRDKN3FnC6jQoeZA6rWo
CSFkHRpyq/QMHdzl/yjWpW5GyeTXoTxEOgRb1xQUyAijKc4PFT1DdOC5K9dx
IVoYGcp9ohes0aHunBBXLUUYDdo1BTlorUL2+/finG4iaFcN7UZ2+iqM/+ht
n7QRRSH96/+EfV6FqMpw59ynoqiZzku1/q+rfMlwVfNOUeSnKeNH52OA6wG/
O15nxRB+XKWwRYkBOayKYwpvxZCYh+ZspjEDDvOKsDfPiaHP6SbexwIYkFZs
68X9tzgilFvlyT5mgML328rmJHHk0uk4Rc1hQIeC+U69vRKIRfC5kjbAgKGg
2YmSAQlkHvz8MuXYGshzWTrf8ZBEyc8ysqrd1sBAQ8HiZbskmirIGU6IXIM7
rrc2NcM29Gjsq6tR2Rq41lybclaXQoMsYoZw2xqMmgXdNMmVQloy7f1j02tA
/THecFpNGnXZjLs8UliH71KTIbogg3b5zqddNFwH0o3vvg2dMijk/mqPjsM6
pHs1UOm+25FstZDTwIN1GDTtO7W9XBb5kSSTP75fB3+ypLW0pxzClxS77+Lr
8PJq1FqnkjxyV9e101xZh2PY89T9JQro81FI4BLdgOFn2TFa5oqIcPl4xy/1
DciVFJPWLldELrfsRD4c3QClqbUbr/cpoYLU86duX9qATUwjM/5iJcQqvRpv
F7EB+dGJJ/j1diCbthvfVVM2YGTSc9KOuAMt8d470d66AUNRcj1bOVTQUaWn
D95NbsDL57JnAgtVUKrxi6YQLiaURF0tpl/ZiShO7wgn5Zhwqc7ux/LeXcg4
sMhCSZ8J5b2GtYvcqig+vuLesi0TrLbxlJRRVNFwXi2x6RoTBJqV7czHd6OY
4V+mAW+ZMOHLr9crrY4Gv5y4FlLFBGNq9ddr5hrI8DmeHNnNhL9c3usLPtqL
knx18dh5JvhErkn/TdNEVPP86Qf8LFgPY5sdPqSFrBQUxJ8pskCOoqlkJLkf
5aw+N04xYMEK4YuEPP8BxNVJuJJhywL3ZZFmCQlt5Prh1pO3Pizg2KHyycFE
B1XGLlTkxbKgUabY62aqLtrqenW0OIMF9ln7lmblD6LrB/sFP39mATX72UTl
sD5qFTmtW9XGgtK1oU93Zw3RbgrRtXaKBadyeeP2Ox5CMXWGf3/jZANzy9Wa
r7IIDb4s/vhDhg0XU5vVJYeOIMNglb4ubTbIHA8kdwuboeRTaTx9Vmyw97SQ
PzxwFNF2b9Ec8WCDJ2LUZTy3RFZcMU6TkWyYXW/y9qs8hXL76FFzKWwQnRdT
/Fpoh2avekr9TGPDyjnejoxd9uj/ALjtbGo=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$4912#3"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.922526, 0.385626, 0.209179], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwllnc8Ff73x6+9Ze+411VWRmYi521TVggpe3wykp1KKklaRiSrErKyVyV1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          "]]}, 
        Annotation[#, "Charting`Private`Tag$4912#4"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.528488, 0.470624, 0.701351], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJxTTMoPSmViYGBQA2IQXfvc99P//8oHGKDgWbXY19t/EHw/gfvft/9E8BUs
C/7mf0bwD3VN4njwDMHn0Lsud+Asgj+lJMG7YTaCv5qxeAmDOYLPmupuJBmr
BOcL3zias2iiIpwfdHhWsuYpBTi/+r1tzoTH8nD+jvJZWev95RDunRSq8G+m
DJzvPnevn8cbKTh/2fHKa8XRknB+XMsU0fMrxOH8ugTmddzCYnA+g678fudW
ETj/VvfM9tVXheB8Ga6MlV+cBOH8bFMt1tsy/HB+svm9FYcPc8P5WhVW//OW
cMD5C+8GH6l0ZIXznzeK2rhLMMH5XLXr30/mQvD1KjwXP/jDCOeX5dVyVT1E
8Nmin95YswrBVzHZWipki+AnPQtZfzeRAc5ve/guSSsEwV91p0Os3A3B/3Rp
T62ANoIvdi7MMFYWwbc6+eHJSn4EP7k2Yw0/L4IPANpnkVE=
          "]], 
         LineBox[CompressedData["
1:eJwVhwk0FGwbQJk0FEKm7EuR0EZKqOZ5kRYhSSKikGwhRbakLKEohbJLUiSK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          "]]}, 
        Annotation[#, "Charting`Private`Tag$4912#5"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.772079, 0.431554, 0.102387], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJxTTMoPSmViYGDQAWIQXfvc99P//8oHGKDgWbXY19t/EHw/gfvft/9E8BUs
C/7mf0bwD3VN4njwDMHn0Lsud+Asgj+lJMG7YTaCv5qxeAmDOYLPmupuJBmr
BOcL3zias2iiIpwfdHhWsuYpBTi/+r1tzoTH8nD+jvJZWev95RDunRSq8G+m
DJzvPnevn8cbKTh/2fHKa8XRknB+XMsU0fMrxOH8ugTmddzCYnA+g678fudW
ETj/VvfM9tVXheB8Ga6MlV+cBOH8bFMt1tsy/HB+svm9FYcPc8P5WhVW//OW
cMD5C+8GH6l0ZIXznzeK2rhLMMH5u6ZquL4I+7sfxjd4efOL/txvcL7Lrtuc
f8M+wPnrVi21PG6N4EvMzs+cqIDgX/vONsn7yns4X7U6TGDur3dw/qHGrzwO
7m/h/D99xmxtD17C+Rfyj6dKrULwFwdEH11XjOB7CTW3XGdD8GdOvcSkqfsC
zjebU/j/dOUzOJ+rljU+wRnBvxs7c98XHgS/Rf5greyCp3D+Y6/vbFM6EXwA
ZCO4rQ==
          "]], 
         LineBox[CompressedData["
1:eJwVhwk0FHgcgOVqQohxRI6crav2YdSmfv+MI6VxDJJSjEkqV+PMWUtILy8S
42hdPUdth0IU/kmGyTGZLCY25rIlWbnraXfb773vfe/bTov0Oi0pISHh8cP/
yz71pFp9uxg/p5TPlyF7kHiZXPZuXYRFQXqNXf94wNsq5fUIlgjL+2uy/EU0
qEm/c1zqhgi3all3Z1hEQSJ99/OiYyLMX2UYDzolAMWpX8vCUITL6tsCTYcu
gYFJYOKLWSGOf2Pl26KeCSuySzzvZiEWCaQmrny6Buy/svZ8TBNibxNVtRSj
PIiqf7i2RVWIuxt7aw86MmGXYkS6w5QAO+9MX42cLYUvDAvF6PsCHPPZkHm+
txwY9vWGwy4CHL3vxAc/+TtgXXnmgbSaAI8lH2P6kWtgScZkj62Aj2s/Jz1d
yK+DWE4VpTCZjxujDeaM4+8DySaIx3LlYzruHI6VeQQrTD36qjofkxLa8jz9
GyCeVnbRr2EKn/GiuTkaNcLijLmrbf8kDuir4L21eQYxjXUnC0MmMb/urqwx
9zmspBjHrEpM4ulU4yjXi+2wpqxX3mr7HtPc16bcGC9gnaSyvK98Aod1MVn7
1F9BSfH/8DBWpbK1uX2Q5RbbXKvw4+/eKwygDABBfaGPUDuGhQlHFuzODYJc
/aeV1+OjWD/LvQSOvgElzns3d6cR3KkrbHBo5IKmdvfaUc23+HrbJdX9g6NQ
LDqg2PqYiwOXt+1AiWOg9aDDUOsIF1N+e6FvvoMHOgdaKBOXhzCpQlmv4eo7
IA/6unaOcHDvGFWJFPwn+IYYVnKT2XjH60Hmpqd8yLkQnZ3f1IsPW6ZGd20V
QHtyV6TXXA9uoqTk+qYIwOhm8H7uKRYOXArgSbsIYQFXjw85dGEmyhaHikSg
a33QomrLM0xk8WJvOn+AeWYx/UsxA08VfQ62qf4bkiULup13H4OLm4kuLKV5
sCoJ7iGS0kB01phEpcwD1XqsLfNAAcyNSSbk9s/DhZDzH18V1IDYsrPg14Ev
ICHR9G9wx1MI329pY8RfhJ5+Xo41tRcQedrv2pE1CE1413PcnA0hdI3kottr
QDAal86Qeg3TFVouCnNrcChpInX4SR8sPmzotrrxFfp/mmTEEjnw3fyCgQfv
G4SSGffHZ7iQ0Jfg35/6HcIq4m/p/M6D5UPhcRk5G9Avlp2PPYqEQJVj6+Y0
bES2wyOFHWwhOKd+tXcRbES7kmYTLdaFoC4X5GihSkAmbA0yIUgE5n6CgfBY
AlI9HfEWm4lBMbddbmDvJjRbpr1k1TENs01S8XMcOVSuEGe7WfwR0pcPH/JW
U0SScWoT9K3zwNCPEQ9nq6CNCh6XaXMrwDE35ZWqaKP0qREyo/o7WJ12NTjv
Y4AelWZ6pC5sQBp5X+sCPE1RZBVdWeQrg3xmKBt8kCU6ea5EJ+syAQGr5fpC
5c/oqmfcHusyebTI0W8M77NB0sRb02nfFNFZz7njlrG70WjflfkMpS3ourvZ
DWN1e+STRCOVdKmg1qOy+XZ6CIXJfzIY9CciQ9kHYW5mDkjvHqtQdkYN5UT8
UUAkO6KkFjJVqVQD3S3brHyt2hlF1YQ1k3ZuRZnZBNPMxYNIQdKOZiXWQuJQ
E96r3MPIKdz7ltnNbYjJIcqG2FGQZqtOpJORLupNM7c7QfdA+YzsvF2jemiG
WnW2fdgTeWW6W95u1kfKA298X9pRkWtlE81t73a0ckJm6LaJN/oPURJqaA==

          "]]}, Annotation[#, "Charting`Private`Tag$4912#6"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.363898, 0.618501, 0.782349], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJxTTMoPSmViYGCwAGIQXfvc99P//8oHGKDgWbXY19t/EHw/gfvft/9E8BUs
C/7mf0bwD3VN4njwDMHn0Lsud+Asgj+lJMG7YTaCv5qxeAmDOYLPmupuJBmr
BOcL3zias2iiIpwfdHhWsuYpBTi/+r1tzoTH8nD+jvJZWev95RDunRSq8G+m
DJzvPnevn8cbKTh/2fHKa8XRknB+XMsU0fMrxOH8ugTmddzCYnA+g678fudW
ETj/VvfM9tVXheB8Ga6MlV+cBOH8bFMt1tsy/HB+svm9FYcPc8P5WhVW//OW
cMD5C+8GH6l0ZIXznzeK2rhLMMH5u6ZquL4I+7sfxjd4efOL/txvcL7Lrtuc
f8M+wPnrVi21PG6N4EvMzs+cqIDgX/vONsn7yns4X7U6TGDur3dw/qHGrzwO
7m/h/D99xmxtD17C+Rfyj6dKrULwFwdEH11XjOB7CTW3XGdD8GdOvcSkqfsC
zjebU/j/dOUzOL9g5fofgsKPEf7jy2t2evAIzv9YpMNXvBbBL7JZqXzFHcEv
Pb/Ib1rNQzj/8yttT9Mz9+H8ki0r4qalIfjfalVLvjMg+D8E5OfvNL2H8K+Z
0Ffb+Xfg/FqGfq75lgj+v5PcCgxXbsP5jLGs3oc4EPwmtaYE5cW34HyWD/9K
W2wR/C0td/+IMSH4AO7l7NY=
          "]], 
         LineBox[CompressedData["
1:eJwBgQN+/CFib1JlAgAAADcAAAACAAAAgK5VNhwx2b/St3NwevJoP4nfbuFp
Qdi/Z11zFjqMkD8Kqev2ytvXv/phQ7kvxZc/i3JoDCx217+ZUPGN9a+fPww8
5SGNENe/JNwfodobpD8Pz95MT0XWv1N1r5QTcq0/kJhbYrDf1b9eXWQ92Tyx
PxJi2HcRetW/2UOtw7rcsz8U9dGi067UvydTfp1Oark/GBvF+FcY07+eo15S
Zc3CP5nkQQ65stK/kn/L0JRtxD8arr4jGk3Svyp8xPB0F8Y/HUG4TtyB0b++
/FVABofJP0POVknB1s+/Hr2Ld/xm0D9WZiOh0nzJv4pb/guTZtg/inFziJK1
yL96/8Ue43DZP718w29S7se/81ZZPll+2j8kk2M+0l/Gv1jrh7aAotw/8r+j
29FCw78OamI5LofgPx4zSCyiEbq/1Kxt1Wc15T/wmJlk8ZlyvwKw/0IWfu8/
fAKUxUY4WT/BvyOWFRbwPy6aY8cUNn8/preTbadt8D9TM9i8hkGUP1NbiG9c
HvE/cWZr6cSUpj9kX4B264XyPwAAtf9jvrc/qpLA7K9s9T/HzNmKM1PIPytU
HxNykvs/aHjax1sryT8TwoSOKwH8Pwgk2wSEA8o/NTRTrWJw/D9Ke9x+1LPL
Px1504pGUP0/zCnfcnUUzz8wqM+EzhX/P2hDcq3b6tI/Do9NFpxbAUBsoHeV
HazZP6zZjGtlJwVAOi3BslCX4z/tcSk5UFkNQAJ2FdxkGfA/6YzTIUJAF0AH
DFCBYu72P7y2lYn+2SBAg+DWQ3Kj/T/JkbIYmnsmQLCbhlB+8gFALiim3l4h
LEBvomQQ5FYFQBsnDa1zUjFAXmqaHYeBCEDeSESquYc0QIlRdjkznAtAOT+g
ezLSN0AEFJXmf/oOQFvE3DKMiztA18uF8IQPEUAiydAFsic/QFV7YjaawxJA
0mMNxKKaQUBsC+sizloUQAcIrD0Jj0NAH6vGlgbqFUBYs2Ll7IdFQHu4Q1MP
mxdAOpqAquq6R0Bwpmy2Ni8ZQJY/VD4z1klADAI3Yi7lGkAlUhb8Hi1MQEVt
VJUqkxxA65fYQEyHTkAYuR1vRSQeQKDuYwSiYlBAknKIkTDXH0CujDH/RaBR
QFKGTy2dtiBAIHMjkbbOUkAqW4RVpH0hQFbIvl6T/FNAlAg6z9+YIUD2XAXR
nSZUQNXmoDI=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$4912#7"]& ], {}}}, {}}, {
    DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
     AxesOrigin -> {0, 0}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> {FontFamily -> "Arial"}, DisplayFunction :> Identity, 
     Frame -> {{False, False}, {False, False}}, 
     FrameLabel -> {{None, None}, {None, None}}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None,
        "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, 
     PlotRange -> {{-9.999999591836735, 9.999999591836735}, {0., 
      80.60338235400909}}, PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{
       RowBox[{"-", "2.5`"}], 
       RowBox[{"-", "1.5`"}], 
       RowBox[{"-", "0.5`"}], "0", "0.5`", "1.5`", "2.5`"}, "LineLegend", 
      DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #5}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #6}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.363898, 0.618501, 0.782349], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.363898, 0.618501, 0.782349], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #7}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.560181`", ",", "0.691569`", ",", "0.194885`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6150173333333333, 0.25708400000000003`, 
                    0.13945266666666667`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.922526`", ",", "0.385626`", ",", "0.209179`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.922526, 0.385626, 0.209179]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.922526, 0.385626, 0.209179], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.3523253333333333, 0.3137493333333333, 
                    0.46756733333333333`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.528488`", ",", "0.470624`", ",", "0.701351`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.528488, 0.470624, 0.701351]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.528488, 0.470624, 0.701351], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.5147193333333333, 0.28770266666666666`, 
                    0.06825800000000001], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.772079`", ",", "0.431554`", ",", "0.102387`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.772079, 0.431554, 0.102387]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.772079, 0.431554, 0.102387], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.363898, 0.618501, 0.782349], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[0.24259866666666668`, 0.412334, 0.521566], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.363898`", ",", "0.618501`", ",", "0.782349`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.363898, 0.618501, 0.782349]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.363898, 0.618501, 0.782349], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             
             RowBox[{#, ",", #2, ",", #3, ",", #4, ",", #5, ",", #6, 
               ",", #7}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.828023422056819*^9, 3.828023518107972*^9},
 CellLabel->"Out[65]=",ExpressionUUID->"15881f4f-2c54-415b-b62b-8e9486ffa763"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"CouplingValues", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.5"}], ",", 
     RowBox[{"-", "0.4"}], ",", 
     RowBox[{"-", "0.3"}], ",", 
     RowBox[{"-", "0.2"}], ",", 
     RowBox[{"-", "0.1"}], ",", "0", ",", " ", "0.1", ",", "0.2", ",", "0.3", 
     ",", "0.4", ",", "0.5"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"LogLogPlot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"CoupledExponential", "[", 
        RowBox[{"x", ",", "#"}], "]"}], 
       RowBox[{"-", "1"}]], "&"}], "/@", "CouplingValues"}], "//", 
    "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     SuperscriptBox["10", 
      RowBox[{"-", "3"}]], ",", 
     SuperscriptBox["10", "3"]}], "}"}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", "CouplingValues"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "Automatic"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.826380762352865*^9, 3.8263807858041267`*^9}},
 CellLabel->"In[59]:=",ExpressionUUID->"23b4fb1f-d08a-4ad2-849b-b5e7ce3886b6"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVk2c81v/fQO29ucyyklBRSWXkS2kge48kMjOKEhV+RmY2kb1n1mUkyvsj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          "]]}, 
        Annotation[#, "Charting`Private`Tag$6550#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVV3c41/8XtT529iZJihYhEd+8bVFEySYrsrOVjCQ7K0p2lFBZoSLuy87e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          "]]}, Annotation[#, "Charting`Private`Tag$6550#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVl2c41o8bxT329piPUVYKLQ0NKt9I/FIUioyE7JFClGwyopSsRAkpZGTP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          "]]}, 
        Annotation[#, "Charting`Private`Tag$6550#3"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.922526, 0.385626, 0.209179], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVl2c41g8bhu295/NYIYpKSDKSHxJRKpJZZlJIyWqIkk0lhFD0VPYWDXHf
IiUUkUQhKzv7sb3/99P18fpyXcdxnlJOl8xc6GhoaB7T0tD8P+dln4dFPRXF
0kZ/jftqxpB5dEu9ymNRPEXJv8dtbgxW3imMv5NFUWjkXV/hZWOowtgQxVhR
tCHZ0x3JNYYw25vB7YGiyGKhN78mcQTIDywCJU6LosZeaHXhOgraayx+pUKi
eOKSntEhpmMwLR1SbssnilH9NQ6j244BxWhznoFLFN+0JS4UGx4D5qQFb0tG
UXwn1aycF3kMWpT7vdbmRLD/yjW1E9zHwcm18qJBqwgyHNG7BzwnIPzbRZeu
aBFUpGnUMFQ2ha7Oin3bwkUw8rJGVPQhU9jVu8l8JVgEkx/fhb/WptAy/iCX
9ZoIrnRVG40Em4IIY8U/9XMiqOt/dZmjwxQK1TauJemIIHeZ5JaNW2bQkXbv
nvkCGSs/hHiZjp0EuWc/7DOmyTh2xvhBI6053MiRVJ4YJyOX9zZ7R7I5SFaU
tt3pJyPru9/hfYfNwe1rB6nsKxl/v+zLVsoxhw3aLc94c8l48lxXmOqlUyDr
WvTqix0ZVzP+jdbxWkLphZhTbjZkdLe4/lJupyXoeFyYY7QgI7Hx0ytb3xJs
vLYqHjAho/xXc501f0uIvZGUmbefjErF5nFLvZawcj/oYQSJjMbyhfn25Vbw
5fVxb71vJISEh3xNfjZgW7mLt6eZhB/02gMn4m1gpIq16FoDCa8m2q/uKrEB
+trasWIkId1aTSnXhA2oN6s5bSki4bUZVlprZ1ug/NliuhZNwoPfTrbIWZ0G
f7bp3RWHSFjC56ZeaWsHFZsS3a66//VpqqwHBNjB4rxJOPkACVXvu4s5PbYD
n9683oC9JFw3YKBP6rWDyy9dYw9uJaGO0p+sIhd7cLPrnWmhIaG62j4jey0H
sCv98nK8UhjHFqpHK844wuOsdYfHr4Tx1M7DbD6XHeFX2i7O42XCuPYq5Oyh
O45gGx7lUporjP907orL5jiC1WkDoWuPhLF9rx5794IjmDFV+zP5C6PMF6NL
e+Od4JBNgabUHmHs/N2oQfQ4g4xi7lVfBWEsMGVbvb3gDAwMWRUNcsJoXb66
3MtxFt4XZOy5skUYxcublKb2nwUd2oQdtRzCqM2vwWqZeha0sq+LuPwVwpo+
blF/exfYs2i4kp0mhB37BG/1rJ8DvkZ99fUkIaTaukEDyRVm0nX9TOOFcKfM
Ymi7iisUG+2fXY4UwuXMfC0jN1dQeLx7zNhfCO3kV13KOl1BTl+wa8JUCAuS
nQbzKs+DRFz/GyVmIQydNf2ok+oG6s7ede10QijG/em+2Ws3MNvL8PXqhiCG
6etKhHx3g/AO2UFcEMRQihvJkMcdpkUucJkOCqJ9nprViXB3qHn2z8mrRhAX
mMY17t/wgLPla+ylNwSR7CFrhjc9ITDsrpCFvyA+4ZLZxZvuCcmWElIrVwSR
0fKKVxh6QvMysU/PTRBbBu797qG/BPu07zh8sxLEhEQrypGYS8DykbV8VlUQ
SVQzMzGzy5DXKWSnMi2AKTqK3HVXveBcpO/KtXEBvDzjTvG96wWSmu2JOCyA
9rahlQTFCxJSH3w1+S2A2/Z+nRNv8oJAOw7d858F0Hqsp4xR6gqYDdLKPH4u
gLSvcp6kfb0CS/8mRpmsBTDYl4uKOj5QmnEkzMRcAOW2TVtVW/nARdNc6YTj
AniLtk++87IP9Je62koZCOCAH3uG41MfaPTtb9qvIoAfdxxN/krvC09WfxRd
5hTAGLOnAvu++II+c61vVw0/Mlu8fC933R+Kc70l1ar4cafniqvfQ38QOybT
GP+aHzlkBtsGiv1hISFU0qSIH1kOz8RLjPhDpvThRkjjR+HDT5qEra4Cq3az
5At/fnTzv+ypqX0Nvvj8aLykwI8pizdUzWVugCYpwq9Jjh8bbLrbhg7egMxK
dSl5GX6s+t6hnOZ8A4LoUvz6RfixSVaz0//ZDVCKPS11ioUfdQ/3Rv2TCYD4
vD9+mgN8qPAbeJgVb4JV/7gU4yM+zGAbKhZ0DILN20qPShL4sHG09YfknSDI
lPLlsYvlQ7pxOuqJF0Ew77CxURHOhxd/HATJsSB40Mvz67wfH7ot7fWp0bwF
n3+pJjad5MOv7/N+vW24BVo/gtgSuPiwn8mkNX3xNgz41d3WYePDKSdSsjhb
MEQJsS5PMPLhwWX+6GLxYOg8Fff30DovBgmOmAodCgbf9ud11HFetDF/NjWQ
EAyFrQ2Btg28eDYoUTBN/Q5INvHPSYfwInPXEtvOByHgemTbmlcQL+rT3SE7
5IRA4Wd1xprrvKjmqVBUUhMCBxrOCNt78aJZx6/F3NkQsKnP2p9qx4sDKgYl
y6dC4SHuD+HX4EXWH3tlt0mHAVu5syDDFA8W+l4hKlrDwXSvn8TJUR50MdhC
wclwSC6L2E4Z5MGS3k/8E6wRsL20QEOnmwdjaNIwUy8C9IuoZwI+8aDfsFZc
UXkEBOVEZ85TeFC2p0NnDyUS5h+XqQ1Z8iBdk9yOLYnR4Ee24Qw5yYOqSut8
Xi+jYSmBZkD6OA/yP7U5M/gtGtaiTe45GPDg4RybXEWeGGC8/newey8P5vCQ
1M/ExICQhWjcN14ebBof0FGNvguXxZh+BiVxo+/o309jz+6DD//JkKp73FjG
txxZVn8frrJlKK6GcuPwcGEqZfQ+3F7SCPPz4cbCbCuFFeFYiGu/qHLRlBuV
WcOvq9rHwsuY73dt2LnRp/eyuctMLCyvPddTDeJCl/p9xw1l48Dp50O2dD8u
TL6VNdqhHgefy8O+sXhyYd5768wbR+MgxfO806/TXOh937qfwScONPt2Bt/R
5EKnUPlJUl0c3KgtrWld4MQnq1xDtufjYT0SdTw9OLF4Z+I1ak0CnDtXwtLp
zInNSSfeW3cmwBc9SouuLSemmvJot08l/PfPOw6CxpxIVRhX4xN7CNoXDW+9
286JdDF+YuX+DyHoxBdgG+DAxWMp+oUqiUAr/Es7y5oD9embJZUhCXJLz++w
OMGBmd4Bb9a7ksDs2IIgoyEHPj39aGFsMQkooVyTTns5sP+V3+x2xWTQn9dJ
leDmQCbn07pKT5MhovUF9WEdO3rW9vanRT8CnphLxSGK7GhXvvCI8UYqvN6+
lqqyjR2zotnH9qamgmNtRHi/GDvuuZHMEVWZCqUrT+102NixvNQ4IHMtFcwv
tHOsDbKhQ4nOUv6tNEg20LjgncKGrxgfZpFiH4MkLb20EyMbSnBaPXndnA7a
D19lFq+xokzOV7ub0+lwWt5j5+YcK5KT17a+ZM2AZNPve9P6WTHlT+mTzl0Z
wEvJMuwAVrSgF1Rk88kAuoNHLxpfZ0Wu18wl0kxPYTDk4es9/1jQsrtv7YMG
BejIxgduD7Ngn4htZrkJBSQLNmq+/mbBeZs4tUpHCpz+7tro0cSC98/4XWGJ
osB3Wc2ezBwW7FT+ulTQTYH6+h560bMsaKGhRd5/5xlkscgdp//JjA2pUmS+
wefAdPoqNfErM56raov/tfYczhZ9St9Rz4z2+ZSMOsEXIGlxYdq0jBnlDh7d
nDR8ASnPch9k3GVGGlaXmpyCF3BPW6H9gB4z0hrsujwUmAl+Pnusr+Yxob76
n1x/jWxo/xhMy0Fhwg0ehUEti2xQEW3LSU9mwlE++Zsy3tkwXeO9Uh/KhIRz
uo9tQTa4cZenCjowoXBUG+/FrTlgl6vWUyrAhGVnxXz0hXLhUJ+W8+RNRoxN
9ndfEc4HFrKyXIEPI4ZtcvvVqufDZ1PZSQ93RnxRZcxcZp0Px2o5/SesGPG3
u48vU1o+WGX2RI2rMOL5opj4O9IF4OZxq3R0lAH9PB7toFErhNilWtrhUwwY
ddjufT5RDGbKr+tfHGVAmxH109lmxSDolh/tcpAByQfnT7x1KYaU7gTBISUG
3O8vXyEVUwzPql3kB9kZcKIq4SxtVzFUhDCb9tfQ49XR/efFrpdAN8+RjB4F
eowgWgYimkqBT89144AsPWZTw/jZ/pSCsfed04/F6NHvze/vTxdK4e33StJp
dnqcufzeX3BLGaSk7Hrwc4QOBaKJNW7vMrCR4Qpuf0aHJz0kG85veQldaq3O
n8l0+LsvutUlqhz4LkzWyPPS4aG2jdyJp+VgnMIqGclCh4HTWQF335TD2zXd
X4ZUWnSjfLspPloOKTWl5h/aafGQ0n2V60YVYHMk4RDep8W3c59Ah/sV/LSz
3F7BSItEhsuZX0Wvwew4d47NGg1GP+0Y2d78Gj4TH3dsztJg/cmLzxPHXsNb
SbXdh/toMO6cq3WP7BtI6xdS7XpLg3RFx0NEn7wBu3MdehuXaNAvcGXY++Fb
qO/OfRfzchP0TJKJPxnvoOKdCH5/ugkMcmd5xt+/g6zHUbUS9zdBMCNkQnzo
HUTYu30uvrAJbn534jblq+DIgHxnu8QmeEyc3Pe6vApaxrLnxCI3IEFjS+2u
79XQvZy5o+DMOqwY1xi2pCBUNJ5zUD26DtSQkVjDAoS4x9sSqzTXQbyw+E0b
IBjrZtJ+EV4HvbpC9p1DCG8jXnROta5B28RXp06lGkgRfh6mZLAGThHEl4NN
NWCl+rS/bPcqmLyzyOYXqIW9zI4kLfFVOERp8bWSrwWen5LH6thXoU95w71c
uxY+BWS8aR9ZgZYRd/3XF2pBrS49doGyAgYR6o2DWAtCJ59oqwmvgA3Plo5Z
7zpov5ya8nZzCWK9Nr6/mvwASjVxws0TS3AmvdTLiKUe7vJGJfT+XIIEr/KZ
Vel6MCi9eo/h5RJE7ss/XGdVD2/mTgUfO78E3DPBjH8/1MMTPx63/hYqDND9
EUnN/AjnA0I12CiLIEecyVgPboC6poAKsfuLoFrgXEJHaQApcR8VxYBFoBf+
67m7pgG6qpwUzC0WoeYK89uVzQYw2SSknrAtwoY0zU6PoM+wJ3iFZY/3AlyL
uXCgL7QRVsMvdVrrz8OJvkjJ6NJmkHxWeK9OeR72sTzhl+loBv3qSX3FLfPA
Yca+MLLcDDHz7iUMK3MQ4Rp4nqr7BSQcz0cVFc2BRNJHN4X2L6Cj6ajFKDoH
1Q8v/U6maYGwCbOM4pkZUKzeaZRJtEIuc5yFaO8MfNo0veJh3gpfpVs5wppm
IIt2LODohVYgWR+/Zps1A1+q/nPs+FbI+XDEjOnMDAw5+jWXjbRC0xN9htOf
pmG3adp2/+RvIE3xaOHS/gfa3/WUDDnbQfdt1FMn0X9QTaciOLW1HRy+ZV+p
WJqCHFv1pCLNdkinGxKwK5sCqb9Hl4PPt4OY8xnrArkp2GSr+ef5oR0EZY71
H+GfhC/WgeTrId+BJVNxPmJkDKSS2BJtST9ge7XJh18fxuC630eZ1D0/wKDD
PVHp2RjsGLQeXT76A0KZstU7z4zBUYaMcsHgH0DnKnlze9so3P5j0PZ58ges
budlqq8agWIlvtLjjZ0wlTMrTB8/DJGmykUL6V3ABUH+7heHQeCXeChtTRco
tnP8aDMchtEsk1j1/i64vLEt8fnaEJz7503Iy3bDnJmtgMG5IeBowmHRgm5Y
Wq3lidQYhKMeB8cCP/wC2mMPWbn7/oBe2sI9YZFekHaWvuD/9g+cqOtpZ9Pt
Bb2rRZ96E/7A2m/uaeXzvRBCaQgvNvoDfoElXWKveoGJus5oVtYHmNfF9Gpn
HwjpNDPs4OyFBdZt3NKtfSA/eGlR1aYLzrWHBoQr9wPpW0MgvUAXCOx1vMpk
2A9MuJW5tfkntKTUvMo43Q/9qT+EPXR/wjZGlSfbI/oh5SShQZHrhD9bW87e
6esHjlquAC7qd5hgrUtOTByA6acFtH//20HAgNKYFXkIeu4zR700aYWfk2+/
PdgzBE03HfmCmVuBMU3Cb/zIEORYC24Vv9ECx1LbSmiDhqBWeiUis+4LsLbd
uwTDQ5BJ+4gryPEzrI/fbM5/NwwXoYOslFoDJTRTnxvujEDEzKf5GN4a0Fxq
SRjLGAHK1sqvo+EI028mPZWrRyCfm2A3HK0GDy8L7T3LIxCU0dKvkPYWGF+5
NjF5jYLM+7kHq7SlUC09s1/5/Bj0Fegw95ReAEVy6GdTvwkQSHK52uFuSmha
u+0uSp6A69JthM19X2L7EcMhzcoJmDV5VsO7Fk2Q099e7aGZhELn6OCLe9IJ
jc2KOLt7k2DwKqik+VwxoaXQGxVeOgUbb9LkHzUjkXl9hGYwdhp64qcExpxq
iHxtgS30WdNQfVGHZv9SDfEiW6jlYNU0BEoNdvySriWC9uwe3jE2DRsRO0Mk
r34gBiVuDyUemoENy7c9WdKfid0ND60l6Wdhc2XcqLuylbDOUT44lTIH//4m
wB2mb8TeU/5s9eVz0Nt+QHWX6TdCLm/w3qeWOYDC+5I3h78Rojf3yB9nnocg
JxWqBG87QesrZOLhNw+bDdefO53rIFp/KHzcd2YBNpNZN0d5ugndSvHAj8ep
oPf+St5Di27C/NbAZoAnFULGuy1107qJRDa33053qcCmXVicvP0XUTBcu7+m
kQpCA2ZOhtq/iVe7NF+2GS/BboW0D8/de4mmZ83d+WbLUOQgXK848IdQe+4f
cCZmFU4sHEljZOonWj9/v0gtWYXZyFtXuuX6iSSLLQUtP1ZBtWxUItyzn/jw
esHqoMwaVDK+8+9Z7ifWgfxcA9fgY66D/F2eQcL1YIbhLMMG9M7nxIwdGCYy
goRmpJ1p8FbMpfY852EiKENj5UYADUrKqIpdjBwmSkvm9ZkTadDhJOb9ax8m
FHTDvlU00GBfScfnefe/RFGSTkmkCi32edKzbj4aIYLJJtah/3HCLaaGEwgj
xEzaFzOj3XQo+eRe8u2hEUJ2osvL6AgdOjST5RiURwnOno9UmjA6/LNTyZD1
0yiRE1FU5rhBh/0jp0P5F8cI37hWz0er9Bh8S7q5XXSc2Kef0HhDjAGlSSMC
ibrjxKrkL9WsAwzoZOj9nHR3nPhbwjj/6xYD9r+IrBWXmSDYycdse9kYccCp
glbObJJg2THAvWM3Ew7+4g7UKvxHeO+OPmz19j8O7jl00qTtH2Hwb2G+dogF
Gf/ckLNb+kdwTmXkP+ZlxfhZ71FCapp4d76bheLGii8Xe+Md3aeJB99j9kpK
syF149XI881pwujFdN7zbHakoTl5UoFvhthRxXy0rZsdWWinqsplZohOVsnD
sf95B4leJr7eaIZQK+Ek4CoHajLHHhiJnyGqyNEammacGMB9Pm6n3CyhEsmv
vUOGG0N46NfLNGcJ/r47Q3cduTGG94mrlsksMXTSyLg4nRsf87drmVyZJXh2
VhtLcvNgtbDOX893s0TNJDrciuNBGkmSVumJOWLm+tHpm4W8yCJVlqnpPEcs
vWZwUGjnRR7pY7y1vnPE5EbJe6cVXpSSCRluS5kj4k/KcHkc5kM9uenYhcE5
4t1q1oLlOB+GKn0aUr82T9DtLM2gOSGAFB1+ieCoeWJW9o5jfIAA4gk7i8bU
eUIqfqSyPUcAVy7P15+pnieG1SM5/ZgE0bNEMjuYfoHQuOkwXPNJEKNr3Psa
BRaIZt8bPJargpjdWkES3LZA1PCyzHzYLYT9M0cjsw4vELrSzpxHkoXQYs81
t6a7C4RsE3tKpZ8weuvVUQTTFwjlB9xPhoqFMdaMu9uueIEgbcvz3zYhjJ+v
vDgy822BSDTJRL5zJNQua90lRFokwoyqxavOk9G2VszFXn6R4As38X6fT8ar
ba6PszUXCYmSUgeNWTKWzq1z7j+zSFx+YW9VcVsEt+3d8c+eskhorsbWv34p
igf1fbfnlC0Sop4j4nxMYuhgjvazdYvExKi2loe1GKb4WLSE/F0k8qpyXwcz
ieOrkKcsX5YWiXDV19beDuLYnjChI8xGJeIDNZ80vxNHrvLgkpxdVCK9xdJd
+6YE7vzQPDp7gEr8va83m/JHAg9/J0lrHacSH69IZU4e3oLBC4VxX7yoRJ+8
Y98DHkmkqnZ7z1VQCQW9+IewIYkCBtvytT5RCdY/IevxhBTusfAaDP1JJWwU
onJbgqXQw4/5FGmNSuSbfHfW5JXGyDCzu46cSwTnw5XYltPSmJn4+EOuxBJh
5Ma0ScmTxr4KlX0HdJeIxPU9dhLmW3G9PtAzzGyJOC5+UNKrcCuK/GjI/Oq8
RJCm3j2x5JRBtb8CvSTfJUKO46qakZcMmlPthZ3Cloj35aKBLV0y6MWcdzwv
aYmglQxKum8si/eEF8Pns5cIXcXTg4dqZPGTWsxSWOMSMSZGVbX8uA3PNqSP
HupZIkBkpPGvy3aksS3rYphZIi7f/hifyieHaRP1jbX0y4RQaf+IVIscagR2
vQsWWiaeKr94QUeRxw7uqQJd+WXioTLdD/q7O/DKU9p0Gq1lojrcWKUtaSdy
qQjGwrFlgpNbalG5aRfm1sndDnRcJtKYvL8qa+xGAwutKwd8lgnvkpsaM1RF
HPh73HktbJnQOCjjnCiqjLeuOZtXPlomxIw2ig4U7UExdv9D1/OXiR3fYij3
LFVxixaLLbVsmWhS6uxv4tTE/wHI4Vtf
          "]]}, 
        Annotation[#, "Charting`Private`Tag$6550#4"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.528488, 0.470624, 0.701351], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVl3c41g8bxZ/H3nuTlYoQhcgvvg9JRUJKZW8limSVKNltqawoe1T0iIS6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          "]]}, Annotation[#, "Charting`Private`Tag$6550#5"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.772079, 0.431554, 0.102387], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVl3c81Y0fxa3rIuMa14ysoqKMZFS+V4nioZDKSEZmSFkpo4SMVKJBJCPC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          "]]}, Annotation[#, "Charting`Private`Tag$6550#6"]& ], 
       
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.363898, 0.618501, 0.782349], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVl3c01/8Xx62PvffKbFBE5CsVb2SEVEZGRkY2KYTKSnbIXlFWyMooZd6X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          "]]}, 
        Annotation[#, "Charting`Private`Tag$6550#7"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[1, 0.75, 0], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV12c8FQwbBnDr2Hsdo4SoVCiUIs5pKpJQNg+KIvQUkREpmzKLjGxCRVZS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          "]]}, Annotation[#, "Charting`Private`Tag$6550#8"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.647624, 0.37816, 0.614037], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwtWHkwFY73te+7x8OLKJWoqJQn8V4KRRQVsmVJJMq+JRHZyhYlW9nXREhF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          "]]}, 
        Annotation[#, "Charting`Private`Tag$6550#9"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.571589, 0.586483, 0.], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwtmHc014/3x+291xvvsjJKGfEWqXil0lShJCTJiCghMqJkV5IR2SGz8FFE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          "]]}, 
        Annotation[#, "Charting`Private`Tag$6550#10"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.915, 0.3325, 0.2125], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwtmGc4FY7//u29OTiOrJKWcESScVoUUmayEhGiZAstW4iIsjLKKhEZEe+3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          "]]}, 
        Annotation[#, "Charting`Private`Tag$6550#11"]& ]}}, {}}, {
    DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> 
       None}, DisplayFunction -> Identity, DisplayFunction -> Identity, 
     Ticks -> {Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& }, 
     AxesOrigin -> {-6.907754997032942, -26.05390756395427}, 
     FrameTicks -> {{Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, GridLines -> {None, None}, 
     DisplayFunction -> Identity, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None,
        "ClippingRange" -> {{{-6.907754997032942, 
        6.907754997032942}, {-744.4400719213812, -0.0005001251827273307}}, \
{{-6.907754997032942, 
        6.907754997032942}, {-26.05390756395427, -0.0005001251827273307}}}}, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> {FontFamily -> "Arial"}, 
     CoordinatesToolOptions -> {"DisplayFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& )}, DisplayFunction :> Identity, 
     Frame -> {{False, False}, {False, False}}, 
     FrameLabel -> {{None, None}, {None, None}}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> 
       None}, PlotRange -> {{-6.907754997032942, 
      6.907754997032942}, {-26.05390756395427, 0.}}, PlotRangeClipping -> 
     True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{
       RowBox[{"-", "0.5`"}], 
       RowBox[{"-", "0.4`"}], 
       RowBox[{"-", "0.3`"}], 
       RowBox[{"-", "0.2`"}], 
       RowBox[{"-", "0.1`"}], "0", "0.1`", "0.2`", "0.3`", "0.4`", "0.5`"}, 
      "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #5}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #6}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.363898, 0.618501, 0.782349], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.363898, 0.618501, 0.782349], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #7}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[1, 0.75, 0], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[1, 0.75, 0], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #8}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"], 
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.647624, 0.37816, 0.614037], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.647624, 0.37816, 0.614037], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #9}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.571589, 0.586483, 0.], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.571589, 0.586483, 0.], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #10}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.915, 0.3325, 0.2125], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.915, 0.3325, 0.2125], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #11}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.560181`", ",", "0.691569`", ",", "0.194885`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6150173333333333, 0.25708400000000003`, 
                    0.13945266666666667`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.922526`", ",", "0.385626`", ",", "0.209179`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.922526, 0.385626, 0.209179]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.922526, 0.385626, 0.209179], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.3523253333333333, 0.3137493333333333, 
                    0.46756733333333333`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.528488`", ",", "0.470624`", ",", "0.701351`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.528488, 0.470624, 0.701351]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.528488, 0.470624, 0.701351], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.5147193333333333, 0.28770266666666666`, 
                    0.06825800000000001], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.772079`", ",", "0.431554`", ",", "0.102387`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.772079, 0.431554, 0.102387]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.772079, 0.431554, 0.102387], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.363898, 0.618501, 0.782349], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[0.24259866666666668`, 0.412334, 0.521566], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.363898`", ",", "0.618501`", ",", "0.782349`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.363898, 0.618501, 0.782349]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.363898, 0.618501, 0.782349], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[1, 0.75, 0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> RGBColor[0.6666666666666666, 0.5, 0.], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"1", ",", "0.75`", ",", "0"}], "]"}], NumberMarks -> 
                    False]], Appearance -> None, BaseStyle -> {}, 
                    BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                    ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[1, 0.75, 0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[1, 0.75, 0], Editable -> False, Selectable -> 
                    False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.647624, 0.37816, 0.614037], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.4317493333333333, 0.2521066666666667, 
                    0.40935800000000006`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.647624`", ",", "0.37816`", ",", "0.614037`"}], 
                    "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.647624, 0.37816, 0.614037]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.647624, 0.37816, 0.614037], Editable -> False, 
                    Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.571589, 0.586483, 0.], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[0.38105933333333336`, 0.39098866666666665`, 0.], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.571589`", ",", "0.586483`", ",", "0.`"}], 
                    "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.571589, 0.586483, 0.]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.571589, 0.586483, 0.], Editable -> False, 
                    Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.915, 0.3325, 0.2125], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6100000000000001, 0.22166666666666668`, 
                    0.14166666666666666`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.915`", ",", "0.3325`", ",", "0.2125`"}], "]"}],
                     NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.915, 0.3325, 0.2125]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.915, 0.3325, 0.2125], Editable -> False, 
                    Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             
             RowBox[{#, ",", #2, ",", #3, ",", #4, ",", #5, ",", #6, ",", #7, 
               ",", #8, ",", #9, ",", #10, ",", #11}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.826380790777837*^9},
 CellLabel->"Out[60]=",ExpressionUUID->"87d891cc-fbab-4f9e-bb83-93056f600b9e"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"CouplingValues", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"0.5", ",", "5", ",", "50", ",", "500"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"CoupledExponential", "[", 
        RowBox[{"x", ",", "#"}], "]"}], 
       RowBox[{"-", "1"}]], "&"}], "/@", "CouplingValues"}], "//", 
    "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "1"}], ",", "10"}], "}"}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", "CouplingValues"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "Automatic"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.826382626243512*^9, 3.826382681123664*^9}},
 CellLabel->"In[65]:=",ExpressionUUID->"631f4061-eb0c-4b83-83b3-62c6726a457f"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVjGk0lAsYgM0QY7tlvVLWj+yVslf3fWUpWpQrSyTZo5v4JksNx66yUyIi
TbgqxsyYDJoYKeR0tKCkukW7bhHSpLjdH895fjznPDqBUe4hVDExMe9f/O9B
S/eFRUsxPF1edC80sAPGZkjFVeIU9LE8ZOaoL4Cy1Rn9tTIU3P7uz+m/FASw
K7wk3UCRgsHvcnn1P6/DjVH+V0MdCi5vPf/Mc/g6nOv6+cQEKMjsZ5/IOHEd
3AuymObHKZjE2ChFTLaD0KTcYtM0BTUDNj0d6W+DC4Fdnh7jVGy831S6n80H
yyO5k+wPVLyqHhjTVc2HO4nep5Z+oeJGMVdyXREfZks/Ce6IiePTeZJrS/LB
dUBd315THG+iqGHckg8zG8hpMx9xXHNE1FcmaIGtKnoFUgPiyI9W+7x++BpM
9WT0tbVJoLO0wxDDiAclJad3lQkl0Er5cWO1Bg/sQpiP4nol8JYLLXBAgQfp
VOFri2EJlGmxt3D63gyqm35QWFMSmLnWjVnW3wwbOTF2TMMlGLLHhPk8qhmy
KvZfyS5Zgl4f9Jou3uCCRrRtrl+MJDp3+5gfIznwh+2n0YEESczivdndHc4B
fwrT2CFFEmsXRW2q/hy4UCjXa1QgiYRRu/PgVg7ocf6hzjVIYsHEi1C2JgeM
pzPiC99L4havzaKP/WywOvog6HaAFCpNhbEsTNnglhBpt9aNhrU94w3HqU2Q
TA2O1fOk4fHOxNaIGRY0Zftx1PbRkP72GeP9OAuWVe4wpkTSMNL7sLz0TRbc
61qj/iCThq17bjoNpLDAXW5GRN6goXNXMDeKwgLPKkZri5k0zvUUJ3bQGsHv
Vq4NyMngt/T61CbLq3D4yGrt5E+yuMyjNmG6/W9QVnK0EvXIY93s4W3aozVw
RuRyviFlKT6K9DyyL58JeVpvv7o0LcMDnroHsyKr4Y/g6p4n+QqY/NEquz+o
Cvh6JSypSEWUnyACiqIqgKfsmnPXWwm73KJChRfLoGpVrALbShkNKh39d3if
BbnwA9lmBirYN0aULhieAQf9rJXOSqpYdbma0+dUDGES+p6Mz6qoNVE5m8sr
hN+UUtO3Pf8dp0wkv4rWFQDTwcvXRqCGfbpzR0te5gJlMtRFeHY5GtRz51bW
Z8NYv6XpeLo6HhrxuWaTdBLYfYwW3cAVKHuG7X72ShZQ51Kt53evxDqffTvr
xjLA3pkSMGWugdrlbkPZAengqtBWlSaviTbcOgFHKg2GZdcNNs5r4vaR7L1G
3BRwM9Rlz41ooZevrW1QYTLot4ULD4Vo4yaP7qH1r5OAVvmMd2VUG+36eyzc
xhkwVpSXJHLRQevF8L6dJschUatKWCzUwdCamic99ARYKph4vNFcFwc/BptK
/4iDzNKheW6lLjZ3Car8a2OhQ1arLE6FwMvv0jSEbkfhU/4lGdd0AqNP2c+K
DOjg9NR3zCaTQO6liiAqQYcKQ6U2gxME7n14kZTTpIOrMOXgkhwCV0iJx+or
06Hui39vZzGBrSFhUxFidAjYszzTmklglWvcaocREh6q51H0uwjU4Hmt1csh
wTjMaUSpm8Bz09Z5+7NISOX+aKLe/vXbPxtfkUrC2u2RAS/6CFxQTfuinkBC
TuLWjvL7BPJr9gYbh5Lg/IKaqPiSQEFRUksRknDetM2DMk7g4PktpW82kDAT
H206+YpAlfiF+Q3WJFxUeDF6992v7tDtMmFGwuJmgd3JSQJjIur+9V1Bgmc+
XTH+C4F3ZAcH21VJaBw1+RA6Q2DEMSU7DUUS/Ojnyhy/Eeh7L5DxikYCt3N3
9PrvBFocA29XCRJk5KVddH8QqBzby2IvxsABn05thQUCLz0PTlWfjwF+Tdy3
xcVfPzXGQPrXGPgPMcJ2vA==
          "]]}, 
        Annotation[#, "Charting`Private`Tag$8772#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwViXk41PsegGexL8fBkFxl+aVRKKRJiz5fkRgSQqco5diiGylRh+Jnz5LI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          "]]}, 
        Annotation[#, "Charting`Private`Tag$8772#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVj2k4FWgDhs8hoyyZbMWU7aBNJCUUz8u0kRIpiYMs2YpEBmPfKZGECAlR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          "]]}, 
        Annotation[#, "Charting`Private`Tag$8772#3"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.922526, 0.385626, 0.209179], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVy3c81Yvjx/FzjihUsq9SxsmMa3ZSxvtD2YpskT2rS0ZRISqKZJa9qmtF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          "]]}, 
        Annotation[#, "Charting`Private`Tag$8772#4"]& ]}}, {}}, {
    DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
     AxesOrigin -> {0, 0}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> {FontFamily -> "Arial"}, DisplayFunction :> Identity, 
     Frame -> {{False, False}, {False, False}}, 
     FrameLabel -> {{None, None}, {None, None}}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None,
        "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, 
     PlotRange -> {{-0.9999997755102041, 9.999999775510204}, {0., 
      0.12259523956400492`}}, PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{"0.5`", "5", "50", "500"}, "LineLegend", 
      DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.560181`", ",", "0.691569`", ",", "0.194885`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6150173333333333, 0.25708400000000003`, 
                    0.13945266666666667`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.922526`", ",", "0.385626`", ",", "0.209179`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.922526, 0.385626, 0.209179]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.922526, 0.385626, 0.209179], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3, ",", #4}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.8263826403720217`*^9, 3.826382682331128*^9}},
 CellLabel->"Out[66]=",ExpressionUUID->"5c90b3c4-9e88-4390-b051-0c8cd1935ffc"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"CouplingValues", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.2"}], ",", 
     RowBox[{"-", "0.4"}], ",", 
     RowBox[{"-", "0.6"}], ",", 
     RowBox[{"-", "0.8"}], ",", 
     RowBox[{"-", "1.0"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"CoupledExponential", "[", 
        RowBox[{"x", ",", "#"}], "]"}], 
       RowBox[{"-", "1"}]], "&"}], "/@", "CouplingValues"}], "//", 
    "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "1"}], ",", "10"}], "}"}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", "CouplingValues"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "Automatic"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.826382720642561*^9, 3.826382771092565*^9}},
 CellLabel->"In[67]:=",ExpressionUUID->"8a06f6b0-f73a-484b-a692-66c98d8db402"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwtk3k01V/Uh83DNd17v5RIo1TIEN2M371JKEKaVKSByJgxikiRhBRJ5qEk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          "]], 
         LineBox[CompressedData["
1:eJxTTMoPSmViYGCwBmIQLbpnTtN0ZhEHBij4Y5R5ax4ngu/8uDZDjBfB75o8
6VuvAIJ/yXl5C6sIgp+05MKCL1II/qqQp3o5cgj+R5Zfex4rIvgNqco3L2kg
+PPVygQ3miH4Sy7HMhdbIZnX4PrVxA7B33ZL5OYOVwT/TN/mBQdCEPxf3z7o
XyhC8MOOZ/u9X4fg82QkduuqiyL8q9ou4yYsBuens6iG1bxH8PmEm1q874nD
+Yudw6Mt9krA+Ywf0jwPTpeE8x+dNtV53CIF5288WbNdKUkazmf63mT+O1AG
znd0Y0z4aCgL53sJ7prfzCsH51/jNrqy7jeC76+htPH7TXk4X3VXxsGcVAU4
n2Pe3a2rbyP4jyb11f30VITza+XnH5x8EMHn3/v6ho2hEpzfNuPq783zEPz9
3PIzy0WV4fx3/Uu4vFoQfNc70Y8s2hD8ORrCu9Q7EHyvg42ZrD0I/vJPcScO
TEbwE0Il28wXI/iXpfoYVQ8h+FrprjeFjyD4TZv/bGA6huAb+GQnPDiJ4PfU
euyffRHBd3vAVCv0EMGfq7MrhPExgv+lolDnwxMEf5Hgg9tnXyD4/532WnV+
QPDD+kuEKj4h+Otua79K+4Lgx5TMmunyA8HffCCw0PgXgs/Fy+mp9AfBT4w8
oCD4D8HfsbT8x///CD4AQkjIOQ==
          "]]}, 
        Annotation[#, "Charting`Private`Tag$9198#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwtins0lGkcgL/ve9dRtFu01KqdI4XakJCU8nt1tui6qEShwkR3NGlSlOsY
mVKrWcIgNZVLNEKlcU0zBhmJ7ebaTiXXzIhmXHbr7HPOc55/ngXex13oFEEQ
f/znt1qLPZ9lcbvKP4y892dfHIFt9g2dntYt5Y5ObsMqRzn4RlZNJE4Ky+ud
ZgVsXvgZDDvOlMRFCEEVVtI50dgP6/ILDGrWtUDtpmb3vNYeYOQ/CrQ17gZ/
LWEvvUIGXNmQ5gi7Bw5f2tqx62sXEN8Zgt81O9mKyja4oqNQNFgPARotZwy5
tYFvcJ+1mlwO0mNW/beLW///FTCuVTvwV2Yr9HnV6BUeH4WoKN3rjT82QU72
N5QQvkf1G5kthhTr2D1Nr5VQ0CP2XfOrGIat/va51aMEmywd6qRCBI6WRkdC
x5TQd1qt+EmdCL4se3J2ia4KbNMOce+EiMBlCcELd1ZBbi/7btyrp6Axn9m5
XKwCP3GUZXFKDTBJf/qVonG47z7r/ByLathZ73B8e8Ik2Ai8Ps7ZKQTJFprL
Ut4knKqVH8z6WQi4QWGFciehIBn8vF48BpNnmUqBaBJoLDVvnx2PgZKqorWn
JqGrRX19nmspFDTnpzYdnQKIj1213/chxPRrym4ZEvhquGncvptFUGl6qvVP
EwKvsme5GR0ugvGj3aJzlgSeVzq3VH95EQQNPMjeZU/gqFXZ44XC++A5SA9Q
9yTwjWnQ4P6qEKw+l6sOJBI4I4SzvVFPAN2KE9pGiMQaT2tZ0kt5kGwWXcfX
ILFp04WCUv08cPLnRhlrk9juoYGHpSAXyt48+LJ4AYnjP1+dUdeaA9eqJl4v
BRIn+CgXFhhlg0sCK2v5GRInaaTdnujiw3RJkocggsSGHjE8zbN8qEDZOpZx
JF5kq1cp0OWD2al6llUyif3WklbSrTdBw0vryMoSEjNkke7e1VlQuTTFaq2c
xDvYXRmi+gxg0nP7hUoSJ9fIyjoCMsAsXci3oyj83lDanamTAananXOxFoW5
6/F5lyoeML8umli3jMLGCf6znxOpYC66W+N4mMJpUib9bSUXPhAVYbVBFG75
yd84VZ8LvNVNKzeFUPjeqO1Ix/mrMOOu/M5mNoUrctLzqQ2J8DHR5uK2WxT+
xyHkw8J3lyHDu8p1xzsK+/iYMyJfxcGKAM7QvU8ULjxJGzstZYMk1C1u5jCF
XywIFAxKYmEkaUAoIRDGlU0v2JIY2NSoZ2hPQ/gayuEPjERAx1tZGc8QYUnO
dJGzRgQwPhW4jZsgvDpwe8lNg3BIV3OIL7FFWC3SNKOhLAwUtifkpu4IlzuY
HxvQZQJ7ox3nwj6EJ5+N+WhEBgNt13TjHj+EG37g5ewdY8DGoPTdN4IRfrk7
Nyl1KhDazx1SkGEIezCH86grAcDgrLi4NxphZXYcHXseA97tukq9RISft/eH
hc48CJbF3D3MFIS//HLm5Tv2ARBX7x9puY6wejrfgjXPF+Tto4sv30PYYrCZ
c3bWXojtq6oaeIBwkHT21LhkN9CUHI8tFQj7dphY1D52BUedRQnqjQi7utev
yedvgzaDwSX0VoQn0jaIBYQjBJk/qq5qQ9gsuI0X0GsP0+yiPfVlCDtzkETY
awNpm51GQ/sQ3sloD4hvNgML93mX38gRjpnvPMzYQINW1lOldAzhAafM6maO
JvwL/l1d5A==
          "]], 
         LineBox[CompressedData["
1:eJxNz2tI0wEUBfBtaD7msj00NW1zuSY2S3voHGYPdbYl2HpoodSUfNSimkgu
0Mq1tPAxUFutNketKJCcJpIMxP0jnSP2ITLLrNBZFAXmRNK10SqIe7twOPw4
n2582Zn95RQSiST+k7+dpMoI3BQUsJP075q0ecsyOjrWyGi3RKHzBh+OnOSj
DXPjgpkcdKrPNJqVjx4NUcgNB9DuBHJnURlaXJzsdV78b7dfcVitaJ2uc5+e
QIvKza9rx9AaCvFp6wQ6cruPbHGjMx9Xi8yJgfif4Vh3sw4dp8xoLaleAS44
rxClFASDS0ZahTvCQsGnz27kXJqjglnMnDSPnQa+7pEYHzWEg9vYn39IeleB
s47fsb/V0sGDCTpLkIIBHmBJW5yHmWDT+nP0vjQWOKyqtDmZHwHO5jXFipmR
4MoAXmHdd/RKplqz98NqsDm7qFg4FAUmz1dIiBvRYNfzbYJZTQy4z1H3hFu2
BkxZUqd7ZbHgXWKy3J0aB5bSrabLtLXgCerm8R4vuiCR27c0yQbzrFXEqXIO
OLjr/UD3FNrV3nbBI4kH17NNRAeBDh/69iYzlQtuvPnK29+FHqay9bUR68Bz
2nuhUg06912xS9iINiQyrfyraCnRcCKwBf1g4eiYrQMtPxTdmG5Gv4xpI/Oe
opMqcyeZz9Dqfl8vZRSdkq+QTzvQLfV7hm+/QIunKfWMGbRRYD1InkUvqpSC
+Y/ou/TpKecXtH/3kOjaPLpQW8NQLaB7pjZ8rVhEl9Tc0ucso/ttMuWWn+hQ
WoiE60OXHrFx6L/Qg/drl/1+9G8XrOYr
          "]]}, 
        Annotation[#, "Charting`Private`Tag$9198#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwtj3lMkwcAxRuOuI3DA+gCRhARUNDIIMIImgeMakRF5Vgnol2gBDrAuo5O
hLFZNpDDA+0oTGEgYkcaoQXdTOtqY8cNhYEgZQ5sGxEKHSvwlR4IjC17ycv7
55e89zyTmbGpViQSKXTd/2YgwXWt3zInmzK8SS+5bsCR9JmA6jKdbNv9J1iO
NsCZnfjHh+9rZQum8m+uiwhwt0RcEfygltEOJbFFpxcRE6FQnQ0eke0/STpw
0XEB9G/lK9+vSmVRo0GZXVI9vF/lPS4tkCLFayG3kzOHSKFoR3vkCDiJHl/z
K2eRLZR8HuarQXG/P52LafAm9XaGEi2yO8hJnuLXIP0nPaJ4hODHJDVuuRCE
IliPbNo+XpWXGi7G5cGnFD12P6DXwEmNKqW1UBivh8ebQsWElRq1d5wYN1l6
VKUpM2w1KjzwDBqPb9bjT49PdfO1KnTsZbW/9J7HiPvxe6FuKixT9BVa5wXs
EKwdf2fTBOhf6oJtF9d/up5ImC0a+38Pgcbe0rGV1DE0HrTjzzgSiFhda3UI
H4POxs/ldz8CqvzOzX0mJdjctMXbyQQoHIc2J4YSRSKNMGCIQDZVTCmNGkXj
jHJXUosB+14cc7n2ehjRMemqGJkBtfSa/AzpMHQtxsoIhQF3xZnKnbxhBFwi
b/DVGtDSvtXP5/AwxBviJ+e3LyFu3um32J+eo2fnwN0r5UsIIosZz2hD0J1r
d3vINKJugDxJ+3gARz/r4tLyjQg1NjNaV/ohYPfa25cZ8XboArWwoR9pVwdX
6XwjLLcTGur0CmjE4xryuBGn/urPjCvog9J5SZAbbQLTy7bgTHE3QrabvXw/
MaGiYTrExqcbPP+31c9TTQjJzdo1K+9CXKTVjT0cE/LTwhcTLZ3oZzp+MfHY
hCOcE/K1cx1o6/EJi/Q2wzdg+hTfKIfXi92P5gLNuPie5GRMoRwF6j1774Sb
4U/NMZZvlgOmQA/ijBmcQ2LXau9nkHjDmn9rnWdudGhmPYXrB5FfxdaZcTia
kTDIkiLnAMWw2mRGjl1lXsn5X7E/7ugUtduM1OK+yylUCUSXqb3vkixQfpeR
ztD8jI1XE6N+cbDAj1nURrQ+wvnKs9LkrRbMzGqbynIfwr85Rfgk2IKPrE+H
6bQi8F8yuVlZFjR51rvI3fiwmWLZu+VZcIPyt/RSUz1SFtiFHcUWHHy1STXa
VwP5Ss4qi2fBhZ6b/G3uFaBKQuqOVa/3B9tx3AfL8Q9BoNQI
          "]], 
         LineBox[CompressedData["
1:eJxFx2tMEgAYhWEgK0LJALU0CCVBTSvJQnMuM9SE2himVMM1pVQaLrNc0iYt
Cc2Wl6broiZW1vzRMhljGZsTnIWs/OG6KtUI27pthswpCpNqq+8727uzJ0ZR
mV9KIhAIwj/9/UAny0R+upRJ+LeMxHpX7Di6S9Aon5hCe3a+O973HZ2XwqvQ
eNHz20drEyJ84PwEgr5OiqYw1U7+GFpNVJa2mfzgwpf7Kw9dWwa75s7SeSuI
e//7jmJEVjBNAs/a6u1mcxCYVbW7uejMKrDkvCo9WUIGFz1rTssMoYBPnd4W
fXEmGBzGyBYs2qjg64ui7kd1oeAW9td50cA68J4Td21TrTTwYOyNx6tVdLAp
TNw0foQB7uGdoxkEYeAQZcnVrXHhYCH3MjOXEQEuD+LKan+h1zK0ugOf1oN7
hYflaUMbwER3mch6MxLserEraVoXBTbYa59wFBvBpAVtqk/KBGflEotn+Syw
mGbuuUTdBH4bvON1vw8tiecYFibZYK5Zaa0ojQaT9R9NDx1oV1vLhUVRDFjD
7rG2W9GhQz/fZ/A54IZbb3xGPXo4mN1RE74ZPNN6nyLWoXM+yF1pDejb8Qxz
XCNabK07ubIJ3ec5NmZpRxcXRjak9qJfRbUQuSPoLeU5k4xRtNboHyA9Rycf
VBU77egmTd5w1wQ610nS0D+ju5PMBcRp9Jy6Ksn9BX2P5nSMf0MH9g2lX3Gj
Za3VdLUH3e9I/FE2hy6q7uzI9qKNFmlVyhKaQl0j4vjRJUct0bRl9OCDGm8g
gP4NEnPmeQ==
          "]]}, Annotation[#, "Charting`Private`Tag$9198#3"]& ], 
       
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.922526, 0.385626, 0.209179], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwdkHs01IkCx4VbYZfyGNza681RQsiw5CuPbrSrPG4Spq2k6GGJRCXVLmpb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          "]], 
         LineBox[CompressedData["
1:eJxFzXssFAAcB3B3KW+6ByG6cxwmtVMLqSWvy53GFHo44oZTLI8IDUM6mnMs
KyEkqaaFm1ncZpx5cyu95pF2nVolE5Lz2qm2+v1+23ffff75/qz4CaeiiRoa
GiF/8reDFpJ1ww2X3TX+HbeuMmzaAj2n6WD80gEtju0Y7XFFs0Z989vY6NQy
wc9KPtpUtdJYnIiWnhfyc7LRaquGsehKtLBF2cx6hbanJgusFejhNALNZB5t
6E4Xb+j8ApfLw+L7PdBPZsfteRI01z9W4d+FnpOoyj3kaFaGiZbdN3SHVtDn
RfoK/rd5UVdQip4L7zNvTVCB/S4Nll3IQjemjujrF6EFojF11CO0smNaaTKN
HqeuNF7jroJ7h22PeDLXwFXOhaFjk+vg4NETCadL1bi3fIVsu41w/L/v83tC
gmaI4MWBG0NSqSbYMulwMS95BzggI86NFaAN5vUVu7rr64IvJ+6n58zrgakU
b+e1AQPw7TVO9bNcI7CY9mWF07ITfCyqbmCyhARut7nTrBVHBrdRuSL5WQq4
1vYqSeJMBevHRhbtszMGezELLNgUE7BAkxmS+QNtSMnL9/uwC1zvdSbUtdMU
TFiI4cjKzcDKkUOOM/nmYMlQ5nMGfzeYqMpz2Qi0AHuwCRGLTpZgLklae91g
D/id3oE3TRvoAHuGRDVBAzOlsbL4aDpYu2a67ekUWnlLnL3GsQJn0WplZTK0
Uef38aNODLDw7tuN1hp0lx6tIs3YGjxf8lCXm4/2eR+qdBWi79lTpHaFaK4s
9+J2EfrxUvhgdxk6IthM6FKPfm0uJjB70A4CnwlKLzqvdbOF2I9mnYyLUAyh
RVm+XVVjaLaCmEX+iK52lAYRZtDL6UmOC5/QD0iKKflX9JZnp9vNBXRISQo5
fQndNLV3NmYZzUuprPBeRbd2ByYdXEfrGuhwGJvoyHPddJIa3d6Qtrq1hf4N
Ozc5dw==
          "]]}, Annotation[#, "Charting`Private`Tag$9198#4"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.528488, 0.470624, 0.701351], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJxTTMoPSmViYGAQBmIQXRdd1f7///v9DGDwwd4z45XBnO43cL5IadQtC/GX
cP5kIcf2VTMfwvl+jmcfxJpdhfNTmg/9nfJvL5yver96e1fTXnsY32n9BqWj
Tlfh/JL1uwqt1R/B+a2nrtrNuPEUzp/29AP3186XcP6vbXmH7zm9hvNjO95U
nfj9Bs7nK1wdZnvjLZx/IDLLaNOWd3C+5DnPBW+U3sP5RU6afOoTEfx5fyRc
XU4g+JHyUq+dHyP4AMBka8c=
          "]], 
         LineBox[CompressedData["
1:eJxFx2tMkgEYhmHAYygafFBpGkihVto0F5JrWSkU1OYspJrOKfM4WyY5tU1b
GpEt1NJZGimVNX9UKGMux0aKC82VP1yHqWQjbOvkDJWFiNNqq/d9tnvPrgh5
6fF8ColEEv3p70sdCmp2kCOZ9G9+UYJu3+3oZpbTOcZHs1ye8edCdNuEV0+P
FK3VEMU3FegnEQnTUh16OFZhsfLmwR6ho/UbcwGcVzHL91lcBM9mW0INpS6w
hl+fOT61DM54fbj0xI1VsN15nhHpRT7w3/fkQzLpDAU8P3Jl1Gj0BoeX7W3I
UviC0y6UJMWl+YOzLA2C5EAq+Oy5XZxLcwFgJpHKd4/QwK1uccfT2mBwI/vL
L3HvevD+vPsjU010cP+2Wz1+JQxwH1OiHjtFgLWRFXQ9nwkOLMq9HhvFAqfw
roaJiA3gQm+erPonOoioUx79uBHclXIyU2DaBCY7CsTm2yFg+6s9MTPKULB+
tPoZV74ZTHHVJXrSw8AHReSc+fhwsIRu1F6mbQG/D9j9VudBp0Vz9a5JNphn
LDKfyeeA/Tun+x5b0fbmxotucQS4hq01t5jRwaYfE/viuWBV2zuPoRM9EMBu
r2RtBc81PaRKlGjhh0y7QIW+G00Yo+rREnNtsY8a3b2Q/XKwBZ2TEaJK7EK/
CW0k84bQOwqFk8QLdJ1hpZcyjI47VpJjG0Wra44MaMbRIhulhvEJ3RFjlJJn
0M6qshjHZ/QDus069hW9dsiUdM2BljWVM6oW0Drrzu8FTnRW+Z321CW0YTC9
LGEZTaWtE3NX0LmnBzn0VXT/o8qltTX0byNv9I4=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$9198#5"]& ], {}}}, {}}, {
    DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
     AxesOrigin -> {0, 0}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> {FontFamily -> "Arial"}, DisplayFunction :> Identity, 
     Frame -> {{False, False}, {False, False}}, 
     FrameLabel -> {{None, None}, {None, None}}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None,
        "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, 
     PlotRange -> {{-0.9999997755102041, 9.999999775510204}, {0., 
      1.3472990129224571`}}, PlotRangeClipping -> True, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{
       RowBox[{"-", "0.2`"}], 
       RowBox[{"-", "0.4`"}], 
       RowBox[{"-", "0.6`"}], 
       RowBox[{"-", "0.8`"}], 
       RowBox[{"-", "1.`"}]}, "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #5}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.560181`", ",", "0.691569`", ",", "0.194885`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6150173333333333, 0.25708400000000003`, 
                    0.13945266666666667`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.922526`", ",", "0.385626`", ",", "0.209179`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.922526, 0.385626, 0.209179]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.922526, 0.385626, 0.209179], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.3523253333333333, 0.3137493333333333, 
                    0.46756733333333333`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.528488`", ",", "0.470624`", ",", "0.701351`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.528488, 0.470624, 0.701351]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.528488, 0.470624, 0.701351], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3, ",", #4, ",", #5}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.826382772435774*^9},
 CellLabel->"Out[68]=",ExpressionUUID->"102603cd-ffe7-45cf-b58c-79b06c5b9351"]
}, Open  ]],

Cell[TextData[{
 "The curves are produced by the Coupled Exponential Function\n\t",
 Cell[BoxData[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{"\[Kappa]", " ", "x"}]}], ")"}], 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"1", " ", "+", " ", "\[Kappa]"}], "\[Kappa]"]}]]],ExpressionUUID->
  "b9d91216-e3f8-48aa-9df0-97caef8a6eee"]
}], "Text",
 CellChangeTimes->{{3.6608489130250063`*^9, 3.6608489299461737`*^9}, {
  3.6608491241044807`*^9, 3.6608491268405533`*^9}, {3.6848556336461563`*^9, 
  3.68485563479657*^9}},ExpressionUUID->"68ee224f-851a-44b9-b740-\
8d468fa41879"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"CouplingValues", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.5"}], ",", 
     RowBox[{"-", "0.4"}], ",", 
     RowBox[{"-", "0.3"}], ",", 
     RowBox[{"-", "0.2"}], ",", 
     RowBox[{"-", "0.1"}], ",", "0", ",", " ", "0.1", ",", "0.2", ",", "0.3", 
     ",", "0.4", ",", "0.5"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"CoupledExponential", "[", 
        RowBox[{
         RowBox[{"-", "x"}], ",", "#"}], "]"}], 
       RowBox[{"-", "1"}]], "&"}], "/@", "CouplingValues"}], "//", 
    "Evaluate"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "10"}], "}"}], ",", 
   RowBox[{"PlotLegends", "\[Rule]", "CouplingValues"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.6848556497147503`*^9, 3.684855651145131*^9}, {
  3.684856525957563*^9, 3.68485653310115*^9}},
 CellLabel->"In[28]:=",ExpressionUUID->"23fa79c0-f3bb-4422-ae96-6a5969f1ba59"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwt0Hsw3FcUwPEVVoTFrmVvvHf9DPEs7ZiRjLgnpK0iERXNVFqt9VjqFVs7
E482olmPSemmkRBGDSKdMF5FU0loPBrvbdhUM+hILclEEl1kxSK2se535tzP
X+ePc3n81I9j99BotNC3s2Plt3xdz5iLvm8fSxpNgftl3OBT3ATspSczC9BV
YPX97u+43AwcPXuEKWEocOW9Ils6txD37GCuwEmHRcmvba/hrMWpJfyuAss2
40sUtjfxf7WhvediFHjv9JzjS9tO/IiDE9SDClw9XJX23HYI12+Z39q+tIyb
mgunk9yn8JpXWsXY+6tYgMevvRDK8VRIusOdmVc4ZcFJ3t70DH+WMezdmLiG
9XkXHGe9FVjM7L2carWOi5kjztVfruIPCmKvnGhRYf70yd+1R9dwa3L9rFbc
Jk774wI/1XED/8b3a73FfoNbzzDC9n/0Bn/qnef0/fVtTNsN1Lvh6qzERY9T
NHjQ5SUqc6FBbtVW6KsXWtATsn0gc40GNNljuJupDTkJZ7iRXVqwu65D3AMO
6XFnTzrTgX3oPR/nK3tgXln6iUGXLqxsBDWII7UhcOGrb35M04M6YYfvkLcO
HJ+XuNjY6IOjcNDM2JgODfU76UO4Jjos9vFzBhsNQBnlCfOTdHBnTVr7f2EI
pXrPi6ubdUEfK6NmVo1AXLXhsZSxFz58GfHaLpxJ1INHvTFaEa4s4j7oKOiP
5CpZxH0QRxc9lkpNQKBRH6KFWR2McjbRgNxlSv7HAN4RuEiDo0yJDPjTxNVA
dNAMpBoN4U5TboWrNgfuajSCLC3nE6P/cCBToxHoz2QGpLQjMNBoDAs3eCVV
2ftBKGMOh/gyoeSAuU5nuDkEljByZ+qZICmPG4jiWkDAsTa5B5MFESWfSxlP
LcBt2F2Q9zUL1o/Wqaq6LSG7ZmIrZJYF8gW/s735VrAi5Pn8628CNYc9Lcei
rGH8p7KiJy0m0CgWsQRuNjDZe7ugjcOG/PDoIG2lDfTXt9adz2SD10z1zfwx
W7BYevCw7wkbypwqGseCuERTOHevqCyok0s0hWOW4XXD9jyiGRxJGTjNKuQR
zaDKim8/quQRzcC+7+erx0/bETkgcet1nui2I3IgblVnfMiZIiLgt65wC3+g
iAiScsO2Ai9RRASisPa/GZcpSNeIoEApKpZcpYgImg6qNksrKSICVY968kYD
RUQgmWAU9Q9QRATltcnxeUMUEUFtutQ/YIQiIviVI9kYkVJEBDMR7HjZXxQR
gaPc3H9OThEReLRn2lxfoIgIDomnVbFPKSKCYMfKlmeLFBGBMJFrs7xMERFk
+5xX/bJKERGIDecepispIoKy5tqL6+sUEUFNjo7g9gZFRNAQGuuXvUUREbTb
3bf23aagTSOC7lUHlVpNERH8DwqsQVE=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$12664#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwtkHs0lHkAhkej0Woq436d+WY+vim5ZFtdNvV7Y0s1OLmHthoVyi1z2HJr
JYXETkuRwrpURxbL0Z7VSpQlCZV0o7Z1SXQ5Iykj1tbZnnPe8/71/PG+Qv9w
t92zWCyW66d87ryD/hybXWlr1hc+pVgsBWnqopy8qT1kyMpRckxNQWaa6w9T
VDRp4ySU6CxQkLyGdMFsKpWInpX0WfMVJGR1VOgHwWkSTMlTZHYK0vUxKEsh
KCXTdr2uS6IVRK2nT/xaUEsydiw9dXxMQQpvFkS8FLQSEhLwVf3LUVJRmdoT
YvWYGGiHscJ13pFAcuf0K1k/kTbnXNG2fE/CBhf111QMk+Tys8MCyQRRFyaJ
/16hILfvVt+WlU6SDI0288IdY+R8Q6mAo5gi/j0eV9m33hNKpUyoHT1DIv5K
8g8XT5JM3X2dT9gqqNrHddffOE3cF+oX3bCbBdb/YL067OPd2CiMDR5Z4s3C
7cmLSZY5bCQWTLm+e6UCgYFR3VC/Klhdz1AXw4att0/+C2vOF18VtiOOlfsK
OWAiAw54mM+G4qi1W06BGgbGs73mXuHgeWtD77HpOdg0uDf+54g5YHPjHZcl
qMNlQL6Yz1fHI785NgZsLsoufkYd7aHCpauSuBi57p9wo3wu7k3n8Z1l82DF
u2/isH0emmttHgSrLYA6GZf2js2H61U7gcJBA46vfT+IPDXwx9k9k/l6PDy8
tkvF14IHrwcnU7pGeLiU0rSNGueBw4nIvletiYDZUc86OjTBvRCk+TRDCztl
sZe4uVqwrbJY7rBd+8s+bZjm6xl83aoN68DFHU5SbdQ7t0xJrHTQqWkxN2ql
DtLCn8SP8HTxZ0XiGQu2LlJeZOlQw7qIVTHffOuJLty6UxOl3XpQ743ZEFaj
hy3SzU4eDfoYPC/MKojTR27jSTtBvgGyFhqo1noa4Piy6McJJwwhzw1okVKG
GPDIdPbZbwTfrO87uEOGEHnVrKb8jDHx3TllQb0RFvGOZ8/yNEH/oP2Ba8nG
cFl3cFUl+ChabWPULjXBB9/HhwftBSg/EsULtOTDoqpRvs6IQrLnTgl7nI/n
C1y+6cukYNtbWJrcLsChE+I+tr4QOYvOlLdLKPh1v5tJyxLix4b0HEktBSW9
d0eHsQjORp7nbpoKERaqVzdQLMLasBY/XqoQv7yJC+bRNAqM/U1vjQvRniK/
vLyChun1C6dc/ETYGj+wV2JjCrnlNfO79SJEWx2g/qk2RcCY6p1WcxqtHUXi
cpjBv+otlfoTjeQ6j2rjZjOEJLpPbTpBgy7xGZfdMEOUe80DbiYNoVuTc+tN
M6SMR2XIT9F4Pdqy5odOM1SsVH7MzqNx6ZFMeOehGZSNM/fPl9GQev364thL
M8jvctObWmjMr/CNns9jkFscGnS0lca2mPzAQC0GxZEdDhvaaHTWWgc36DD4
XVc+2dZBg0lfXi0zZNDrqxXU1U1DHNhCHtIMxP0GDn39NOxXkC0XljFYUhPD
Lxn89EPbTK7KSgbfHulR7h6iEdmiMua3ioGTOO+34REaxRPNfRpgIAum+KOj
NPzWr22M3cggzu6QsnqMhqVoovy+hMGReX33IsdpXA8aqbRxYZBTWZw2MUGj
b8XBsSE3BkUJqoGXJ2m8deDbOngyKHPdbR83RaOr7n1avjeDGlGzyZp/aewf
4EwofRjUjzHKmRkabz46R3tuZfAfZvMrRQ==
          "]]}, 
        Annotation[#, "Charting`Private`Tag$12664#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwtj3s0lAkAxUfINFRGEmF84xMZrxSZr1N9F5sVakl6jLO1RigkRN6U8tqW
RmY9kqQ2ZWOiVCeVJDV5dkpbW+zWIduhbOMxMZ5bZ/udc8/9695zL1d4YEvA
HAaD4fVFX70kWTjXds/x9ZnsnaYMhoxu6iQ8thP7aC3jbdfamDJ69lH9UYKI
o/NcflY+zZbRJQ3ZRqpEFp1UGP2vEymjQ9dF7x8zKqL/0Q4arXGR0Z2Te8Uy
owo6qXHBkHaOjFbr6jEbNLpF+x2Mt5ZxhuiyltKID0bNNCsltVxt4zAtuZLV
FWr9mq4dOLLA8vMoHUQ/LfoY2UvHsCZNz5SP0WF95r21kn76bsuTdrXYCZrF
PWb2hi+jyz2Y9xJvT9M5mq28sp9G6Ai7e87PEhkQdm29p9z2mT4XXNUgFsxB
xMNjwgNmEzTV4sn19FBBTbiGt+7Gadq3utxcvHouGP+DuncvktM/q6EsIWRg
xXYGpqXeXp8CmUgtnfIa/aiEWKKhWZjNAqPzLe7EK8O67kKDrFHjW14FBuNr
4xKz5sM0KjB2K08VwQEGL0c3LMQ7ecE29btzMaZ5euRDtybc+oKTTkYwMU+h
/bwYWtj8TmTB4bCwsye3OqNqES7//hUWGtL2vbLjaWPggfDw4yp1dMflJy/U
XAxr9gtD593z0TNwOHndWR2waLlf98gCPBhuzygy0cX3g4IxYx9NpCcYrGx4
pYc/G/coCSzZCCxa1Xu2QB/XM5t2EXI2OJnBE2UuhghUjX7b0aGFlsEKRiLP
CP6RCdc1Ti2Cs7W/WJ5MfPunDYGpssb0GAGbIIsODz9tXHN0WJ8SwsUTLUv1
aGoxyKYpfmyUMW5LUostlXWQteFS7sWDJBKUeJ5tf+nAPfVO5EeRCVjd8a5h
tUtgnz2w+nXNMvSVc8WliboIO9HruEVqCvFyPZVbPnp43D8unZ4yg+hUoNSP
WIqj/eETAlNzCMQ/dmi8X4qQDxVEtRcP499dUJTW68Pq7k4rr1IL9PY5xTZm
GMAhc57Op78tcW6drX67nyFeFsTZ5zpZoyotmh1kxYFoRtfFo8AGGT7+7spy
DvwrWU73D62AfXdZRUa7EVzf+w4P29mi0Ly4qt2dQKo89ETbgC1SGrIL3W8R
kAyxb5bWrsQmfZ8LLSZcJPixKktSVsExTOrLzuJCMrWjzdvRDqUGQpM2ORdn
Xt1kLtO1h8mDi/mbfY2RM0Wq2PfZQ2TVyHtWb4yrO5YvTLq6GoEjKk+beSQi
JyXmM786QFgzTGSdIHHklyW+bWF8hKZ6T7nlkihTGO8+E8lHtHftS408Erkb
j9eFH+IjUx6dI8onwdkdH6OTzIeEUkwWlJAwH2eyArP5UNyffVF+mcRJ79N5
iyv5ED3TyG6SkngjWeNyo5+PU+f3701vJvFu1SUqb5CP81Edzq6tJFpeVx8L
H+Ljho5oorWDhK9nMddSwUe3YNHezj9IhJqox1xkUjDr1XPu6f2yN4Z5UmJG
YUVtPOe3PhKulZqMXAsKa9K6FAHvSYQodT48aEPBw6ykun+AhKA5e8caBwqR
IQRnaIhE+gi7uNWFQuLaI4qrIyQ2/cCVVrtRSJvf8zxKTiLGacItfzOFwivn
j4+Pk3h+SVMg3Ebh3GGVoLoJEtJxrz5XAYXLXgFOiVMkxEV7HtvsolBr/Mhw
/QwJ/zg35SVCCvUjporZ2S/96ar5MwEU/gM9OzaU
          "]]}, 
        Annotation[#, "Charting`Private`Tag$12664#3"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.922526, 0.385626, 0.209179], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwtj3s0lAkAxaeGMEw1Rp4Zw0fjHXpsefTdsiXPpW9ko7SkIQnJnIQ2SQ9Z
Gq3y6DjjsREOpdSu0qxFoTySZXczqQ0Vqp1olimydbbfOffcf373j2scGrNl
93wajeb3KZ+78PvQBfZhGet6jvZa0WgysqWX6xXA3UO2dqr7LGLIyLm7kmNc
7iFSza7RhtCSkYWNmUbK3HSyV5Bttc1cRka5CPdNGeWTAzNnPD76ysjeDxE5
MqMK0n0kJ8y2VEaqDDzjvTaqJ8cP/pJu5/mWLL4n3j9u1E7aK7aHsSonyJrL
6QNRto/Iqy1LCd6YnAwne/JfxQ2R4UFPIpX9FGT0iMVQXc0o+Y9v6AOR8izJ
ME7jPVkjI9PviQdtJDRkLb5vWfzdJOnefGRLmQYdoQP8X+kd/5LMsW+rPLuV
sf9OWmgM7z2pb8o/a96pitpYDUrXfZasfLGPzpOqg/Y/IGMf1yY0LURx0t4x
uwAabnBUkhuTFyFVPOP37tU8sKuVFZG5LNB6n6IhkY6pcl7l3iz2l70Seo84
Jrht1MKyeEEC31IZRZFZtcWDSzAsz92qfnsBAhrmnB1W6cJjJPLw2f2q4Mle
Wbyc0IfPsMiKw2GguadU01xqiKrKzzBQv9rrjN0QB2PNoSlt1epgXjvntCiM
C1tWv6HrTib6xMIYuq4JGKQ8RDq5EJkrpc13Ogi4vQ6cMvFfDJwwKzBrM8Of
TWHzAq1ZYPfb02MGebh+qiWYK2fBMo3HKJ+wgEBZ+LSrSxPO3wiilwdaY1dc
0nWNAjauHTu9KV9i++WfFsqvTNOp+XZYHm7V5RWihXhm5a3RQjt0a1qrC9cu
wUVBmViV6YBbNakXrOna2Pp3nTAvYQWS5ln6djzWhlX+eHCk/iowpImbo+t0
sGJHvmNJ52qMlBnniJN1ceCm4ZP2xDXIMddTqvfXw7LhUbdEgSNEBYLWEK4+
VqzUusz0dEZgzo4ujRf6SIjeHvZm3TpMf31RIZYYwML6r1q2CzA0siGh6eRS
DKeruZ72X48SF3uDzhBDzDbJFUHnNqD6uJAVbsOBTnDuc79uV5z03+VJl3Pg
1efkMMXaiFXS4oqTnUa4Up80q310E/IsLlR3enLhPV4yG/DSDUcaM/M867nI
T/O6ui3FHd4G/hfvmRqjrogtiDX1xPro1iBWujEq7rppeki8IF4aatohN8aB
g5eI51k+MG0uP+8TZIKMqYUsPUtfiGyaLB9KTGCjp/LyBssPgkmlnnZLAjrn
MrylOlsQWjvBTT9DIO7Rzz9MalOISqVmPLIJKDGFM4cNKAipuj80fiTQlH3C
SY1L4ZRcmCU6T6DIQs3cxIJCzVrFh9xCApvUyds7nCgofpvrL6si4Gi+4M2H
nRREDzUyW1oJ7HFha/dVUCgo3Rdxop2AeOSrokM1FErju1w33ycwHdE2w7lG
4Ya26P39rk8+X2QT1UBBGsiO6O0joNh6IHZxNwXekJ7rsyECG8Vu2SnvKNjV
JXJ+GiHQ1vzA20FBwfH4gGL3CwKBN6nB4VkKXrzCK6NjBC51VSZ5qfARt5fL
efuWwKOGgEozfT6SnY8qrk4SOCbaozbI4eM489nv8XIC7hJnl1yCj7zLpRnT
0wTUVDMIDRs+SlKUwm++J1Cgs6a/1Z6PKr/dG5JnCAgjGMFpq/moM7lruO4j
Aea0/q31TnxIJpcp5uYIBNxJfT1H8vEfox0tAA==
          "]]}, 
        Annotation[#, "Charting`Private`Tag$12664#4"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.528488, 0.470624, 0.701351], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwtkXk01AkAx8c6ageJkCNmxs+6omifrq1+X8dLhCfnRpdxTXIfrTDGGbbU
1Hq51kpKjkWsvE0ScoSMNGV3o7ImyrmjoVztbL3t+973ff/6vu/7vC+DGers
9xWFQjn0yZ+zMIEpY+Z7bt/Gdw5mFIqQbOfT7T3oJ8nXjChNZ1khKe5sTqHT
z5BZ3CSiUlVIFrZk0aTpmeRvLSs/R5gIyaC90cEfaHmk1tIPJR5eQpK/wsoW
0srJ7SdXoti/C8k1Q6P6M7Q75JGhadI1Zo4s7ikKn6J1kxvaKgJUFERkdU3m
UNCW56Rl41TSK9F7MoAcyJuOEJC7CLmVOfMVMmTMUFBfPUHOnScy22QooDJS
9V/tFJLOb7d2bleWxIX1vUbFJ0QkWy/ORoUpA+aQ633JR+9J7lgq43I6FeEd
qcxQ/WWyeDYlYDcUUBsm56Jm+5E0HNC11DJQAuV/IZe7TTD9qzKK405NmnpQ
kKjikd/kqILkotVD89MSyFhr7ltepQYKfwRNsZKQEjoOOYRu+tKXQvQeB0On
l1rQi/KPcTWShlJoXdR8Jh2vF3LcZe/JQDRQ87iBQsBuLJB9OXwt7ufNtJz7
Wg+Or7mbtbWpaM2360gJNkRlxWdR0ZjCalKuMcLkA2biwypZnFFQ1pwWG2OL
4qCW1XF58ANl3XNqTEElF7yHRetAE3sFeaxug82M5wcdt/XYllawv6nHHH+2
+Up4GitihH+adXh8J25ntB+jLyhCQlxR9+abPfCXjh7h8ZQQa/OhOUcW8ImI
uy2XvwEdmeOseA3LL3zKMCuxrTaDFbYGbObZeyuDYNHtkr63Rr+SsWz0LhVU
nV0OmZa3wd3q5AJjSVWYuLjM97+wRZyEkdOjF6oYWL0V+Z2FA6jDsQdC6jeC
1p1l1x/rhLFSRnZRvBoC1thfjvR1RraButQdN3V8DGt9Ol/jCm6+f5c3XQOl
Q2G2+9d5wDP7KE/ujQb4eY8N2gsOY9H6xlJRsyZCr+p1TZUdgWDMMqYtfRN4
7puXyd7juLbXTLPPWwtqW8ru6rR7oyotWjHARBueyZM3I2p8kO7mc1ByQRuy
Xjq1R5v9YD5cXJ7eR8O38+W2naYs5BoWVPUdpOMrducJ3dpAcFqycg/eoeNe
zAvBbFkwHDTdbvToMmBvMf9LUnAYLEK6vBQzGeiZOhY8IR+Bok1M3UcLDFi/
NNA7sxwJ3Qc3rzh66SAwzOP03/9Eg2vSZvSkWQcHMh67H5eOgb9IaqDbiEBB
2lWf27GxYNa+o2deJMDOWBmZXYxHULLLqt0lAj+Oh8qclWIj2qX+D7mfCCjk
8u0Y69nIWIi+wL1CoHgg09xXn43qXUsrOYUE1B9WNGi6s7HUKh4srSRA0SjL
cK5ng/tELqu9i4BU6Xzq28gE5JcEs852E0icb6LKJCagJIpndaCXgFcl8Zf+
+QQ0qHKXe3kEdFLV3WKuJ2DYcwOL/+zT/szgrP2zBOgL1K1GBQSebbR42rGD
A9P6WO3rYwT8TzekGFhzsDttaMnvDQHmrP6li04c2OsX3pqYJMD/yOwNPMlB
xCm69twc8em/tQK3Ag7i9yQt1YkITNw3Khu6yUGa/OjTqAUCV7dqDPrVc5Bb
U3JucZHArG9/eHofB9cSpQIalz/x1jqMaj/noPKQn2X8KoEditTWxnEO6nU6
tfb9SyCm2lvBS8RBs0hvSSwm4LiujicWc/AfuWRE4g==
          "]]}, 
        Annotation[#, "Charting`Private`Tag$12664#5"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.772079, 0.431554, 0.102387], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwtlnkwFnz3xi256bbedrJFZekuKTuP71VS8gjdWSoSEtlJElmyhRBJIRUq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          "]]}, 
        Annotation[#, "Charting`Private`Tag$12664#6"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.363898, 0.618501, 0.782349], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwtlvc/FX68x4/DOTjWOUcKCSmbskXxedvZ4hhJCBkNSYhkKzKiQpIRkdEg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          "]]}, Annotation[#, "Charting`Private`Tag$12664#7"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[1, 0.75, 0], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwtVHk4FHobnd3McKOxRHWRJVuF6MrW782ahMiWhOyUNZUlsmUJJXuJshSJ
FCqmQqWyZS3XUlFkGTN2YWxf3/N953nOc/553/PXOWeXo4+ZCw6Dwcz94X81
J8yRpOiccIhP+/VhDGYGNXSLHrMS9UBHisyrdnDPoM0PtVGiokGITW39pC88
g3Lqk0SIovEo7pWXQZfGDDqnecFrSeQWwlvfeF4RPIO6V93TZkQeIipR1qVq
ZQZxDPyUYonUoLEMz7CTxDnUW//6AnO8Bgm9u9ldJTCHSh5kvZt8SkfZmxWl
4tJzyNjf2J6h/Qpp+7uZaBjOoQwKPWPMrQ5VxisK26bMIdqdRtzsv28RWTIg
65zEPMprvus3KdKEciLjyq5bL6CFrGeyg4lN6NGD/mJN7wWk59Yy3LXShLa9
/p7LEb2AmPglC3p3M/p3TzMfq3wBHdQ0VouPbUW9NGtPHGURdZSv4aSm29Gc
qWdOUP0iutK856iifTcq3E95Xau7hIqfvtj9KL0bhS88sNxpv4S6srTwkq3d
6FK3k8rtwCUk4Wb9SlD1M+L97nVt6dESasZH78PwfkGumTvLNPmWkYDmAK39
Qw9CAaPZYpPL6HF5/MC5ff3InxLV+6ySjRg0RuLRM/3oqk4Pf1Q7G0ldPHpI
Oq0fNTP9hf0n2eieBmfe8Eo/Cq9+afRcfBWlNCW62rwfQBGtYjZf01fRxeEb
s/q239BjPGpvCl9DdzlborbqDiH1vazIG/4b6JBmiG243RAafGlIXkrZQF+9
5Q5MXxpCN9isjpjKDSTYlTDaWjKEks8obRVd3EA3s44ZxPH8QKeWnR6g4E0U
Jdm2ZXPgB6qb6r6upIcBN9R5i+k/jEzU5+8RPbCAyf9NH0gYRln0c3q3QrCQ
Rdj5tblwGJ11xPLoXMdCY5Or8MOeYYR4k1MHK7Egbb6W76I2gsaJAYT9GByM
u0uVfsf+QvbfYyIcc3DgmRJa25E8iri6PJVZDDzgFvMH6x6OIp6ezvMPMQS4
bdWIKX87irLeGDpHCBCgeSevdtLCKGpkirCjtQggV1T80cB6DN1jnD8/mE0A
5svujrci46jy4JgRyZwI3r9khqseTyDnHnyLyQAJhuz6yG+aJlCsAvc8ZokE
J3rj9n0amUA2Uqf0+2gccLBlLOjXdgbK0yvzxxhyAP7pfR6BWAbSKwoalH/N
AVmXdx26ZD+J1Pms5489JMMbXqFbqjwstDzJbnbPp4JyUmOtrhwLUT7qJUS9
o8IDUuCIqR4LJbgnWbaNUCFxpUfe8zILedByjBalOMFyMO3D7XEWav6Zn33z
CScwSngWVuun0Fy96r1jLVxA3RUtNXhwBikbcQ4oy3BDo8ZN+TDdmT+5Fjpl
bMwNMda5Kn+bzSCVinCv7PPcgEuu1j91dgYdax6/2VnLDewNptu/OTNoG704
eECOBya/WhS342ZRl8TQ5d4mHmjNkpKpb51Fq7kuOed20OBalbKifd8s2lJ+
rTtWkQZHOg6rbvyaRVbP6Ylv9WnQwGFroLE5i1gHdlpmBtDg1cVkj2rFOZQQ
85n9rp0GZeYrD59kzCEl2zsh1om8cJ2nRTbPYR69+1z2jS3CDzUqfz8z9p5H
430l0SUa/DBi54PWQuaRmPuu4tiT/KBexmtulTmP6lMky1tS+WHcwDb0r7Z5
tExzYkVRBUA7itUepL6AbASdxzDYbbC8yH3BTGARLZXLFD7hEwLxvx03N8UX
UdU2hulrFSEw1qmKL1NcRLZFnxTxp4Tgfop1LtloETU6Titp5AuBmXzex9qo
RSTyvW63j9J2KHPfv11udhGNTv9kqp3ZAY4D5nX41t+o0aDocxnrb6Bm//Dy
6/uNUnGpnMvbhKHCxnvn4OhvtBZlR43TEgZ8f2wQHbuEho7G+IRkCUNhL13J
T2UJWYhFXK3VF4HRL6LF3/OX0Iy4QtQ2G1Hw7GAm1wQto0+5lmMOI7uAlhyE
pGKXkaV7t5/Xxi6gm5Cm0tKW0f5/+3QfCIoBZ7uIoW/5MjpT1SieayQGpZ/M
iFK/lpFaKt30SY0YTDVXB6UdX0FBd9Paym6Lg9/7aEcfKTa6rurWWREpCcIj
eqTfSmzkYC5m0nlPElpwlJLLwEavjFY4d9VJgiQkzcafZCNVD7kqizVJ6KOn
hxcmsP/s6UescNBuOPzkQW7fNBsJc2XClctSsDW7sV+7ehWx7bWSLMpkoLbm
WlhzwyoakMvYbtclA2d7j4mZdq4i/npHjvtLMvCev9PDjrGKKH1CP6a1ZCEo
uW85cOcaElEMYcd8lYUfVxkCj//s2j9ZG54yO/fAU1+uE4IG6wjdxAdPd+yD
1tu8QWWW6+huWrB0AU4eRhu239VyXke3aLUXqpTlYbuQzOS5K+vo4v50/vlb
8hD9RjfqzbN19DY7IzhEWQEsaVcqPcU2kIqjLxeDSxF8NWL6NuU3UGpovNFP
aUW45pq0maa5gW4PlUjw6CpCHT3bsM56A909xT67JUwRpJ2qh3lvbCD1v1Kv
5M8pArtylvf16gb6/UOlMH10P/B9X1Y1o2wif6bXL1GSEuwjYxzGBDZRudp0
/oSkEjjZbindun8T7SkqrD7gqgStBDkdV/dNVLFBL/g4oQS5Fs4B3F820cGW
+lrTdWXA/A+wc7v2XF/AAcgLOctQsMJATrW57xEbFVCopyzHnsYAB19zx9IV
FagnFBMHnTDwJdlZZeq+CgwljYgm+WJgWzJXaMWcCgjfO201EY+BPJ9LpJIb
B+F2g0lD3isM1LW3Tf/8VxVkKVOdy28xIOac+t4IpwY1RomDJk0YWL3wm0HZ
qwa9PY3s9S8YSFLLmQ6MUgOBicOKNtMYKL210KarrA4pfynn0sSwkOHPrQUP
NEDMrOuRhzQWyg98vL6tRwOeZvjW1O/DQmTOVZljJE3oECn77K2OBQUJRkKH
myZs2b+bs8UcCzrBnYan5Q/BNUvBwKhYLKiF5K3odyKIvLtmusDEQsy7GVX/
ysNwXKHaT24eC9p/tQtd6jwMwm/O33RcwcJjur3Pi+nDQP/J6Ogg4qDgMyuT
uUcL5iT7TMqEcaBqsHtWpUQLHEufGbkex4FfQsNi+xNtUND088qxxIG03FvX
ti5t2Pi0J+mzLQ5aDxg7bV3UhtvTBa1aHjjoCj8Y5KamA137UwxFInEQJm3c
7NKoA4drvA16K3GAm92z7dCcLmw5KuvBTccBbciI1C2oB1/7f8Xp1eNgSP/U
vhdIDwLXbBurWnFghYp4fa/rwRNkqH/zFw6KDWbJNfL6IPpBSvfoNjxESf5m
mUceAUz3ELwKxsN6H9ffN70MIXqiNi84Ag9ewS7hIwWGQMHk4FTj8PDr+Y6M
Z/2GQNt7suFZOh7oYrXcPQbHQDK2U7/8CR5GHubeOrDPCI6qvzXOH/3jd7vb
pYjHBNpM75U7sPBQxiXVHGxiAifcw3hEFvCwgE278/O6yf/zSAAbY9Mlbsxx
sE1X68rGEmC+rJ9B5zgO3tMVFuk7CSAlM90zmXAcUgsKbOPMCNArriMQUWAK
gvSI13on//y3PH45+9EU7nTYCxMdCBAwkXaCzjSFB+s7hiK8CKDq4CFjpWIG
1VZpTiFxBJibGKL3d5jBV86rnt61BBAVGBKf4jeH3QGugeayRMjTddMVnrGE
vWqDJ+4qEKHGwBdvImwFyhhrecY/RLCSF/FkHrMCrUSD0SvaRJjymf7g98gK
7Ar3mD+yJcJw4eVbH72sIePznDzuBhFk5pXDK8g2kHP7LKdROhEq7qh1HDpk
A4UOI6OZ2UTYerO0xTDABiqYX3L2FhMhaovLcdYPG2gj1HCefEME8VeCFhff
nQLigStj5fNEyK8+a8C6dxq4VpfesleI0BU9s6A0eBpob3xzdTEkcOErSeQV
tgNRI0eLAS4SGPcTNQtz7EDDRfcdaTcJBs9KLKoX2kNAOufd09YkGAljss6N
O8DIYqYl52sSLLyQ8/1m4wT5h5XOf3v3557f2xkT7wT2SW03yptJENc10lfw
wgn6JYhNJ3pJUK3/u8pfwBk6T/ir35knQX2GraddvzPUPj0muleWAyZzX7Hm
L7vC5fUxjU0FDgg1dzvErHT905uok50qHOBYFZwSMekKVUM1KRd0OaBlx0vT
BFs3KOGWItY6cECgrZychZ47ZHphGcaZHHC+vGdvqZonmNfcIe3K5QBDn7dO
Fy97Ao14UHy+kANkc/t8sHWecP2Ot21mBQeUspUUnPXOQnTLQNvgJw4wvgdB
9+3Oga/M80pfAhm6wy8RI194w74Lph1anGRo1wlczSX5wGQ9k8lHI0OxmSJP
iaUPuFqL764RIUOCvHhA5ooPnI5NzsKqk0GNkr6Zft8Xjv7yDE3xI0OPysYZ
KR5/0AsVL9UJJEPHpq87H/IHLf6v/b/DyDBl5jl6w9sf1HSNDp5KJIObuuRf
au3+IHtffl686I+/fxUtNfM8cLosuD/7SoZvRfd2KOldAPJ6aabbMBmkQuI1
sVcuACHd5YMQgwze0f4F4dUXYK2hRzxsiQymI16yOXsvAlOi5rsejQKbVz+M
polegtaRUPNefQqssQcsh+SCoPHyP1HXjClgoYh/hPMLgga+6acaFhTAyGhN
4l4EwSsdB+48Rwq0ihWuPtQNhtJCrWaPUArkY558m/IKgSRnDlitoICR69Q5
V1YoXFur8y6rocDYkZmLBK0wiEkLzLGvp4BWgbMEPTMMwhom2O8+UcDDqQwf
qncFvCVanyWOU+BTSVMI0zocjEeS5YSFqdAaSy9f+BEBrXIjn20lqBA8gNaj
aZFgeF4lLFuWCt8Kmr3KtSPhCO57h6AKFSLPHYkYKIoELVG5i7zHqaCuRptr
vxQFB2zf15MjqWA5vlSme/AqVBUIeurHUWG60D8i9fxV2D95ljfmOhUEtuK5
Csqvwr5gmis+mwpZe1EhTTYGHpX8F1TwSRhjxTTFgHSWPedGJRUIQqHbY6Rj
4b3+cVZNPxXG4w0nGF1x8B9UfOtH
          "]], 
         LineBox[CompressedData["
1:eJwVkHc81QsDxomMMiJ0Ofv8RGahcKV+j4yICFH2iJAQl7g5GjKyIpWZ3DIj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          "]]}, Annotation[#, "Charting`Private`Tag$12664#8"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.647624, 0.37816, 0.614037], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwtVmk0FQrUvRMilQgNRHpFZhoQOsecWWYlc8aovKcMDeahyBC9IgrpoYSi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          "]]}, 
        Annotation[#, "Charting`Private`Tag$12664#9"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.571589, 0.586483, 0.], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwllGk41Iv7xr+WsZQsKUciSzGiLKkky/MIiSzZS6XSQkiIShM5EQlFBynR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          "]]}, 
        Annotation[#, "Charting`Private`Tag$12664#10"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.915, 0.3325, 0.2125], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV1Pc/FQofwPGDzCIRlcyiY0ZZEb7fjCJSRnQRdVKIJDdlZEVkhqwUDRIK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          "]], 
         LineBox[CompressedData["
1:eJwVVWcgFX4b5S7XNe+1R9ZFSEaDiDyUUSiEv4pIWSmjUIkSFWV0jcxsSqWU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          "]]}, 
        Annotation[#, 
         "Charting`Private`Tag$12664#11"]& ], {}}, {{}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, {}}}, {}, {}}, {
    DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
     AxesOrigin -> {0, 0}, FrameTicks -> {{Automatic, 
        Charting`ScaledFrameTicks[{Identity, Identity}]}, {Automatic, 
        Charting`ScaledFrameTicks[{Identity, Identity}]}}, 
     GridLines -> {None, None}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> {FontFamily -> "Arial"}, DisplayFunction :> Identity, 
     Frame -> {{False, False}, {False, False}}, 
     FrameLabel -> {{None, None}, {None, None}}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, 
     PlotRange -> {{0, 10}, {-473.71542035127914`, 794.3647027875386}}, 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{
       RowBox[{"-", "0.5`"}], 
       RowBox[{"-", "0.4`"}], 
       RowBox[{"-", "0.3`"}], 
       RowBox[{"-", "0.2`"}], 
       RowBox[{"-", "0.1`"}], "0", "0.1`", "0.2`", "0.3`", "0.4`", "0.5`"}, 
      "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #5}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #6}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.363898, 0.618501, 0.782349], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.363898, 0.618501, 0.782349], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #7}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[1, 0.75, 0], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[1, 0.75, 0], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #8}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"], 
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.647624, 0.37816, 0.614037], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.647624, 0.37816, 0.614037], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #9}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.571589, 0.586483, 0.], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.571589, 0.586483, 0.], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #10}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.915, 0.3325, 0.2125], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.915, 0.3325, 0.2125], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #11}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.560181`", ",", "0.691569`", ",", "0.194885`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6150173333333333, 0.25708400000000003`, 
                    0.13945266666666667`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.922526`", ",", "0.385626`", ",", "0.209179`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.922526, 0.385626, 0.209179]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.922526, 0.385626, 0.209179], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.3523253333333333, 0.3137493333333333, 
                    0.46756733333333333`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.528488`", ",", "0.470624`", ",", "0.701351`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.528488, 0.470624, 0.701351]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.528488, 0.470624, 0.701351], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.5147193333333333, 0.28770266666666666`, 
                    0.06825800000000001], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.772079`", ",", "0.431554`", ",", "0.102387`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.772079, 0.431554, 0.102387]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.772079, 0.431554, 0.102387], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.363898, 0.618501, 0.782349], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[0.24259866666666668`, 0.412334, 0.521566], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.363898`", ",", "0.618501`", ",", "0.782349`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.363898, 0.618501, 0.782349]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.363898, 0.618501, 0.782349], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[1, 0.75, 0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> RGBColor[0.6666666666666666, 0.5, 0.], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"1", ",", "0.75`", ",", "0"}], "]"}], NumberMarks -> 
                    False]], Appearance -> None, BaseStyle -> {}, 
                    BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                    ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[1, 0.75, 0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[1, 0.75, 0], Editable -> False, Selectable -> 
                    False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.647624, 0.37816, 0.614037], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.4317493333333333, 0.2521066666666667, 
                    0.40935800000000006`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.647624`", ",", "0.37816`", ",", "0.614037`"}], 
                    "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.647624, 0.37816, 0.614037]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.647624, 0.37816, 0.614037], Editable -> False, 
                    Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.571589, 0.586483, 0.], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[0.38105933333333336`, 0.39098866666666665`, 0.], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.571589`", ",", "0.586483`", ",", "0.`"}], 
                    "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.571589, 0.586483, 0.]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.571589, 0.586483, 0.], Editable -> False, 
                    Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.915, 0.3325, 0.2125], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6100000000000001, 0.22166666666666668`, 
                    0.14166666666666666`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.915`", ",", "0.3325`", ",", "0.2125`"}], "]"}],
                     NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.915, 0.3325, 0.2125]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.915, 0.3325, 0.2125], Editable -> False, 
                    Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             
             RowBox[{#, ",", #2, ",", #3, ",", #4, ",", #5, ",", #6, ",", #7, 
               ",", #8, ",", #9, ",", #10, ",", #11}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.6848556449899597`*^9, 3.6848556531587267`*^9}, 
   3.684856535080687*^9, 3.800107821248217*^9},
 CellLabel->"Out[29]=",ExpressionUUID->"00fd225d-e400-4ebf-a43f-29849c4cced7"]
}, Open  ]],

Cell[TextData[{
 "The curves are produced by the Coupled Exponential Function\n\t",
 Cell[BoxData[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"1", "-", 
     RowBox[{"\[Kappa]", " ", "x"}]}], ")"}], 
   FractionBox[
    RowBox[{"1", " ", "+", " ", "\[Kappa]"}], 
    RowBox[{"-", "\[Kappa]"}]]]],ExpressionUUID->
  "312f8279-bc21-4fe6-b4f5-fe326ec79d22"]
}], "Text",
 CellChangeTimes->{{3.6608489130250063`*^9, 3.6608489299461737`*^9}, {
  3.6608491241044807`*^9, 3.6608491268405533`*^9}, {3.6848556336461563`*^9, 
  3.68485563479657*^9}, {3.6848556733813257`*^9, 3.684855673381452*^9}, {
  3.684856610085166*^9, 
  3.684856615175021*^9}},ExpressionUUID->"cc367a38-1217-4c79-8feb-\
ffa640a4e277"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Graphic of Coupled Logarithm", "Section",
 CellChangeTimes->{{3.660848838447104*^9, 
  3.6608488445977583`*^9}},ExpressionUUID->"20f42ba2-4670-471c-ace7-\
e85f27b11c7e"],

Cell["Graph shows curves from linear to logarithmic", "Text",
 CellChangeTimes->{{3.66084903460938*^9, 
  3.6608490462009687`*^9}},ExpressionUUID->"7487311c-0c8a-46a7-a5f2-\
93a40b940424"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"CouplingValues", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.5"}], ",", 
     RowBox[{"-", "0.4"}], ",", 
     RowBox[{"-", "0.3"}], ",", 
     RowBox[{"-", "0.2"}], ",", 
     RowBox[{"-", "0.1"}], ",", "0", ",", " ", "0.1", ",", "0.2", ",", "0.3", 
     ",", "0.4", ",", "0.5"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Quiet", "[", 
  RowBox[{
   RowBox[{"Plot", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"CoupledLogarithm", "[", 
          RowBox[{
           SuperscriptBox["x", 
            RowBox[{"-", "1"}]], ",", "#"}], "]"}]}], "&"}], "/@", 
       "CouplingValues"}], "//", "Evaluate"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", "5"}], ",", "5"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotLegends", "\[Rule]", "CouplingValues"}]}], "]"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"Power", "::", "infy"}], "}"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.660849068393867*^9, 3.660849104658946*^9}, {
  3.684856099862259*^9, 3.6848561129408197`*^9}, {3.684856208458323*^9, 
  3.684856214832344*^9}, {3.684856669002672*^9, 3.684856678737275*^9}, {
  3.684856721966455*^9, 3.68485673334907*^9}, {3.800108689910988*^9, 
  3.800108746560917*^9}, {3.800108887852624*^9, 3.800108975319233*^9}, {
  3.800109013022809*^9, 3.800109061524088*^9}},
 CellLabel->"In[52]:=",ExpressionUUID->"e104abe3-0b70-48d6-96a1-aeaa5993f10a"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwBkQJu/SFib1JlAgAAACgAAAACAAAAQD1a5esQ5D6mGhTv6///v3wClMVG
OEk//5pO7rH5/78ummPHFDZvP2acOOvJ4P+/3hmx7gsPfD/MnSLo4cf/v1Mz
2LyGQYQ/M58M5fmu/7+22VeCh3uKP5mg9uERlv+/DcDrI8RakD8AouDeKX3/
v3Fma+nElJY/zKS02FlL/7+iOStMxbGZPzOmntVxMv+/1AzrrsXOnD+Zp4jS
iRn/v5xZNTpjhKE/ZqpczLnn/r8AALX/Y76nPwCwBMAZhP6/mekUMeRMqT9m
se68MWv+vzLTdGJk26o/zbLYuUlS/r9kpjTFZPitP5q1rLN5IP6/ZCZaxTIZ
sj80u1Sn2bz9v8fM2YozU7g/Z8akjpn1/L9oeNrHWyu5P/OwBIeU2vy/CCTb
BIQDuj9/m2R/j7/8v0p73H7Us7s/l3AkcIWJ/L/MKd9ydRS/P8YapFFxHfy/
aENyrdvqwj8mb6MUSUX7v2ygd5UdrMk/5ReimviU+b86LcGyUJfTP2Npn6ZX
NPa/AnYV3GQZ4D/8E9VHNs3vvwcMUIFi7uY/8udf/Toj4r+D4NZDcqPtP+j7
SOFt5MK/sJuGUH7y8T8Au2kI5SfPP2+iZBDkVvU/vImSQZBb5T9eapodh4H4
P7zUNDsOA/E/iVF2OTOc+z8So+xyZjj3PwQUleZ/+v4/Cigqzf/0/T/Xy4Xw
hA8BQK6XC+EJHwJAVXtiNprDAkCq9sRsNIcFQGwL6yLOWgRA2BbWRZy1CEAf
q8aWBuoFQD5WjS0N1AtAe7hDUw+bB0D2cIemHjYPQHCmbLY2LwlAcKZstjYv
EUAMAjdiLuUKQAwCN2Iu5RJAaRCEStl8C0BpEIRK2XwTQH6KSEo=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$44224#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwBAQX++iFib1JlAgAAAE8AAAACAAAAQD1a5esQ5D5k3sZBsf0DwHwClMVG
OEk/1k3lFwDVA8AummPHFDZvP3QHqHQagwPA3hmx7gsPfD/2W2EWU0cDwFMz
2LyGQYQ/1ZZJRBYUA8C22VeCh3uKP0j/myru5QLADcDrI8RakD+86Oc1SrsC
wHFma+nElJY/EU6MEmBtAsCiOStMxbGZP2qEt6QrSQLA1AzrrsXOnD+A2L4g
ZyYCwJxZNTpjhKE/HlrQd2jkAcAAALX/Y76nP/1OxMseawHAmekUMeRMqT92
UgzHik4BwDLTdGJk26o/1LfoYYsyAcBkpjTFZPitPyp4a1oe/ADAZCZaxTIZ
sj9X6B3ympQAwMfM2YozU7g/5mN1M7ur/79oeNrHWyu5Py/s2k+rev+/CCTb
BIQDuj/MBPceJ0r/v0p73H7Us7s/Ltn/Narq/r/MKd9ydRS/P/CWDi5PMf6/
aENyrdvqwj9mC8qOktH8v2ygd5UdrMk/+qpZVGBL+r889vezMRjKP7bCpsID
Jfq/DEx40kWEyj9PkXcD3P75v633eA9uXMs/LbPHjSaz+b/uTnqJvgzNPyUv
7ugGHvm/uH6+vq820D9mmLSoKfz3vzotwbJQl9M/1Q9KD/DU9b811NaWvsnT
P/r+5tzPtfW/MHvseiz80z+htVMQypb1vybJF0MIYdQ/zz/xRwxZ9b8TZW7T
vyrVP3an6MK+3vS/7Jwb9C6+1j+EKO/4l+7zv58MdjUN5dk/bcnPOEke8r8C
dhXcZBngP2qP+K0jZe2/BwxQgWLu5j/42jR6td/fv4Pg1kNyo+0/iPtKN0Lj
v7+wm4ZQfvLxP5QijhWTdsk/b6JkEORW9T8OpUZSeO7gP15qmh2Hgfg/vzhk
YEFM6j+JUXY5M5z7P8qhuDkkjPE/BBSV5n/6/j8MngS4MiP2P9fLhfCEDwFA
vZUSIqBF+j9Ve2I2msMCQPGxDzrunP4/bAvrIs5aBEBEYgepTUYBQB+rxpYG
6gVAVwwEWaonA0B7uENTD5sHQILfmY8PJQVAcKZstjYvCUBOARtdmvUGQAwC
N2Iu5QpA0caCmC7iCEBFbVSVKpMMQGt9P4u3uwpAGLkdb0UkDkAyD5HjJG0M
QJJyiJEw1w9AOluNMbU6DkBShk8tnbYQQMrNCTVD4g9AKluEVaR9EUAJaOtG
hL0QQNbmCaKTVRJA69S1Vt6XEUDOYuVBEh8TQDv+NMGdYBJA4QSn+JUiE0Cg
sj/OF2QSQPSmaK8ZJhNAm0rOpJFnEkAZ6+scIS0TQEJzrK6EbhJAZHPy9y87
E0AiiuA2aHwSQPqD/61NVxNAUly0JCWYEkAmpRkaiY8TQATVStF2zxJAOUfb
0AyTE0AP2Tko6tISQEzpnIeQlhNAUpJMSl3WEkBxLSD1l50TQNKyD/BC3RJA
vLUm0KarE0C3aG3DC+sSQFLGM4bExxNAJWB7lJMGE0BlaPU8SMsTQCyJh6MD
ChNAeAq388vOE0AeQn9+cw0TQJ1OOmHT1RNApKVimFIUE0Do1kA84uMTQK3j
UF0OIhNA+ngC82XnE0BTYBHNfCUTQA0bxKnp6hNA0WwfCesoE0AyX0cX8fET
QIDVVObGLxNARQEJznT1E0BPdZSHNDMTQFijyoT4+BNAMSxS9aE2E0BqRYw7
fPwTQIAPmi8POhNAfedN8v//E0CkL3g2fD0TQAOaSZU=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$44224#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVjX841Accx487P+7rGvleq2NG6ZElETmp2z4fz81qJqmmLenXOU1GfoSu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          "]]}, Annotation[#, "Charting`Private`Tag$44224#3"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.922526, 0.385626, 0.209179], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVjns4lHkfh2c84zwbevS2tu1SvQ4RrUNs4fX9FZMllUhFSY2xSUJOO4um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          "]]}, 
        Annotation[#, "Charting`Private`Tag$44224#4"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.528488, 0.470624, 0.701351], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVkvs/1PkegMcYuUR2fStku8i8iJCIsOX9KSkk6whH02U1Rk2ZSmodrZBr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          "]]}, 
        Annotation[#, "Charting`Private`Tag$44224#5"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.772079, 0.431554, 0.102387], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVlGs4lPkfxieJwWispxOSvxLWKVFErr6PJpeUyrJsSarZUdaqJGk2Q2tJ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          "]]}, 
        Annotation[#, "Charting`Private`Tag$44224#6"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.363898, 0.618501, 0.782349], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVjnk4lHsfh4fGEop69KLSRkKR0pEm9f2V5uRoE6VSbzFndLUocqTpWFpM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          "]]}, 
        Annotation[#, "Charting`Private`Tag$44224#7"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[1, 0.75, 0], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVkmk8lHsfh8eaNdXtJOGUkkopsoScfrckKZXT4ogeaZokqTQiWSYxSXik
xlLhICWlJEso+d/2MmEwRhlqshvr3JbI9nhefD/fN9eL68WlSb1y9Jw4hUIJ
W9z//+D9gDb9r5lgaSAIY1BkCVq6YSVt6jXgXAeXQH0ZYu5ezGRX6BtgLDSM
U58vIdjmv1k54dmwUj2+RnLDEoKm7Vi5QzUH9p+J1A01lCb01OS+0wU5cG2H
f/LmSiliTun9ZM7zXBg+knFPxkmKiJtW3WRgnA/RuZvyKmIkieov38INjhXC
rawL9z4bSxBuvq3VTluKwDtx3qlRIE7IaPElmRLv4Uej/6pNUeLEAf82Bjf3
A+wbyi3jicQIoXb7h9mIYvDr+O8l9nMxIqKxfUqL9hFmW1z1LV3EiC+bf9C9
lREIos6PaH6lEB7NP94kDSLw3MBdujeeQoTinpNUKgGC/P/wnR0oxMZBT4MI
s1Kwi3EwWU8sIDdL+mu+sAyKze6PIal5ZPKIZI/Yl0O7osuu4//OIZkRulCi
vBwG38qfDjadQy8ee23cmlABIQ4JnobXZ5FQdC014GAVpE6y25QUZ9B76wkU
XVAFsVy1/SEFv1F4kvf3p+urQdq37MaXs7/RFhsftdqZalh7PvZdfOk08ki5
Hqvx6jN8DRPK2UZPIfPJX7n6q2rg+ob3EinWU0jB1rfRilkDrPl/BEcoU+j1
L1+ly6fYoNyseyDG+xcaPux3t0ShFlQzYspuX55EhYySkjfGHJiSS6YbPRhH
KS/ma4s8OLDV08ik3HEchXF3t5c/4UD+2u6/tLTGkYMOmmlZ2gAjCWMGNkVj
aLoZmVB6G6ByslY2YpBEZrqlOXbxTQCSu5UG6CJkxOXFlXxugmpj6QpHGxHS
8x/02zrbBAL/AHurtSKk/VnFUuYMF5IkjxUF1Y8i7NzlJqTTDC0r3G1sDUbR
YKLa+LYSHhh+eBGTqD6Mei31vyaO8uDyj+M9ebwh1CHcVyy3vgWGj5GJN+8P
oW8mdGbPnRbg9S0zXiM7hKq4n5T//fsr+DzLLNKjDKJkBR8jxe5vUGa/Lmiz
vBA9zo1Q9VdpBR3S85dcfT+Kc0yd67NpBTP1UVcvVj+KzPhSWZHVCk+We93i
aPQj373r7QN8+UDrS8u+ZdKHjgZwfAbk22GZ3+qdK5k9SNznjzaaqgA6wuca
8t06kaPLy8ZCQwE05ufJaBt3opwD8FnBTgDZDtP20xKdiLrmwru8UAFIX9ib
UZ7Sgco+FUdLji/yrlZvS7//RCFqNKundT/hVQcReeiiAEmW5mR1hnRCtrEt
x4zBR6cyrZ/tTO4Elxd3PlF1+Sgvti0h4n0n2PJccxTbWxHNXfqugWiRP/Z6
T89fragSczwb4twFia56KjekvqFQV7FV6027Ac9AYSMveWiJgt0t6nAP+OYn
T4Xua0QOGx0JY9leSPEIon6UbEQv99AW5LR6Ybzg3ZhteQM6fMM3MPdkL3RF
tS2/ZtGA4npSbkhW9oJM/iCrxoKDtEtFV9MT+qDv1U/1mrQvyMqHRRXuFwIl
fdjD70QliruflFpyVgjZ7N1y5t0VqO/Vc8EDhhBGNVh+wd4VKLLzg/OuPCGs
Suu2fRRfjrh2XY6RawbATbyPXf+zFNG2GB3bNjkAa5c5NvWu/ohCBDxLetoQ
6OXqdGd5Z6PvRQcv+RYPAaEqJ9K/+QaZsYg4RvMQpBycJd2Vs9Co1cv+8CXD
EGS7ki5mmYmcMwPvpV0cBiLwjZ5/9jNk5qPF5xqMAIXMlVkdG4umpUyL1Kij
EDT20IK1KQayE0LtGKQIMlLO60XvLQQ2FtqrRCHBTkfK/HZHIfRE3GakKpLQ
dzXy8cjNIlAPYL6q2ERCdIE0d2PRewg7FSwjf5qEDCeJ07s3fAQXDQYRzybh
y9DCntMkAcuSvfXePh0DFx8/IYRUwpUntGVd9hNgR1+F5c/VQtSNR4lRZycg
knGpdsq0DrLs6jaZek5A0PjJsRU+dTA8v9Mi6u4EPGUb2rOH6sDDSZ5uUjwB
TKSmd5RfD+7KOU2RmpNgfrzGmd7CAdfQhTijwUmoi7kUWZDYCM7ujzXu3JqC
dYTXfD3ZDHf/9jE1SJyBA4eqIJzDB0nl2J6bv+eh/MXVzWSWAFrYt0eZSmJ4
t0z77eXTHXDCn2r8uFwcj4q9d9tIvxs85AfW1TlK4iy/DCe+Ui+syayKkxZK
4QmfSkWhxX3gX2h5TClhCX56XmRNOS8Ez3SPd8bbZfH7M7G2MrwBUBDfSd3W
LYfbbNfMTd43BFaXjsfqsBTwOO5CWm3cMKwq0rhipbUU1wxjOuwuHoEH9LD7
ei1KuJ+UoYWf1ygcDT2im/RuGa4QlEQSk6Og48Q88TRwOe75vXOG5SkCXo/s
E9aJFfjlFBXf2iER1FG7LVaswXD1PFKj+BQJPyWfT8Wuw/A/H5nYJDuTMJ7u
9kZVG8OX/iGmGOxCgurAgLqmLobPWpU476ORQPMif23fheE6zLPbay6S8Dtk
4fWhfzC8rJ6fUeJHgvYz1dV372F4l4CHnXpIgqk1n6PAwnDioXPSjsck2PYn
3omOw3BlJ5cKmUQSruqunYhPwvA2hYsDuckkFOdrc9IzMVzqU3Wq1HMSjlYZ
hFZUYzjT+jUnLp+Ec24T5vvZGC4oemh5voAEX7mCMXYdhp8sdLDfWURC0mGz
M03NGP5ERejEK17slIebd3Ri+Dd5hwGlikVfX/Gxc70YjmsMu7RXkqCoVvGi
X4jhg38yrr+sJmHHaWsVkQjDg9dqhFkudmslJlt3bQLDI5jXgpVqSXBIq2FO
TWG4Xqr9n/w6EtytIncFzGK4lk+vUzqHhMDeQ+TCAoYvHzmw62ojCf8Drk9p
Tg==
          "]]}, Annotation[#, "Charting`Private`Tag$44224#8"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.647624, 0.37816, 0.614037], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVj3k0lH0fxsdSZK3uCkmyNEihplVO37tEogWVSo5qTOWJVJR3SqZHI3sl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          "]]}, 
        Annotation[#, "Charting`Private`Tag$44224#9"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.571589, 0.586483, 0.], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVink81Pkfx92hkdZXB1IqqYhEuVLvr5XupJQiO+w0NhUdwk45dkXk2BTR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          "]]}, 
        Annotation[#, "Charting`Private`Tag$44224#10"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.915, 0.3325, 0.2125], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVjns4lHkfxgeTVCMy3mJ1zsqypBwqqe+za0VCKmZnh0oTXbYmRDQl1sYO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          "]]}, 
        Annotation[#, "Charting`Private`Tag$44224#11"]& ]}}, {}, {}}, {
    DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
     AxesOrigin -> {0, 0}, FrameTicks -> {{Automatic, 
        Charting`ScaledFrameTicks[{Identity, Identity}]}, {Automatic, 
        Charting`ScaledFrameTicks[{Identity, Identity}]}}, 
     GridLines -> {None, None}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> {FontFamily -> "Arial"}, DisplayFunction :> Identity, 
     Frame -> {{False, False}, {False, False}}, 
     FrameLabel -> {{None, None}, {None, None}}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, 
     PlotRange -> {{-5, 5}, {-3.1252413457317814`, 4.8719226496797114`}}, 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{
       RowBox[{"-", "0.5`"}], 
       RowBox[{"-", "0.4`"}], 
       RowBox[{"-", "0.3`"}], 
       RowBox[{"-", "0.2`"}], 
       RowBox[{"-", "0.1`"}], "0", "0.1`", "0.2`", "0.3`", "0.4`", "0.5`"}, 
      "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #5}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #6}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.363898, 0.618501, 0.782349], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.363898, 0.618501, 0.782349], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #7}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[1, 0.75, 0], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[1, 0.75, 0], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #8}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"], 
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.647624, 0.37816, 0.614037], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.647624, 0.37816, 0.614037], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #9}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.571589, 0.586483, 0.], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.571589, 0.586483, 0.], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #10}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.915, 0.3325, 0.2125], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.915, 0.3325, 0.2125], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #11}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.560181`", ",", "0.691569`", ",", "0.194885`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6150173333333333, 0.25708400000000003`, 
                    0.13945266666666667`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.922526`", ",", "0.385626`", ",", "0.209179`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.922526, 0.385626, 0.209179]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.922526, 0.385626, 0.209179], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.3523253333333333, 0.3137493333333333, 
                    0.46756733333333333`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.528488`", ",", "0.470624`", ",", "0.701351`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.528488, 0.470624, 0.701351]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.528488, 0.470624, 0.701351], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.5147193333333333, 0.28770266666666666`, 
                    0.06825800000000001], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.772079`", ",", "0.431554`", ",", "0.102387`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.772079, 0.431554, 0.102387]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.772079, 0.431554, 0.102387], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.363898, 0.618501, 0.782349], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[0.24259866666666668`, 0.412334, 0.521566], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.363898`", ",", "0.618501`", ",", "0.782349`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.363898, 0.618501, 0.782349]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.363898, 0.618501, 0.782349], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[1, 0.75, 0], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> RGBColor[0.6666666666666666, 0.5, 0.], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"1", ",", "0.75`", ",", "0"}], "]"}], NumberMarks -> 
                    False]], Appearance -> None, BaseStyle -> {}, 
                    BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                    ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[1, 0.75, 0]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[1, 0.75, 0], Editable -> False, Selectable -> 
                    False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.647624, 0.37816, 0.614037], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.4317493333333333, 0.2521066666666667, 
                    0.40935800000000006`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.647624`", ",", "0.37816`", ",", "0.614037`"}], 
                    "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.647624, 0.37816, 0.614037]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.647624, 0.37816, 0.614037], Editable -> False, 
                    Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.571589, 0.586483, 0.], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[0.38105933333333336`, 0.39098866666666665`, 0.], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.571589`", ",", "0.586483`", ",", "0.`"}], 
                    "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.571589, 0.586483, 0.]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.571589, 0.586483, 0.], Editable -> False, 
                    Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.915, 0.3325, 0.2125], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6100000000000001, 0.22166666666666668`, 
                    0.14166666666666666`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.915`", ",", "0.3325`", ",", "0.2125`"}], "]"}],
                     NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.915, 0.3325, 0.2125]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.915, 0.3325, 0.2125], Editable -> False, 
                    Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             
             RowBox[{#, ",", #2, ",", #3, ",", #4, ",", #5, ",", #6, ",", #7, 
               ",", #8, ",", #9, ",", #10, ",", #11}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.800108747533544*^9, 3.8001088544042187`*^9, 3.8001088901478367`*^9, 
   3.800108924037816*^9, 3.800108982531206*^9, {3.800109038547076*^9, 
   3.800109063983571*^9}},
 CellLabel->"Out[53]=",ExpressionUUID->"4a552853-74a1-4f20-8fc2-f6ae7a2d11a6"]
}, Open  ]],

Cell[TextData[{
 "The curves are produced by the Coupled Logarithmic Function\n\t",
 Cell[BoxData[
  RowBox[{
   FractionBox["1", 
    RowBox[{"-", " ", "\[Kappa]"}]], 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["x", 
      FractionBox[
       RowBox[{"-", " ", "\[Kappa]"}], 
       RowBox[{"1", "+", " ", "\[Kappa]"}]]], "-", "1"}], ")"}]}]],
  CellChangeTimes->{{3.6461331535627427`*^9, 3.6461332333256826`*^9}, {
   3.6461332741041546`*^9, 3.646133400122376*^9}, {3.648723647988269*^9, 
   3.6487237693720818`*^9}},ExpressionUUID->
  "f6f5f867-cdc5-4528-80ba-e472c16cba7e"]
}], "Text",
 CellChangeTimes->{{3.6608491458973303`*^9, 
  3.6608491821510983`*^9}},ExpressionUUID->"a8f9f400-2914-4806-9775-\
aa29956ca18e"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coupled Normal Distribution", "Section",
 CellChangeTimes->{{3.800109223396997*^9, 
  3.800109227884129*^9}},ExpressionUUID->"1775efa8-b2d8-4d30-8ab7-\
d8ebdb9e0d7b"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"CouplingValues", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.5"}], ",", 
     RowBox[{"-", "0.25"}], ",", "0", ",", "0.5", ",", "1.0", ",", "2.0"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Quiet", "[", 
  RowBox[{
   RowBox[{"Plot", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"PDF", "[", 
          RowBox[{
           RowBox[{"CoupledNormalDistribution", "[", 
            RowBox[{"0", ",", "1", ",", "#"}], "]"}], ",", 
           RowBox[{"{", "x", "}"}]}], "]"}], "/", 
         RowBox[{"PDF", "[", 
          RowBox[{
           RowBox[{"CoupledNormalDistribution", "[", 
            RowBox[{"0", ",", "1", ",", "#"}], "]"}], ",", 
           RowBox[{"{", "1", "}"}]}], "]"}]}], "&"}], "/@", 
       "CouplingValues"}], "//", "Evaluate"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", "10"}], ",", "10"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"PlotLegends", "\[Rule]", "CouplingValues"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"PlotRange", "\[Rule]", "Full"}]}], "]"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"Power", "::", "infy"}], "}"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.684857042303303*^9, 3.68485707094108*^9}, {
  3.684857115704288*^9, 3.684857130250449*^9}, {3.684857160525826*^9, 
  3.684857192687024*^9}, {3.684857222850348*^9, 3.684857223807577*^9}, {
  3.6848572799195547`*^9, 3.6848572851817303`*^9}, {3.684857321756291*^9, 
  3.684857329932355*^9}, {3.732616832133396*^9, 3.732616878955141*^9}, {
  3.732616929380588*^9, 3.732616964713625*^9}, {3.732617452944779*^9, 
  3.73261746499343*^9}, {3.800108765930394*^9, 3.800108822196589*^9}, {
  3.800109082748824*^9, 3.800109104440156*^9}, {3.803650632043682*^9, 
  3.803650645547255*^9}},
 CellLabel->"In[47]:=",ExpressionUUID->"c9127869-ff3b-466a-ac82-2d5ae3a0d997"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVlnk0ld0Xx0uRDFEoQ4WkQYNKvCJtmRpeyfSa0oQ0kCRJxldkLCWZQqbC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          "]]}, 
        Annotation[#, "Charting`Private`Tag$25712#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwVlXk01P/3xyWJkpI2/EpSolRKyqfUUwtZpkKfVPokIW1KWUqoiCQqkZ1i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          "]]}, 
        Annotation[#, "Charting`Private`Tag$25712#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwUVXc8ll8ftvf22Osxssn2M+773FGkkFRWspWiZUVmVhIKkVk2kcim8hwr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          "]]}, 
        Annotation[#, "Charting`Private`Tag$25712#3"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.922526, 0.385626, 0.209179], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJw1mmVQ1m2whykB6S6RerBoMVBqb0DAAEVBEFRQSoxXASkBURQBFQnpklJK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          "]]}, Annotation[#, "Charting`Private`Tag$25712#4"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.528488, 0.470624, 0.701351], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJw1WWc013/YtvfetPBDSAhpoPsjISVZqTSNJMm/VFJEQ0qJ8COJyt5bVvh+
7Jm9N9lK9ub39JzzPK/uc53rut5cL+5zX+cWs/zP+AYNFRVVBzUV1f/Op+Nn
5ikUEiZ1Dz1b1SHDmIvAUs8mCbvIfxE6eYYMBlwDKzlrJOzB7PktxIQMokfu
bv23QMI3Doq+OXmdDCVv/ZkGx0i4/27ZZOVjMjDJd+zGP//xBfGBZslkID+4
fvrZZxL+pmzaggUCIZH6fhTVIRJeM8+jObwQCPQ3dJWEr4hjbWml5WXqYODt
LLeP8BPDGb6nb5kqh4BxaYiVTI0oLv1cuNc+NhRc/mrYf/i1B+/vUOC0yvsK
uY9C7FLP7sYFlc+y1zzDwcD/nOj2p5248Z6JZ3dUJOiGFRqc/C2CeRY46Z/t
ioHZ972bdGMi+Pn+4Q/kvTEQ4raRUDwggsfv+C2lHoiB39eOMKq1iOCbrw9Y
bmjHgL/4d2J/vgguoI12GnWIgf7YJEXeNyKYTamZj60kBpwyPvP0k0Tw4Q2L
8cY7sRBT+bj9/iVhXPXthjXujYNPSk5a8abCuKTFb7F0Mg68wxzT+s8I4+uO
N+KbluPA8f7tt3pIGE/ETCSwcseDxp4rx/ZICuPGsOCDG9rx0OqEYmpmhLDP
/budTenxQCPF6CT2Qgi328hnJ/olwFUPMn9DnCCudIzVz3mQBPWwzOgcIYhV
zS8SH14mAWycXxcNFcScOvtyH/ongajjjgFHX0Fcpe5x2zItCYavRcQJOAni
XeGJe6t+J4GNWpraVS1BfDLTNKzhZjI4zNda/OkTwHcYbKn23kwBt+u0Kay8
AjhS+73wuk4apDXo+rSyCeCCIwLf+s6mwfAxb4cwBgG8I4XhctWFNNDZxa+g
sM6PRR/QBmTapQFHt1Sa8RA/nqKb6xjySYMvJqcyPqXy4/k3u7KiOtOA0PHL
3qvPj+07RaO37qUD1f49hNYrPpzB1/JivjADXAU0r75y48PNPIyS7dUZsEKx
3Kp4xIfTb9Eul7VlwExzjNopOz7cR9N+qexPBvQ4y+canuXDpyJ58endmZBd
ppF+RZgP30vicvV6ngl3Ll+OfJTMi0U4clZljLJgQtvteF4ML6Y++27T92oW
WCt8G1r/yosDaKMEaO2zwJxmZI+bPy/2FqFt4nqdBSfjbod6POLFF6JW1eoK
s0Bi8UmgnyYvXqWMKDbv/w7d7z69TmzjwRKz5OQQoWxoKRdNut7Ag++HlHYU
S2VDHSW2kb+aB1M9j7BbVsmGovvZws8KePDahzPsb4yyIeJSS6JpJA9WYaYR
PfsuG+z2sTdu3eXBdrwBX1yoc2Cj5pmQIRsPFnqaWlK3mgOLdEwa9Aw8WHuD
w6qUKRdmjvla5FO4sVvf3ZxqoVwYzAhLkFjgxmUjdj/pj+RCWUi++loXN+66
HvhH40kueNstXo+I/ccnmNtgSi7sZLGNXzzOjY8We/jHCebD1aI6b3F1bvwk
gkVFe18+hDseuGt4kBuLukmSV47lg1TPumrSXm68dDec44tNPigkvS+zYuPG
yny1DkvZ+XDcIHOguY0LIxH/xKlLP+CW/zZ/ui0XXlLNJg3nFECSjtVavwUX
tjRDp53rC2BmvbKX7RIXrljRDZEeLQBHK79I2zNceGPY/kwnTyG4qkgcEFXi
wlZdzx2c/yuE3ckBwbtXOPGpkTdVA/uL4PZBWfqenZw4F6smuWICHOnzYp04
OfFOymtPhW4CnrTpnuKh4cSlp4pmqBYJePvwhq/eOAf22z7LS8WHIeH7N+Hc
dA4ceF6hIP8khsmDgvKBOhzYPCDtifJ3DDdV6c6fvcuONyq9NiZDisGBIWBt
yvIfHuYp7YsvBqd2sVDPc+w4OOOP5EhuMXg6wVDhUXbMHqykKtNRDDHZT27L
0bNjoV617o98JTCqOufOHMKG+6oyS8UDSsDqUH9caSkrtqCTZ+j6Wgo2cYfM
dHJYcaBECiimlYKdsB9tdQIrvjbifeEzLoW7G1pX6/1YMT3Pz+jkoVJwJeJ5
u66x4in30zIVEmVA1n3oNrPBgtnp7jn9SimDivNspsIqLDj47n8fi1vKobrq
BnXoXhbsyHxgX/tYOdQdIVJ272DBzRWx9LTr5dC8w5FZgoYF/w40lI8RrYCB
gY4i+UZmTBewt1DgvwpYvRkpc8KeGReH8BuVc1WCrPNRikMUE3aXENp6b1sF
uM9A9lcQE568smIy9qwKzLSszp33YsKXjtb2WXyqghcc3ongwIQPP2X5UFpT
BV1Rfee5DjPh7e7J7MMK1fCm6Vlaei0jvtr5Tl2KUg1jspUWi/MMWGSg5D5X
YS24fujxth1lwNKav+0vdtcCz/LfnN4OBmzqzbBav1ILx4qFOMoLGHCLgvNu
TeU6CDxvlx/oyYDbMpYMTyfVwQkPdt5DIgz4OetfAd3YnxDeZ1L2WJMeJ/dk
vH1d2gBKI0rrrSr0ONDmR33eSAOUTXErKkrTY8MdSX0ctI0wttLweYyDHidd
56AtkmgEWR79+ya9dDj5EMXO4FYjZOicEN//iA5XDckMWq40QlmqyrPBJFp8
VZVRuVa6Gc5l82arfaPFJbsesg2rN8NYwfx0UAAtZmAfuidk1AxMNWnnzzyh
xQnIqXPkcTOcGZFTyNelxXaS7ms0P5uhXUiyP2CYBt/hdnn7/nELjD/nV9cV
osGddeI7S/+0AsvT1L8BLDS4hSPf/ApTG8g760UOblJj1kvHBPeS2sDJ4SnL
kyFqrIU/Jp662AYMl0Y7kxKocTvLtZG+yjaQUPn+kEeDGkcbxnMEprTDSYWz
MtcUqLHactlh87p2sJed7E0Uo8ZDHWoD56baIUt01wltBmq8yjnlOS/VAVrs
r3icG6iwrHRmdX1EB1iOmab2WVBhwYz3J15Gd4Ln0IylrCkVvlpn6ztd0QkJ
vW8EHulQ4a7suEf+E50w31zwlGsfFb7sf8D0i1wX0LjrBXtOUIit47NzhTld
AAcmm4zYKcSBud31X7q7IT9QWnvCbIvQUluW1jzfB0yXxq2o1beIE3uN24ee
9YGZaMwLEbEtQsZ64FZzQh/MJ5Cw/vQmMfzpAlMMpQ9k8G61dPdNIv7OnXbZ
5H4InuI78CRug2hedfgoxTYIY6ktZwPebxCF9wLlw0mDoPLQ3yHJcYM4Po6C
9dQGoYHCmdSvvkH4Los+lLw9CAz8rHu1mtYJv+pcV76fg/AQqHexra8R14ve
3t0XMgQldFhNcmCNeOsJbZlZQ8BV42Z+rGyNgH0qCXcbhiDJdPPjXZ81YuRk
yA0v+mEYubXC0ya+RmwrCvtoPRgGY/Ifpi+nV4mbcmRFdPkXKE52LSqELRPm
V3qlYi6OwoPZ/TKCHsvEjzvpPecfj0LuyvMr23bLRFeMyIUTn0ZBk0G2ou7w
MrH846NOQ9coGJOefLzVukQ0/+U6n3tlDB5eETkaybpE1CsvSindG4f8JnN3
gScLRA1X6PCH6knY7kzJ3Lq+QDSe0zS2nJuE44M0EyO6C4TZH3YhF+EpqJ1J
MMzkXyAuXA9k87Cbgl7WDTHD9HmiQzhy9xbnNFC0P5d5TcwRWu4tO1tsf8OJ
/B7mLbNZQuzFxeJ+k7+QkhB9pFJtlrh0OqXP0OsvCH3+75af6Cwh3nLz0R7i
L7SvMPifbv1LUBNSpEn+WZB0MeMKW58h6E/XLFV4zELJ8yU2pPuHmPNbyxC7
PQebPsoMnoOTxG6qBMR5cQHuxqeucvP+Ik6rvvx7PXQFFDkcXh4fHCauMdhM
Xi5cgTlHOY77ycNEhLxJxaX+FXBUjye16g4TL4fuOl8WXYWHDREGQa5DhKDE
Y0kUtQoLU/v0DtYNEPu7TBNkEtbgQVbc1SCbAYJH5rdvcM0aLD+VfLBCNUDo
yZ7a/Du1Bqtce77mHewnOhi+VBjuW4dNVZ4lja+9BKmte29kwjowvFz9puPY
RXhkKe4rCt+A1/oPs2PZuogJwcTOEmIDmATma5liOwkeLet4n74NYImfXq7p
6SAokz/jHIU2gbOhX/+sdjvRMxYtKPduEz4EX7JMG2gj/lOiLMrEbgKPZecj
7idtRE4vM7m3ZBP4lpoiWlJaCQv/FyVma5sgtKN89bxQC9G+OeV+xXILPo1o
cuRlNBPStm/5sMsWiKQUkUTONBMUHFhRRt6CXZq5Br3Pmwj2U3GnEsu3QKve
TK+4vYE4tJ20nSS+DToWhg9esjYQNK96wgYPb8PJRb2v2qieiAhP0o8y2AZ9
EY2l6vg6IkpDWrnFeRvMbEjhza7VROiXMmly5Ta8vXf/jf/3KuLblT3P0ru2
odC19D/jmUri6k7X2ybT2yARYHWs+VoF8fVNZvsudgpc+JIp6R9cTjw/6Ndz
fBcF3sXTshs3lRE+AUbuo3IUmCcie5qOlxLuL1JIYacoIFW7UOLnUkKc2n2J
OvUCBS62ayUYZRUTVzk6Dh+2oQD+/cu5SRIT1/WNVxLcKTDx9yYWLywiYg8r
Yr93FDjLwOQbIV9IyJ4yuTkVRIHdyiflIrjzibcfwP53IgVe6U2si73MJYwv
HnYN+E751z/eVIcvZhPg/aAhtogCJk7SwWI234mB0VRr6UoK5HtX2YR3ZBL2
k6qn+BooIBZpe1BML4MQ0ZP5YtdOgTd5THTh+WmEgnquj2wfBfq/GkVrXkgh
mCKvm5v8okBynd9xTbVEAvWIGo5MUMBlrWkA7Y4jxKQTGnr/UEBPiucpoo4m
JgLf66nPU0DQxFgEjYQTxsgaaJYpUEuR+M/dJ4wIGp/Xkl+jgO3F6Hg3048E
R/HJDWKDAnSZkiNPRfyIm/e7RHK2KICE9TXW8Svio6L/LV4KBWaDP1nPfXIk
woYnidZ/2JWGXK5z+CLoWP5m3PiH5UOsKvlU3eFMBtLw/If7hdNnWDm9QUNN
9o7T9r88lDsLPDXJkMz4Krt2kwLZXedfvHYJgUyaQj3PdQqw42lSmfk3UNXA
tjErFICARM+yR5FwX/O3jdwiBe7Z3J4sI8eAndtynNAsBSKO7NMv/9er+jsa
om5PU6CVbTqlvD4JQm+9HJIcowDDYAJ3xXQqZPpyregOUiD04sGjVh7pYKiL
fJq7KHBQK/D2L55M8N9+eL2imQL1ckuhVuFZQEy7HJKopQAV1XeKVVEO0BbX
3xbLp8CnSb4DI/p58Dg0KqkkjQJKLQ8srXvywZnoyG2KoYB1jEq59WohBKjO
RWj4UWDLl7w84kkA1/KDvtBX//JXXw/IeIvBTWXms8Pjf3op83Il4RKQ4F5Y
dbhGAcUdLP2OgyWg189m9c3on58zfzkjthTCdz4d19OiQNCasLSyajlEpqRY
sEpSoLKu662ySRXEPZUIeji8DbbO3ZWX9lUDkL4ssDRsA5NED50HbQ30pKqf
4/qxDadcet1aM2uB+hFX0gO/baiTGXB8yNcAre+O/5w49M+v5ZjcM9UMs6zH
8svvbMHhT/O1f81awMt9advbeAuY/jpO0Za2AOurpwYzqlsQH3J/r9znVjDa
P3f87fYmTM09CHc93Q6ju8VITl6bYP/tUeCupC54tZz4w/fDBqgvr2QeEOoG
qoxbvTp3N4BN37lZ26MbzkYzni07uwHJK86cDpd7QMifPXsnxwbMGDzxKmLr
gwkP+z/v3qxDrltRUarqIIh3pMr7OK3B0f3FGYYff8GDdSmOrzorcLC1Paio
+hd4dhr0r0j+2/8uv5/Ibf6C0fBvby7Sr4BUtaAWk8UIfE5XPqZetgy8Nxxa
CNlR4CrP1f56fBl+h+5YlC8ag6MTAjplx5fgK5vTQfbRSfBME2K2NFuAkMx3
wi6CU8AeLgnRhxcgyDx8a0JvCn5sHM5m2bEA3nF15WUpU3As0onu2OA8OJ8g
mbk6TwP3rT0WhvbzYOza6DTN+gcq5l+53Hw3BzRO/L3WwrNgqmAQePXnXzC/
ntCcqzILyU65XOof/0LGKahmM5yFkVCVbieLv2C551Z2lucsKGFpAbqVGSip
KvhAtzgL/gPtik+kZuDlDmvtqPo5uJBo75MU9Bt66dfUVifnYEXlQ4y+zW9Q
mX2vpE8/D++rdzfpqP6GkbKcPYtq8/DSYwftyY5pOOHAuq4VPw/0J3YV3N81
DXTFGSm/Xi4A1g2REcyfhMuJutGHvi6A2P6us7K+k5AV2Pv5Xf4C0DM+2vpg
PQnWdgxeynMLsGvZnvk81ySU85pbvby6CL+GWy/k3p4ATxtqIdKRJaCPS1S9
fmAcGNkMn1vOLENu/gPN+MURuLDXHKsyr0DW0qvV+pYRSDhuTWGRWAFWMwu7
g5kjYPDY+WnmxRU4NhboVeI4AkFj3x7Tla/AkUZfwcbFXyBVPHcv5vMq8NZH
DurT/QLn3o20J9mr4BTDZXtifBhqVuhnDZpWIZ/IYXpYMwwO8iIOKwxr8JQ3
mXD3G4bsz8ftTjquwXpcmr68+DBoOwVYTp1ch9ZiWj3vM0MQ5BcWXmS1Dqdu
OEcOKA/BRFLsoL/bOmS0NbHcEBkC718/rqplrUNJidE5uolBaDUcMffeswE6
LRvnvF4NgvW+gybyyxvwQ24jIsZ/AF4Otms5Rm6BUVNk7LPhXujPO33HuWAL
StnljvYV9MLRABzk1rYFwWfSJF0/9sKsdsLkW8ZtqBoUmnI70wtXE5/6Rt7e
BqvjBNmroAeOOkn0tCpTQCbGUfN4TDes0R/J22FJhXh1Jd//juqEGMeFn18c
qNCd+pKz4Z6dYDqQPCzmQoXSwuTsvWw7IS1XnE2aTIW6Vfh45uU6weY22zWV
CirkevrPoHBuB7Q2DtCdkaVGtn9Cftp3tEPaZ09Dt3lqNFQiXkA+1Aa1vJ7j
nFQ0SNKUUjKxsw3G3r1yC2enQQx+r5Jv07TBTlePpDJpGqRyPC9k82crvLn8
gon1Gg3KHdBgQDdb4fouN/yxlgYZWgp9WwtvAa6vDxXTo2iRzPX82qqDzbBP
8GHl8QxapDTwrV5HtBl0fB9cbSVokdjt7eIJlmZwdb/vvdJNi3p+Emc/DjbB
xLV7ExrcdIhOyoD+4/smwKJ3vtU8pUPPOp1Xh/80wn8R1lwjZvTIwHZgovBd
A/g8/hTqY0WPSpSpKqhuN0CKYb30kbv06GqbvMSDUw0ws31I08eLHsXbn16f
Ym4A+0usjocL6BGFy8CM81092PFltHiLMaAjhknPxwN/go0nJejgbwZ0on1x
82ttLXheUSENrjIg+ZLvL4zSaiFG5VbqW3pG5NSRQpYJrIXR4ebKgd2M6LO8
rp7+tVqwhphVLyNGdCwv743pUg1Yruqb92czovp66jpV6Rq4ahey6/VzJsSi
t8zsF1sFG13M9XPvmZCekTANm18VBOs9drscwoQuD15Mj3pSBU3SFwYOZDKh
Ao4Ne3mDKjgxzh/eO8KEzEvYl8KWK0HG2k9C6SQzEn6VfKtUvxIWr7zZ18fB
gurrzOqm2Srgw8+VHt0dLCjlIXNM6Vo57Ne46Z2xlwWN9HJHZ46Vg81OnT+v
EQuKeDbl1IPLoaubNlXJkQW5CsWq6DiVQ9F5d6U3bf/0bGeu3xwtAy8jpyPK
oayIKfrKxmBLKfjXydonxbOiX2sed3hKSiFUd+CLZA4rSipu3nsprRTSNE7S
CTWzIuNX2x0H3pdCp4xIwyYjG3Iczf7zV7cUpGmKrCvusyHqInUJn+ISqMig
/3DxNDv6c/Db+UtlxdC4P7+k+QI7Kq09ZqbwvRi64xyWTtuwo6RjipoCMcUw
86XD/NhzdiTH8ddi55tiEHgXL0nKYUdqPpyvGM4Uww2rMz9+i3OghuKf+HA3
Bjq+wDH3dQ6U8L7ihwk1Bp4ZNw4eRk4UbLAsbj5FgFiVrWoULyeaePexdW8z
Acdc1T2r5DjRmavuKbwRBDwe+SXJfY0TeYfnMspoETCTpXQjopQT0a3bPJN5
WwS/ZSfuWepwIdsOfaY3soWwKNj3KlGfC0l8c0s35S+EDbrmT4vGXKglomLt
EFUhsAz8KPa8xoVqnq9NarYXwN4AX+5EZy708b7o6LWXBWC5oZq+kMCFxHZ9
boof+gEdta9mPTi50Ux4w3Jicj6Qj4VpnxPgRr7bHoIen/LBOD0rRHIXN1oa
Hw91fJUP9UG/TlTIcqNhj3UT/yv5UGGh+YlRhxvJJqctdHLkQ/bKpuZbV250
roUr8+bDPAgiPQjwneRGWLGp74dRLpgGvZu4NsuNaF6bEJIoF3iYIzUUV7iR
yyGqKxHyueAz0zTeSMeDZj18rVtYc8EzT0GdR5QH0U8qt3JX5oDT2ekRshkP
qmaMQFMoB865WKqGlPKgh/YOApaa2RAUxsewUs2Dur1+KK4pZUM7UdFm0siD
+CTOc36RyAYzun0P2fv+8fhLKxdTNlx4v5D5bJkHsY0pRwY3fAfzrx4Hbsrw
oohYulJxi+9wvTRGTsWXFxWaGaamv88Ce9Zp8XpzPnT34kdfL+ZMmH6cT8dq
wYc0oy89Md7OALsJrzHdm3zI6+aQrORCBtwsl04ovs+HjrC7pY33ZoClu82B
7958KKzyhdBgWgZcXBiC0CI+ZDRjOEw2zwCdns7LduL8aPvzkeDyrHTYk1gR
xDDFj5Qfr8nIvkgD06+5eTWz/MjGc2bR0DkN3gYk9Pqs8KPfBs8ePHNIg0UX
H3FBegG0N4X61pp5GtTom6XsFRVA4wrmnMYqaeA0M1p+0kwABX30fLqvKRV+
HqBfelsigG6P3MzTS0kBl1wtE87PgiirRqpwjZwESpcbZ6+FCyJmFP1HzDMJ
piiXfdJiBZGu6R+fc4+SwFzXqdooSxDNsFkt9V9MgqPtcUD+KYgOXWngfLwn
CVYX2OWEqYSQkuPPYJyUCE4KnXSkG0Jol+NWkWFDAtyNsc9WVRBG/XUv/DSU
/t3xDTsCOVSFUeHyj/RxiXg4s1Zzf0xdGI0qlBmECsaDrL7MgaBTwsgqrKhF
YisORuZGE5duCKMTlxp70qvi4LzGtfCsUGG0utgblWERB8dajbyVWETQEzW7
JsbPscBGc8hSflQEsSWEPp2RjoG8R7QmY9MiqF/jwOtdu2PA5k+D1pd5EfT5
me5tU94YwJ22khxUO1ASQ3dU11Y03E8NmfgjsgNNHExn02yJhu7L2w7JhjtQ
Lm/+iafu0RCXU+4iV7ADnYtyIx73RYH2HdNA2YCdqEeHR5YrJRKE8nb9py2x
GykbjB38kRsO/o5v/BQ79qDIvx+OpTR9BWPPs/vDskWRkvuGVexkKMhe8jgX
9VQMmbIVZQQ6hED7GHNEwDlxZNpEfuLIHwz1lqOaPHtIKJjEIp8pEwRDdLGr
geIkNPhUXy9GLAgWY2xThaVI6KiWf0mYcBAIT0/vFNtPQj9PB5V8Yg4C6/vz
KwpqJLQmJxZePhkI6y8pyWfOk9Bz92tSUwmBIBUtLOLlS0Ic70QNjRQD4Yhu
TyNbAAmVK3aerJEOBP3J0NcfgkjokLIc0hULhHv7RZc+hpHQf2m39pzhCYSC
71KNMYkkND3CYPxmkQzGFcqeZZUkRBEl7c7PI8MN2yX1k7UkZLtHNeteBhmc
WXIWautJ6Eg5Zef+RDKEGRy1aGkjoUeuXJdTQ8kw0Y7Uh3+RUEaFiM6QOxnW
nWkWboyTUGFv+d4cZzKw7yiLn5wiofO7BSo/3COD0jVdwbk5ErroclLlrBUZ
tKmZ6x8skVDWBeot5ctkuBBZ47G6SkJ/B9ke7zxHBjttbzXXTRISoe6NZjIg
w//9V9H//1f/Bw5gmVI=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$25712#5"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.772079, 0.431554, 0.102387], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJw1WWk0lX/UdS8umedCma4ihYwl6vwiUxlKojSYGiRJikyhkpAGGSpDRbky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          "]]}, 
        Annotation[#, "Charting`Private`Tag$25712#6"]& ]}}, {}, {}}, {
    DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
     AxesOrigin -> {0, 3.1799838796970296`*^-22}, FrameTicks -> {{Automatic, 
        Charting`ScaledFrameTicks[{Identity, Identity}]}, {Automatic, 
        Charting`ScaledFrameTicks[{Identity, Identity}]}}, 
     GridLines -> {None, None}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {True, True}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> {FontFamily -> "Arial"}, DisplayFunction :> Identity, 
     Frame -> {{False, False}, {False, False}}, 
     FrameLabel -> {{None, None}, {None, None}}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
       AbsolutePointSize[6], "ScalingFunctions" -> None, 
       "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, 
     PlotRange -> {{-10, 10}, {3.1799838796970296`*^-22, 
       2.2794989552796068`}}, PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{
       RowBox[{"-", "0.5`"}], 
       RowBox[{"-", "0.25`"}], "0", "0.5`", "1.`", "2.`"}, "LineLegend", 
      DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #5}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #6}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.560181`", ",", "0.691569`", ",", "0.194885`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6150173333333333, 0.25708400000000003`, 
                    0.13945266666666667`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.922526`", ",", "0.385626`", ",", "0.209179`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.922526, 0.385626, 0.209179]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.922526, 0.385626, 0.209179], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.3523253333333333, 0.3137493333333333, 
                    0.46756733333333333`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.528488`", ",", "0.470624`", ",", "0.701351`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.528488, 0.470624, 0.701351]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.528488, 0.470624, 0.701351], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.772079, 0.431554, 0.102387], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.5147193333333333, 0.28770266666666666`, 
                    0.06825800000000001], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 
                    1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                    Magnification])}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.772079`", ",", "0.431554`", ",", "0.102387`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.772079, 0.431554, 0.102387]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.772079, 0.431554, 0.102387], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3, ",", #4, ",", #5, ",", #6}], "}"}], 
           ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.6848570629873466`*^9, 3.6848572492418633`*^9, 3.68485728762533*^9, 
   3.684857339151576*^9, {3.732616912669895*^9, 3.7326169670592623`*^9}, 
   3.732617473132894*^9, 3.800107821818284*^9, 3.800108798584811*^9, 
   3.800109106890115*^9, 3.803650662210698*^9},
 CellLabel->"Out[48]=",ExpressionUUID->"d80098b6-c459-4c84-ba25-e74c34a74aa2"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coupled Gaussian is Scale-Free as \[Sigma] \[Rule] 0", "Section",
 CellChangeTimes->{{3.800113441936368*^9, 3.8001134714202538`*^9}, {
  3.800121423987226*^9, 
  3.800121483076736*^9}},ExpressionUUID->"d1f5de3b-05e8-4d89-8860-\
af0548861a12"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Parameters", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "1", ",", "0.5", ",", "0.1", ",", "0.001"}], "}"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "1", ",", "1", ",", "1"}], "}"}]}], "}"}]}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{"Quiet", "[", 
  RowBox[{
   RowBox[{"LogLogPlot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"MapThread", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"PDF", "[", 
          RowBox[{
           RowBox[{"CoupledNormalDistribution", "[", 
            RowBox[{"0", ",", "#1", ",", "#2"}], "]"}], ",", 
           RowBox[{"{", "x", "}"}]}], "]"}], "&"}], ",", "Parameters"}], 
       "]"}], "//", "Evaluate"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0.01", ",", "100"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"PlotLegends", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
       "\"\<Normal \[Kappa] = 0, \[Sigma] = 1\>\"", ",", " ", 
        "\"\<Cauchy \[Kappa] = 1, \[Sigma] = 1\>\"", ",", 
        "\"\<Cauchy \[Sigma] = 0.5\>\"", ",", "\[IndentingNewLine]", 
        "\"\<Cauchy \[Sigma] = 0.1\>\"", ",", 
        "\"\<Cauchy \[Sigma] = 0.001\>\""}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"LabelStyle", "\[Rule]", 
      RowBox[{"Directive", "[", 
       RowBox[{"Gray", ",", "Smaller"}], "]"}]}], ",", "\[IndentingNewLine]", 
     
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0.01", ",", "100"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{
          SuperscriptBox["10", 
           RowBox[{"-", "4"}]], ",", "10"}], "}"}]}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"PlotTheme", "\[Rule]", 
      RowBox[{"{", "\"\<Detailed\>\"", "}"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<Density\>\""}], "}"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
     "PlotLabel", "\[Rule]", "\"\<Coupled Gaussian Distributions\>\""}]}], 
    "]"}], ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"Power", "::", "infy"}], "}"}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.800118785080984*^9, 3.800118837852675*^9}, {
   3.800118887713697*^9, 3.800118911087638*^9}, {3.8001191104030237`*^9, 
   3.800119206020259*^9}, {3.800119240278945*^9, 3.8001192880931873`*^9}, {
   3.8001193205991707`*^9, 3.8001193210650883`*^9}, {3.8001193687328463`*^9, 
   3.8001193913916483`*^9}, {3.8001194386762543`*^9, 3.800119478245614*^9}, {
   3.80011951283315*^9, 3.8001195151948633`*^9}, 3.80011961082471*^9, {
   3.8001196763784723`*^9, 3.800119679383091*^9}, {3.80011974024636*^9, 
   3.800119817480957*^9}, {3.800119863736347*^9, 3.8001200190289783`*^9}, {
   3.8001206429057693`*^9, 3.800120705222536*^9}, {3.802623591144639*^9, 
   3.8026237129578753`*^9}, {3.802623799322633*^9, 3.8026238564584618`*^9}, {
   3.8026239097472973`*^9, 3.802623919326303*^9}, {3.802624009708894*^9, 
   3.8026240163479967`*^9}, {3.802624701349518*^9, 3.8026247121421537`*^9}, {
   3.80262487526271*^9, 3.802624921300693*^9}, {3.802624957887767*^9, 
   3.802624983545682*^9}, {3.803650715519816*^9, 3.8036507340012302`*^9}, 
   3.826204037478997*^9, {3.826204068486326*^9, 3.826204089158214*^9}, {
   3.826204148643865*^9, 3.826204189981329*^9}},
 CellLabel->"In[77]:=",ExpressionUUID->"24d0e0e1-3ad3-4539-a6a7-7ced6dddb143"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV13c8V98bAHB7bxpGSEhWSyRxf5JU0kBFUhRSZhqyV4oySrKKqGTPskLP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          "]]}, 
        Annotation[#, "Charting`Private`Tag$24688#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV1nk4VVsUAHCua7yGSyikiDwRIVGJK5WU9GRIEpkjRJOQRIY0kClCNJky
ZJYia5NHeYQyNRAiszj3utfs7ffX+X7fOd85e6+9zlpL1tHbzIXExsamyc7G
9v/VRGJDYbGHCHIx3SS35wQBFyJBtcVNBBUO76j63zEs11ejLiJoG3mTxV7s
js7ygs32IiivNyFSB/t0nHl+lIUIGvGmLutiu1IevHTXFUGbH9gLHsAOWuHM
kBUSQcNhnWMm2E89C+R0+UXQHm+ep8exa79bvDjFK4Ki4g9Y/Y1Nfvvi+QMO
EUSP7Plgin3Xd/+zFZYwGkvKrjTHTvwTmP61Xxg9ltzVbI1dOEB/FFsijFIV
bT+5/r/eGPnWC4XCaDZGtOoc9iWaJeexfGH0Qlv/pRu2VlrFRa4sYdTutnLn
PHb1Kf+j15OFUdN+U9sL2E2tq0vOwcJIXc7DxBd76B3v2d3HhdEFr6/Ot7Eb
PPc+FDMWRn0+rWGR2NlSHv8Sh4WR98G7WXewz/s3axfsF0bETes/97BnNR9Q
t+zC3luQGIO9midaR9kojBwpRXop2BIpm7f+HKOiLO6jvIXYzam20bO/qYjd
+LhpEXZwWiqLY4iK1JI0kouxR5+ub1LsoyJR0yy1MuyKbCHvS5+p6PGJSN83
2Gbla5WcVVR0K67Mvv7/+LX9PLb9PhUlrAS9/46t91m6Qu8OFY0NKx3r/X99
X2w2n4igIrL6+u4+7FPd3bNXg6moVLmGPoCt0NeaCFeoaIf7C6NR7LoJ6Dez
paKptQ9Oc9hLXE+v+Kvi90UE+QmbERCQKxqXokxFg5JvDNdhL5rcKaxSpKLG
j/fXi2HPJ1waX9lCRSk+grUbsJlbDtnfWk9FzG9DO2WwCb1x43tsVHSNTnu8
A7tjys1wb6YQaqsfjTfF/qJpFhL4RAid7pypNsP+fF2nuiZZCCU+Fh2zwG7j
FdQ4EC2EzmvXHrXGbt5aKn3smhAydlTRdMSut11h2B4RQrv22764gl3aEvsi
ZEoQrTn1iT7G1kiIMj48IohmWj8MpmOXnL5D5x8QRIyJdeXPsItGgg886hBE
V2NHXLOx89l9fr2qEkQrvoZspdhZu47L/bgjiHxfilH/xU5J43uhpYCtPdy9
ii3hzGW8vFkQcTgs0EjmBDxSItFrJQTRa2RXwImd9HrBwERAEL201XvEj53Q
PjroxBBAj0P3FUpiR3E2bomtE0A8JVVj2tjBXqHPJ+wEkJ7q51Zf7Bdfk4uk
rAWQ2j79/ADshkNFNcbmAsi22CQ6CJt/U++3vMMCyPLROaeI/7//SWudp5oA
WnW9aZaIXbhjPHSSXQDJeWcbv8bumzV1mcrgR15VJSMr2Ox25y5LP+FHGtd6
IkkWBMg33QgxSeZHal+r1Lixzz/PTSuI4kcfegYTqNhMM3KXly8/ch1pLJTD
Fix7bTh9mB/lK7buNsbW9d2k+GecgvQ/EANPsPsdDr94PkRBHI0GvzOxw0x8
Np3so6DnkcKMPOwm+TrRd+0U5G1ybnsl9skvLuz3Kilo9/fHwu3YPmoF3xQi
KKjPsLGKbInjMa4TZSdLQexPFtiuYBt2OfOISFGQ1V02i+vY47VRof+IUpBU
x8OSW9hqyT/9t/NQ0DO/srhY7OrDoa6L03zoF19jfyF2V0YTLbGaD7kOf6yf
xuY9a020WPGhtzotWr4nCTgY/acz/QQfcrLzYgVhB78Lf+NtzIcoKqn/3MZm
SpUEC9P40Lp6+t1k7MEePiFLBT5EL5ioeof91qxa+QeDF63rXvjIY0WAp+Fm
l/FYXhSQV7krFzv7arlR1T1edHNO+0sZ9mCG8fb74bzIYLNCCGCf4vCjq/jz
IlJrmWAn9iHUFnLRnheVuXjS2U4RsGnvrfR5VV7krnt80Qa7VWW4m6uZB2n6
lNVstibgFSlR1qCBBx1w0EXK2Pe7DT1uIB5E3iD5RRv7aEjOKr2MB6FmDRVT
7IYOj60DaTyoYCqjPAS7+jpxqcqbB72WkFUdxc79uCbgs44H7Xw06v/+NAF3
0ous8gR4UKqX+Nd2bLfLDs9+c/MgHdbasX5sBen3mrbL3Cgh98blFexnPhE2
R4e5UeKDizW7bXC+rhd4Kf+aGz2q2XuuFDvcRfLg19Pc6KJe4vCbM7hev+ld
TLLgRtuYvNHN2AcFnxWfPM6N1D7mmf/EJr3+a3PHfm7Uw6OmzWmL+xPPrsVP
itzI093Q2QzbN+/vonomF3rYQb9JYLvNhm0siuNCUjpr4sZnCWg8ZPTlwn0u
ZJNO1DlgK6RQ7qpEcCFpc8s4P+whgzhWrj8X+lx36HEWtn3Ck8+Z9lxodHnG
kWxPgLX228hUVS60dTZkqgH76I0/jIgmThT6pzTF3ZEAk2stwkX1nEhMwMY0
FNv0Yp7q1xpOtFU1UDkN+6TLOTflUk7EZ81l3o7tYPLze2sqJxJ8rO+l40SA
n3Rr3QYvTvTud/iauDMBme9exeQKcaKpJEmtKRcCcl7fL/jCy4m2WUXz8bvi
8yk+37TMwYnWfo1SlLGLMhXIf8+TkdLiyUR37Oqo9Gv0fjJSzTRuH8X+bBtt
t6+EjEgXr0jOnCOAbdVr+ycLMlLxmB6SO09Aqs6bYtG/yYi2gS5jhK3lT9a2
OUJGtm6ZMZ7YHoyU/aO6ZLR2fDd/BXbXeONJ0l9k9MN4Z6CxB66n3bIhWgsc
KOdeRUiwJ87n4s7O9DQOZJQ0eVTMmwD6tIzNcBIHavRIi9PHjt7u2a8cx4Fc
JefWe2K/zyZNVEZwoDfyzB/vsVXTd7B98eZAvpeHrX198Hxx744StwEHWlM/
eGHkIt6vs+5N798kdORHRuzyFQL+lU29uKefhPIvleZoXiXgd9+8E8c3Eto9
aSXuhb3RuswoqYWE6HJXWX3YkSbKIlBGQlmZr342+OLz1pLIFAwlocHdSijb
jwAqz1xTwWYSWvj8O784kADlf8zeXZMgIY94cvYctuGtosL960go/GMf794b
BAQueyR0cpFQF/0wfz326Myg7coUO9KTTBPrDSIAfW2bOVbNjrp4ZprkQgjw
zssXnzzFjl55NvnxROD9Cm71ljFnR/xa7caW2JM+aY0WJuxIcIkj6jn2ea1o
v3f72ZHziO0ZvdsEONd6f4tWYkdBt7bqX4/E81y3Rrr6MhsarhnK4buH18fx
Zuu1J2xI+vT9hLAYAjhd1YPyktlQjMeiRz92/YeXXT/j2ZBwRUPDvlgC9ken
3DaKZEM7n9jxsrD3SQSOSVxkQ+GvObovxuP+uoNWUG3AhhToKjohifh+xV13
cvMabDQpesifhutfwFPGZViDUzt7Y0Kxd9Mqbv4qWYPHN1YOLGFrfhh4WJe8
Bs8qr3ZNphOw/dvuuuBzayDhu/3fnqcESK3+llwhrYFTu8FMRwaePw4daJ7b
swoW5wNcXAvw/8FnbeWisgrDhmGpTGzmpwuDHTKrkNKSVnj7FZ4/rFLmS7lX
gVG551p+Ie4P7rPylzpX4J76updsJQR8i3pyY9pnBU4wgv7pqCDgTeeS6kj2
MiiUzb3ZVYvnX9nJu7Gpy6AxkKE0gC3q9eO3zoNlcFbXTouqw/2J/C49xncZ
nslPRI29J0BxZ5Dg3kPLIPPAuLGgAc+3MWzT9weXIGJD1LRbM55fjTkLNKSX
wP6sj1x5DwEySXM8vdQl0Dnu/SX0KwGfBoedb5OXoMr0XqX5NwK2BTRs/DGx
CIyP4ttY33E/zbl9P/ztInTbDnUc+0nAcS4+zx6rRcix9BzX/E2AUq2gclDc
Atxp/ncr5xwBm09pmq9ELMAsZ5b2FLboH+vrgdcXIGC+/24Xk4C1jZn/Bjgv
AO3y1uX8eTyf+el4XtNagIkq6h73ZVxP1dwKfL7Ng7+l82MZMh2+PKnd4bRl
Hq6o6PomidLhg9aI1aD4PAwsvBXPE6NDTQt/sANlHlpnmOpInA45S1ZtZxks
2CXg2j+9gQ5BVtM+ZxpYYDrVaG4nTQdFIakSy/Ms+Cl25MRtBToEBl3RNCph
wp+UQC7qXjpcEgWtT5lMCFj5k3lJhw5uubx7LJKZYMZUPNG9jw7mXem69sFM
eB9i0pVFo4OS6kdDv+NMuA4tE3aH6NDTK22dMz4HBbLun3edoIOmbuMNHrk5
MF97WB1/Hj//RTg4WnwOWgpIKbqedJBxP3NLlG8Ohsbmhsa86EB5OBshM8vA
+0luO3KRDoOTUjG7gQEyh1ts/vKjQ8xjn+duNgxo42radSaCDpNLEo0f4ulw
WzDP9p8MOmzrTX4oF0kH7lrzoGdZdHCtkXAOCqTD9mG9TcE5dBgIlmDXdKFD
ZbjU6MF8OnRxSug80aLDX5PHFidL6YCENhRexXlR42jQ+eY9HRLkxJO2bCbA
XMFj5MwwHdrJiS431uG+cURg/YMROgj+FtPs4SZA5GqiSv0YHe7kiLVHzcwC
Z6mktPY0jr+KGGWhdhZqGqpc9Vk4ntqiN1udZ6EoqLTsFy8D9hmLnAvMnQFk
Kno6Qp0BAcqvhfrSZqD5a22NuSYDKilnKmmxM1DiUbtuqzYDdjZn8nL4zUBP
1rrnX/YxQMlkT/6dQzNQbOnVY2/EALV0NKhS+wdoMVuWafYMGMk8w3JUmYZt
Xfc9NOMZ4OiaZaQgOg2RwBV+J5EBfQozyWOLU/BIRlT9VzIDOrJD9/l8mIKC
/G0nM54yoO5lbsgNpylYtW9wt3vFgPSCecqj5EnICthzoaKJASfLE2RaOCdg
/QeqxUbuOVAPi1/HNTUObQ5T35bwufKbx3HROsZBjvnncr/gHNTNPJgsej4O
OS5rs5U4D3Zsv1eZQBsHA5el8myFOeB5EWJ6xn8Mwt7Wbwk5PAdvYy4ET0yO
gP27q8pHo+bg4Vmvy/IdI3A7zjSpK3YOfFQ9XW2rRiAwTLXBLXEOFJrdj7Xe
GYGlsvDQ9CdzEM/tsqFUYQT0un5LmpbMgWeQTVGA/W9Iu3W2pK1nDqQ9jfp5
O4ZgtdnL3luJCZHtxR6T1UPQvsPpndMOJhBaUqxPmUOgmHCs2laTCQ3s0wIJ
14ZgNOpeiaMeEy4kxetskhqCTvUnpGIzJkBdX6KG4y9wPStLcwpkgoPEVROb
mQHwJBQD9DqY0BzU16P7dQAqF+z4Zr4yQWvosLNM3QCEKYy2Zv9kAl+h5PXh
+AGQipcQUJpgQslBlOOtPQC39LM/+ZNYwOFDIYcF9UPXRjW7oxosOBJ14Kq1
RB+c2RFf9ymRBTIWuYw9I72gGPtR/kcaC5iSwlcky3vhrU2b/2wGC57n9F36
btoLbWHzwpqlLFh87+9jG/kDmLEOsRJtLMhdLPRwYH2DAn+Oy+dxXeF32+jo
3tkNb8rXTx6OnIc2g7nj/rHtMFs36rOWugD1Wupr+Y7tIH3hkX1CxgJUKnkV
9u9sh0uXzhxUL1iApyLDQkZdbaB+3Ylxq2YBLg52tIlLtQGPcc3RiIEFkCe7
Z7odaYEoiuMZoW24rt4aye/zboSAyHCmbs0iOK59uPbQ/zWIsxK/32VfBjUD
B0PhugpgaQeWLFKWYTVsQTSKrwJM6iuLL4kvQzKfUklYahkc4j+kcl15GdrE
7k1eflcM+bo6O6ctlyHwVEpdBikH4m9wa9rmL4O+nPLpMVkH4A6v2+R0dgVU
TRQGfc9V0M7uZeSu61sFpgtXriHlM43357XlykPsSGZW2/VXdi/NJ+eVsYMu
BxrVLV/9vGmYpjP+PqboMSeymuDrNT42QaO8SrelyfOg0ABb08LbM7R7XOmR
Fhl8aB/MKd7KptMOsoVcsfATQGIaOgzZOCZtd/R9v4ANVLTQ0xcpcGGBlrC+
eoLtqjACMEgwiFymEfs1VHsZIiizwzzuoNcaLci2bk+TnSiqSIjjMDJi13+W
+SCWfVwMKXWd0LYAkv632EThqTPr0Qt19DfXDrK+51A5qfnjBiRQEC3X+45T
/2isTUT4SUl0G0p9JDW49S3ffmo791EKEbO7/QP/4dE3c73a1mgjjZz0vFzD
TvDpiwfnqsiPbEJDsvPNRB1FX55loiuyXQZZbv01HW4uoG82ZVOvwZBBXSB/
9+ZHQf3FkL9OihXKItW3fOFCG6n6MTHu281dt6DrRTM+F2Sp+uKW5VXFHlvQ
f/I0fBg=
          "]]}, Annotation[#, "Charting`Private`Tag$24688#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV1Hk01O0XAHCGwczYxlJJ2UqpJElaGZTlTXpLIknaREi0WEoikRbKEi8V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          "]]}, Annotation[#, "Charting`Private`Tag$24688#3"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.922526, 0.385626, 0.209179], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwV1Hk01P33AHCGGTPMjmTfSv1S8vQoPRW3VNZQkpJIRSmEFtKikRSVkkcK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          "]]}, 
        Annotation[#, "Charting`Private`Tag$24688#4"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.528488, 0.470624, 0.701351], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwBQQK+/SFib1JlAgAAACMAAAACAAAATRkYr7FrEsChXf8c0V3yP26Ivy7N
aBLAsf6el+dG8j+Q92au6GUSwL3Qt779L/I/1NW1rR9gEsDRziwUKQLyP1qS
U6yNVBLA7Qfn6Hum8T9mC4+paT0SwAu0pZ0S7/A/fv0FpCEPEsCOz5NpDgDv
P6/h85iRshHAqIf4gEZC6T8Zmh8l3+kQwEnyPyIKk9k/+wagdHkuEMAmnx2v
+O6hP7PNkaOB7Q7ADBqEuiCp1L+AbYOi614NwAz/jp49w+a/PXYeKO/qC8Dn
PhHxoy/xvwlfWfLNVwrArytMhLp697/PLeXXBswIwHlAg3rjqP2/hWUaRNla
B8Dd2bHJfrYBwEp97/SGygXArc4YLenWBMAA/m0szlQEwOdfl4Q1wgfAsGQ9
f2/mAsAzjTNz2Z4KwG+rrBbsWAHAUIbYQM25DcA8toppBMz/v1iqHGvKTxDA
udX7Luen/L9biXeA1OERwCrBDit+kvm/Nvvi/oVsE8B8fnQ0SLL2v1Aiygqf
3BTA7PsZx8iT878RgbVP3WsWwDxLEmd8qvC/Eekqk4LgF8BWtZQgzQXrvwOb
8c5MdBnAHWxI4AnU5L8OOWUqvQAbwEmNQ3VZGd6/xrMHKZRyHMDQQvVOeZHR
v53CHvaPAx7AYv/Id5V5t7+WqKlQ8nkfwC5NJuJ2Z7Y/AzKYTX10IMD2ZQaq
RgvSPz8EndOTOyHAIPE3ZuOn3T8tsZCZXfUhwEbXjtiShuI/FlW1u7FrIsCl
wiJl
          "]]}, 
        Annotation[#, "Charting`Private`Tag$24688#5"]& ]}}, {}}, {
    DisplayFunction -> Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> 
       None}, DisplayFunction -> Identity, DisplayFunction -> Identity, 
     Ticks -> {Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , Quiet[
        Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& }, 
     AxesOrigin -> {-4.605170185988091, -9.210340371976184}, 
     FrameTicks -> {{Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}, {Quiet[
         Charting`ScaledTicks[{Log, Exp}][#, #2, {6, 6}]]& , 
        Charting`ScaledFrameTicks[{Log, Exp}]}}, 
     GridLines -> {{-4.605170185988091, -2.3025850929940455`, 0., 
       2.302585092994046, 
       4.605170185988092}, {-9.210340371976182, -6.907755278982137, \
-4.605170185988091, -2.3025850929940455`, 0., 2.302585092994046}}, 
     DisplayFunction -> Identity, PlotRangePadding -> {{0, 0}, {0, 0}}, 
     PlotRangeClipping -> True, ImagePadding -> All, DisplayFunction -> 
     Identity, 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None,
        "ClippingRange" -> {{{-4.605169998021961, 
        4.605169998021962}, {-5000.9170588722645`, 
        1.1479048725693752`}}, {{-4.605169998021961, 
        4.605169998021962}, {-9.210340371976184, 1.1479048725693752`}}}}, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {False, False}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, 
     BaseStyle -> {FontFamily -> "Arial"}, 
     CoordinatesToolOptions -> {"DisplayFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& ), "CopiedValueFunction" -> ({
         Exp[
          Part[#, 1]], 
         Exp[
          Part[#, 2]]}& )}, DisplayFunction :> Identity, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox["\"Density\"", TraditionalForm], None}, {
        FormBox["\"x\"", TraditionalForm], None}}, FrameStyle -> Automatic, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, GridLinesStyle -> Directive[
       GrayLevel[0.4, 0.5], 
       AbsoluteThickness[1], 
       AbsoluteDashing[{1, 2}]], LabelStyle -> Directive[
       GrayLevel[0.5], Smaller], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> 
       None}, PlotLabel -> 
     FormBox["\"Coupled Gaussian Distributions\"", TraditionalForm], 
     PlotRange -> NCache[{{-4.605170185988091, 
         Log[100]}, {-Log[10000], 
         Log[10]}}, {{-4.605170185988091, 
       4.605170185988092}, {-9.210340371976184, 2.302585092994046}}], 
     PlotRangeClipping -> True, 
     PlotRangePadding -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{
      "\"Normal \[Kappa] = 0, \[Sigma] = 1\"", 
       "\"Cauchy \[Kappa] = 1, \[Sigma] = 1\"", "\"Cauchy \[Sigma] = 0.5\"", 
       "\"Cauchy \[Sigma] = 0.1\"", "\"Cauchy \[Sigma] = 0.001\""}, 
      "LineLegend", DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #4}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #5}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
          Directive[
           GrayLevel[0.5], Smaller], FontFamily -> "Arial"}, Background -> 
         Automatic, StripOnInput -> False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.560181`", ",", "0.691569`", ",", "0.194885`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.922526, 0.385626, 0.209179], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.6150173333333333, 0.25708400000000003`, 
                    0.13945266666666667`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.922526`", ",", "0.385626`", ",", "0.209179`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.922526, 0.385626, 0.209179]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.922526, 0.385626, 0.209179], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.528488, 0.470624, 0.701351], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.3523253333333333, 0.3137493333333333, 
                    0.46756733333333333`], FrameTicks -> None, 
                    PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.528488`", ",", "0.470624`", ",", "0.701351`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.528488, 0.470624, 0.701351]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.528488, 0.470624, 0.701351], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3, ",", #4, ",", #5}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"Directive", "[", 
               RowBox[{
                 InterpretationBox[
                  ButtonBox[
                   TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    GrayLevel[0.5], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> GrayLevel[0.33333333333333337`], FrameTicks -> 
                    None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"GrayLevel", "[", "0.5`", "]"}], NumberMarks -> 
                    False]], Appearance -> None, BaseStyle -> {}, 
                   BaselinePosition -> Baseline, DefaultBaseStyle -> {}, 
                   ButtonFunction :> With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    GrayLevel[0.5]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["GrayLevelColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                   Automatic, Method -> "Preemptive"], 
                  GrayLevel[0.5], Editable -> False, Selectable -> False], 
                 ",", "Smaller"}], "]"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.800119291895893*^9, 3.8001193225605793`*^9, {3.8001193706140203`*^9, 
   3.800119392265835*^9}, {3.8001194894439487`*^9, 3.800119516468252*^9}, 
   3.800119612943879*^9, 3.800119776670923*^9, 3.8001198195773783`*^9, {
   3.800119890984486*^9, 3.8001199136896057`*^9}, {3.800120008930027*^9, 
   3.800120020269775*^9}, 3.8001206636597757`*^9, 3.800120707114155*^9, 
   3.802623699850091*^9, {3.8026238334790297`*^9, 3.80262385740079*^9}, {
   3.802624011372662*^9, 3.8026240184685802`*^9}, 3.802624714370175*^9, 
   3.8026248838038483`*^9, 3.802624922383251*^9, {3.802624960326703*^9, 
   3.802624988882711*^9}, 3.80365074151621*^9, {3.826204040381658*^9, 
   3.826204090697297*^9}, {3.826204171974531*^9, 3.826204191847451*^9}},
 CellLabel->"Out[78]=",ExpressionUUID->"4468abad-242b-4405-9b2c-58716012bf24"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Multivariate Coupled Distribution", "Section",
 CellChangeTimes->{{3.8110008508242073`*^9, 
  3.811000864380494*^9}},ExpressionUUID->"76ae8b4e-e4fc-4d67-91d4-\
1e141bb7087a"],

Cell[CellGroupData[{

Cell["Multivariate Coupled Exponential", "Subsection",
 CellChangeTimes->{{3.811000955308819*^9, 
  3.811000971071398*^9}},ExpressionUUID->"6fa9b034-ed26-48ef-837b-\
de92d5d4dc40"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"PDF", "[", 
    RowBox[{
     RowBox[{"MultivariateCoupledDistribution", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"1", ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", "1"}], "}"}]}], "}"}], ",", "2", ",", "1"}], 
      "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", "0", ",", "5"}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"PlotLegends", "\[Rule]", "None"}], ",", "\[IndentingNewLine]", 
   RowBox[{"PlotTheme", "\[Rule]", "\"\<Detailed\>\""}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"PlotRange", "\[Rule]", "Full"}]}], "\[IndentingNewLine]", 
  "]"}]], "Input",
 CellChangeTimes->{{3.811000902301285*^9, 3.811000950494342*^9}, {
   3.811000989157936*^9, 3.811001123238167*^9}, {3.8110014924976063`*^9, 
   3.811001516017907*^9}, 3.811002375003347*^9, {3.811002414073853*^9, 
   3.811002419161593*^9}, {3.811007396142239*^9, 3.811007566819419*^9}, {
   3.8110076208606586`*^9, 3.81100763415203*^9}},
 CellLabel->
  "In[228]:=",ExpressionUUID->"8c1b54f9-e42c-44bd-959f-0c2c5ecc0949"],

Cell[BoxData[
 Graphics3DBox[{GraphicsComplex3DBox[CompressedData["
1:eJyF/Xc8l+///49rIRWiUKkkLSXSnudDk2g80ybtaGhqaMrOKNkrMrKSnZUK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    "], {{
      {RGBColor[0.880722, 0.611041, 0.142051], EdgeForm[None], Specularity[
        GrayLevel[1], 3], 
       StyleBox[GraphicsGroup3DBox[
         TagBox[{Polygon3DBox[CompressedData["
1:eJw1mgf8jtUbxt/nPee8tKxCSFYIJZXQVJokQmhalS1pyaiQhoamlPYiSnsv
WtJUKZV2ae8kker/vbrO//P5nd99Pec57/Oc54z7vs91302GjO19QrlUKj0X
SiX+StP4VxN5LJX3x1KpC3gRdRMojYpSqQfyurLrX+V6d66/B7dA3kr9QeDl
1N8G7gp+G3w7uBv4HfD14H3Br4EP4zcR2Qk5BtmLUqHMo01v2qwEPxuNX6HN
0+Be4JfBwymHcn8I8h7aH0H951zvwfUP4JbIPbiuSvmO67tpczjyM667UDaj
/Mj1fsgmlJ/Ai2lzEnIt1/eBB4O/Bi+Jxh/wzF6Un8GtkYO4N5DyC9dncn02
+AxkF40heB/kX5SLuH8Fz3iMZw4H/8y9ranfSWML3obyK3gs5Sfqz0JeSPtF
tB8H/oP7T4FPBP8OfgZ8MvhPjQ/4FPA68HPgU8HrwS+Ax4M3gJeCJ4H/BT8P
Pg38F/gv3nGIvh15KfWdNc5cP8H1l+CFyL+pfwa8L7JM314EHwTeQFkM7oKM
1L8E7qaFBH4BcQB4C/D74CPBBXgJ+EDwX5RF4H2Q6yhPgTsj11OeBu+NrEX7
d8GHg2uCV4D7aw7pX11KHUoN6t+hvh/1m4PfAx8Brgv+AHwMOICXgruCE/hl
8MHgdvy+oHytdRkt70WuoXwDbkbbepSPwANpvyX4Q/AAcAPwJ+DB4K3An4KH
gHvw2zc1PtTVp3wMHkT9XdSvAt+JXKt1StmBchTv/6LkvXQt7Q5BrtC+oGyU
1+3Whfujdb8ruDXlXZ5zEs8fBT4ReVxijMG/U/8k5Sva3oM8MPp7+3B/9/y9
T1H3DyWBF0X3rTHPaFph7WjOaD+be6doTMGzwDPAHcHPg88Hd5C+AD/Ob9uB
b+b346hvDL6L+gngluAHwR+DLwfX1t6iNNG6on5K2WPwBPjRaHyj9lTy+ulC
3QauLwZfCX6J9meCS8iXy94jBfIh7tUAn03b18BvgXsiP+LeZeAtkJsFr+H9
wNXAz4L3B39CuQJcB/kp5UpwXeRnlFngLZHvld2H6shvKNeDGyH/kX4E71L2
HrkP3B75L+UBcAfkCsqF4M2Qa7UOwG2RFfrwBHhP3ae0Aj/COLwFPhe8EfIN
ynRwFeRXlGvBDZFfU67TukC+T5kJroF8t2w9Uw35OmUaOCHfpJwDropcRjkb
XEG+RpkKjsgLKNuCH6YPF4Fbgx8FXwxuA34MfHzZ9uFy8HHl/6ahdCl4mPYV
+CrwcH0b+GrwCHAV8BzwUOkIzSN4LHhT8FzwKHBV8A3gM7WXwPO1pjVn4DvA
k8D1wLeCT5ZekC4FnwauDb6m8LqSzboMPB5cB3wdeKL2rdYn+GDKGulPymTq
6yNvl77WfgbPK7xOtH/7UCejuAn4NurHgDcG3wIeGbx/GyJnUr8d+HHqp4Ob
SYeAzwY31R4E30K7HcHTqLubQWjCdXMG6Vyum1N/P23OAW8Dvg88GrwR+Cbw
1eBdwS+CrwHvBl4KPgvcUPoEfBl4Z/DirEuE75PdKdm+3Au+nDbtwc/QZja4
E3gJuDv31qkdcg71u4Nfkj0A3wJurrni3hvg3por8KvgHuAXtW7AmyCXUGaA
N87rSvtuc2Rv2m/B8/ZDPsi3T9bzs13T2gvIEcG2bzvp5uj9sl7v4d6l4FrI
gclrW7b0Kco/4A+jv1O6U999d7AO0dw0qti+qv5n2vWgDxuQy3nW1dTVQ94W
PFbDqP+TMgq8Fvk2964B10e+Q5kDboD8TvMHbof8nDIPvC1yFWU+uDVyNeVR
6WnpAJ41Ud9I369Ltv3nZL10M/XNyv4O6eoelKH0sSPlX/DT2mjSnbT/kHY3
cdkU+QXlDnAr5PeUe8A7In+g3Kt5RP6YddHOyN8oj4A7IX+lPAzuiPyAciO4
CfJLygJwG+TSaD/hTWR73v93yX6A1tA+Ja+pTRjbKlwfxDd9l7wOWiK/p7wi
PYL8NnlNtEB+Q3laa11Kgt/+Cd4TvFbttC6Rv1BWgrdH/pQ8B22QPySvg1bI
rymPao8hf06em+2QPybPa2vkKspD4MbILygPghshP6c8AN462aeVPZWd/4rr
R7RXkV9SHgY30fspN4A3R75BuQJcHfmZfl+2b3gfv++Q50j2+NuS7fNLwfO5
a7a1suGyvV9F+52/6f2UfcHfys+i7CNfFPk5ZW/tY+kJxmqq9A7v3Bbcgef2
kZ/IvZE8c4X8HfB21H0f7RuXwcui3ytbL/93Jb+/Q3oQ+Uy0X/G1+hXt++q3
K6L98J6UD4P9zDOp2yV4rLZBPknZs2Qf5kaed5j2IO17F/4++e27UapQjuHe
8MI+xtHg44Prv6TsGuzLvC89Bh4m/ap3Zr9H/vrowr6NnjmYextoP15+DtfH
Fr5+K9rvnoFsw70TwHsjB3O/G2W19mfhNau1e2y2EZfkOeiSx0Ft66uOe29H
66ILkGcVHm/15yDKntknW8m9asgXoverxrlnyWMtfGjJc6q2Gu9jGfftwe+D
P4seh8XI82TzKEHPle/C73pqvsGvgw8t+z36xjElz7Pmd1DJvmspv0tz1qrw
HF6EvFA2k/JRtH6RvmxbGGtdfhC9r5+g/o/o50jfSE9K98iY75D1kL7rxeiz
0RnIcwr7mTr/tOU31ZALNDbBbVohuxZe25sWXsubFD6PHcq96uDOweeowSXv
o5nUXUy5QN/HuydQ/yP3B2gdFT5bvsP1Uchf6MMsfnMA+A3uXQU+UPpKdhbc
H/wJ+GZwP/DH4JvAfcEf5bUzpLDNk53YK9uNK2mzP3gZ9xrmNbO31krZbV7W
d9BuGbiX3pnXg9an1rnOxdo/svGy9U/lZwg/rTHjt+tLPlvpvevzu7fKfZEP
rb0u+SR1D0f7KUchH6FsnbH2V5vCukbt1F5+yO/89jHwbshx1B+Q121P7m2s
fYmcz71jafMt1wvBwzTO4LvAQ8E/gNeAH5e+Qt5JOR78PfULwMeBvytsX26W
TkD+TLkV3AL5i2wquGXZtkNrUueZYbnP0gnSl9qT/+3H4LPMe4VtoHCPPHYa
N/kk7+a6Ttmf0W81L6chj6C0pByV7DvclLxXahXeL9pzOoOPLnlP1yh8vtba
1BrV2b97nm89c3n0+UJ7vzZ1zQtfn1r4zNlC7wo+jx2S+9Mw93Vq8H6eknX0
uDz+pwTv2Y7Bc3Jirv83uo+Tov3SUyjV9a7MXYjD0PfpOz8rWUeNz3pKnIP0
gvTd6nxPumtlsG44mnE4o7BeaJR1RuOsN17OunfXzE80zeOzObIZ5YySbVHX
vH/fiB4z7f03s29wXvR4aFyWc92/8G/U/n2eO5C6I5P3h/TmgJKx9oxs4Tf5
2zUGu+d67aPReU8NyLpUbWXHuubna74G5HHsmO2FsN53DNf7F9Y/4nV2zO2l
q4UnI08ovH+0d6Sn9W7ptwMz1vuk8/Vu6dUfosf4+WwrB+Ux3ij3U+0/iV7X
78r+UPYqzIMdHHw+2h7ZLfMDXYN5AZ0vDit7futRtszzr3XQgPIqz3xeuiLa
V5bPfH70/GieJuc+qW/6ruo8821wX+mw4DnsE2xrOtP+02jfQmO4KtoP0Zhp
rqSb1UZrTr6H1sJz8k2C5311YT9G+1d+jcZfdqhv8D46Oc/7YXlt67mLs42T
zdd+7ZH3V5HM10mHy6aLd2oa7D/0ynOxIfvq4mFa5TUsHdIoY+nYiYV/q2dO
KLyHhU/PY6h91K/wulTfVkevqfd418nB98QNTsn7WmtY729O3fvJ+0x7ir//
5kPPlC4emM9KOjO1yL8VBzW18Fxqn8q2yMaIRxuafeZGOt/kfSbOsFn2qeRb
rc3njgnI9dFzNyP7F9rn0mdvB/tQf0dzjYPy3lefe+e90zL3QWtJtvv4wvZ7
Xe6T7IR8JvVtXZbH5X7ukrHujwrWxZoX+TY6k3yddczgPCajg+dB3K3GVGOr
daU10Lfw/uib14TWw/jsvw3Kcz4hz5f8DPVR3OPjZZ+V/i7sY0wvPC7yAcfk
b5ye78n3ODJjtZlb9vh8wfW44P2tudRe156vktf42LzOF2XuazDvH4EcWdjH
kK8xIuOx+Tl6nuZfa2tN1mOjc39GZazvlz3fqbBNlz9wbuF3iGfTMzV38nHk
7x2V+6o+i7tW2+F5bZ+fse51yLZDNkQ6Sbqpai4a2/+fK3rnvTOjsA0KeS1o
XdbL++aMvI8mBX+fOHutX+0Bcakr8v3Tg3lK+Y/dS96LE/N89cl9kG6Unzk5
v2tiMNZvpSc07tKJ8jM1JgvzWaRd/q3OUvshX9fzknmQTZBzta+o3xR8j3xW
cDXwedzfBfws7WdTXxdcVee54Poa4HnBfONmyXyg7KH8ffnC8i/lWwpflPs8
M18LT8pY3zs5t5mUf3Nxnq9Tk3mlKsl1M/Nvz8xthE8vu/8PgG+iP43AGyVz
FwdwfSdy/2guupf42Gjeuxt4N/CTshHiPpP5QZ2v94jmww/WM6I59t7gvaK5
90NkZ6O59O7JfJB4oR34fR3KSvDR9Gt6MG92aPQZ/RJwTWS36HNHP37bNZrb
6Qs+SO1kp8AHR59H+oMPieaCjgCfE8wP96Kue7RvfDj1lwXzsRXwpcFnLvFz
I8ueu+vp42vB58f6tHk+2Ievm8/cfbR2Csej5OfXof7ZYP+8NnhZsG+/lc6z
wWtpC/FDybElndl1ZpG/N0Tn9+BzQUNxCcHPrwf+Qz6ubHTZNk/6eWK0Ldf6
ld2UfTupsB3RGXoI9d+AXw8+UzTQNybHq3S+/qqwnyB/u3M05yxO8Zjkc8+V
yFuSYzniQ74M9r3FP+jso/jRNuI6gv1zcR3ixGT3F4jPCObea4rfCPbPxVeM
TOZbk/ZTMt8nvuLn4PiR+LbPgn34xpnDUP2awtzMFHAZuTZYD4vP+TP4LCYO
56Fg/r8WeEAyn3cz8pPgM4U4llnJ5zk9+6PgsRLfsi6Zq+qg30XHjw4F7xsd
D+oJbhjNR6nNVuAbZJfALaO5td01Zsn8u/iiRtFnEJ1FWkRzYrtRv3U0V9YR
3B78ELgLeOdo7mvfZP5Je20v8UfRnF5n6tuAF6pe6y2ZrxdPtX00z7Y3uG00
z7aPOJlo3mwP8C7RPNt+OjtH82+KCWwTzRPuSn1z8Fxwp+QYqnwanRekGztm
O6tzefusi3Vm3LkwX9wvmIdrj/ytsP+s2KLOI50K8zOlrGP1nMOzHZc9F8em
79qh7DOW/DmdqdS+U9bJ0nszsh4bnu2Q+icOQf6BYknyEYTlM4j7+n/crE1+
puyl7HnHbI/kv+lb1O8Lsz6NfHsVfnN+yXxdJZo3FtenmFATcLtgflH65xD6
PCuYGx9A2wuDYwH9wBcEc/t9NYeF+fYdqbs4mM8/XHxIMM/fHzwzOL5wBPjW
YG58KPj6YA7/uOiY0wuye7I7wfGFPupvcEzhMPCc4HjNYPA1wXGTQVqz4O3B
x4OvC46bHAu+NjieMgR8bnB8rTf4kmBdfST4quBYwECdPaM56h3pQx3wbI0J
eMtoLnpn+U3B/Lm49n+C9684UnGo2juNkVWj+XlxpAuTY2biZq8MjlMcw/26
0Vz3TtQ/nBxPEgd7RfB4Hs39y4PnRTq8UzQ/cCBtOkZzDgeAO0Tz3vuDq0fz
/+K5N4u2NW2prxbN5++gPZjMp2uP3x4cmxjO/S0ot4NvRv5A/engczV3yfkE
p+o9yXG7G8Grg2NkM8ExOmZ3DXI59SPAp4N/yWMlHfhbcPzrYup/DY6RXaQ2
hX1Jxe6fS47t3UH988nxvPngcnSscLb8meiY5lUag+TY4U3gP/j9edRfJt2f
bcw88O/BMbhLwGuCY3yXgkN0zPFq+aRaB+A5yIYVx5hlTxtnfSJd92BynO86
6u5Pjheq/XfBsf7p4AeS447XSiclx19v0N5Jzts4Wb5ocq7GKeD60XbwFuS/
yRyQdNpDybHP66nfuuI4t+z+Xcmx2FnUN4iOad6qPZgcd9SY3JMca9RYNYmO
k96m70iOR2qO7k2OcerbhybHF+VLlPL60dpelMwx3R59ntTZTWe4a5NjAfre
DclxDunYOcl8/tla18l8/jS9Pzjn5AStx+R8lNP0nuS8kPGaz+Q8Eq2Z95L5
RPkktyXnW2jNfJjMhclvuTU5X0TrZ25y7oXm94Nkrk0+xu3J+Rlan+8m85Ly
Z1Yk85LyZ3TmFBem89ziZB5trtYa+N6yz4bzkvM2tGZeTfaJ5TOL+9KZUWfH
v4NjwZoXxc8UR7sc/FZwzom+8e/kGJVs5dPJcT7Nyz/JHKLs7NXJMZSp2uvJ
sXDtxyeTY8+a69nJsZUpWrPJsZWzwEcHcyDiQu5IzkHRXtA5R2d/cQBPJMez
td7k08i3UW7M/OQ8FfX5+OQYs/xV2Yzu2Ya0rJjrkV83NjmWLH0+MTl+LB3+
SHAO0ljpiuAcpBPBJybHlaVvT0+OAUsnT0qOp0pvn5Ecx1VMcHxy7Fl6fkJy
nFj6fH5w/HQEuGl0rF/r85vgvBqtvVoV56bIjlSvmDuWf16j4niq/PYlybkI
C2h/QnK8WTZrVXA+j8a/RrSfI116WnI8W7bmlOT4t+zFjOTY8Ljo+LLsrfJ3
xFnrPKrcjseC86/U5uzkuP5o8JTk2L++ZXpyPHuM7EZyPPgkrQGdgUvG5yXH
jzW2U5NzF0aCxyXH5mUvpiXnEIwCn5ScRyJbc3JyfonseNPsnyhX5fzkOLfm
6MzkWLhsweTkOLps9ILgOLjedWdwHFzPr1ZxvETnowYV5+voLFO/4nwdnWWq
VpwDJNs0KjnnQD7DiOR8Bdn00clxd/kVG1WczyS7VqXinCf5dfUqzivSmWhM
ck6A/IpNo3NCpCeHJec6yL63qHg/iGfYPJobl43YJDp/Qzp84+i8DunqrSrO
GdI5a3hyXoV8j2bUbxvMqbaumCvRGWHjivOZZIsrFedXyb+tC64dnPOj2IP8
HHHVC6nbo+T5rRkd35dduCs47q/1UKfinC2dMTevOO9K/nntivOrdG7douLc
LPnwm1Ucu9IZVjF0xWzWac9WnB8mPzkpISTYD68VHS+Q7f48OIdNOiRUnBMm
H7tccR6YfPKi4vww+eSbVpx/oDPy4uCcQNmyG6RTyualxU0p3qC44jvJcUqd
DcV7TBNnob0gO11yXPW15HwmcQ7iUcXnHRjMoyoGoHO7uCBxPcq3Unznz+xn
vp6cFyXOQTG+0+SnVsydKudBuQ9vJcerdF4TTyV+RrEC+efi6Y4I5vx1xtTv
b6Tdk2XzvTozioMVB/969Hv1/tbRuUPyK9pFnzF11tgJ/FzJvsrS5HwsncEV
q1DM4rjgfC09R897JTkHSxyFcjsnZy5n72iOWudW8dXiFMVd7xidNyW/6NE8
hhpLnU0UX+wvv5LrB0u2X+I/ld82LDgv7ci8NvYKjgMqHrg6OZdC/u2nyfcU
v1Dcp1V+pmIVimEplqVYUes8br8l50bIn/w9OTdCPvNfybkOOmf9mpzr0DaP
q2Ieys1ck5y3Ib/3j+QcCPnVijPpbCSuW7FqcYfKr5Mdlv1SzqnivuJnv812
TVydOFrxw+KixXOK4z06f7vivopFKaYvLlo+jHwZ5cOI31Uce31yDofOrcuS
447ivqQnxZkqT7NPnkPlAryZHLMU97WqMNeoGLryALXGW1WcN6tx0/jpzK5Y
lmJc+kZxjYoXK9dO671mxfmfn5ec3/RSMp8lvmuf6Jw62SlxauLWFNdQjFI8
U+OK4++KsSmXR3tRHK3yZL8omf9ulHWYYsSKFS+VPivZB3sxOHdXvv2nwfm3
8iU+So5BimP5JDlmKQ7nxWQuSVzHx8E8hjgH5f5prWrNfhWcByvfb0lw3q98
3Y+T453ijpRvuUrnKe6fk5xHJf9weXJcWVzHy8m5jOIbFSvXnGpu/wfpumyW

            "]], Polygon3DBox[CompressedData["
1:eJwtmgf8T9Ubx+8959xfCCFRRkaSQkNpKUWlhaK09057TxUV7aWdhvZQf7S3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            "]], Polygon3DBox[CompressedData["
1:eJwt13WglkUexfH3vtxLXbo7pFNRacFCGqVdWF1XBAR1ZV0FdQ10JQxKBZTu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            "]]},
          Annotation[#, "Charting`Private`Tag$396905#1"]& ]],
        Lighting->{{"Ambient", 
           RGBColor[0.30100577, 0.22414668499999998`, 0.090484535]}, {
          "Directional", 
           RGBColor[0.2642166, 0.18331229999999998`, 0.04261530000000001], 
           ImageScaled[{0, 2, 2}]}, {"Directional", 
           RGBColor[0.2642166, 0.18331229999999998`, 0.04261530000000001], 
           ImageScaled[{2, 2, 2}]}, {"Directional", 
           RGBColor[0.2642166, 0.18331229999999998`, 0.04261530000000001], 
           ImageScaled[{2, 0, 2}]}}]}, {}, {}, {}, {}}, {
      {GrayLevel[0], Line3DBox[CompressedData["
1:eJwl0ksoZ2EYB+DPYNzHLGRCjSSsWGiWY2GLUsr9bseCjeuOppAVEtMoTUJS
siElO7OwsHK/X9ZYiSw9/2bxO8/7njrv953znZyOnqruqBDCqAwoInW7ywt/
RYfwlVPM5l8WcYMl3GUFY2JDmPBMo/rEvVPpUj9ziO8cZ1xMCHNM5wrzuMUf
/McDfjNnWI70fcyX6k8hHOtreMJanrKOZ6znORt4wUY2yaW6mVds4TVbecM2
3kbelXfs4D3L7DXDmiPyoO9ngVTa26G+lHss5iZzucw0zjKWY3wzZ5BPfJRO
9REbuM9y7vAn11nIBX7nthmvHLRuKnuZJWvyRb/KFK4wmUtM4iITI+fEhMg8
xnOefyRO/ZufOUvHFmZouTBNy4dJ+hShXJMZ9f+f+AAV3kEn
        "]]}, 
      {GrayLevel[0.2], 
       Line3DBox[{899, 1205, 611, 898, 1381, 1104, 900, 1382, 1105, 901, 1383,
         1106, 902, 1384, 1107, 903, 1385, 1108, 904, 1386, 1285, 1479, 905, 
        1387, 1109, 906, 1388, 1110, 907, 1389, 1111, 908, 1390, 1112, 909, 
        1391, 1113, 910, 1392, 1114, 911, 1568, 1206, 1115, 1207}], 
       Line3DBox[{913, 1286, 1480, 912, 626, 914, 1393, 1116, 915, 1394, 1117,
         916, 1395, 1118, 917, 1396, 1119, 918, 1397, 1287, 1481, 919, 1288, 
        1482, 920, 1398, 1120, 921, 1399, 1121, 922, 1400, 1122, 923, 1401, 
        1123, 924, 1402, 1124, 925, 1403, 1125, 926}], 
       Line3DBox[{928, 1289, 1483, 927, 1208, 1209, 1290, 1569, 929, 1210, 
        1230, 642, 930, 1404, 1126, 931, 1405, 1127, 932, 1406, 1128, 933, 
        1407, 1291, 1484, 934, 1292, 1485, 935, 1293, 1486, 936, 1408, 1129, 
        937, 1409, 1130, 938, 1410, 1131, 939, 1411, 1132, 940, 1412, 1133, 
        941}], Line3DBox[{943, 1294, 1487, 942, 226, 874, 354, 249, 844, 285, 
        251, 944, 1567, 227, 945, 1413, 1134, 946, 1414, 1295, 1488, 947, 
        1296, 1489, 948, 1297, 1490, 949, 1298, 1491, 950, 1415, 1135, 951, 
        1416, 1136, 952, 1417, 1137, 953, 1418, 1138, 954}], 
       Line3DBox[{956, 1195, 1196, 1570, 955, 1243, 1197, 1211, 1198, 1573, 
        1244, 957, 1245, 1199, 1212, 802, 1275, 1262, 1280, 1263, 1276, 1200, 
        1237, 1201, 1246, 958, 1270, 803, 1269, 1202, 959, 1419, 1139, 960, 
        1420, 1299, 1492, 961, 1300, 1493, 962, 1301, 1494, 963, 1302, 1495, 
        964, 1303, 1496, 965, 1421, 1140, 966, 1422, 1141, 967, 1423, 1142, 
        968}], Line3DBox[{979, 1144, 1426, 978, 1143, 1425, 977, 1503, 1309, 
        976, 1502, 1308, 975, 1501, 1307, 974, 1500, 1306, 973, 1499, 1305, 
        972, 1498, 1304, 1424, 971, 1218, 825, 970, 1214, 1231, 1576, 339, 
        383, 316, 1213, 315, 338, 337, 1497, 1239, 313, 969, 824, 1238, 335, 
        381, 890, 380, 396, 395, 1284}], 
       Line3DBox[{982, 1265, 1427, 1145, 1271, 1272, 1273, 1268, 1380, 1577, 
        1146, 1219, 1250, 1251, 1241, 1379, 1572, 1147, 1279, 1256, 1257, 
        1221, 1242, 1148, 1220, 1260, 1575, 1378, 1223, 1149, 1232, 1373, 
        1428, 1150, 988, 1429, 690, 990, 1430, 1151, 992, 1431, 1152, 994, 
        1432, 1153, 996, 1433, 1154, 998, 1434, 1155, 1000, 1435, 1156, 1002, 
        1437, 1158, 1004}], 
       Line3DBox[{1003, 1157, 1436, 1001, 1511, 1315, 999, 1510, 1314, 997, 
        1509, 1313, 995, 1508, 1312, 993, 1507, 1311, 991, 1506, 1310, 989, 
        1505, 689, 987, 1222, 1504, 1372, 986, 1258, 1259, 1240, 1571, 1370, 
        985, 1254, 1266, 1255, 1217, 1253, 1261, 1252, 984, 1267, 869, 1216, 
        1249, 1248, 983, 1247, 1574, 1377, 1215, 1278, 1277, 980, 866, 1281, 
        1264, 981}], 
       Line3DBox[{1005, 1282, 1316, 1578, 1283, 1203, 1438, 1159, 1233, 1224, 
        1439, 1160, 1225, 1204, 1234, 1161, 1274, 1371, 1440, 1236, 1162, 
        1235, 1374, 1441, 1163, 1006, 1442, 1317, 1512, 1007, 703, 1008, 1443,
         1164, 1009, 1444, 1165, 1010, 1445, 1166, 1011, 1446, 1167, 1012, 
        1447, 1168, 1013, 1448, 1169, 1014}], 
       Line3DBox[{1016, 1318, 1513, 1015, 1319, 1514, 1017, 1449, 1170, 1018, 
        1226, 1450, 1171, 1019, 1375, 1451, 1172, 1020, 1452, 1173, 1021, 
        1453, 1320, 1515, 1022, 1321, 1516, 1023, 717, 1024, 1454, 1174, 1025,
         1455, 1175, 1026, 1456, 1176, 1027, 1457, 1177, 1028, 1458, 1178, 
        1029}], Line3DBox[{1031, 1322, 1517, 1030, 1323, 1518, 1032, 1324, 
        1519, 1033, 1459, 1179, 1034, 1460, 1180, 1035, 1461, 1181, 1036, 
        1462, 1325, 1520, 1037, 1326, 1521, 1038, 1327, 1522, 1039, 733, 1040,
         1463, 1182, 1041, 1464, 1183, 1042, 1465, 1184, 1043, 1466, 1185, 
        1044}], Line3DBox[{1046, 1328, 1523, 1045, 1329, 1524, 1047, 1330, 
        1525, 1048, 1331, 1526, 1049, 1467, 1186, 1050, 1468, 1187, 1051, 
        1469, 1332, 1527, 1052, 1333, 1528, 1053, 1334, 1529, 1054, 1335, 
        1530, 1055, 749, 1056, 1470, 1188, 1057, 1471, 1189, 1058, 1472, 1190,
         1059}], 
       Line3DBox[{1061, 1336, 1531, 1060, 1337, 1532, 1062, 1338, 1533, 1063, 
        1339, 1534, 1064, 1340, 1535, 1065, 1473, 1191, 1066, 1474, 1341, 
        1536, 1067, 1342, 1537, 1068, 1343, 1538, 1069, 1344, 1539, 1070, 
        1345, 1540, 1071, 765, 1072, 1475, 1192, 1073, 1476, 1193, 1074}], 
       Line3DBox[{1076, 1346, 1541, 1075, 1347, 1542, 1077, 1348, 1543, 1078, 
        1349, 1544, 1079, 1350, 1545, 1080, 1351, 1546, 1081, 1477, 1352, 
        1547, 1082, 1353, 1548, 1083, 1354, 1549, 1084, 1355, 1550, 1085, 
        1356, 1551, 1086, 1357, 1552, 1087, 781, 1088, 1478, 1194, 1089}], 
       Line3DBox[{1103, 1229, 840, 1102, 1566, 1369, 1101, 1565, 1368, 1100, 
        1564, 1367, 1099, 1563, 1366, 1098, 1562, 1365, 1097, 1561, 1364, 
        1096, 1560, 1559, 1363, 1095, 1558, 1362, 1094, 1557, 1361, 1093, 
        1556, 1360, 1092, 1555, 1359, 1091, 1554, 1358, 1090, 1376, 1553, 
        1227, 1228}]}, 
      {GrayLevel[0.2], 
       Line3DBox[{400, 612, 1381, 401, 626, 429, 816, 1569, 641, 444, 797, 
        874, 656, 459, 858, 801, 1573, 820, 854, 855, 471, 824, 852, 876, 895,
         877, 484, 1574, 860, 878, 806, 827, 891, 892, 1577, 497, 810, 849, 
        699, 1438, 510, 1514, 711, 523, 1518, 724, 538, 1524, 739, 553, 1532, 
        754, 568, 1542, 769, 583, 1554, 784, 598}], 
       Line3DBox[{402, 613, 1382, 403, 627, 1393, 430, 817, 642, 445, 818, 
        844, 863, 864, 887, 865, 802, 875, 856, 857, 472, 867, 1497, 805, 881,
         879, 882, 880, 485, 869, 883, 807, 828, 885, 886, 1572, 498, 833, 
        834, 700, 1439, 511, 712, 1449, 524, 1519, 725, 539, 1525, 740, 554, 
        1533, 755, 569, 1543, 770, 584, 1555, 785, 599}], 
       Line3DBox[{404, 614, 1383, 405, 628, 1394, 431, 643, 1404, 446, 251, 
        296, 1237, 231, 257, 1213, 234, 265, 1217, 237, 1221, 270, 240, 1234, 
        291, 275, 1450, 525, 726, 1459, 540, 1526, 741, 555, 1534, 756, 570, 
        1544, 771, 585, 1556, 786, 600}], 
       Line3DBox[{406, 615, 1384, 407, 629, 1395, 432, 644, 1405, 447, 798, 
        1567, 799, 870, 871, 803, 821, 804, 859, 845, 868, 1576, 822, 884, 
        823, 862, 853, 1571, 808, 829, 809, 861, 1575, 832, 811, 873, 812, 
        1440, 851, 837, 838, 1451, 526, 727, 1460, 541, 742, 1467, 556, 1535, 
        757, 571, 1545, 772, 586, 1557, 787, 601}], 
       Line3DBox[{408, 616, 1385, 409, 630, 1396, 433, 645, 1406, 448, 657, 
        1413, 460, 668, 1419, 473, 825, 847, 826, 486, 1504, 830, 848, 831, 
        1428, 499, 835, 850, 836, 1441, 512, 713, 1452, 527, 728, 1461, 542, 
        743, 1468, 557, 758, 1473, 572, 1546, 773, 587, 1558, 788, 602}], 
       Line3DBox[{410, 617, 1386, 412, 631, 1397, 434, 646, 1407, 449, 658, 
        1414, 461, 669, 1420, 474, 679, 1424, 487, 689, 1429, 500, 701, 1442, 
        513, 714, 1453, 528, 729, 1462, 543, 744, 1469, 558, 759, 1474, 573, 
        774, 1477, 588, 1559, 789, 603}], 
       Line3DBox[{414, 619, 1387, 415, 1482, 633, 436, 1485, 648, 451, 1489, 
        660, 463, 1493, 671, 476, 1499, 681, 489, 1506, 691, 1430, 502, 703, 
        515, 1516, 716, 530, 1521, 731, 545, 1528, 746, 560, 1537, 761, 575, 
        1548, 776, 590, 1561, 791, 605}], 
       Line3DBox[{416, 620, 1388, 417, 634, 1398, 437, 1486, 649, 452, 1490, 
        661, 464, 1494, 672, 477, 1500, 682, 490, 1507, 692, 1431, 503, 704, 
        1443, 516, 717, 531, 1522, 732, 546, 1529, 747, 561, 1538, 762, 576, 
        1549, 777, 591, 1562, 792, 606}], 
       Line3DBox[{418, 621, 1389, 419, 635, 1399, 438, 650, 1408, 453, 1491, 
        662, 465, 1495, 673, 478, 1501, 683, 491, 1508, 693, 1432, 504, 705, 
        1444, 517, 718, 1454, 532, 733, 547, 1530, 748, 562, 1539, 763, 577, 
        1550, 778, 592, 1563, 793, 607}], 
       Line3DBox[{420, 622, 1390, 421, 636, 1400, 439, 651, 1409, 454, 663, 
        1415, 466, 1496, 674, 479, 1502, 684, 492, 1509, 694, 1433, 505, 706, 
        1445, 518, 719, 1455, 533, 734, 1463, 548, 749, 563, 1540, 764, 578, 
        1551, 779, 593, 1564, 794, 608}], 
       Line3DBox[{422, 623, 1391, 423, 637, 1401, 440, 652, 1410, 455, 664, 
        1416, 467, 675, 1421, 480, 1503, 685, 493, 1510, 695, 1434, 506, 707, 
        1446, 519, 720, 1456, 534, 735, 1464, 549, 750, 1470, 564, 765, 579, 
        1552, 780, 594, 1565, 795, 609}], 
       Line3DBox[{424, 624, 1392, 425, 638, 1402, 441, 653, 1411, 456, 665, 
        1417, 468, 676, 1422, 481, 686, 1425, 494, 1511, 696, 1435, 507, 708, 
        1447, 520, 721, 1457, 535, 736, 1465, 550, 751, 1471, 565, 766, 1475, 
        580, 781, 595, 1566, 796, 610}], 
       Line3DBox[{426, 814, 1568, 815, 427, 639, 1403, 442, 654, 1412, 457, 
        666, 1418, 469, 677, 1423, 482, 687, 1426, 495, 697, 1436, 1437, 508, 
        709, 1448, 521, 722, 1458, 536, 737, 1466, 551, 752, 1472, 566, 767, 
        1476, 581, 782, 1478, 596, 840, 841, 842}], 
       Line3DBox[{597, 783, 839, 1553, 582, 768, 1541, 567, 753, 1531, 552, 
        738, 1523, 537, 723, 1517, 522, 710, 1513, 509, 698, 1578, 894, 893, 
        496, 1427, 688, 872, 866, 483, 678, 897, 896, 846, 889, 890, 888, 470,
         667, 819, 1570, 800, 458, 655, 1487, 443, 640, 1483, 428, 625, 1480, 
        399, 611, 813, 843}], 
       Line3DBox[{604, 790, 1560, 589, 775, 1547, 574, 760, 1536, 559, 745, 
        1527, 544, 730, 1520, 529, 715, 1515, 514, 702, 1512, 501, 690, 1505, 
        488, 680, 1498, 475, 670, 1492, 462, 659, 1488, 450, 647, 1484, 435, 
        632, 1481, 413, 618, 1479, 411}]}, {}, {}}},
    VertexNormals->CompressedData["
1:eJxcfHc4l2/YPhEloySKRKUhEilSuJIWSRKJ7Fn23nsmeyvZe+/Iuu29yd57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     "]], {}},
  Axes->True,
  AxesEdge->{{-1, -1}, {1, -1}, {-1, -1}},
  AxesLabel->{None, None, None},
  AxesOrigin->{Automatic, Automatic, Automatic},
  BaseStyle->{FontFamily -> "Arial"},
  BoxRatios->{1, 1, 0.4},
  Boxed->{Left, Bottom, Back},
  DisplayFunction->Identity,
  FaceGrids->
   NCache[{{{0, 0, -1}, {{0, 1, 2, 3, 4, 5}, {0, 1, 2, 3, 4, 5}}}, {{0, 1, 
      0}, {{0, 1, 2, 3, 4, 5}, {0, 
        Rational[1, 2], 1, 
        Rational[3, 2], 2, 
        Rational[5, 2]}}}, {{-1, 0, 0}, {{0, 1, 2, 3, 4, 5}, {0, 
        Rational[1, 2], 1, 
        Rational[3, 2], 2, 
        Rational[5, 2]}}}}, {{{0, 0, -1}, {{0, 1, 2, 3, 4, 5}, {0, 1, 2, 3, 4,
       5}}}, {{0, 1, 
      0}, {{0, 1, 2, 3, 4, 5}, {0, 0.5, 1, 1.5, 2, 2.5}}}, {{-1, 0, 
      0}, {{0, 1, 2, 3, 4, 5}, {0, 0.5, 1, 1.5, 2, 2.5}}}}],
  FaceGridsStyle->Directive[
    GrayLevel[0.3, 1], 
    AbsoluteDashing[{1, 2}]],
  ImageSize->{426.4297432514154, 336.6096882915044},
  ImageSizeRaw->Automatic,
  Method->{"DefaultBoundaryStyle" -> Directive[
      GrayLevel[0.2]], 
    "DefaultGraphicsInteraction" -> {
     "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
      "Effects" -> {
       "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
        "Droplines" -> {
         "freeformCursorMode" -> True, 
          "placement" -> {"x" -> "All", "y" -> "None"}}}}, "RotationControl" -> 
    "Globe"},
  PlotRange->{{0, 5}, {0, 5}, {0.0074754895810681, 2.475730591407155}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]},
  Ticks->{Automatic, Automatic, Automatic},
  ViewPoint->{2.133598968806885, -1.807242983228646, 1.9056831425703993`},
  ViewVertical->{-0.42973683720945594`, 0.36400415215608906`, 
   0.8263336057297564}]], "Output",
 CellChangeTimes->{
  3.811001126348185*^9, 3.811001375484942*^9, {3.811001494264475*^9, 
   3.8110015182763433`*^9}, 3.8110017610834703`*^9, 3.81100237722338*^9, 
   3.8110024224170637`*^9, 3.811002765216105*^9, {3.811007389643259*^9, 
   3.811007568105299*^9}, {3.811007623182618*^9, 3.8110076351583557`*^9}},
 CellLabel->
  "Out[228]=",ExpressionUUID->"24fe5063-d0ca-4355-8896-335074916c65"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Multivariate Coupled Gaussian", "Subsection",
 CellChangeTimes->{{3.8110024509282722`*^9, 
  3.8110024611176443`*^9}},ExpressionUUID->"4e3e3e17-754c-4c95-8967-\
86eb6298b902"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"PDF", "[", 
    RowBox[{
     RowBox[{"MultivariateCoupledDistribution", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"1", ",", 
           RowBox[{"-", "0.01"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0.01", ",", "1"}], "}"}]}], "}"}], ",", "0.01", ",", "2"}],
       "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"-", "5"}], ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"-", "5"}], ",", "5"}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"PlotLegends", "\[Rule]", "None"}], ",", "\[IndentingNewLine]", 
   RowBox[{"PlotTheme", "\[Rule]", "\"\<Detailed\>\""}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"PlotRange", "\[Rule]", "Full"}]}], "\[IndentingNewLine]", 
  "]"}]], "Input",
 CellChangeTimes->{{3.811002473726088*^9, 3.811002497789809*^9}, {
  3.811002534798026*^9, 3.811002598289109*^9}, {3.811007644293721*^9, 
  3.811007660599759*^9}},
 CellLabel->
  "In[230]:=",ExpressionUUID->"3bd5f0dd-d54f-440e-bbdd-a74e20b6579d"],

Cell[BoxData[
 Graphics3DBox[{GraphicsComplex3DBox[CompressedData["
1:eJx1nHk8VG/7x2Uk0ihbO0lKilJZosw5VJKl0EaWslQKpYVChUgpJKmQhOyh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    "], {{
      {RGBColor[0.880722, 0.611041, 0.142051], EdgeForm[None], Specularity[
        GrayLevel[1], 3], 
       StyleBox[GraphicsGroup3DBox[
         TagBox[{Polygon3DBox[CompressedData["
1:eJwtmXfAj9Ubxr/Pc863QkJWiMioiLaISNGwygyhUkpDFEWFJCLR0kBUfiqy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            "]], Polygon3DBox[CompressedData["
1:eJwtmXPAJkcehKenJ94kGxsb27Z5sX2xdbFt27YvzsY2N7ZtX3zPszV/9Ddd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            "]], 
           Polygon3DBox[CompressedData["
1:eJwt12fYFNUVAOBldwGpIggoKBaKSg2gIF2pkSo9oSZUQzVUI02liAKiAkKU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            "]]},
          Annotation[#, "Charting`Private`Tag$397575#1"]& ]],
        Lighting->{{"Ambient", 
           RGBColor[0.30100577, 0.22414668499999998`, 0.090484535]}, {
          "Directional", 
           RGBColor[0.2642166, 0.18331229999999998`, 0.04261530000000001], 
           ImageScaled[{0, 2, 2}]}, {"Directional", 
           RGBColor[0.2642166, 0.18331229999999998`, 0.04261530000000001], 
           ImageScaled[{2, 2, 2}]}, {"Directional", 
           RGBColor[0.2642166, 0.18331229999999998`, 0.04261530000000001], 
           ImageScaled[{2, 0, 2}]}}]}, {}, {}, {}, {}}, {
      {GrayLevel[0], Line3DBox[CompressedData["
1:eJwt0rsrRnEYwPHjfr8lpYj8AWxkYJSBSTKwkYGBkoHRIDYbfwEriV2xufO6
35Pktrhf4/Mrw/f9PPX+Ouc5nVPW3tvUExdF0bBeFeZHXWo2PopyucBSrrOC
p6zlAxv5zTZmJERRN4tYrEFzNUdZzwm2cIqdnGc/l8K1eM13betZ91q2UIwr
3OEqd7nGvbAT97nBA27ykFvc1pE5xmPu8IS7POUez7jPc057jht+6EIvYS+N
2W2LQ1xkH+fYwUk2c5x1HGEVKzVgLmQX09jKT/do4B1reMzy8Fws4a1zV3xT
jp7CWRXYMZv5zGIeM8MZZjCL6eEdMI2pTFGqOSnMTGByeMdM4q8S9RP+05es
Ec34uf3/Hv4AxpdK8A==
        "]]}, 
      {GrayLevel[0.2], 
       Line3DBox[{690, 1001, 473, 689, 1107, 912, 691, 1108, 913, 692, 1109, 
        914, 693, 1110, 915, 694, 1111, 916, 695, 1112, 917, 696, 1113, 481, 
        697, 1114, 918, 698, 1115, 919, 699, 1116, 920, 700, 1117, 921, 701, 
        1118, 922, 702, 1106, 1119, 923, 1002}], 
       Line3DBox[{704, 1007, 1210, 703, 488, 705, 1120, 924, 706, 1121, 925, 
        707, 1122, 926, 708, 1123, 927, 709, 1124, 928, 710, 1125, 1008, 1211,
         711, 1009, 1212, 712, 1126, 929, 713, 1127, 930, 714, 1128, 931, 715,
         1129, 932, 716, 1130, 933, 717}], 
       Line3DBox[{719, 1010, 1213, 718, 1011, 1214, 720, 504, 721, 1131, 934, 
        722, 1132, 935, 723, 1133, 936, 724, 1134, 937, 725, 1135, 1012, 1215,
         726, 1013, 1216, 727, 512, 728, 1136, 938, 729, 1137, 939, 730, 1138,
         940, 731, 1139, 941, 732}], 
       Line3DBox[{734, 1014, 1217, 733, 1015, 1218, 735, 1016, 1219, 736, 520,
         737, 1140, 942, 738, 1141, 943, 739, 1142, 944, 740, 1143, 1017, 
        1220, 741, 1018, 1221, 742, 1019, 1222, 743, 1144, 945, 744, 1145, 
        946, 745, 1146, 947, 746, 1147, 948, 747}], 
       Line3DBox[{749, 1020, 1223, 748, 1021, 1224, 750, 1022, 1225, 751, 
        1023, 1226, 752, 536, 753, 1148, 949, 754, 1149, 950, 755, 1150, 1024,
         1227, 756, 1025, 1228, 757, 1026, 1229, 758, 1027, 1230, 759, 1151, 
        951, 760, 1152, 952, 761, 1153, 953, 762}], 
       Line3DBox[{764, 1028, 1231, 763, 1029, 1232, 765, 1030, 1233, 766, 
        1031, 1234, 767, 1032, 1235, 768, 552, 769, 1154, 954, 770, 1155, 
        1033, 1236, 771, 1034, 1237, 772, 1035, 1238, 773, 1036, 1239, 774, 
        1037, 1240, 775, 1156, 955, 776, 1157, 956, 777}], 
       Line3DBox[{779, 1038, 1241, 778, 1039, 1242, 780, 1040, 1243, 781, 
        1041, 1244, 782, 1042, 1245, 783, 1043, 1246, 784, 568, 785, 1158, 
        1044, 1247, 786, 1045, 1248, 787, 1046, 1249, 788, 1047, 1250, 789, 
        1048, 1251, 790, 1049, 1252, 791, 1050, 1253, 792}], 
       Line3DBox[{796, 1159, 957, 794, 1160, 958, 798, 1161, 959, 800, 1162, 
        960, 802, 1163, 961, 804, 1164, 962, 806, 1165, 963, 808, 1166, 585, 
        810, 1167, 964, 812, 1168, 965, 814, 1169, 966, 816, 1170, 967, 818, 
        1171, 968, 820, 1172, 969, 822}], 
       Line3DBox[{821, 1267, 1063, 819, 1266, 1062, 817, 1265, 1061, 815, 
        1264, 1060, 813, 1263, 1059, 811, 1262, 1058, 809, 1261, 584, 807, 
        1260, 1057, 805, 1259, 1056, 803, 1258, 1055, 801, 1257, 1054, 799, 
        1256, 1053, 797, 1255, 1052, 793, 1254, 1051, 795}], 
       Line3DBox[{824, 1064, 1268, 823, 1173, 970, 825, 1174, 971, 826, 1175, 
        972, 827, 1176, 973, 828, 1177, 974, 829, 1178, 975, 830, 1179, 1065, 
        1269, 831, 601, 832, 1180, 976, 833, 1181, 977, 834, 1182, 978, 835, 
        1183, 979, 836, 1184, 980, 837}], 
       Line3DBox[{839, 1066, 1270, 838, 1067, 1271, 840, 609, 841, 1185, 981, 
        842, 1186, 982, 843, 1187, 983, 844, 1188, 984, 845, 1189, 1068, 1272,
         846, 1069, 1273, 847, 617, 848, 1190, 985, 849, 1191, 986, 850, 1192,
         987, 851, 1193, 988, 852}], 
       Line3DBox[{854, 1070, 1274, 853, 1071, 1275, 855, 1072, 1276, 856, 625,
         857, 1194, 989, 858, 1195, 990, 859, 1196, 991, 860, 1197, 1073, 
        1277, 861, 1074, 1278, 862, 1075, 1279, 863, 633, 864, 1198, 992, 865,
         1199, 993, 866, 1200, 994, 867}], 
       Line3DBox[{869, 1076, 1280, 868, 1077, 1281, 870, 1078, 1282, 871, 
        1079, 1283, 872, 1080, 1284, 873, 1201, 995, 874, 1202, 996, 875, 
        1203, 1081, 1285, 876, 1082, 1286, 877, 1083, 1287, 878, 1084, 1288, 
        879, 649, 880, 1204, 997, 881, 1205, 998, 882}], 
       Line3DBox[{884, 1085, 1289, 883, 1086, 1290, 885, 1087, 1291, 886, 
        1088, 1292, 887, 1089, 1293, 888, 657, 889, 1206, 999, 890, 1207, 
        1090, 1294, 891, 1091, 1295, 892, 1092, 1296, 893, 1093, 1297, 894, 
        1094, 1298, 895, 665, 896, 1208, 1000, 897}], 
       Line3DBox[{911, 1006, 685, 910, 1309, 1105, 909, 1308, 1104, 908, 1307,
         1103, 907, 1306, 1102, 906, 1305, 1101, 905, 1304, 1100, 1209, 904, 
        673, 903, 1303, 1099, 902, 1302, 1098, 901, 1301, 1097, 900, 1300, 
        1096, 899, 1299, 1095, 898, 1310, 1004, 1003, 1005}]}, 
      {GrayLevel[0.2], 
       Line3DBox[{251, 474, 1107, 252, 488, 280, 1214, 503, 295, 1218, 518, 
        310, 1224, 533, 325, 1232, 548, 340, 1242, 563, 355, 1255, 578, 1160, 
        370, 593, 1173, 385, 1271, 608, 400, 1275, 623, 415, 1281, 638, 430, 
        1290, 653, 445, 1299, 668, 460}], 
       Line3DBox[{253, 475, 1108, 254, 489, 1120, 281, 504, 296, 1219, 519, 
        311, 1225, 534, 326, 1233, 549, 341, 1243, 564, 356, 1256, 579, 1161, 
        371, 594, 1174, 386, 609, 401, 1276, 624, 416, 1282, 639, 431, 1291, 
        654, 446, 1300, 669, 461}], 
       Line3DBox[{255, 476, 1109, 256, 490, 1121, 282, 505, 1131, 297, 520, 
        312, 1226, 535, 327, 1234, 550, 342, 1244, 565, 357, 1257, 580, 1162, 
        372, 595, 1175, 387, 610, 1185, 402, 625, 417, 1283, 640, 432, 1292, 
        655, 447, 1301, 670, 462}], 
       Line3DBox[{257, 477, 1110, 258, 491, 1122, 283, 506, 1132, 298, 521, 
        1140, 313, 536, 328, 1235, 551, 343, 1245, 566, 358, 1258, 581, 1163, 
        373, 596, 1176, 388, 611, 1186, 403, 626, 1194, 418, 1284, 641, 433, 
        1293, 656, 448, 1302, 671, 463}], 
       Line3DBox[{259, 478, 1111, 260, 492, 1123, 284, 507, 1133, 299, 522, 
        1141, 314, 537, 1148, 329, 552, 344, 1246, 567, 359, 1259, 582, 1164, 
        374, 597, 1177, 389, 612, 1187, 404, 627, 1195, 419, 642, 1201, 434, 
        657, 449, 1303, 672, 464}], 
       Line3DBox[{261, 479, 1112, 262, 493, 1124, 285, 508, 1134, 300, 523, 
        1142, 315, 538, 1149, 330, 553, 1154, 345, 568, 360, 1260, 583, 1165, 
        375, 598, 1178, 390, 613, 1188, 405, 628, 1196, 420, 643, 1202, 435, 
        658, 1206, 450, 673, 465}], 
       Line3DBox[{263, 480, 1113, 265, 494, 1125, 286, 509, 1135, 301, 524, 
        1143, 316, 539, 1150, 331, 554, 1155, 346, 569, 1158, 361, 584, 1166, 
        376, 599, 1179, 391, 614, 1189, 406, 629, 1197, 421, 644, 1203, 436, 
        659, 1207, 451, 674, 1209, 466}], 
       Line3DBox[{267, 482, 1114, 268, 1212, 496, 288, 1216, 511, 303, 1221, 
        526, 318, 1228, 541, 333, 1237, 556, 348, 1248, 571, 363, 1262, 586, 
        1167, 378, 601, 393, 1273, 616, 408, 1278, 631, 423, 1286, 646, 438, 
        1295, 661, 453, 1305, 676, 468}], 
       Line3DBox[{269, 483, 1115, 270, 497, 1126, 289, 512, 304, 1222, 527, 
        319, 1229, 542, 334, 1238, 557, 349, 1249, 572, 364, 1263, 587, 1168, 
        379, 602, 1180, 394, 617, 409, 1279, 632, 424, 1287, 647, 439, 1296, 
        662, 454, 1306, 677, 469}], 
       Line3DBox[{271, 484, 1116, 272, 498, 1127, 290, 513, 1136, 305, 528, 
        1144, 320, 1230, 543, 335, 1239, 558, 350, 1250, 573, 365, 1264, 588, 
        1169, 380, 603, 1181, 395, 618, 1190, 410, 633, 425, 1288, 648, 440, 
        1297, 663, 455, 1307, 678, 470}], 
       Line3DBox[{273, 485, 1117, 274, 499, 1128, 291, 514, 1137, 306, 529, 
        1145, 321, 544, 1151, 336, 1240, 559, 351, 1251, 574, 366, 1265, 589, 
        1170, 381, 604, 1182, 396, 619, 1191, 411, 634, 1198, 426, 649, 441, 
        1298, 664, 456, 1308, 679, 471}], 
       Line3DBox[{275, 486, 1118, 276, 500, 1129, 292, 515, 1138, 307, 530, 
        1146, 322, 545, 1152, 337, 560, 1156, 352, 1252, 575, 367, 1266, 590, 
        1171, 382, 605, 1183, 397, 620, 1192, 412, 635, 1199, 427, 650, 1204, 
        442, 665, 457, 1309, 680, 472}], 
       Line3DBox[{277, 682, 683, 1119, 278, 501, 1130, 293, 516, 1139, 308, 
        531, 1147, 323, 546, 1153, 338, 561, 1157, 353, 1253, 576, 368, 1267, 
        591, 1172, 383, 606, 1184, 398, 621, 1193, 413, 636, 1200, 428, 651, 
        1205, 443, 666, 1208, 458, 685, 686, 687}], 
       Line3DBox[{459, 667, 1310, 684, 444, 652, 1289, 429, 637, 1280, 414, 
        622, 1274, 399, 607, 1270, 384, 592, 1268, 369, 1159, 577, 1254, 354, 
        562, 1241, 339, 547, 1231, 324, 532, 1223, 309, 517, 1217, 294, 502, 
        1213, 279, 487, 1210, 250, 473, 681, 688}], 
       Line3DBox[{467, 675, 1304, 452, 660, 1294, 437, 645, 1285, 422, 630, 
        1277, 407, 615, 1272, 392, 600, 1269, 377, 585, 1261, 362, 570, 1247, 
        347, 555, 1236, 332, 540, 1227, 317, 525, 1220, 302, 510, 1215, 287, 
        495, 1211, 266, 481, 264}]}, {}, {}}},
    VertexNormals->CompressedData["
1:eJxcnHk8lc/7/1W2kqKUiIoUyRJZst1jF8lOlKxFIfueJUsoyV6RFrsoZSfO
jDVL9iiFqMhahwrZf+cc53P6/t790fvR83G/55655pprrut1zxkuKye9y5up
qKgO0FBRbSH8N5KX16YpjwtXKyFxHD3iwVGR/swAucgHcbBSH6cpJLv5KPd5
Crd6pHK1/3EQbjBU+YD88m0Kb+T4NZqmmIM7lGm6I2FLKYVf0b//I6K6CWfA
pIwexPRTuDn9ToH2L19wfEdinzs0LFI44a/TP8VmcbIZmr8LY5jh/+HDIO43
bjZ7lubvQa7/y29wNH3GSQ2evi4ey0vhIoxJOUFq3+VO9pewGsGjFB5ueNSJ
+cAfuc3OjvIM7QcovNRn50k9XbzciUAD2eZgRgr/FB++mt4xKBev/S6/Rv0P
pZ/SA3PubIJ1cis2fI6K13oofIvBHmPjjFS5sPAHRx4O5VP42wuhU1d2WeLQ
q0CTsK+WFB5ZMSuHPU/E5WNBChMNSRSu7Nup7fG5AVe+Q8X6C3M7hR/61vY0
vX8G90M+sn7Ee5nC/eWnH/jR7IZa2Xr1fjSHKf3fP1KiGzwlCE+Yp/nwmipR
+Nr6uvXBWgwKurpc+yViCgn/JvzBg8X19UrxM4qwxspwzRA5wD9kPr6+jj7a
CMLr0xOmJRLu8DuZ04V/cX0uLY7tNBx/3OThBsfJXJDPzjw5TxFrdDpW9ifv
Kpwn84OikztO5MlgTpJbZr2unqO8d/jV5ryJKD7s/u6eD02bMUo/Y7CJfKmp
7Zj2FB2dybP9FL7nynf93NUxuUrrvusPPv6i2IHJ+5wrz8+HuHJvuSE7/D97
PuU9HRTI1YP7WIPvZTrVS+EV8VrCcrO0MEjf7tHmeQZK+7Jidgpv205CbVYq
6a0PZSk8RdHdlbbCGEaPHfDpmrhMGdeCUj2Hl5AHrKvjMP92JgLqkHnU8voQ
VAuHOhr7sDOTafDr4gYXH19XU1u6A9PaDgmdFSqGr0c3+Ju+dXCvzQuG7egI
uLsZB5XfkudFnvUa9ZEgTGBvtvyxgUoo1rHBz+8Yu6dqGoVpzzuryIwUQL6p
DT7yV8z+QWEoxif8U3GU4QnsXt7gawkj4VkcLli+n3d18a1g+IDczz20tULc
znqYwqYzo8VK5nCFzB3EA24HxB/H9tShy05iJyl2GHB2ZRcq7ca10nI4h5/5
559ZizeN7YJ3wILjDwwPb91OeV45p5R3LE4Jivo8eLinVZnCNZqFD7YOuUKa
+fnzfvVe8A35vVI9TgqDNMmQ48/sQtv+DLhtYYO/3nwuV1axHNp2WtPKKTZB
9tPkcaUu9q9db4WCQ+lU/IIjsLPvL4mfGpj+vXq7G0Z+m3YRC52HBn8/k7il
NM5chxvBelZ3c+nbVGjGrJLEe31dFDrM3mBFBTvLl16tQ3dcNYnfTtFZwLK7
MfGpZv6irt/wNOt3Et8SbKkjSd+M1bPk7Vu9OAy7Ty6TuBhvrHLXmSLsQ6Ag
ZiVVC4sTN/pJ7XD6yhhDPDa5ksA/wJsC/cjzYhml2LtWcRXTAgIap+WcKOtL
zTQySv39FnikzfrC0KV/6/3a7pnHRvyK8HSEEGe5zz//vGtzR+EJiy+8KzVp
9CrNCxaT24l3UdlzbjYPWu2xiZXckk/xt/bMx9OGQr2wPyHt05Opfkr/JT3s
F+quzEGFrI++DYqbEfVqNIkrla2cqW3ehthM2FqkhQ8hro9iy0TOtDtFdUF8
F7LbfnjFc0AUnXX3Hidyf5afjlVS63CbFM0+PU1Z9COktI3IV4zClI9a0YJs
myKhGQYZdMa+pYtkT63hMR2RXUAjki64wO8E0uor+0Hk04r7CtLU6MBBe+V7
6gtsaEbt/QqRh7aYJuxvnMF+9GckjtItQ2fyfP01zKz55daOPfBRffFe9B2M
y14l8cQmPd/lnDSs+vEnvvv66fD1nw073BTqeae3+wSc73VkkDbnptjzpPbN
g9IXneEZ/Xy559KXKfEtSOXrTZ+tuZAG3qDaH54O3fEbPGXHNZPq159h5ibX
9Vydfnjv8iKJf6nS7Odz2IpmTIbudVQyoFfcj9dIfrLpFq1GwXGkthtGiN05
iVaKuQeI/BbDA/GLForoXkeV3kULPfRX4qvJGnFdvLBuwW3WQLfrnIUCztgg
LteHcQsEXh355aI5TgLla1qfoNvhgh7ktBX8IHCvT0V2PUryoDXsrGqPkjPa
eZFJHE/gAeyDK5Zd6mA43hVtvWiNLp68579I4LknT90RSAHgdm9JWJHgWcRy
pHia+N5OvC+Msz0CgnmYt7PdFkB8FpzTpHilsUQbv7gJxJ36PJpRsxlZqJ4j
jffhqNIcFU8Pxmf+95OjUhc0kN+wf4JSU3Wrsj4k/ENts5MCJf5zDuEd0Y1E
eG1tXSFsTxgsX93gCR7vJdNLuqFG2VpRgeQbePfoGon/MY1hZKCnR1umokQl
J6hQwiG+DfsftKIq8BJHDBcHL6vdFkVcoRv+1p2dsHV80RhJTe/p4Zq8gORb
50OXCJyrrCZWwsMFJalzcipkXEcPs0xEcUS/ClKm4072QaMHg/xczWNR3kTs
iWdrBP83LMaa622Quu73g7lbHyLO7bjXHKt4sK36raWggjPY+270bnhVEgrG
H91iQuABdZydHco+YPkqnqFs011EBVsW5wntJG1mPJ1fcw2YU/cm1UMP1DRu
n9JDeG9mjKfK5V364HC5+JUMLgN08PLq2gqBP7kq8OfXpCD40/Yufq6UD+10
3Vhfp8+dmxdeXcMGLr9Rk6WbgyPnckh2mBarZKuNdybu33TG7EbwJ9nOLHxG
bLY/iyA2us57l/8xfPyNvL8MLkiA7F9wsenXs3N6w5T4pmaPHOWeH0MLvYcW
sp+wo7r2n7+I/IeDaMkHe2PUVMi1bYRFG12r2POa2M85nF+07Ko/+vBLytax
0Bsh7ercVALPEeKL2fo5EWV+7lVkCElCtV8rcu4t44HhW2e+l75P0e3DjgIv
fV8htwmazJpJPNjxjofpSk4UqnbHu5//XYFoaQVEZnrwgM12192oI4kg1Tgq
sSOvHD33EtTj7MMD3b/qUg6iT8E1T5YyOePnSPH7pocGeILdmNMFhgPjALMJ
g6HgdDyKGH1Cmi8luYU6meeeQPmCVpRtmgsqi7d1aiXaua7PKb1TBxxd5VB1
9FZG0rSBfkT/n4t6yXHI8BA4oHTO+pAhE7qItSyR1umJj8W18yGwaX29P4vD
krKfcneORq8ZNcNN/uvpedk5cMlwg9eKDDqe2LwVjWTbP/HknoZXyOslP79Y
4wWbImoPuXhflpsPqYesZRB5S3RSoASdK7rxLvgd/42LyNPbSX2cmF/Z+7Sv
095H21h3LdZZRiB7cWGSv2mnVnTbqhWg2E+Di/unniF73cdpLV/xYGD9DIOj
5WukWuhY4j9XjYIKe0UbI/HAIvSrRUNPNtol6+J3saMFpWUZt4srEPzZXESn
ZfkVuHM7VvulTzOa1L95qFCdEFcvS+Rxj1SANNz1wOxDOHTgfOOZm0/x4GvV
MVND9hfgIbd9w/SRDJTln6VpM40HQj8zfHpOx4LFgD0BvaeDUIauWthtgv/T
WnVw86dcAVq2KaUxivpoS6Hj3jnCuBy/BH7G2iWAn8Dh67p6nCjs5UY8vPBw
2X8sJAYWDHHOhfKbwJ9rG/b85qxtRJPWC7U934eml6TBO+T4IDHrpshLsxdt
Pvdp9SjNd5hyazcp3ubu/va3+KoOKhXx+dXmdhj5cW/ENzrhYoPKlhvo/m32
tGIHYyRif1CwjMAHmO/WT+Ez0SdquYfxmaHobKfw9Z1/8QC8vVF77UU1Uvoh
NRMnm4W0WlKOmxfiQRfNZ9v3yc3ocvcyhinh0AMl+QuPDuOBt+yZw68el6Gt
Qd+6Xz1uRIkJ9xtqtfHgmjEVMK6tBolGF854G79BaxJDoXHSeFB0q6tmhqUZ
lF9nOPuLpQLtdFNxWD2FByenzGO4t1eBhDNF5pGKqagihkObqoXgP/pxjBE2
T4HhvXbfHXz+aKe6Cml9We3du6jZ7gkOy7SGfXqtjRx8QoM/E+MhX4Jlb7Qy
kOCdcukIZEUW2JEEon2cn/uX8sTEwa/HDFoN1WWxHLKdJ3tzf2YH9MGxB/Wi
8sI3sf/tp5mmFnLCnGxokK8lWJCTkAflRayS5sv5difXRz0kFGT0JNuIFuzW
VEwh2tljq9ejp7YhyCX7ItMjKVmwa72MFB8+h/QGXbj+DNE6SvY/XLYFmxyn
93bOEdb7wokj7PZ1aIFGmI9r5BbgF1HRJPpbGs7xQMF0GzpdhXscwZYK2C/s
KT29Dw8YjmwqkmWpQg5WTmniLM/Bb92dqcdrCfPe+Mw77lMdOH9g7+Ea71wQ
57q0tT4WD65oWf6UVWoDnDqKbJJKj8Cewkkn4nz5lp45d1q/GkxUT/O+Zg0F
Q+edlIuqCPu4TuTeNPp0YHhxpE9qygLYsUSN6C7hwYPTLK4Ver5A/tDryTZt
USDDZEuK562IZ1tjghpAqo6yx38vYS4M7e5E+5jejTX1Ur0DZ+unk5X4LDAx
ctxQcNgzXmHUDVkHHrW6tWRjfVkb/vyOaphZwn8XsrowhHVWTmG3yfnDvh2y
if3aZ1D91psay6p8oPmwliyRJ2s/sWEqvY60AF/ForQZkFXbiG90vVq8PxdT
EdPKi9H3FrcAzWwNqf/l3neE9slWIirGXldW2TxwQbQvoq+BsC90qPUJfKpH
TGstD+OHasCznk3NxPhgUt2/4ylVEbr09UX8zxutQP/RnaID+wlx9WTA3sf0
lSBMlVrgCf1bYFZjMnWOMC+b9mYky7jVA+XE+Ld9zyE4b/6ibPE8HgiPq7a9
HC8FnRr2oXePZAJR05SzxHie7+/yXHYiGSiejcB1zwSBy3ybBfpX8KCpLEyP
6qMrKGv4ea4pVx/ElNuT4mGu7WzDlcMKQNny7GZVS07QkiJaRLSD3eeFhMOm
/tDbNKmHodYM+199sVWH03dJrRaamYyFWuAysV2ZG7zGvlJfvmQTiljiHpU0
Gce+kvfT2UPnVByFpFAPf4lJZigPULHZyFc7+AK+hr24glr3uF8Le2EMbLS2
7P9DzA+3rhxI2huN5q43Tb7gDAV0Brak/OGLFI/Gof3Z6NhHnyRRoUxwCJzR
S5vBg1E8nXh9QgEKexTjlfq0Eox0y6kQ/a3G8wc/KH+CmqtfX89tbwC/32X1
Y6GEdbGtR+pZShb45fH3mMrrevB88jAfMZ5na3zCRyoWAL3K96I3KsuAp1GQ
JjE+TPFT73D3Swc35Y1frvs9Bfii1T3E9aXy7UNY+6VwYGxVNZd13w8sj2Is
yoT+G/fR/z4zagHGysavyh3TATfK50j5koJvGZfue2GwXc9hoWsLG0jr2cjP
QWpYOGK5DG3Ve+6179TB/pLt/JLXa94lJwcOCIfwXdVPxP6Xb2eFX6XP9vwO
tfUlorI832PzPPMkbhXzSG6V+wD6aj8g19fPDEz6NuoLLP38t32bNdEUOu/7
yEwZiJbz9hDjScWrEg3lfmcUrfVT8tNrJ8C+byO+Vfn+tTsiFoGMo+VM67hj
AEOZMc9Vgv1fFL62/8IUh0oLeILiMzNA8TE/X2I839LHO3xN9Aay6+TivSb6
CvRcv5BEzB8ShTfZcO0PB3k5jfbv2/NBRZmxLnG+WvVtdtA0xQKcqu86dVMq
aOJ9HNa9SKi7eZ4VTIaHApvCJZpu5Vvg/MgTUr73c1AmhDHxKvB7tyBvibMB
Ed3NpPycJUY5p8lMCbizaVoI/pYGx7ZsxIcSpieCocq7wTDrvbPHaGgBRt6/
5LoH5J2sVWC8pFKHo7U49r865ar6lsof28PhjEypswm/L/Y//WHY+DZDhlUd
5A9s/ObpUkGoh8g8nX/ZheMv7Cm4ytJS/Qt7/LKBxP2bRUz4p3gQq2CxBVo4
DHCtG/neZgO6yn1VakjgvAF9ve5ZMCGwEc/pzSbye5UsEE35ZHSJpwOYbmUk
1RfRLwcCHkXYos/W7p0eWUEgOkDO8xmB9zB57LR+a4DELxoOz2hEghenAcnf
xF4v80uWXgQZ0bR3nNZvgS3MlXH6BF65P+/kvKotkHeQ/MEdFQDOvtIg5ecN
ojYJP7sugAOI7oyBtw14732NlD98PsR1CRNXBC7FyVQHMBVQR7cRDxNO1xhm
6HAC9UdU5kz17OCLZN8iaf32jAyrquOxy0+O75rzHMP02MdJdih//EJfmt4U
d/COqdtyoAWl7v77nNqutlAB9+UKyhd9oEThY7dAuKvhB7m7SbJylqqjFG7p
17sePVcvx6Y7l25o20Xhqh9fc2Ud9YZOLm77zz2xwXrI89Wf46UqwWMEZRWu
aTsEKWH/q49yzCRZA9LZgfD8rxAf/d2g4ms9aV3s+8AwG+9GC6bKZO9p+m8G
CaECpOcDhu+/ZGOTwfnb7ZGqbZenvJd2xX585mCRXN+f992z5rUUXjhSNHiz
xwaqtCYYX1M1xv6na4VHC7XNi28GJ4qou9yY/mI7vkeQ+Fvbd4mBdwEObyP5
cr+/IqUdXNTUzazvVXK5xlRW3m+bKdy7om9ySsEC0sFYuuBQbex/+pLqDonJ
Py3UYKTIiZm5eA3btm5M4oVGb5J96kRwbFWa7QyiYpR2fhdXT1BJ5cp1y9dp
mFKXU/jQm2M/lDAjWHQYHcFPq1LspiI8JSd+9C/mVCLMnEs/iz2waybx61F2
l90PC+MeHVdxyS4XpbSTH/Ot54ROkVyMYGRSu1c1hRdVTtG8ZNWGa+z2W0bk
5CjrjtOlQViknbCv73I7zsMwh52yKyO13xb/hY1KJh73dE+ZaGnPPUo77X+Y
6Whyr+MOqDSoBjHepPDQibc/pFhqcLS9gQYPLjRSeH1fouXtzqe4cekWFrrt
//Rk2zG/CNuDYzhNlYDIywn/9M+sT7uzRbog7n3KtS8wopPCtSsOBl2WpYOe
O0dMk9z3UvQNixgN+5tHenG0XKJyCrNTlOfLtzq20/FzwbKH8Q96PMUoz59V
ZWjo2vIdx1JtKm3lupnCo8UzyvrGRWHYjV9dex9rwnWy/TdRUV1/cGMWV+pj
fi85a/f/0Z8Dv+f4AZiWYvzkVr85XCU/T/CLIaLO//s/Oj9hP3HiMP2Ki5t/
1MpY90/PnyPr1Q3/0aun19cLW51EIDuNKHPP9X+69Ej8qKlMxUe54ofJM4y/
/7XzXENPVjOAD5t8faV9S9a/52ezWcRPVs/IiZTQttXx/tOFhpVN0oI0CdvS
kZyrHemOcJb8/Imt55QvfpuVaz6niS9r/Gfnx0+dTibJy2O4m73njF9ZwWXy
842sbgdrj03I2Tj67Y0DtJTnfYetPiaNimPeI4Mtgq66FHva2NULmNX0ybFv
3mR3JGCGMl93Hpzo55biwSyitEWfy56itJM10iDW1VAt13l8cfvMo3/xZ149
7y1i3IYtvaLJXWrZR3n+adeWT8d7u3AJyjMxP+q7Kc8znrg5tNi7CZbKIj/F
NBrK88IHxdr7x/hhyLbUb9dO/NOHKzvaFX2en4URk2ZPRf1NKPM7Fn7OJC/F
FmrLCK8uM/vATjKXFolvKA+7Dsd+ln5Njo2D3uS6o3JxfXrnWjhccx/kSnyQ
CXn/bnBhsn6e+R/9XKBvXd1jlx+cUq0woB2vhIfJOrnHVNpurno37AqLSEzt
kSpo3b7Bb3BLNUY1RmBAKTNELLUEHvu6wb9Q7whyoruFWYQI3BPLfgb3z23w
ioXrfms8N7DjHlOCCl/uwWKyTqV06CXz2l97jMN+q1NLrT/E/U8HgLu/2cfo
YLxPOuq+/LlAsYM09VDD3Mo2qGXWsJhYT0+xm5yA0Q5HHQwmVdLXim2Vp3DP
/uLu0GtX4dKahjcu558+zNUko96Juwt39OiEXGS5D9dXNvg2DbGMYP1ciO6o
cpv5lsMKsh3uXxjyfC9ZDd/Dsq/2wu+ggNZGfWH21NMfi26DxlYMg7GnxyDL
i418TIysn0f/Rz8X9fqcVatQCx0S1Escatfh9rQN3bWOR6YlraECA4zfUqap
qdCul4jEbe0KrqUUd2DOrxrWr6n8hdInP5G4iGfIk/PHO7Dic1xlBVTTMKT1
F4nnBu7rMFGqw3raAlqw4Q9Q4OpGvfmk5+s7Cf2XGNXDHwdj9+FgbeXGuEav
JHjqMcdit3bYv+k9mARtyHao4hU0K98uD4cCQJ8GlxTFnvFth462HHKHxpcz
uKaOu8Jusj1frV0caXuZBnne/lpLG0yHluR8dZ7qdMeN503wjnhnK+3zDkh7
bcNuH6ej3jG++A7VqYz2ju74BVMTv5H4kbMLB93dVmGme9Z3rS+M6GXiJCl/
kxn90EO3xoDk+IxPuAxzo7amXFIewkjWzx3J+rk2WT9Pf3JXsVpiM6LayVpi
dEAG0biWkuoOBpXTZ1rqF7HBSpVn9KYy6F0nI4lfqRfwd+PeCXYEf9py1VUM
8TtPj5A4zZQCdy8joDJC4yFLR9Gky5d5kt1iyj6xF24CPbcEQs/U7ULhebqk
fl7Y+/z2OPMUdlRxz0M2nzloW/mBNK7D9koqVdJvMXphBbEWti4oxr1hh4sP
vTe7fXOACTY7+feUW1H0Ro3hy3PU2zIgb1z+V/8/j6DUPDk+xx6fPHatB/5Y
e3G4eqULntq/Mb9yb0u/31BahRWTjAsjR9bh+vsoEm8a0teqL+VAvTZmHwPX
uNETG54FIpfNGfbIjTuJ+KmdXGpD5dGlQx/SiTzyzl3PZwZKSOTyjK1ItiEy
NqpoWyX6M1k/v/sf/dyLa36/tY00sow0017TcUafs3pPEPNYxespn21KRQHV
ykkWS09npIGfXPxJzCuOFp0/K6sK1scH3tKcvYK2pShqzBP4Q1eN0KoOZcBb
/vKgV855dEaVuY2oi/YuK3jQNUgC8w7OjOAJZfSu+IITsZ9DS+nyaoJcQJZZ
mO1F/FFkMu/wm8iNDiw8m9lGBU7mJFIH5FOhgr2+G/HTLZ5dZCEONq5pBxR5
hEBpcpy85i7V4VvVCg1F6Sz2b66BchMb83Jg2eTAnrfrcKss1OcSmocpu9NI
3NjY4mqhNx9i/XROzzGbB92TWcKT5rcLv+W9szraZa16qfO1BmpyiMsj5vkf
9h+YN9xnhVYkX/L9Ur+C5A95584SeEqLtWC/jysKO7Q1ADndQN1htw5mE+Mq
WT+fIOvnz8n6efL4Tp5aPTvksX37LYeeJMQU3fGVqFt6u9MI3n9lBQzEPqEP
O5NRY4yRuD6B8+n0nAj75gEk1bnaNeniUXHSAbwXoR2VOI6Rr7vcwbiPk1mK
cyhqGBO8GEB479VX7s0hAjYgJvLbcvMfR6TbuM94ghhnFjU91Hy1gFqGxrnv
OG20JWBkP3FcWmeOnbv1jB9sdXr/7uSfI+h6kimpnt3ab/a6JrYA/pzcudsl
Nhn+ntiws7m6IoNe8jQ86xddJvinD75rmtmItxI8F/2nDqE52M6aacuMZmfO
kfThU6lVP8xTz6CsnqFPDmIKaKDg8ALxvdl72fLyDzoiWvccAS5LW8QOutSI
/WTl/zyvKhSGRPVe/66gDkNFO8SbGQj8J/VephsK99Gw1EvdipjH6KPAniuy
S3igT9bP75L1c3eyfp4SLfu+el8soukMoOmvL0fU7yRF7fsIzzPqOcxrRQDn
5PRSt7VydO/BoOdpAn+3ne90hlAKUOQ9SLvKUYjcDvxMujCOB78SzPbHuCUB
WRqaAWa+dPTxZJhy5QLBn8cbyt0tIsHz4OP96+J3kHZr+1N/wrxEXtzTKr3D
Fbx1o1/jrbRHh1O6Hg4R+v9oIfNwI55QN1ZXMJfwK6LmMec2oh0UevSUthu8
gR0PJP3xxZmQym/DztsPep6l4t2C1C6fzPLyG4WWex6RODvVAF60VRJV7ZU3
u+p3ENlaVvaR9qOYqB1OGpfQznOeN2UUddB8XEvEXwLHe45caR0LQzkDE/yZ
o55oJrt4pymBvzMxknurlopu6Cm0sHLcQ6OScnQOhPp9UEQAQIdCNOX2UXDt
+Ev0+DjVRGg/wf/J+rn6f/TzIhPLJmbPPHRpNMbh2v1mlCl5xzRGCw+KI2tl
wm6ng/3vhYtZPjSjVe9vBXK6hPVe8sX/XVwpcNtmnZ/1vBbNbVEzYo3AA12t
m0axcsUAzzSyHPe4CMXdNEgVbifsp306GrVOmaBa2TIGc0pB91Swouw5PDBR
9oL8XpGgdNxYMI7rOrJbcL60hzAu654LAZ+/XQblsox131h0UV1UkDkxvi0n
Bvm27ngHEwdPJLF5P4UxMRvxgZPxQvjlT0yo80Nqa1H7MNwaPUSK/8I1paIW
d1SQBJi8tm7MhkbOXrlF5H5ip6SXPriihsHk49L9GsgsVIed6LcKIn8mDxYn
IqeC2OWxs25oL0tSMjPBH6bsY7GTCa8QbRh7gOVwHGrW7Aq58gMPuof5dTLb
ahC38buBtOZcREd/gZshEw9ayfq5/X/087PsW2c/slai7MazH06HvkH5S1GO
IQR76tOWBTHqFwHnjIpT5g1vkApfhV78ZcK8Bx+TTl1pAO40bI4ZstXoD2Nq
2A4lgj+z5Pv8TagDfanf2iO/vELyYR8/hEUR9q+OLKnVx8VAp15162f9B6hf
fc5p5zc8aK8xXTlKlwymBbOuqg94opPr7nVShLjkoTEWUq/qDuYVU1ZvcGki
u5Sl8FFivvpYUfp3fC8s8jsVnJoZjO0Q2rCzW+qmHZ8ldyN3Ca7DT/jrsM1n
75LsbJdQXtBUdRpx6dNOvGRbxTj7222IPPvvNqmuWg/0O40uN4RaGOjGyBt/
JcZz/UldbqEkgh9mcbbbGAGfYvfvIYT+5NzOFC47XYTeqjoe2TTvAagvzOg8
JKzfA0fLAzxx9Sh16qJNmmQ0WM2DZ5viCOuRrJ+fJevn+8n6uUZpymO2bdXo
jJHdtcXXueAw96GTx4vx4Fu6bmnQRCnwovb6psuYB3axRN1Oe0fwNy6rtLLq
ZpAWc/ddy550UJeqav1cDw+cdx4zkXv3BlR8d3e1z48Hr/gPlR+7Ttin1BsP
v1ovA2aYyYM2HT/g6uag/JqwvhjFb/a0iz4Ga2kP9R3DzgMb1gb2ayt4EK2V
yVFu4wX2JI1frZI5AXKDY0oGCXaItXTiPsbXCSOY5D95zGZg7H827FxprDwZ
prUdTaTeWKMRGMUevDhB4lG6+f2NeAXkv5WPNiLzIGAXz8kj8uqkrWF7Djkh
FdWaqtlUHTCvIdpE3PfxIwcsrTpj0XaLFKHAWi/Q9uesyROCPxtGncie4MxD
6zZWDVxH7wN689lZ2194QE/XuTV+Jw4xoZ9+BZdeAcOvj/UPvsaDObJ+zvIf
/Xz33cdtj/pLUJTT1spLom9BdUH2U4OjhP7oZjVN8bwEtdP344/6vQVrOZk0
ViJ4EOfdNlVB+P/XDkrsW0J1ICaldW+2JqHe+TElIfgbgbNFb6u1s4uBxxzT
R3rCOnJayXQf+PgKOCd9mkzvfASYNHKKqgn7Qnva3Ota00Tw+Vd0qxKPHygS
jmOlJYwL3jmz6ljrDJ5JbZ8XddYFNp3NCkR90inNUescRzX8tf3gDU+edMy6
YiM+Oyp76h7iXoZfpeavDSV9xSITWjfy+axfv0NiT6A0azej1WZ20HlkG+mc
wHW22zlG7y4gt20WDJzYGZBeKSpPzIvGn3LWWtAFos8tRQyBs64g65jF5RfE
fNKR+yur5H300e9OMePhOFBZzPXjIcEfbvDOZ+U8zkFHfb2/mLk+A05LPNrW
0wT/JOvnUWT9fJSsnzvihpwE1NIQZzvW7dRZD4KCa9KeRxH1vd9NgVTJoE7L
oF5+cwNwCN+1LHUXD0y1XFbe7soHNFau9B2XIfi2W+p+axkecAiXJpRQ54EA
1gbs2dpLMOjz4X7UCKGO46aTSkUpYCHqlNKVM/fB+IV+pyzCvvwl4mTdalIw
kGxJYLdy9wRbukuWbhHG9ZiR6cljJjNQ6ndCUOa5JqABsbnLBF6jUyYRhcuE
B9yePt3fHIfJkc87KQ0f2/FE5gv0zom8qyHUhf0UWyJxr5zr+x2/7UazZ59y
mFnQAZ33U6TvblfaZ+q83wCk/PxO7r5kCfAQt/CUyP/iWb40U1uikvHi5opj
ZkCNLnOKuC8sXWa+lgG90dLwLFyg9wNlZfoPc4j1IO2Pgd2Ft5AI/YeJZbV4
cEKvl22MEGfyyPp5JVk/LyHr53XNJ4SCXwUjrKreUeJHPsjssxYtmiHke7tt
e8ujfcGbRfuPT8ReAqbanu1JBM4aQBPaO3IXaJ38/G1bXhagjhOrpSfkFTtX
d+723X4HiMu8OZnN9gDcVTp3fp4w7zy76p3/egQA5m+t4R+mg4D3No4XHsR8
3mxesPzyJXBYLfpF+qglkJCKCyB+f+H2MM7dJK9AeJ+x1cC0JPio3BtLtIND
CWfuUmkwbF1knjzG44FFkeuUL0W8gRdbcTD+5oOYWv8irO7VBm/z2NPf9gcP
/fdJC2iljWOfojfORRzmd0ou/8mKXL0XM/q094I9dl0kPXblYeNstNEpNH26
lhpJywJuOoFnRB7TGbTkTq+Lyh2V42ikTUA5zt+S6P/WLZJGzgKWKPbzy4i7
g84gLpnnEXE/vU3Wz7/8Rz939uoVPsRijAb2RLZb770N3Pd/vEfUyad+Xs/q
2qMLtk3tuq6rdxusLl/P1CXwwOM+jyMOXAI1FSXz42dCwNPh64kpRD804DVc
um8F9pw/Eps74g6e8D/UIeZp/NaG5SJtBuBqtp0o/VYLUDDcoEH87rNMo/hr
jEcOdJ/wYyqrlwd2Zo9DSXX6pvr72gUxOId3la2OffEUvWgt+0h/iXM17tiE
O6dv8pt/536VD3YvCYziqKp+2L0FsxRex1P+eZPEFphl4TQdIvJPJ2wxn756
aXovFP8mLLojQZDCvYuZTDMlD8Gd8hE3T9rKUTg/VXV/8469MGB3Y2zAmX/6
yRny+eG2nP/qioHhavT88NFTvWdr1S7wB9kfnmjoubkmC2MH09XrZ+9eg7/J
3NEf8zuXJIltPUa/8DPegnJOMuz+wBHmHwLY9QPbMo2N1Cnti0uD580DLFiK
oubusAPHKXwHw9q9yO1/5G6f3H3E0Pmfnta45fzxhNQuXPSxJtPwxH/624ux
pM31YBNUC3w1eYnm3/M7fKtqGL7ywcvBGoJuCyL/zq/6LlB5rJyG3GvZinWH
jCjnrCw6o+v/KFhA+RPHF2geuMIxMj8V0rPclH8F3h17y/qoOwQmkHnTHH6X
+SdLqH+KrbBOKRy6k7k4FVUVUXddsf7/dVe1L3gtJk4P6Br2m3oloAIqf9jg
/OPcM+V+Ptjo6kzDongJrCCfO2JmenbYNCkYo9kvH9Chmg1PkuMeTcpxZr1B
D4zh5cR0waFYiCPXuU5ex7UqVi9isp83a9VGu8AJ8nt96b+1m2TKYLT0iXWj
XzUoemnLgyNDTZG7sRrbkFbRsUMU+xh1xt7bV7ENPplLafC9+09/e6C8W8bP
WQ7ey5T2NVYD/84hF9RuPvbaFh5899TjztA1OElufzLAw5HF8Dasmru8jdow
AZaS9cCPT1afaLxIgwrRfAPYk0I4PLLBZxVf0J73zod2LOzPynUa4EQgeVxW
+OKXadnwGePNMffsFvhMZYMHk3VOg/b/X+d8lDDljC+ohMqV12oU2Vdh0XDL
Rh16L1n5xv1qLGt9h/KbU3/hLqZ+Es9gKeuYLmvEki0iRlxYJmFF9x8Sf1/a
WbE8+BqTrDQJCk3uhm+9N/IoeUHB+vriVOyRBWvQaGghzCO/Fxs4qXXpSyBm
MZCRKb4cCVfJ81J+RfrxoxgjzM/4nuDpREu4SLYPdkHkhes7ANXgpeXdv/7p
0vPRhlnlj93gWa9P3fRNLvB/3790VmibRBNSYa64er/YlzSYTNZdIwQ5OHt9
GuCs987WAo1WeL9+o5/nR0bOvIoZhlEy25UxwWmYp4Un8Y7hypWFnT/g1vSB
LEO9TYjxbTaJ261F07O6jMOvrTRCxqp0KEXWaUO/Iuuc3CP/v87Zrb+ld+D7
MmTWf6PQiT+F7g7MkurNzaq+e40jqYDT03dLKm1iqJztFUlnezzRvT6oRAdY
I/0yj+07ilS29pF0KrsnzjG3UxaxCAY/y9KpHejr0hgp/5+Z7NN/FDCM3ePK
N8+98RPukfixoTc+BwbvC3GYro8KjYdyI4xQ2LBDF1OPyTODREyIxsfpjdcj
2Lu0wXWZZvqZXjjAfsuOH8LJ//S3vVn011WS0qGV1udT3LaP4BOy/sZyufHo
hwvvYJcT//hn8S64zXJDf/s5+l3yZ/8iZD3t7DFwcgXiAjJI3DIjevTNgz2o
SOD07pCrB9DKixiSvnHkKXW9sBUPcs9spX66VwxdvaA+TNK1VkVHgga5kdhc
Vh6fhRyqjAuqJ8UBss5Z/x+d81v4mNolLzGUP3hyFxWPI8oIXyoi6ku+1m+i
P45LAnYPjQXu01fRS9b7Q8TvvOM+Nwv5n8mDGKX1B/a488g6OgFP3E+l5uh9
5FlEQcmvWGnaUkWkV8NAOtcULBg4xmTACq5yhF9i6uJGJ2+e+0va34PfiB6I
m8OcZ4TTYdYqXFvII433WXZNlnt+B7avfKVw4k83LHu0YR/ftuv8BX1xULy0
YS+NaQgEZP/fSbfptLBIK1wakf29Dquh6O8N/9zlXCzHb7wGT0To67CbzUGt
yWck3nmeelvKryPIW3tx3+0FbtQq8Yn0nV3j/I+LORVK6D23vPYMtSoalw49
QuQH3+rlDDAaoCuHbx28p2WOPHqLkoj78vDawLzWBwN0icaOq+TqVeR11kKC
mP/okHXO9v/onHcva29yrrqEdhfKOr/b9ADtc8nsLyfkdVeeXGrY/9wWPLZd
9g80S0Cdj15rGhPqgujvp50V7zqD76NKRV/bbiKsUnKzN6GdaAupYZr9VqDn
W1jL2WtOyDOTw3WKwIWK3JaThE6DrlzentFcHVTdtn6OqPNQmbC22kXwgssH
pIV0Eo8jZ2+PKeK4jicGZZ7M3QS4F54wjlRsQkJO1iT7DDZFpsj6FsD9U2qX
8zyToeDkhp0jWwx1h49Mw+Amt8j87D44hs2SeMOxCzqNSofQk4v7mZ2/MyEu
yeqN85ngUM4+AXVkulWNq4pZHs0X0nOR9M/nbh5Gx68gg+Nffj3/a41Oicym
zxA4R1TBMf8oX+S0uvI6FQWgnj3bPjwg5jlyh9131QUi7u8LJicS7iB52Qvw
D8E+sWSdk/rQ/69znhqjO/MmOhL1HFgPVlYsQ/d+JNeZD+KBuuZi8yadu2D5
ZfhHDrFixPmD8eqdUTx48LO2T/ZNIhgTDlRKj8xCu2cchU/O48HW/IJk8a4I
AM9uW3noGYtkRSU3ixPaX/dYC+JscwGRgysO3GLuSKy6Mr+bmAcayPT/CdID
XyoSuNRzdFEP70sXou6dnPsj6dQpARBbyjF+JfMocpSKJNm/anDzjULZN9Ar
KTy8MycTbvffsPMbPTuuoInNKLzhz24ho1Eo7v2ExNGvrbzUTJIoqsRlOvvn
AcS+l6afZM9TIfcf9lghGdb4eH5+bZQh9T6YeJ68McLHZF0wBOXvqtuhVeaO
FGJSxYl1kEE56FuauIfS+XZVWiXFojFfL0YcIc9fevV6xwvBp4RxVuvyPktH
/tyhYJCod5F1zgf/0TnlBxUNfGeyUKJNj0R0aBPS22yD/2mOBwzHgsLcc7LB
ME5C1dfqDaLzquMb8saDtRsC/j2ChSBnYGXAy7cShagLfvpcSdhf/F01aLOz
wNzK2StvQA4an++ofvOTsK4l0ts0GeIAZ4GPXpZVNHrrc2xXPWG9zGnuZi1m
cQdr0ZjGTL8DUtuT1fSRMC5cyH7azZ+0wCSDZ8czM0U0UJ6PI/r/kre0Y+Wm
d7Ck+rclX9BT6B+3ER8Eww47RZYyITUarh/Xvw/DafER0r4wYSp9Q5hdBe0o
uyBonsaG3JT23yVyRbcb62bTLmjUx7Nqq8sZ9HaiSZi47h6YHbefV4lHY246
ajxs7kg176WCG8FPXl4OVmk69wztaUCl3OoJ6JPdO12TP3hgrrbT6EtkCWoV
OXW1ny4fvVhs9G3swYN+ss55+j86p6WQwKH1hTLULBQf4dHyBhl+kn8eZkyI
n3waTjbSpeAPNJEoimhEg5qm+Z8N8CD7jh1tzlQtcNlC1WHkWoOep5wYKiQ8
P+OatL929TWgPSUi/32kEKWkcUoPEeyf3j9sfsA4B5juuzlxZ/oh8mnA67z8
TfArlwO9fI13QFCjhZf4lC+arhfP3kLUwQ45svAI2oAB68vPsK266KjfQyFi
fKN92IU+RfdC/1f39+XzBGN1ZP2NkyE1xF5gN7pRTT0ZnluD6eyKIdk5LGJC
L+f+adSsM+5w/u0CtqyQe4WkMzdfnfzm7oHgSjRNTSA3mP91TewbgWvuuu57
9OID1LGUHKnceARYHVtQzyf4Qz+/rsrYnwIUmZPkVsLgiixm+QTgFKGOsDJN
KjSpRdQNlYI/DVNRWbTMYmoWHrSTdc5zZJ0ziaxzqm9+MBRHj9C8FU/SkHoq
uCVDMzUMCevan2XvJZ5ysDNHhZfZKwCdiXt+RakfD56/154qLm8CorhZUdGb
OSjan35E0JSw76td1eNE9aAsT7ph8VEWYhSnV4onzGP9MS/5C5wlIIdqUwfW
moByL9tfv/SVMF7sg6odSAY3KpW+HEv0QAnBst0ihHGx873H+vTcQdZXFe+7
OWfQ9aPTTkSd83tL4J/NIp1Q/Yx/jmRfBsZI3u8E9b3OvwncjpbHA5wYvo5g
ndxCJO4wPRChBhTRDR2vJuaMA8CohJVUFz8daGQQkXBGJ9YnG8p+aAHJoqp0
4ve1r2lMW87eiUfjcSd4Lh52B0H27/yuEPyZ6pjZ/Tvmr1CD02LDMxQN3ESY
quUI61Qu8jeb/6M6NHk4NoJV+hG4X7UPDjzEgxSyzmn/H53T5O6R7OAT5Yg6
rWZPYnQz2CT9rvueOGGfDVh6nvgqF8Tmb/1VL/gGPNx1xK7Rm/g7mqblD7L1
QKtqYYblaDkYUrk3O+9GiKuubw8OXakHNGb0EbvxqeDN5bd6bHGE+miUlp1l
rhR80pM6fEA7BGiyytc+/UxYR3NQTKXyEXiX+PqhlqYZqFiVV4ogxD3npPud
vzS9AN2blOvDMyfAgX7ddaLOqdUWxxYhWQ2zNW8sUx9Nx8LKN+Izq8sRDs2S
ZXiQhaVyd9pXzOD+Rv4vHMC2qdZbBFFpWvmk/GIHyVl7Jkm6xK1W98Agc9Ql
w6ljW3oGDJtYmRN1p+zUT2FuuqGo4uL2yagON1BRHqV4g1j3vRTZl6uYjujD
bE9+3p8Azktv+vn4Lx607qRfa6Z/jVSeC7Skw3wgsZjGmfcGD36TdU7m/+ic
ps+8K/7q5yCj2LHa00YNwJDJbdd6CKEOcjj9I/5LFLDe0p1fWPIG9D2QMDzq
T5jfxiaN4fligHVzFireqwF/th/sC7uHBzYupZdmLlUAT70V0fmQIlAlVVOQ
jvDgk6pdDFN/PtA6jqOOPPwIhO8zcmn+gQfXq2Sl7UoTwDWRoXls+joInw4X
4yb4z878+LayR86gz+H1SW9bXZCWEhNN1Dn7uFdUeuizIN/q/v03vsdhNOQ6
tD939off6hfYn8H0cyysCzsFN37PtV6VZvSJdy8KXVOLPP6QHpTV+pP0t1zG
zD05msrIq1N4/E2gFOiyMXYmcqGykdVxJgc0l83xWTPFGtAxD+UT33t0SO4+
tVkUStzZVZXLGw5enprRSyP+PkLwpwa1SSZSZm+gM+bMBA5s9C+/z+LBF7LO
GUrWOb+Tdc5MOwFu57poNHkxxO9RagEYP8/nFE2IPwXDN4/GRqoRiomJyBC/
UuBl3HXfcJgQx3j1gvGHUgBzavEHxiMlYB1PtWucwE/ssNlV9zEd8BoxHb8z
nQtudVReDSSsr4tNf1/yxiQBHqX+0ev5iQDHXsmdsowHIgf4Unk/BgF7V13/
J4MegE48yyeOMK4bJR892ssugkjWYA0Jd01wrNOxluhvit0+TOFaoVBw/8LC
CzdPzJdcp2TJ7OQLZqmGaZ8u4rTtijFt8nnsTIfYu7Snf8OGe5t7ZLKnsMHO
ARLfwmDsZ7DGheT1hW/kjXKAzyMvSHXW6vBKX5+tBkpS+fz7bYUa0Nk5V086
J7wzunjA5Bo6dEXmV9eEI4jg8X1IjOcGMdW/8RM3UaIZ81ihy12QVBgl9pRg
/xyyzln4H53T1LJuXI7fHl0u6HewZ48FDCjlRwfheUFqq4F8KWX0/bRj3Off
ScB8TfLPRUL8tE56b5ua4g1qbx4Kjf32EKRuDT7RSVjvK39D537eDAFJIx8a
dU/EA30vnNsJ4jnhWhPruFBfoMbno56gcQOAgW2FscT8ykDt/DU7a3CqoKE5
TskSPKXFuxHrILbOweDBdkPcmysXZJI/XqDoRaL8xhfubG+RY0mWdeGe+0jh
x6aufNhrT4tJ9Ap3iF/d83/O7409uw3SoQ4Vdv9WUQx2kPw7ZcwE//NSOQY+
ulo+T7IXB79mjHKIXFrz2JvzmRxgWk81Z3JsH5Dbb07Kt6nj2+GPln2gUvbC
pxKrvQCIZ5B0Ud/vUxEWATq4nZ0dTe5FhpT+eBQZ/xY8EI+Tjja87Fzz75yk
8cHw/LmuCzhlT9aBDtWLFC4zn2M6olKHm53bV/bwZsu/33GLiD359uoH7mDc
XoMO8b8U3qBddTcKxwS974XXX6z+p/+8VPmQ3qvLD90y2BdstylQOA0Vk4HK
dhkoMfjsZm+7MUU3W183V7bcIw+rk+4J5an/+z377/XAjDd9x2DHQuN3p0YX
OE3m0nhBm8fPRDEtqsNf2jhdKPV48NZpHhU7eUzlVehx5UQb+L9z8heWgfl+
w1MY7ZGDVW+TDCjvvfj1CdtK0hGsRVJSfou9NKWf/pvn8wTct2IV2BZ7+ov/
zu+pNPefTB1qx1m991Hh0f13/nO2ivOR0PVE3N590uczNif8s9uJ1+W2dzbB
r90p15v/jw7phzuyHiUjBMOwuAD+r+L/zm1+lW769F4PanmPM7pVWsAlcj97
H0ucf3XAGdpwyB65EBYMk8lcabXRRr8oCMpIfMo61ZkC3cnnsk7P4jW/UofB
83f5OV3K8yHpGAWBl40FWq+xuML+v3eLG3aVQu8vG9zO0dHQZbsvFr7wd9e8
aAlEZP1wfC73/DftMEyNnbaSXi4PTv8mx+0DDJaxvoFYy6v5rtis+zCerMsZ
Twa7ijDaYfYMLMrJqn6wntxP7vzPAsP7NbFMCxama1/OUc7plSsIxSbuOIK9
sQ0YcLv8T3+e5rR2YdlCDa8Zhw0Zhf47BwuP2ht9K2zCzV6OyLl2tJ5iZ81N
Ax3421LQPX6IK11LhvL84dLt+ZduXoHlxQ7DQ9udKOdORZ8kOFH1xcBbow/V
yo4lw1Nku/XtV7Vb1HkJ2ZVcZ857IthRuMFzhqpqBU1qoZlIGZ3z/k9QfvOG
XiF16hHHw/5GmDR9X+Aa6w/ofGrjfN0HefMaJepiKBCtr9V3fg762mycW7Pv
m1cF9VWYoci93AaPP9ArbZTE+eQKTU9tasLO0tZWdadMwB/CcyT+9mU1jx5X
Ncax7wqsE+6Fe9c23sshxXKJczoHG02EqsJZZfDcu41+Mj2NHOB+HoE9G175
EGgQB/eR9ZOcqAjZ4goLrDbjkG2twyU4R7bDbjdvZf9dYrAj6kv4wJIwxW5O
uR91dgrO43Zq2arrCOEpdi6WCTm6EuYA209f3HZS1o6iD/90Op6kc/YxHPWz
uvvN/ynkJ58jVT5fWl3U/gb2NqYmsTm1Q5t7G3ms0xeu2rHp71DsttaZiTe/
Yeq5IRI335YlHRy7Am/pFNoxujKjuYwUUh3x12ecL9tsC0ovc1N9rcyDuuck
SOf6+k13VM8kzsBIJjHHpItC6NvNsFkiH0tdBbubFrADe8W/rdMKIiOtQpL+
k/mu6GP9hS1AJnCotLWCC3UGVpDi6skzGUeP9v3FHLtmpLZu2Y7SFPZs9NP8
RU7f8heMttlzkFn7B9QM29APt7ue+1BmV40F7p9xMpirhx1aG+PF/8qyfaic
gJ255P/CXi0RppL96u/KL/ldUmbw3ZbuXzTmBpTzyYXdi1K/tQ/A2PJPjkI7
d1Hsb5qVTWuQfB+eOjbXhoziYAi5HYlnlz83XH0LO621flIJNsMXmRv23PPi
CbOpxTy0HA3Je35sEbJyV2zs41sT+Iw496G3XpdV7oVwIfYwPEmXQ7uf++6q
FkacPhOG2dry6JJ81QsiNxQX3RtwXgLNN989uStPH3Xu+XmRuL8nCSDjOwGH
0bj/4IclKjPUMNLjQdQD+8UcTVlHj4Pvnp/qHz8xRVYst6qJnHVYaBmKSYBL
2tJFSmy6yLW2s4rYDv/zLip94+PgVZGs1uSMFBL+q9BCfK/qZhGLlue7gGeX
ZV77UzZ022UX6f4Bw84P3r7leEyB47akIPMsTN+1oRvf4WbacoWmFjtmJnVT
BY/gLqcN+7j+UA8x/HYD5phxlddRucE0sp2jbLKHNPQBvHUovvOaxD8/3x/K
0HLCpwr21wsvBTsWQjZy/i+z+vD+1sZZGM3lrOfFOwWXK4ZJvN1j3Vie6iAa
5rusXDLDgZxsFkn2jDoypimar4B+851ky1RXQfkHouWJ3L/1sLaFiTFiEzPT
Vy+9hCRd+ZaJ5xiDL/4eevHCEn3lyXrwvdIHLcX+rugicJOCtNOCmtrI1MGc
xnAyBKnaVHx9SOAsxsqnBs4bgcwS6rXxhWB083DEvgzSeGWScT6WIFhRufWD
kieqT8nCBgg8QXVU4XC7AZgb45yaP2eBmpNfchP1zP31POmybDJga86QkGMv
hrKEXt4j9vNjTdNjpqI9oEphMXXfpd1I5mAByf5VAoLn9m0fw1SVaUvHVobh
7InfJDs0/a1i7G9MgQaLaIvP4G24k+yfOb13xFmOmkBaKqpN5Y3KlDh/vMvY
4W7hR3jjvo/eWNJbKPxihcTp0hlEE34yoVdb7mosudEgGXlH0np/CdZ/ZK1h
qOSMSLrkmiQyjV16QorP78M4chyskEGFwkdGayvkz/n+KjFPS35xbr1d3Bd9
r2EMbvIORnopnLWBBD76sMUiziUEXR/kTetyuY8+XBysuk7IGyV866STLV1Q
bLhu2R29NGR93frdiyVCvhHFqvK8zRtkHxilQo6pqDg06D4bIQ+/rTl+D88b
AqbflH30lExABekC/gyEfHL/9uzZVmVPYGVwkEdw1Q+9dXK7/pxYp+C8LkrR
moJ67TajLMfz6PKxxZNE+5dtZtrNESgOJH0TObQGhJHpZsV3pPhz0nCbYQUN
oEVVMyMxS9B5/CLJPgfJ9wmo/Oc+AT//AGEa+0YYH7PObO6RDc86bvA/5HsJ
/v7nXoI23MfAo0HBxHPYN8rOW8AsMqf+vHEvwep/7iU4y+hpYLadDi0b11a7
5kxCGlZnEjch31ew6f3GfQUN5PsKan2H7n5Ml0dszq/C8I1HkdjevEIiXyPf
Y9D9n3sM5LRG/+bfckbp1/6mu9dcQIeOJo8SdbbtcON+g2Hy/QbV5PsN2LfE
lnPuTUSlgS27mobDkPwEo1M1YR5fk+89eEW+96CefO+BytoOu90pL5FIdDdu
QDUHzXLUifGMEeoFss4Z+x+dk5eq+6b0oXJEPyOy/2wNQoXsTckrD/HgCPme
hJb/3JOAe7Nwy+5tJsqMeekqy9aMuG0TpWxMCPGNfH9CHvn+hBfk+xPMzHVh
MtNLYJWQ3xr+sRFpnfGcF7Qk5J/kexW8/nOvAptYSU7EizLQWZ5gpKJSiWhV
RkeY8/HgBfm+hX3/uW/BiifEBD7JBV1Jj1tdh9OQek32ihGhHQPyPQxnyPcw
lJPvYUi1EeI8nRsNhns+1ExI3UAuZk+u1xH8OY98P4Mg+X4GWfL9DJM7Wh1e
JtoADrozLQ1cemii8DckxrF18r0Nh/9zb8P1Nj+a5kUxwHo6wCSvcD/azBX0
hchHzpV3W196B+/UPL1zRDQDxgdt7F+SJcKTkXVR8Lnt+yOSjhfgIbJ/8u7D
BR7O2I3ssDPffe+MQd1oD1J8kMrW61riOYtMO9XTrj7hQfJKm0jfcXgBh8pE
gT/aLyDjk1BggsLxi1pvCFzwZY/PZsN0FNI7cfRDwk2k/uwubfEiHrySfWXd
a4xD04tnJ1z5s9G3CyP+d2oI9uSMXc+5+Qaptgzm2rlD9IzJPnKvCmEeMzDG
+cJiJJw3eMnNoQldvbnj9TEtPJDdu1eu9zIOjCyb4hwON6JHBxWV8AqEfXMv
R9dJ5jfAdxNyred8jUrcjxb8IvjJXExFPHN+Ofh+9efFlJVUNESny1pH8KsO
DkdD/ukUcG7H+PrT4AB0DvzlDCLUs5G7eLuAkDvQ5udsLy3VQXY3PN2JOqHc
LYWxbBVFwvxrLzj92YcYHrHlEu2wHBcxN2n/ASpnM2p/yvSHp/o38sZt3zVr
ot7GQhHxvx2zzGKwjpwfKiWcAQJ/WFHKPANDgGENZJiuIekt5b5Vm0zwuuhA
Y7X+Pqc1GOrHAYn+ICH2zeqtQzDKl0uJZlwXRR9+nNmWR+A/orjHpHhzkN71
G/rX5s2RNONOkcF5Qnzj3nZu4XItKpdfqz8sFISaT46EZuQS1q+T9edYy1ZE
+3pQ6GbCPaSe8pp9uygeHOtkpLO79xrNW9Ucfk2VhjCuO2wP2/EgRn+T+EB7
LTDTzQ8d252KcArqoj5peCCwjf3wgeq3IGanDcd583hUorZSmSODB/TTQp9L
f0KQTacnW53niz7ysve21eFBpd1U/hhDGjBdxKceumeEqOkvjH4nxP+d7AHi
zJt8wO0EqZ6wYj40PUD/rY8YVxtH7KddVUFF0eiRKttfMKT+RxDRPrp8FlOv
7Hqh0Hbtj0p7H2E0ohv+3CVlLfrKJAZKangZOBboY7/Jdmb8mpMt2rYHSVV1
dERt+oJZCxwi+XPW34KucHkdxP1uUipzgBM83mT9h2hn2RzpfMPUGyisdnyq
IEMPHC2ivVFB4C8+2u3pf52JxtNOiG63CAQHTw91WPwl5J+v80puh1WjMmXH
otiBp0CULd98rhQPAuUvdEDnZtQaz/ciTqQUqG39VM8qhwcZVuyc21LL0Ivw
9a/be2rAkb7Rfb138IDBwN4+HFaDwMmyicdfq4FNkJOdtQsemFqwolejTeDd
3s7cTtoi0Bp1W9xZEw9KMC+V9uZK8Gh8N7otmAwaFLi/fm/9f3V9eThV3fu+
oYTMyRBFKhEqJUNlL5ImydCbqCQvmSIiQskYKkPmMWMoMs/TWmSspAzRZMqQ
iGNIyvjb55z9y/fjut5/7+tca6+19jprPc/93OveJFBn6b5QiCUA54+HRqGE
A7AXKDOpxvdn5WjwtCvMAQBBgbQK7ATIDNR/2IOPK0KafoT30lEw/djOgpGd
C7Q944sgz09D56CGScMbqOtUGuS3Ng1j2ETEz7YvVbWkfOCgo+EkX7kB5kDs
G/sDWgtvV7GiW/DU5k6HEexqDS8FTwr/OfX4lSryDjrLL9AtCoqLBSnv8dTs
WdK5KXuUY5YYMLnjEviuvf0N+Z57DX/7oaXTscj5d6zEhKkP0EvLHDyH/x/T
3RPMxPQLEfcB/fiWP0/Bnt+mkskf8bxMo1xNTq8KGZzdXtR6uQpI98w953Ii
gcA0CeOw4kykzL3g2kz7CnjHCNusPU4CW3KPRrSKFgKV2/7n5rMbAbOXLu+T
kyQQyVLltZ6pCqhK5ERobKoA9Uo0zwL8SeBm7SMhbC4XpP4y9Jo0TQb02q+/
9QySgDDpkXCLTBhQOLvZlz3EDdyTa/T7g58L9/W+GWCDVoD5K0nswAVtcN5K
roRch81qXZS2NT4EpPy8PA5CAbDzE18zeR76++dm1rpDeGqs0tf+dxKmUEWd
z5my9IS8NEeYzyt2787Gi1gnMc+WGzIvF15ehA+671VCzkGsXbSGmh/NOD+g
q5NB3hcUWWfjtoJc2r0U3cKRwUtXjx8xRhUaTgf+sJ4DJax0n8l1pax1PIw3
Le+jwKd6iTRc7mCpdacfWW8/c7dQy/RDElI/e3y06lsiOJQ+pWo9QwLJmpff
H0x9jtjv+3DLaRWDeOGMf9hbSEBmO4PE+g9RqPVhdnCWyQtwP4rWVPUpCdDw
bLHWskwCfa7WmKtLNehKuZbbnEm+b9IR1yz8HIS0l5y4xVgAuGX1mRw/4ev/
1WXt4KZ4sCdy7OzAQAxIeJr/mxY/L45sVfwxs90TvCmNVHM46QgqmdvdyHGm
Alvw9dc1F8Hk4+NyR2LVQHSjD0aOc3wt1a6on9wFWM6ce+LhuRFIHrtKuX/k
rO+247VaKtx553fzGecQrIrQjRhwlh7/dmQAIqksW327dix7epaC393FJOou
JIDWL415G8lygK93+SnxeZ/UL/E94icRM89ii4/tESA4mcZCqUf8eO0SrXgd
6QBFcfYsSzBb0PJ5AMe/T4Y9a82+h1ozS0xShP1BnnrCqSL83N9u+DNnt9Qj
5G7+xc4kLgl0XA5kDsTHq6zAEJO8fAc5LzjEJ+zPBIUzRQn2UyQQkBDAczTb
CwhZCFX01mUA/72zSvem8XlblAnM/BAInFpYW6yZEsB6rrFzi/i+WtXVWQZM
PMBEjmr0unfe4OC1HUrieH+SR7lzeH6agA3G4Z9SA42BtparOFmXEjFXK77P
TBnUHfDChi3kQajcAqU+q5f9fX5rGQcYaTW4OXaPHkSM+FL2z62ct8PCdtZU
TsxfqpLSWeEtz0wINkjdH6v8ZU674M2xwlv+kfHN6krngMHfX3OcSV/hLU9M
z6jZHtwFtQce53jPrugb2difq1bOH4Rufvwd4SUrvOXScsMU/QYlyE1/d3Je
foW3lNBR7Zjy3IdphHhXJWxZ4Sfz9mi1nzJUwh5k2jVNJa/wk37yo3sYLstj
JplGTIOlK/xkhRPjprdlOzDR/ld9ziUr/OSGA3RMnHVMmInv4+nbHSv8pBZq
Zx8fHlCk7b7vtfR65f5y9xbr84GvpCBzgfQXDc8VvrF3+ZZTQow2PGun4M/v
cuUvLxGB6T6pHLWGAYP8aybUPOBjAmfP10ovC3GHDIW8kuLesfAekfctTDQU
KPy5B1UEWNI7XbIg7SQVH2TfcCp9jzdWVCQstc42A1YR/OG2hld2yomu2HHS
aIQaTyS8S/CHItJnTp/1tcCCxmvjRVVdYBXxXFmZtl5OaXVsr4e9wk/tlfvO
S4feeGi7iGKMWlWC37fu/TuurjWNfy4bPoI5tnX06epRf3nRMezo8K/YLDgI
zCruh0L4vYiK865ty2JmfQELxEd36bp/hAzbqfHSp1zJkWulDZDm4FctFP4D
0mRR+T3z8nqF0+eqMK4Z3bE+kQ6o1kb9/aLtix5P9nSMGSyrfcgphduaqO3r
G72/rbTmIVZfmH+3nCsMBhDj9fnDmqV63whrL9bjX/7XHE4Q4zJXd7P8/G4Q
yh2rz5MJmYL1y18peJkX+772h/PwEVsqVHnEgR4oQMr6v5K2X6mTnx7Nx3w/
0+uyDfFvLKTwYzcIXVzgKl0cc5DVXZHhfszIiJbTNmYSPrtOzfusOWdzx1hq
sQ4j/SYfjSYo1EaNZ+aWHHaTdsdi7wO44l/kJ8AHBL+nXKVuVH5YCnm1Oj+v
WTyM0joEISU+94pwb406gM449zNpLmgh+k0X48nxjA+hE7NcpRM7Ej3nHj87
i509/lNc4tcS3LsngdK+4XYX+QrOFsxlvzlkd2qFV30I/vPOYqECuIJUUmcl
E1tuIq+F4y97cZz2P3RQDvcZj7ZfoAXa5wwPbMujQW/pqbpEROhznFfpc/id
dzR+mFmL56V5etoVw3B3rwsFF/8R2ajYCtBCBSqJwnag8P3PSinnXZVw6h9k
jba0hqmYrL2A0oSfRpPrj7l806K850PRI2mtuY6Qe2gklPR2DD+XM49H9g8V
ZaHqDQXctFxp6P548L7vw3h/jl7sqn5ZjDjuX+xodYMoSupu4kQiCYhqOlXN
vHsO7MU0+bvGG1CPqraGxr94/BDnnJOhXQwCZOybCkIr0Be1L4/c8Lh6hNNf
Kc0lAyhYJO1xk0xBW5sZRXeN4XE+b01ppXUQyKI7srllkxda/ratzwLf/zm+
90YX7r8GDPV2Wd7M0EMVp490kfdhtufh7xovHQZcD+m6DnLsRPaO5nXk8UoT
uhq1VbqaroR2nUi6kyh4s0xDI9sWFM3Nd4WMHzRnox2vckIeJbP88VxnkYqU
sPB7HI+573vk2ZpElH7HtyPTzA0p6374eACPD9mHmJuVb5eh9koT5+PySajS
K4mnA4//I8NVi7xM61Cic9M+/cOlSMBBIFvKmAToRvmas+drQafg3elPSQhp
up1j9NLC++MbfMnQuxx0HpxliPHJQDKanAYRtfh662AXuhiaDGYuJjJn8gSi
i88eSnPg550p+5W+bml30L35X4U1h01RjsNQCvm+jPlVdz6GNf8AzU3l333T
5dA2f/3T5PXsq753093XTij6jqRJ0oIE8CxK1O/A8fXPXtN9dktE1Q8WRo+2
/gPCDpQ4y+DjalxvnT91rQKZ+Ck3cX21AeiLpM7JNyRQqxOhxET7EiXsfubl
VGMLmt9GXOfF3++zML4rgqk1IAZeDxSyikceiIuGJgYfV8GnRKbbOSCNRzP/
jaoPUnjWxuA8jse9F+CGIxaPwFCm+ZS1jTE6sDSxPIq/30+kujuLL41B2O/K
wmdQFn3ZOr6FrB973XNh+GJ7OJLfLDohreIMrl7fkvgaX5/JjEu3MZoCdE0g
PpuEYoHBMRvdkn4SCKd5Oy4mVYvqo3+J3/HOB/GvbZUs3fE8CFiNVRXng12B
hbWJzJ5gjrlFQuMb/n8nCUYkGIeBL6bZJ8KrL4BznUdc9uP9qdfK/73nliV4
xeSis/OpOGCz9Ts9jfenrpurZmwoGZ3yW7bxbIkA2+ziD0TMkkCpV9hgWEIx
Up9uW6/flwfe2goJJ9aTwLlj9098iw0CvY06l8wnbwHjKa+xFLx9lg6TB6fO
mAFhntOVz6tPAp5DrEJkvkJ1cDalLTwZHV1fM27JkgLqHxqf4MLjli9vz71b
k6KH79qnAibDTgCkpNhJjt9yfvPNvz9zpTLy26/4/vsrvuJfdozASPV+xXqg
cl3Dbvwv/uyqR9HXr9fgZ9KHu94Xz2NTxH5+g6j/yq6q/64Zt7vOE/ovNIT8
vzh+n8b+f73P04L76RHBjYBl6XBFjBQzoF0XQcnH36POdC2V5kpPEeWG+cG3
f597brhH/4GlLQxZ4H1oPl8IS4i63nbiXkDfqnsBGS7PuOxF6eGN+6kp4lG0
K/7b2NDetjB5KOmKBEhbVuINBVrlEKmWAmg/ZVPfvWMGik1S61ks6IJfrmk5
FrONhfn5nZ8wPIxaz7r4e/v8C4190KKKTVKjSPJvO2m2ijxPtCwgg0XY4xef
TP/eB1nWHshNArEw+G77N1OdOHiKmB/FWulFxfFJuKHj5rWwAik0ckSEwjf6
Nky93GA+gzUY+9+T2yyF3rU8ofqmcpkZHx+gA30ew3uqb4mgT/e1KXxUcfb6
eAaSLmSVT7vkslfzb/zwydTr/XhfCCxmPsvP9NEfhhPncgjXRYNbAQ1Qy/Zk
buZULQxeR+TvD7bqvo+dhkYlyg2j/85AB4PXFDyFH33+BkXR0knXa+4+l1DS
P0svyevHLdmeG8zsAGmRPmNZLZfQsMxYKRn3Y+i5Xih4AORVNiSkIG109Emi
F3k/2Zqp8PGBsRQYG+vs2cSnhMI/w1xK3aGlSzNN2QHO770/tzBpAj8S/edb
rzVJ65wPVU7219+QTYO9PVRc03591R2l73B4osTBr60PKnFRdbw3ZmbYbId5
kNvIO1v21g0o+7IFJY+YzOxdYrl0CM3YefpBTww9mFtDybvt/XvHq7adQ+6N
enlq+e7oLL3WWDaO9xxGk1/mj4NXD9DQOxVPxCc2mpqE42pi0drXLhuA0RR7
v0/7b6Mm7d3Db8jnbIy2nEGgLmilk30iedUcOccJB5H/72H7Ri1rjFSBsrra
LN+0BrKhswogz8P5DZZ3x5i8YfBMz9lrizbQnhhvvfCWnf/Q1sGYf3br5VXk
werbVNyqmV30T80izL9T8bHjNglqbaPWvyBP25foTCm0LDb0yEFxG1ITuEm5
L+mWkvHqQ85Z1P1ZNcTNVBOlJUpYk3W8VQ1F5vQmNqj+2wDfli5bFGfSP03m
f5x9xNT02l3QI3nOvLm9sciy84tDJpkHuJFMe756G5hc5+7JQpuE+q0kFsbx
fEeOifltpeVdIO3AE9Qb8xg9FJNV0sN/n5vxJwK75Aqiczyl4g8HIiuugJBm
8vmb40rnW2ANTHM1NBQO2aNe5ewtZH/L1vFTnFKvtAFjwIe6XwPa6B+jKQVy
P2MJvXEIoTe2JvTGdLJU/p+Globx//L/O8SpdQTrof+tI1QS/L/zy//l/5kt
qTx/2Cqe/6oVlc8/lv+/fP5pgrfnm/5f3r6e4OebV/Hz7a1UXv3lKl5djuDP
n6ziz3UIntziP/yHWVfx4UoE7628iveOJfjtrav47fP2O7xvGL6DrHQidy9E
xcLbE9S494+6SgizVgm6Icr2gjRThwYuAa/xK3h8O6hY62b8Fk4a3W7cIOSI
3WGizmekSWYqkwwHynooeMjiazkWr/KHEhfVCB+RHrF7gYRjZd48euCOvonK
uQjhcdp0oyV9k3IO8DWfVWemS0YscNGm+TueR9xx2V/hWgMnG2QdGegjsbw4
avsJZvRBwmvXIIYNC8Cf4wvGXRpJ3SeDjqa522HoRcXnbTYZfODrcnoNhQ/f
Gna85WMN8jASjsn0rQIi4t7xZufx/Vm4TMe2Th8xo7HSn/urQVq5OOlZIX7O
as5MT4nXgqziRVGJ34VgGPtpKHgb3zdeJ+nPdWTDS2yNGVJBQdjMOLU/j6U3
qbnsmIAeo/2zZVvasS6hcQo+gb1JmH4mhpRqHquLBXCC52GnZij5r+RpDYH1
F1GVSZa5ovoxsGeL5lPy+nlC6CQLVukkrRolnOT5KtGhLdfO7TauAe1NjJvR
NTyvvOHsUxoch5Zl/qhFJL8EjIaRDEo6JADC7tR0mRWDf191hBUa1gEZsyHD
jRYkUM5hGnf+ZCVgGAg2BFpFgL50k0UrPv/P2Xalub4PgstTBlde897GQgme
9nQmja0WTyu8Vs2G0h9D7Nhd6noQvbur+7oeC/rw1EBGfXEOO+zPRMGBwXg8
6YYS2use38zSJw3uHjan6LiKbwHnRpIFykhgrnu5xxDY9guxkPm3qG+F2+Ov
ZiDvAz2u2o0loK9bzbAcjwMfdL6+ZDMainjV98qfU38BFALiDvHj/bzv9uLb
2QPJQFIuQ6opswrwROR36BTgeev2kKuB0hngjaNTtG50HlBEVc9QNwlwYfdY
njLGAYN3Rm98baNA2Y2ctkk8vu10tKosPJoNL4fq2BcbxmN9P6jjPaosnjqe
9R3mqzXUyTJ2YTEm1DzXitdV0ixKCKXTeLGpx3GDkcs2lPMUTAf06jqpIa5c
5jN0LqpAPuDzW/L/aGLv1ZJ9vtYoJfOimI62NfgEc0T6yPyArYDl5slAxLhZ
9pfDaBKIHhi0MMP7w1jHOq5x7TaqGTqzfbt4JpAJamv0miIBhpAC9ZQpL/CH
8W06q00GCBz9/rQJj9OO1Sx1VPkHgmvIQU3PMR4wSclcFMP3k2r6C6VGAh5g
vMIv5sUxb8AjO/pLGn/uSfaQuEpXE8DwqG+PM40xUJfo6SDnNeqETk9vlU6v
utFx4NaLqko+gzl5pvmVe7VXGfLVVJyGKve/+P3+i8iKH6BU+73IpoMMsKa/
nqY9eaWdS5w6JzkYhOHpDVl8qv+s+LZdGCq4QicoDV/XWWicP33qL76G5lFH
c/0hmL4uqNjS8dLf+GSe0MXVrtLFpZ5qPW/AIokp1h/ThbIr/NKGzJQwu/LD
mNCdzeMfp1d+P0DfP+mzdAiz9TOM1Ni/8v2Rwmcl4vcC92ECT6Yl6FXV/97r
ZHr15uQB1a1YcG5e9NCTFX9CRWLeTq6aN2uGQJvUAHFofcEU1Rvs+4svbO/f
eusfNahz03NyrnlF97Xpw0GvsAxjmEcf2tDk7AA/ETiv31r6KQcHeEfW/2Gm
fiAcJf6P6vMNdk/MPWCD6j45E55EKEh81+MIoXMzXKVz28z7cc2Lp16YuNJ4
Bt1EJgwj+KitIy1HWebdMYEszfp6gUS4mWhnSCDKU9/AAUv98c3p82wAnCSe
G9prdvNrkxG21UVy3/zMTdhD9PPwFo8wu0snsIdLkeNRwSt83UHOnVdVrB9C
7cwEs09rwuADIp48mHvsUGhFCqwIkPk98KsAinZR8XOVD37/eF8MtbuPyjV2
voEfFqn7DG+4grGVZQ2M36DSKyTeC9f4U78bso/QmyWs0pu1VgSybuyuxs6O
hB4+/PMzrEmi6iu6Dowf+15SgNX+EsrmeNoIpdSpz72U3Z6Vm5aI0Yhojcyr
5UBf4v6XZdS2P7+LvLD5/pRvU5EBMIOYh9nDVm8CX/ZBwbX1KVU1Y7CAuEc5
diKSabv7NGw54MK55L4WNYXcpOAcZt/VWOeW4K8bInLO7Twou6WMkr9METqu
56t0XA4ET/V4FU+lxLpvSvLCKHZLrFp8k+w8FGSsp553zZIbuZc7sOtyJ4ad
pL/CvTXU79FUZYwU0hmVYcnHB10O1VRBxQBq/xl2T/wq9NiJjHuy3QSALIpz
uU65Z/p6n2JCYqM0usVS56TRdQw9CL1vTsZPETqopVU6qHCCv7q9ir+ibWKs
NrFZA8r93NzGTNcj7lefKf2vbU9bM7n4Desob/oO33+HtSE/KP1pmnxh//3a
JcSpadJXI2mNUjcXPiXzRU6ELoi0She0nuC1bqzitX7wClc7X2EDdmYjh3dV
rkcs07qU534kdC++q3Qv/3UfzUnBpliJkR4x8XB3aZsPwsrjcRT8fcOP547+
cuij4uJXMXUhNBbt9pGMu4d1Ss/9MELHmIxnJ3k0UcPnhUayH1T+KK8cs7g3
8mWOC5qzd0B+3vG5V3Fc8HPE08saCWi2trk672IYgqbfZNjxOPn7nsr0s9M5
yKXz5jk630ykZnf584VeEhAh9Bhsq/QYIbXZJNboFLDRbm2X2eZGtD/p9/3e
q/j6DB3/MCpYCFJ4jWmvMlajwKEN6xajScCnfUvqT/o8ELTz3zuvUC4S/S3g
ONZJ9sd2YrD6mQQ87q45pNwfhV7aGCbWzOJx4HzK6/JAX1D6UtnjOb0T0mDQ
4TyC9/9FLW+sUZcR8BLnMpMe1kA+Ycyd5PH+1/2yvazl5UBJFaW6+o99v8+P
1gr7+5Fxl5qWOMl/8bxi39ivraJqSNHfXmyEHGdOmNxQKQ9FMvvFpGTi7ZCt
cQzvQTwvELf0dC33z0KyhW08Nboh6B2DYTAJj3v/ndWPsIlC6EPDYip3aQbS
r+3mw/D47TKhW1AndAvphG6Bxu9zde5kDYila/e5vr4aZfj8mODWxs/H3ZqT
Eg+rwDbmycPBA7norGdI0cY0vP+PlsRNsVwQIMK7cU1dFPrZDnRfj5CAY2Dc
1vWi4aBhKoctW9YRHe7JKpvA19WcNUtuUeoNIPbvLsUN7eooS9blKVnPI8ge
hBn42iOtG1LZkwk7wEV1oSWyz5Lkh9AWmBOJtNl8S0T2qoJ1JxU/P8bbibnk
9q0vLg+dyP/++442ALtucFefxp9bfE2ya6GkBrV2KPt5195Ch91cf6yLw88v
Qj8wV0rVD5wi9AOf0xjguWN1gKO2TFR3zWO06G4Kix/g7Ys3BZ5QLQLh4r9O
fFTzRBXXihpD8PXWsTNkdpdCLEgalu+W+KaP8mcDm3rwfG19tLkO+0YHMElT
Gpn9RRI9u/XNkRzPNIrrfvLaH4L8lbLQqy83QaSOA9dN/H2JsG8N/9SdhaTR
Vtac08FApDyX/SX+vhSMW1ilblSjBV3eoWGzVHBZyTjwWQYJ3CHq8u2r6vKd
edIpXyoKAbPCMReeKw+BzuuOt5l9+LlZ8axOYjIaOIfRowtfLIFKQdMcOd+R
sjy/Vd31JrA1XB589FIFLGd+uUjWh1jtqcuUW0hEQVxg9G1DCNizTWq3BR53
jXueL8rcUoReHfd/ZKWYCSQMWiQn20iAn6hfG66qXwtYmr/yfxsM/FqvzS3R
3QbhSuOeNuTxOilWHmG0BBbnWUs2+miCHS8i0qbI52w3TVqyXDKiZ2rOFXFO
BllXkmXUfq7UbTlW1W0/C0buMCu4AKK4277x858CrRGhfuS8YNt/1CUDavWs
nYKaK1v3+3Nziqzo8LHh7Cu1Srbwl3NbPO+6IhhJ8GA7/oMHkwxxXrhfSg9r
zIYcpg6s6Mw7279e9fkmD6P2vtos4bXCg8Un0OQFnSiA/7jkv95zcgZ2hRI8
2J7AV72fy7HG94HAwPknHI6i8mBXQ7flNSfuh46ffuwtFV/Rne45f2WLpuU1
aKn3p815vTkcJeIHq01FEmw1sTD4MUchWo6DawgezD+Dh2PNjwlIX/Luuu7+
3ehIcyZF5yw2m30vJPIX9qFdr1CASwq5PM2g5OlZbW+rRenowSEJyHn0hAjK
yLekxO3CPkHCKrv0oZNyeOKI79m/9T7m6iZ0eXM4bClIdmP78wjOEvWyP5hk
sBH/SxgwkfsyJrkeXvlKjU+wSOscnV0/oUeOZ4Dq9l8wdbKBeq416dxYu2M7
Sry5WHKnUB9ZZzD5kN/jVbWO4p33xIGmNn9VSv4lpGzCCMm4asnh86V+B4B/
fnhFLo82ulNWH0M+Z3fPL33Nm5cEHFVD3XoTGCp2Yikmt+/48ZTZrs+u8Pyw
1m5HdANmEv2vu/7G3cGpFEpvipniFsuBWxuouFhA+uxx03G47nz0gLvLEDye
9p2CB2XFirHE8KMWHcvSY1940Lqqq5R7NKOzP37a5B5GP4tZf+xSUUKlXlYP
KO9lbUWv+SNN1KY9trvvoSf6VcBnHo/jN50kYmKHtIHi/OCUCqcncts8VpiM
4+n0A1tcQq8Aw+dZBt3ljuhEwgZHcr1DPUbE/NeZ82D2zVOG+ismSF4vhpXM
S6dXtasqbjwKgqo5DPOl1dEkJ189ZR4CrpSexqIh1xy7UqqiL6wk3oscSShW
7OJ7uC7XY0nTuB4Gl1HrYlWdxkNpZUxo+8BZrzSVZehmvpmCF01NntKoOYDy
LbVmzgTuQQdH3rdS4qiEHwmdEbpIO6k9zlhBBzWxqRSS75GltxnAj0030Cd/
AcV/bO2R2NNl27c4LnSxb+0ZcTsEN7te0i1JRKOBT9No8PzrLvOe4QODN0A0
tyXHaGQi8lvLN8mN4wLHl22sbrmDHqUniduxSBS5i3PEFN+vqtK/yHQ43AaF
vU8ut7L7oveMwgJKePvJ5soalqkWYNOpvKQnrDfQNX2aMvJ54bGv7rRfjyaI
OnFdLK5GE9WMKb/6v/fug1fxYNyEv+jMKn9Re8Jf9Nwqf9FNhL/o4ip/0QuE
v2gu4S/aRfiLthH+onyEv6gA4S8qT/iLHl3lL8q4luovOkf4i34i/EWfEj6i
7Kt8RG8SPqJOq3xEfxA+osdX+YjShFF9RJVW+Yh2Ez6iOYSPqCbhIxpA+Ii+
WeUjmkD4iI6s8hFtGb7+fORrC3zW/+GqOUiE18up+4CDreyTMx5FaFq7IjlZ
ugE57eAcFtIlgWFUxp/3oR0+Ol7Vv9HOFBqrU39fw6bsZfGSC/EyvWi3F4nF
HNbHU+KlH/t/LhxbD1GbP+2bDQPR6NiVDO2HtSSwbLn3H3GjYrD29xVM1zgJ
GZ9LXryOz8PBvlnZBddWKMjM5H6zJQYrJ77js2FBZ+LFW3Y0GNNj3c7Ug+3C
lintn8lQDQ59qIL4eTKsKmf5QXDSl8eUOH+2OYomtRylXnC3mveqBlblD8x9
PPC4YoL2h9KjHCC5HjVv4KoG4Rofb7ml4PvJ+Tk2B706oMupXuF0uBCo8KaR
YmxIwDs7qvaoxUvIkHQxPkDyCSauS11vHqIyd8K516Ikl9YvLN/6sd9FD6j5
pvovde738ihDh8ezoU0QOKTLdpDxFzZl0WGRRmjh0M+Fen51oIIdlSTX6bII
fqx8FT9mV8fXyVuUiw6ftx7RzmsE5fVq2px4XPfS72rogGYySOqNtWwXeAli
hL7SDl4gAdPzc301/WXgjF13wwf7F8BO2/9XwB0SeJPzJHCSsRzM1xWwCb/J
B1suvynyKMXn2etxHBIvgwfiZjwfGiRgjC3UccWtTWE8xzMD9frS/JqedGPy
D6j3I8xGmiz2aoqhrT/tihOWuMH7HGYK7yej1hL4fkEbZaxDjjTLKqB0r8d+
8rpaWrPOuj7PDhUZLFv15ZsDxncMVuR72ZlDyYIpmXEIximPcZpWg436pmPy
Ofi6OqbQUxIRBOTbmnvka6uBloV30e1sEtDgcBTZkZkGZHtywvcXlIHwmXxz
11ck4LtLhtPeOg1oeTIwfbXPBB4MSnVt30mg7H2DwmJ8NJCqFVncKh8Ooi4I
cKnj+1WH2FQ0ZO+GAhcYhul0mjHFYGre3ZcRSz/Vw4EG6i2ybm5cAzpM7lLW
VbJtkXgK90EkVy6oOFa9D3CecEVkPPP2dG8u70W0xHniwhYOXbBNYrabrP/P
ltfof11oh/S5yjdPQ3vA2tIp/grH/RQbFQYW3ZDO5a9v6sozQLgBS3fONAnk
fSkL3FBsDx784GV3l3gOjspcHHTC8Qb7GW8dqYfgZtUbGv8fT8DBFp991r9J
oGThbUnXyH0wqfXE63xPONB7p/T2NL7fYruHrrnvcAGtG+nmrxm6g+XP5wxd
8edOCP2u5Ro2AjYGFodjQg1B0ODjObK/jQdRrzy2ql75/wCLiarh
     "]], {}},
  Axes->True,
  AxesEdge->{{-1, -1}, {1, -1}, {-1, -1}},
  AxesLabel->{None, None, None},
  AxesOrigin->{Automatic, Automatic, Automatic},
  BaseStyle->{FontFamily -> "Arial"},
  BoxRatios->{1, 1, 0.4},
  Boxed->{Left, Bottom, Back},
  DisplayFunction->Identity,
  FaceGrids->
   NCache[{{{0, 0, -1}, {{-6, -4, -2, 0, 2, 4, 6}, {-6, -4, -2, 0, 2, 4, 
      6}}}, {{0, 1, 0}, {{-6, -4, -2, 0, 2, 4, 6}, {0, 
        Rational[1, 10], 
        Rational[1, 5], 
        Rational[3, 10], 
        Rational[2, 5]}}}, {{-1, 0, 0}, {{-6, -4, -2, 0, 2, 4, 6}, {0, 
        Rational[1, 10], 
        Rational[1, 5], 
        Rational[3, 10], 
        Rational[2, 5]}}}}, {{{0, 0, -1}, {{-6, -4, -2, 0, 2, 4, 
      6}, {-6, -4, -2, 0, 2, 4, 6}}}, {{0, 1, 
      0}, {{-6, -4, -2, 0, 2, 4, 6}, {0, 0.1, 0.2, 0.3, 0.4}}}, {{-1, 0, 
      0}, {{-6, -4, -2, 0, 2, 4, 6}, {0, 0.1, 0.2, 0.3, 0.4}}}}],
  FaceGridsStyle->Directive[
    GrayLevel[0.3, 1], 
    AbsoluteDashing[{1, 2}]],
  ImageSize->{311.4730343179637, 267.1552288911929},
  ImageSizeRaw->Automatic,
  Method->{"DefaultBoundaryStyle" -> Directive[
      GrayLevel[0.2]], 
    "DefaultGraphicsInteraction" -> {
     "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
      "Effects" -> {
       "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
        "Droplines" -> {
         "freeformCursorMode" -> True, 
          "placement" -> {"x" -> "All", "y" -> "None"}}}}, "RotationControl" -> 
    "Globe"},
  PlotRange->{{-5, 5}, {-5, 5}, {9.48933381114217*^-15, 0.37965298448884305`}},
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]},
  Ticks->{Automatic, Automatic, Automatic},
  ViewPoint->{2.6606631418277713`, -0.3481712331424651, 2.0614675447678676`},
  ViewVertical->{-0.6040693506182975, 0.07904780105453756, 
   0.7930016801949608}]], "Output",
 CellChangeTimes->{{3.811002476272387*^9, 3.811002498924062*^9}, {
   3.8110025363285646`*^9, 3.811002572477056*^9}, 3.8110026025410957`*^9, {
   3.811007651318042*^9, 3.8110076613800077`*^9}},
 CellLabel->
  "Out[230]=",ExpressionUUID->"28e833e2-9314-48b4-ab30-fc126e736626"]
}, Open  ]],

Cell["Test Normalization of Coupled Multivariate Gaussian", "Text",
 CellChangeTimes->{{3.821633903972995*^9, 
  3.8216339198203506`*^9}},ExpressionUUID->"32c1f4ea-bdaa-411f-8aff-\
22b0011841a9"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Assuming", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"-", "1"}], "/", "2"}], "<", "\[Kappa]", "<", "\[Infinity]"}], 
    ",", 
    RowBox[{"Integrate", "[", 
     RowBox[{
      RowBox[{"PDF", "[", 
       RowBox[{
        RowBox[{"MultivariateCoupledDistribution", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"1", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "1"}], "}"}]}], "}"}], ",", "\[Kappa]", ",", 
          "2"}], "]"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"x", ",", "y"}], "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
      
      RowBox[{"{", 
       RowBox[{"x", ",", 
        RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"y", ",", 
        RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}]}], 
     "\[IndentingNewLine]", "]"}]}], "]"}], "//", "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.821633952264729*^9, 3.821633957298665*^9}, {
  3.821634023739246*^9, 3.8216340267331457`*^9}, {3.821634065562706*^9, 
  3.821634137538972*^9}, {3.8216342240865393`*^9, 3.821634227741158*^9}, {
  3.821634334773877*^9, 3.821634375519059*^9}, {3.8216361745768147`*^9, 
  3.821636175036734*^9}, {3.821636231404373*^9, 3.821636232654811*^9}, {
  3.821636515260367*^9, 3.8216365325863237`*^9}, {3.821638214527314*^9, 
  3.821638219410569*^9}},
 CellLabel->
  "In[197]:=",ExpressionUUID->"a43d4dda-722c-4c4f-a373-26b0c1d25106"],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{
  3.8216341829118633`*^9, {3.8216343520428867`*^9, 3.821634378688587*^9}, 
   3.821635267896777*^9, 3.821635344490197*^9, 3.8216361765046177`*^9, 
   3.821636630550881*^9, 3.821638322956417*^9, 3.8216390743338747`*^9, 
   3.821639440812779*^9},
 CellLabel->
  "Out[197]=",ExpressionUUID->"08356ede-357b-4ffd-a73d-6fa4182c73d6"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Assuming", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"-", "1"}], "/", "3"}], "<", "\[Kappa]", "<", "\[Infinity]"}], 
    ",", 
    RowBox[{"Integrate", "[", 
     RowBox[{
      RowBox[{"PDF", "[", 
       RowBox[{
        RowBox[{"MultivariateCoupledDistribution", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0", ",", "1"}], "}"}]}], "}"}], ",", 
          "\[Kappa]", ",", "2"}], "]"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"x", ",", 
        RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"y", ",", 
        RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"z", ",", 
        RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}]}], 
     "\[IndentingNewLine]", "]"}]}], "]"}], "//", "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.821634785518567*^9, 3.821634823041181*^9}, {
  3.821638242530678*^9, 3.8216382451520987`*^9}},
 CellLabel->
  "In[198]:=",ExpressionUUID->"c247d8a3-5009-4d11-833c-a4c9f5caaafe"],

Cell[BoxData[
 TagBox[GridBox[{
    {"\[Piecewise]", GridBox[{
       {"1", 
        RowBox[{"\[Kappa]", "\[GreaterEqual]", "0"}]},
       {
        RowBox[{"-", 
         RowBox[{
          FractionBox["1", 
           RowBox[{"2", " ", "\[Pi]", " ", 
            RowBox[{"Beta", "[", 
             RowBox[{
              RowBox[{"-", 
               FractionBox[
                RowBox[{"1", "+", "\[Kappa]"}], 
                RowBox[{"2", " ", "\[Kappa]"}]]}], ",", 
              FractionBox["3", "2"]}], "]"}]}]], 
          RowBox[{
           SqrtBox[
            RowBox[{"-", "\[Kappa]"}]], " ", "\[Kappa]", " ", 
           RowBox[{"Integrate", "[", 
            RowBox[{
             FractionBox["1", 
              SqrtBox[
               TagBox[GridBox[{
                  {"\[Piecewise]", GridBox[{
                    {
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{"1", "+", 
                    RowBox[{
                    SuperscriptBox["x", "2"], " ", "\[Kappa]"}], "+", 
                    RowBox[{
                    SuperscriptBox["y", "2"], " ", "\[Kappa]"}], "+", 
                    RowBox[{
                    SuperscriptBox["z", "2"], " ", "\[Kappa]"}]}], ")"}], 
                    RowBox[{"3", "+", 
                    FractionBox["1", "\[Kappa]"]}]], 
                    RowBox[{
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["x", "2"], "+", 
                    SuperscriptBox["y", "2"], "+", 
                    SuperscriptBox["z", "2"]}], ")"}], " ", "\[Kappa]"}], 
                    "\[GreaterEqual]", 
                    RowBox[{"-", "1"}]}]},
                    {"\[Infinity]", 
                    TagBox["True",
                    "PiecewiseDefault",
                    AutoDelete->True]}
                    },
                    AllowedDimensions->{2, Automatic},
                    Editable->True,
                    
                    GridBoxAlignment->{
                    "Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
                    
                    GridBoxItemSize->{
                    "Columns" -> {{Automatic}}, "Rows" -> {{1.}}},
                    GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.84]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.4]}, 
                    Offset[0.2]}},
                    Selectable->True]}
                 },
                 
                 GridBoxAlignment->{
                  "Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
                 
                 GridBoxItemSize->{
                  "Columns" -> {{Automatic}}, "Rows" -> {{1.}}},
                 GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.35]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.4]}, 
                    Offset[0.2]}}],
                "Piecewise",
                DeleteWithContents->True,
                Editable->False,
                SelectWithContents->True,
                Selectable->False,
                StripWrapperBoxes->True]]], ",", 
             RowBox[{"{", 
              RowBox[{"x", ",", 
               RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"y", ",", 
               RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"z", ",", 
               RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
             RowBox[{"Assumptions", "\[Rule]", 
              RowBox[{
               RowBox[{
                RowBox[{"-", 
                 FractionBox["1", "3"]}], "<", "\[Kappa]", "<", 
                "\[Infinity]"}], "&&", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{"-", 
                   FractionBox["1", "3"]}], "<", "\[Kappa]", "<", "0"}], "||", 
                 RowBox[{"\[Kappa]", "\[LessEqual]", 
                  RowBox[{"-", 
                   FractionBox["1", "3"]}]}]}], ")"}]}]}]}], "]"}]}]}]}], 
        TagBox["True",
         "PiecewiseDefault",
         AutoDelete->True]}
      },
      AllowedDimensions->{2, Automatic},
      Editable->True,
      GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
      GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{1.}}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.84]}, 
          Offset[0.27999999999999997`]}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}},
      Selectable->True]}
   },
   GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{1.}}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[0.35]}, 
       Offset[0.27999999999999997`]}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}}],
  "Piecewise",
  DeleteWithContents->True,
  Editable->False,
  SelectWithContents->True,
  Selectable->False,
  StripWrapperBoxes->True]], "Output",
 CellChangeTimes->{3.821634833050921*^9, 3.821635351025865*^9, 
  3.8216384083844*^9, 3.821647496273*^9},
 CellLabel->
  "Out[198]=",ExpressionUUID->"eeead421-f566-43c4-8ebc-4e6c7266380a"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Assuming", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"-", "1"}], "/", "4"}], "<", "\[Kappa]", "<", "\[Infinity]"}], 
    ",", 
    RowBox[{"Integrate", "[", 
     RowBox[{
      RowBox[{"PDF", "[", 
       RowBox[{
        RowBox[{"MultivariateCoupledDistribution", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0", ",", "0", ",", "1"}], "}"}]}], "}"}], ",",
           "\[Kappa]", ",", "2"}], "]"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"w", ",", "x", ",", "y", ",", "z"}], "}"}]}], "]"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"w", ",", 
        RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", 
        RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"y", ",", 
        RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"z", ",", 
        RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}]}], 
     "\[IndentingNewLine]", "]"}]}], "]"}], "//", "FullSimplify"}]], "Input",
 CellChangeTimes->{{3.8216360462794313`*^9, 3.8216360837245913`*^9}, {
  3.82163825126274*^9, 3.821638254114012*^9}},
 CellLabel->
  "In[199]:=",ExpressionUUID->"ac76ebda-4da4-4cfa-b4a6-7b8b32ec1826"],

Cell[BoxData[
 TagBox[GridBox[{
    {"\[Piecewise]", GridBox[{
       {"1", 
        RowBox[{"\[Kappa]", "\[GreaterEqual]", "0"}]},
       {
        RowBox[{
         FractionBox["1", 
          RowBox[{
           SuperscriptBox["\[Pi]", "2"], " ", 
           RowBox[{"Beta", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"-", "1"}], "-", 
              FractionBox["1", 
               RowBox[{"2", " ", "\[Kappa]"}]]}], ",", "2"}], "]"}]}]], 
         RowBox[{
          SuperscriptBox["\[Kappa]", "2"], " ", 
          RowBox[{"Integrate", "[", 
           RowBox[{
            FractionBox["1", 
             SqrtBox[
              TagBox[GridBox[{
                 {"\[Piecewise]", GridBox[{
                    {
                    SuperscriptBox[
                    RowBox[{"(", 
                    RowBox[{"1", "+", 
                    RowBox[{
                    SuperscriptBox["w", "2"], " ", "\[Kappa]"}], "+", 
                    RowBox[{
                    SuperscriptBox["x", "2"], " ", "\[Kappa]"}], "+", 
                    RowBox[{
                    SuperscriptBox["y", "2"], " ", "\[Kappa]"}], "+", 
                    RowBox[{
                    SuperscriptBox["z", "2"], " ", "\[Kappa]"}]}], ")"}], 
                    RowBox[{"4", "+", 
                    FractionBox["1", "\[Kappa]"]}]], 
                    RowBox[{
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["w", "2"], "+", 
                    SuperscriptBox["x", "2"], "+", 
                    SuperscriptBox["y", "2"], "+", 
                    SuperscriptBox["z", "2"]}], ")"}], " ", "\[Kappa]"}], 
                    "\[GreaterEqual]", 
                    RowBox[{"-", "1"}]}]},
                    {"\[Infinity]", 
                    TagBox["True",
                    "PiecewiseDefault",
                    AutoDelete->True]}
                   },
                   AllowedDimensions->{2, Automatic},
                   Editable->True,
                   
                   GridBoxAlignment->{
                    "Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
                   
                   GridBoxItemSize->{
                    "Columns" -> {{Automatic}}, "Rows" -> {{1.}}},
                   GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.84]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.4]}, 
                    Offset[0.2]}},
                   Selectable->True]}
                },
                
                GridBoxAlignment->{
                 "Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
                
                GridBoxItemSize->{
                 "Columns" -> {{Automatic}}, "Rows" -> {{1.}}},
                GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.35]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.4]}, 
                    Offset[0.2]}}],
               "Piecewise",
               DeleteWithContents->True,
               Editable->False,
               SelectWithContents->True,
               Selectable->False,
               StripWrapperBoxes->True]]], ",", 
            RowBox[{"{", 
             RowBox[{"w", ",", 
              RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
            
            RowBox[{"{", 
             RowBox[{"x", ",", 
              RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
            
            RowBox[{"{", 
             RowBox[{"y", ",", 
              RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
            
            RowBox[{"{", 
             RowBox[{"z", ",", 
              RowBox[{"-", "\[Infinity]"}], ",", "\[Infinity]"}], "}"}], ",", 
            
            RowBox[{"Assumptions", "\[Rule]", 
             RowBox[{
              RowBox[{
               RowBox[{"-", 
                FractionBox["1", "4"]}], "<", "\[Kappa]", "<", 
               "\[Infinity]"}], "&&", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{"-", 
                  FractionBox["1", "4"]}], "<", "\[Kappa]", "<", "0"}], "||", 
                
                RowBox[{"\[Kappa]", "\[LessEqual]", 
                 RowBox[{"-", 
                  FractionBox["1", "4"]}]}]}], ")"}]}]}]}], "]"}]}]}], 
        TagBox["True",
         "PiecewiseDefault",
         AutoDelete->True]}
      },
      AllowedDimensions->{2, Automatic},
      Editable->True,
      GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
      GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{1.}}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.84]}, 
          Offset[0.27999999999999997`]}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}},
      Selectable->True]}
   },
   GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Baseline}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{1.}}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[0.35]}, 
       Offset[0.27999999999999997`]}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}}],
  "Piecewise",
  DeleteWithContents->True,
  Editable->False,
  SelectWithContents->True,
  Selectable->False,
  StripWrapperBoxes->True]], "Output",
 CellChangeTimes->{3.821636089612121*^9, 3.8216496538172207`*^9},
 CellLabel->
  "Out[199]=",ExpressionUUID->"6533d30f-dfd1-44d2-b9e4-30d66bce565a"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Normalization of Multivariate Coupled Gaussian", "Subsubsection",
 CellChangeTimes->{{3.821966432033431*^9, 
  3.821966445173772*^9}},ExpressionUUID->"e4adc312-eea2-4c76-a3ad-\
330210a25043"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"Evaluate", "@", 
    RowBox[{"MapThread", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"NormMultiCoupled", "[", "\[IndentingNewLine]", 
        RowBox[{"#1", ",", "\[Kappa]", ",", "2", ",", "#2"}], "]"}], "&"}], 
      ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"1", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "1"}], "}"}]}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"1", ",", "0", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "1", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0", ",", "1"}], "}"}]}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"1", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0", ",", "0", ",", "0", ",", "1"}], "}"}]}], "}"}]}], 
         "\[IndentingNewLine]", "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"2", ",", "3", ",", "4"}], "}"}]}], "\[IndentingNewLine]", 
       "}"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"\[Kappa]", ",", "0", ",", "4"}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"PlotRange", "\[Rule]", "Full"}], ",", "\[IndentingNewLine]", 
   RowBox[{"PlotTheme", "\[Rule]", "\"\<Detailed\>\""}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"PlotLegends", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "\"\<2 Dim\>\"", ",", " ", "\"\<3 Dim\>\"", ",", " ", "\"\<4 Dim\>\""}], 
     "}"}]}], ",", "\[IndentingNewLine]", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<Coupling, \[Kappa]\>\"", ",", "\"\<Normalization\>\""}], 
     "}"}]}], ",", "\[IndentingNewLine]", 
   RowBox[{
   "PlotLabel", "\[Rule]", 
    "\"\<Normalization of Multivariate Coupled Gaussian\>\""}]}], 
  "\[IndentingNewLine]", "]"}]], "Input",
 CellChangeTimes->{{3.821966504172275*^9, 3.82196670106744*^9}, {
  3.8219667353986397`*^9, 3.821966759051133*^9}, {3.821966797879859*^9, 
  3.821967068892712*^9}, {3.821967153714713*^9, 3.821967277781251*^9}},
 CellLabel->"In[52]:=",ExpressionUUID->"5da59c09-9105-4fec-8faa-ae5233a1627e"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{{{{}, {}, 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.368417, 0.506779, 0.709798], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJxd0w1Q0ncYB3BeVGAoSJIKioYjvWl4m+e1s7X+mBZoup35cnVnZjXddOXK
6DqdVjhtK992m0vDMnPWLCbYTXLOaRtRm+bbMNa8Q03GqUstLKUUlAnq87vr
OTjuc9/n+b3AH+7Bz3anEXA4XNby2/bpnU8pNcfhMMvW8jgzlyWIEWl9wqUM
zJUvtTuvnU/KvBKE/fLHPbsjAkJDt7FSMU3qYLzNA5ESIol2GNvQw0uw+XOm
XDiaK8bqGJF2W6IPVR7wP4V58lesTUpcriLMY9VkfUFdkn8Z5Ex7VYA5Praq
Adeu//DseVY9zK/kcsgl5J8CzrGawVVDuRHKpQ4wzl6/gb/hRS2/VOA9e68q
jDNqzH3Vifa6B75or07oD/mR4eRN7gJTrZJkoU/fa/v1Y6y19RoqHxx30UC+
Me7pxN3ih+CV/bTgXjqTGSx4hLFXfbI3DKuaHQQLf79G68nVQb9HM2MYjx95
bf/H4AMp1/c1DD+G7++2PqK7b6seclXmZa+9UwawTnRkPZE/Bp7zf5/eeGQc
5i3ls/SPOicgJ8zLXL02PgF755091JIxCd5sTb0dL58CdxeG9t4PfAZetNoK
eft/BU1O241g368n5288NcL9HdVpU3fKZ+A8ky9HhrVvPweLZnoSskeew3z9
k7YuWtkL8OAX7wzlhM2CaZwfnhkMyLERwTJV4Rz8/iUZyvRangnWD09suuag
MUG/SSAzfHL6JVjZ2tKNcV+BxSHqZve7r2D+5tHIDeLsechH5Oriv7gLcL/p
ku5vF3sWoN+c+bD6rVwz5G5/T8f8ybaAg8JJS+ltFliv9VhFSEnGIswL67gf
3/JYgvwkZ7/faLIVvGkqkfooFieA/9ui3/nYemQ3upGsXsAJ1tZzHt+hO74L
D7nDnWrFm5eQf86RqS+8QP40tG2QsosAlkUdrO0wIfdLt+3R7CTC+nOTbNex
C8ha31uU9mki9CvjhcSKSAcw+4P0uPrvkE+Liq40jyFzh6L/+b7IEbzl3cIK
VogTONmnbx4/gNwgDs4KCySh83SV6o9KSIK15+H++AnZqIYE/VUOKeLdPDI4
aeZccaAY2W1YWUdQI5c0Vpsvsylgx3xJQtgxCuxHcDPiaSqU426kKPTr3gC3
cr+syc1EzpYqStepkD3/9XrA8KSC24O+IjOyqIK152V8S740qgPljOjsTQU0
Z7if6T0u7lSaM/Tr+P0DOb86w/l2jpEschcXmG+qwXiGFGSngDbpGTnyPsVm
BmcJOYt9uDMvgQY+U3RVortJg/3cW/aXnTCjvHGCU02LocP+foVMEek6cqUv
dc5qpUP//z/ghg8=
          "]]}, 
        Annotation[#, "Charting`Private`Tag$11974#1"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.880722, 0.611041, 0.142051], 
          AbsoluteThickness[1.6]], 
         LineBox[{{8.163265306122448*^-8, 15.74960879671338}, {
          0.0005077978790941904, 15.743619278616483`}, {0.0008561643835616438,
           15.739509606141983`}, {0.001226871682238507, 
          15.735136379778561`}, {0.0024536617318239528`, 
          15.720700671916275`}, {0.004907241830994845, 15.691939369915593`}, {
          0.006131897157752762, 15.67763839035626}}], 
         LineBox[{{0.04889455557892998, 15.200032660190649`}, {0.0625, 
          15.060140519825024`}, {0.08963358069509919, 14.781151126921214`}}], 
         
         LineBox[{{0.09218460110326246, 14.75599619465426}, {
          0.16666666666666666`, 14.09013893288165}, {0.19872448979591367`, 
          13.803547333068133`}}], 
         LineBox[{{0.2012755102040769, 13.78285797241524}, {0.25, 
          13.424648026157737`}, {0.3210305952755042, 12.902449316169282`}, {
          0.3320578231292517, 12.829367767384063`}}], 
         LineBox[{{0.33460884353741494`, 12.812632582740333`}, {
          0.48443648039284554`, 11.929507319231714`}, {0.4999999999999999, 
          11.851218469188403`}, {0.5699233045152912, 11.499484720016781`}, {
          0.6538510836938587, 11.118946887387729`}, {0.732138185275487, 
          10.795646535811485`}, {0.8170295072193599, 10.474761650791661`}, {
          0.8962801515662935, 10.199424903479189`}, {0.9739717509693491, 
          9.949381007915333}, {0.9987244897958938, 9.873469556172534}}], 
         LineBox[CompressedData["
1:eJwVkGkw3GcAh7GOjWPfIONdaxe7ZFolJaaii/b/k2mdoV2k4txExZ06J2cj
ZGyYNglRQxtnQ9xXZE0atkiYznQ1I2oTTJVEtiYlSpo4SkP1wzPPPF8fYUxq
0HEtDQ0Ntx3+d/Cw3WmxzjKju9g1eLDXBgtrM9OPnP5mWtf7WsNWRKib71Vy
rr5iVsVGjo55InAEDUtq9Wsm9cSR0ES+CJcTu+NqbFcZSxH3Ws49IVbRok64
sMack7Ib22KFyHIekpsNrjOfBsc2rZgIMdM+9PWocINJVkjV3v3W2ExSldud
3WSKd3fcrUy3hr2n3lZc779MRVF2jcrBGt43hPG36BYjyM/vmhi2wimBVPQ0
cpv5oPfJtDLACg4vDhuMB2jA3cvo4p1xS5iSZfbQhgZ8Ivn1IVGW0O4v77Cp
0ETxH9IR7rIAye/1Tu7y18Krm5MLfdkCtPjG1PStauHxdXmWyx4BVhZ4u+dK
WVBJ71Uo5Xx0B3uzSj7SRoqrZeQRCR8XfGTV8jltBKZeTSDrFhD+7jdRK9PB
2vz5noZaC7i55pWYO+tC7OFpH+FvgUjLkX80x3SRYuLs93SThxXlldm0XD2U
8FgPuht5+FY7OivIlo39f92P74riwXS6+4bWEBvNDuy0HDMedM7nhojTd+Hj
HnJ7cNgcGk3RHbMm+jgHmylxgTkyrndcMbmvj4eb8pdBnub40b6AbfyFAXzb
A9XP33Bh7JfhcJFjiOE7ZHR2gIupfQ/HzigMYdjACa/P4aKzirFVRxvBfT0w
OsGLi6iOA8aCLSPMTJSZ/cbhIkf2fe5UMwfOdSeaPFQUbc8F5ZxDBBJBJ1en
mqKgUnZ0OpAgU5lZzauiiA1a3NsuIfCVnPR0qqTgKxSdAaEEn02IxiPKKb4q
jPjpcgxBTfElH3kZRcKB717qnyEYufbsw6QiClHeHh+9eoLGsJLD87kUW+9/
aTTeSKATx3Vj7fTk4rNf61sIEqNcxPwciqLQ21Fetwj8WvfmB2ZTbL8TlClT
EFilSY7Jz1JMjRZWscYImvfH68syKX64tPb52COCxWPmpdUZFCXuUrvaCYL+
UlOPnnQK/5vvyg9OExSwfnm8lErRc/rBz7l/EtgtDHhFplCU7nMplLwgULm+
VXcqmSJ9tiJEuERwt7TN4pskircPpcwMvCb4pEU3TJlAoa2pqitaIwjPGrGc
i6d40u2edHRj58/Jui3NnVYk1To6vSEQ9OWsC+IoyqwMVra3CWi41NDtOMV/
llOLSg==
          "]], 
         LineBox[{{0.008682917565916028, 15.647964801470499`}, {
          0.0196078431372549, 15.522635796444817`}, {0.02631578947368421, 
          15.448033551474955`}, {0.03925736321938733, 15.304104186740906`}, {
          0.04634353517076671, 15.227387375665584`}}]}, 
        Annotation[#, "Charting`Private`Tag$11974#2"]& ], 
       TagBox[{
         Directive[
          Opacity[1.], 
          RGBColor[0.560181, 0.691569, 0.194885], 
          AbsoluteThickness[1.6]], 
         LineBox[CompressedData["
1:eJwtkHk41Ykbxa+l3Cz362spu2QZ2dJyuYj35RZNUj97hFTKUhNhJsxUc+NK
xdBkKyrLJY3srgiR0DJ2SoRrSUWMRtkSpl9Pn+c5z3nOf+cclcP+dkf5KRQK
66v+73IuDqMx55PNIiu5L02qmVh3KUvfYcIM9m96+DmsnImCTLkAnqwDzMso
2w8UMnHrreTUElkvsO8riorlMNG6ubv4hmwwJOe0DStHM/FhiPaigl04GBVm
bC46yERnr7nqP2UjgOefYzxzgIlSJkWSJkVXQVd3n4kGPxP7N11gxsjGQ9wY
L9F12QI1C9KrIqevg7bnlVFanQWmSe+LvCTLgV0mF76421jgr81J9U73OSB0
pfp0pYUFOn4jC14GW8pcYFigE3uQX/VAFsxQ7lupf82iM6fOVaZmw28BN+MU
VS3Qcp17cPvO27ASGfVum4wFhnTFh0wo34EH7TNhNxfM0fZqr99ejXxg91/4
2JpjjixqyQ8XZUthI7/1xROEOWpZXYkqDiiFAaF7NJ9V5tjB9h979bgUZk2G
/+e/iN/7cCFTYb2B23tENQGdXL1fuHC6064i7S1i4xJHr6ujDAIChd5E1yKK
fkykK0dXgHf43rTQYMRr36iAu/LWUVJBiMn9YUzu8gN41NHd/uMTQHMFF86A
Yw0EHq/ge1sGOO5quIqa9zWrlegkZQFSvlELfJRpH8MEQKWJ0CN5urUwp6ky
vykc0N/XQJ3SVwuJF3jiCIASXgU5WYw6OBvtfKb2vhnud0kv+PBvPRhc9Lf8
/YEpTr1yEM6Qa4B1ocsHrQpNMdKDetSe2QDHFYdEFTJMv+9tgEAfzTZeiimW
HPGXK4tvAOVa7c5JtinSTpqyfzVshBr/G4v6e02x4XyP6+ozj8FrKfOs+tB2
3HKXXK1AfQaX7h7asUd8O4qssNyslFqBfPKGIR9njMm+zh46lq3wojD0yWKE
Map16XiSJ1thXryJMxpqjGZ3uo+8qm6FlTY96edHjL/vbwPOF5ZAua0xVtxJ
5N7Ta4PcxFtXVRjGOGk/UWXd0gZL4U7G3sNG6JiT9HeQWAfckCg75WpqhOq2
/7x7dLkLbHp4rTXCDJQ7Z2Xsnd4FPBFT334KA8Xz0i4L3+uCy9wxZYE5Q/xM
tdezHe4CMWHX6mMjhthWey9ogPEcpgmzrJgqQwzTZy3Pjz6HalXlOtsAQ2wh
pKT0sBskSA9aUL8Bnm4xguRPPaDx9HHjziY60vS4o0+pvbD+aQm1sJ6OnBj9
6EWFXpjukY9TrqZj+54fejx29kK+TrALLZ+O2n9LBakn9sLG4hAzrTg68h5P
ZRcbvoJmSn6ukyMdrR5m0ZrD+mAgxcTJY2QbrislB/j4eHDlahhrvdi2738N
AnU89NDPdVuxbJjZ1Lp9GOpzzKKWArdgnd8NeZeJ1yDE7dYMlt+MMxqmRN5P
b0HNSSVl8yp95F/IFZdXH4cdruFHCxd00WDFs8w+fwKaz5hHqFnqYFPEtpZG
rSkYy5yKT3uihcpx7xfu/PMB8nzE9pU5bsT3c7yB5/rT4GdN7yaVNJEzXvmM
9sdHMLk0neo7oYE0xdtTr19/gtE5qu3xPnWM9uUeS1ObBU3OEuPiJzWcxdzX
Pufm4AnL4HGHmBoGb6kvXftoHlLrr9KLrVSRl19/uV3lM8jOueXHszbgol9X
ysawRTjGuuk706yC2uZCy8cqv4D1T0PtsEUFrTJUvIvWLUNMhehLwYT1eFrx
4IYhtxVQOhuQISS6HnUmHEW6bSiowTBrlFVQRkniA7X+MwWbfvai5MkroWBN
SoFqKh+m9K7x7DZWxOPbKnvWWPPjYMY1S7ajAub+eDjtwSw/qg40z+ew5HHm
vZz4m0QBXLvbNW6sUQ659lYC8TsEUUwvKd5SSA7P7WLfKn0jiC/cKlVb3WVR
pX/3y0z2KvTXcpnprpFBY8OIeNktq5FiIdb2VkEG3ZRaF/g6VyOrXZDGjF6H
M89ihgNYQsgpbAianF2LyYIewXZqVLTP7JcQCFyLkgPcDP56KpIRBbM+M9K4
6gzLwejUGmSx5fcZBUsj5Y5HwbCEMO73buobpUhj4PWCGIk6YeT702Jlx0Up
rNaOopInRbBHsiVrREYKyd2BOudpohikO1hjUyqJfbptnaFVoihVXmKSZC2J
hTdB7bWHGJpWHKfnT0qge4EBqbgsht4tB8YFEyTwd3Y6q+8vGvozOQUnGBKY
904xhbaHwNVNBhKGoyRG3WB7DuwlUIuQ3vFshEQvu0n1fFsCqRuohe7DJCpU
VRXaOBM4xZyhs3kkXoo90Bh9mMDF3T7zL3pI9DG49q9wKIHkgWxrdguJGyKk
dgllExirSmsSLSdxmfGbWHcOgZ/kl9Rvl5HYMznSkZ1LYHK8Tbk5l8Q45xJ3
yyIC5+/3s08Xk7iiZRfEriLQmWv5aPQuiX3tsTcFOgn0eHW7simdxPLIuSOd
zwncmhaS4JdGYrzJwY2ZLwk0vN6dTb1FonWWXqnFAIGN70UCdqaSeD+k+Slr
jMB0+aHrDxNJTNSlx9pOEDgeXrbpUAKJp4ZTHVSmCPxFqHiREk+i5p4TvNpP
BOakvWWYXyFRkK+LEzdHYMKg0F9DsSQOck38PD8TuDNOzu78HyRW+WVu0l8i
kNIqsVk1hsQkZZGZlRUCF2pGLOsvk/gfK/+M5w==
          "]]}, 
        Annotation[#, "Charting`Private`Tag$11974#3"]& ], {}}}, {}}, {
    DisplayFunction -> Identity, Ticks -> {Automatic, Automatic}, 
     AxesOrigin -> {0, 4.386490924502135}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, DisplayFunction -> Identity, 
     PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.05], 
        Scaled[0.05]}}, PlotRangeClipping -> True, ImagePadding -> All, 
     DisplayFunction -> Identity, AspectRatio -> 
     NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> {False, False}, 
     AxesLabel -> {None, None}, AxesOrigin -> {0, 4.386490924502135}, 
     BaseStyle -> {FontFamily -> "Arial"}, DisplayFunction :> Identity, 
     Frame -> {{True, True}, {True, True}}, FrameLabel -> {{
        FormBox["\"Normalization\"", TraditionalForm], None}, {
        FormBox["\"Coupling, \[Kappa]\"", TraditionalForm], None}}, 
     FrameStyle -> Automatic, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {Automatic, Automatic}, GridLinesStyle -> Directive[
       GrayLevel[0.4, 0.5], 
       AbsoluteThickness[1], 
       AbsoluteDashing[{1, 2}]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "DefaultMeshStyle" -> AbsolutePointSize[6], "ScalingFunctions" -> None,
        "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& ), "CopiedValueFunction" -> ({
           (Identity[#]& )[
            Part[#, 1]], 
           (Identity[#]& )[
            Part[#, 2]]}& )}}, PlotLabel -> 
     FormBox["\"Normalization of Multivariate Coupled Gaussian\"", 
       TraditionalForm], 
     PlotRange -> {{0, 4}, {4.386490924502135, 39.478388870941714`}}, 
     PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{"\"2 Dim\"", "\"3 Dim\"", "\"4 Dim\""}, "LineLegend", 
      DisplayFunction -> (FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           TagBox[
            GridBox[{{
               TagBox[
                GridBox[{{
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}, {
                   GraphicsBox[{{
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                    Directive[
                    EdgeForm[
                    Directive[
                    Opacity[0.3], 
                    GrayLevel[0]]], 
                    PointSize[0.5], FontFamily -> "Arial", 
                    Opacity[1.], 
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    AbsoluteThickness[1.6]], {}}}, AspectRatio -> Full, 
                    ImageSize -> {20, 10}, PlotRangePadding -> None, 
                    ImagePadding -> Automatic, 
                    BaselinePosition -> (Scaled[0.1] -> Baseline)], #3}}, 
                 GridBoxAlignment -> {
                  "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
                 AutoDelete -> False, 
                 GridBoxDividers -> {
                  "Columns" -> {{False}}, "Rows" -> {{False}}}, 
                 GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}},
                  GridBoxSpacings -> {
                  "Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], "Grid"]}}, 
             GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
             AutoDelete -> False, 
             GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], 
            "Grid"], Alignment -> Left, AppearanceElements -> None, 
           ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> 
           "ResizeToFit"], LineIndent -> 0, StripOnInput -> False], {
         FontFamily -> "Arial"}, Background -> Automatic, StripOnInput -> 
         False], TraditionalForm]& ), 
      InterpretationFunction :> (RowBox[{"LineLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.368417, 0.506779, 0.709798], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.24561133333333335`, 0.3378526666666667, 
                    0.4731986666666667], FrameTicks -> None, PlotRangePadding -> 
                    None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.368417`", ",", "0.506779`", ",", "0.709798`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.368417, 0.506779, 0.709798]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.368417, 0.506779, 0.709798], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.880722, 0.611041, 0.142051], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.587148, 0.40736066666666665`, 0.09470066666666668], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.880722`", ",", "0.611041`", ",", "0.142051`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.880722, 0.611041, 0.142051]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.880722, 0.611041, 0.142051], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}], 
               ",", 
               RowBox[{"Directive", "[", 
                 RowBox[{
                   RowBox[{"FontFamily", "\[Rule]", "\"Arial\""}], ",", 
                   RowBox[{"Opacity", "[", "1.`", "]"}], ",", 
                   InterpretationBox[
                    ButtonBox[
                    TooltipBox[
                    GraphicsBox[{{
                    GrayLevel[0], 
                    RectangleBox[{0, 0}]}, {
                    GrayLevel[0], 
                    RectangleBox[{1, -1}]}, {
                    RGBColor[0.560181, 0.691569, 0.194885], 
                    RectangleBox[{0, -1}, {2, 1}]}}, DefaultBaseStyle -> 
                    "ColorSwatchGraphics", AspectRatio -> 1, Frame -> True, 
                    FrameStyle -> 
                    RGBColor[
                    0.37345400000000006`, 0.461046, 0.12992333333333334`], 
                    FrameTicks -> None, PlotRangePadding -> None, ImageSize -> 
                    Dynamic[{
                    Automatic, 1.35 CurrentValue["FontCapHeight"]/
                    AbsoluteCurrentValue[Magnification]}]], 
                    StyleBox[
                    RowBox[{"RGBColor", "[", 
                    RowBox[{"0.560181`", ",", "0.691569`", ",", "0.194885`"}],
                     "]"}], NumberMarks -> False]], Appearance -> None, 
                    BaseStyle -> {}, BaselinePosition -> Baseline, 
                    DefaultBaseStyle -> {}, ButtonFunction :> 
                    With[{Typeset`box$ = EvaluationBox[]}, 
                    If[
                    Not[
                    AbsoluteCurrentValue["Deployed"]], 
                    SelectionMove[Typeset`box$, All, Expression]; 
                    FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
                    FrontEnd`Private`$ColorSelectorInitialColor = 
                    RGBColor[0.560181, 0.691569, 0.194885]; 
                    FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
                    MathLink`CallFrontEnd[
                    FrontEnd`AttachCell[Typeset`box$, 
                    FrontEndResource["RGBColorValueSelector"], {
                    0, {Left, Bottom}}, {Left, Top}, 
                    "ClosingActions" -> {
                    "SelectionDeparture", "ParentChanged", 
                    "EvaluatorQuit"}]]]], BaseStyle -> Inherited, Evaluator -> 
                    Automatic, Method -> "Preemptive"], 
                    RGBColor[0.560181, 0.691569, 0.194885], Editable -> False,
                     Selectable -> False], ",", 
                   RowBox[{"AbsoluteThickness", "[", "1.6`", "]"}]}], "]"}]}],
              "}"}], ",", 
           RowBox[{"{", 
             RowBox[{#, ",", #2, ",", #3}], "}"}], ",", 
           RowBox[{"LegendMarkers", "\[Rule]", "None"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}]}], "]"}]& ), 
      Editable -> True], TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.821966663493232*^9, {3.821967006411293*^9, 3.821967022959064*^9}, 
   3.8219670701789618`*^9, 3.82196727967971*^9},
 CellLabel->"Out[52]=",ExpressionUUID->"c9057bf3-93f1-4746-843d-94e51be3f9f8"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1118, 718},
WindowMargins->{{63, Automatic}, {Automatic, 28}},
Magnification:>1.5 Inherited,
FrontEndVersion->"12.1 for Mac OS X x86 (64-bit) (June 19, 2020)",
StyleDefinitions->FrontEnd`FileName[{"Report"}, "StandardReport.nb", 
  CharacterEncoding -> "UTF-8"],
ExpressionUUID->"50d7d726-bb1a-4906-b6db-f5cfbc104991"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 178, 3, 144, "Title",ExpressionUUID->"a118a5e0-a5c9-4e86-a00f-f7d4c46520e7"],
Cell[761, 27, 850, 13, 466, "Text",ExpressionUUID->"2df1e6b4-9101-433a-b5b1-aeba2eb8a8d9"],
Cell[CellGroupData[{
Cell[1636, 44, 177, 3, 104, "Section",ExpressionUUID->"c4ec21ec-5129-4d93-8bf7-20c01238b74b"],
Cell[1816, 49, 410, 9, 87, "Text",ExpressionUUID->"19e09968-b236-472a-becb-a9a83dd0169f"],
Cell[CellGroupData[{
Cell[2251, 62, 1639, 36, 132, "Input",ExpressionUUID->"f464a9ac-8fef-4c2f-86fe-a490d2ae34f6"],
Cell[3893, 100, 108941, 2002, 378, "Output",ExpressionUUID->"8c42d79d-7e69-4608-a328-b6f5bd4aeb57"]
}, Open  ]],
Cell[112849, 2105, 395, 10, 52, "Text",ExpressionUUID->"ba0f4fb5-ab15-4edb-b55f-92c2211dc171"],
Cell[CellGroupData[{
Cell[113269, 2119, 1593, 33, 126, "Input",ExpressionUUID->"836c154e-d64c-4d99-9bfe-fce529c935df"],
Cell[114865, 2154, 61474, 1159, 401, "Output",ExpressionUUID->"15881f4f-2c54-415b-b62b-8e9486ffa763"]
}, Open  ]],
Cell[CellGroupData[{
Cell[176376, 3318, 1084, 29, 132, "Input",ExpressionUUID->"23b4fb1f-d08a-4ad2-849b-b5e7ce3886b6"],
Cell[177463, 3349, 137533, 2473, 354, "Output",ExpressionUUID->"87d891cc-fbab-4f9e-bb83-93056f600b9e"]
}, Open  ]],
Cell[CellGroupData[{
Cell[315033, 5827, 822, 22, 132, "Input",ExpressionUUID->"631f4061-eb0c-4b83-83b3-62c6726a457f"],
Cell[315858, 5851, 34207, 653, 378, "Output",ExpressionUUID->"5c90b3c4-9e88-4390-b051-0c8cd1935ffc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[350102, 6509, 939, 26, 132, "Input",ExpressionUUID->"8a06f6b0-f73a-484b-a692-66c98d8db402"],
Cell[351044, 6537, 41241, 796, 381, "Output",ExpressionUUID->"102603cd-ffe7-45cf-b58c-79b06c5b9351"]
}, Open  ]],
Cell[392300, 7336, 601, 15, 91, "Text",ExpressionUUID->"68ee224f-851a-44b9-b740-8d468fa41879"],
Cell[CellGroupData[{
Cell[392926, 7355, 1018, 27, 132, "Input",ExpressionUUID->"23fa79c0-f3bb-4422-ae96-6a5969f1ba59"],
Cell[393947, 7384, 98678, 1837, 356, "Output",ExpressionUUID->"00fd225d-e400-4ebf-a43f-29849c4cced7"]
}, Open  ]],
Cell[492640, 9224, 700, 17, 91, "Text",ExpressionUUID->"cc367a38-1217-4c79-8feb-ffa640a4e277"]
}, Open  ]],
Cell[CellGroupData[{
Cell[493377, 9246, 175, 3, 104, "Section",ExpressionUUID->"20f42ba2-4670-471c-ace7-e85f27b11c7e"],
Cell[493555, 9251, 188, 3, 52, "Text",ExpressionUUID->"7487311c-0c8a-46a7-a5f2-93a40b940424"],
Cell[CellGroupData[{
Cell[493768, 9258, 1542, 37, 202, "Input",ExpressionUUID->"e104abe3-0b70-48d6-96a1-aeaa5993f10a"],
Cell[495313, 9297, 75511, 1453, 374, "Output",ExpressionUUID->"4a552853-74a1-4f20-8fc2-f6ae7a2d11a6"]
}, Open  ]],
Cell[570839, 10753, 726, 19, 93, "Text",ExpressionUUID->"a8f9f400-2914-4806-9775-aa29956ca18e"]
}, Open  ]],
Cell[CellGroupData[{
Cell[571602, 10777, 172, 3, 104, "Section",ExpressionUUID->"1775efa8-b2d8-4d30-8ab7-d8ebdb9e0d7b"],
Cell[CellGroupData[{
Cell[571799, 10784, 1963, 45, 281, "Input",ExpressionUUID->"c9127869-ff3b-466a-ac82-2d5ae3a0d997"],
Cell[573765, 10831, 97227, 1722, 377, "Output",ExpressionUUID->"d80098b6-c459-4c84-ba25-e74c34a74aa2"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[671041, 12559, 248, 4, 104, "Section",ExpressionUUID->"d1f5de3b-05e8-4d89-8860-af0548861a12"],
Cell[CellGroupData[{
Cell[671314, 12567, 3498, 78, 448, "Input",ExpressionUUID->"24d0e0e1-3ad3-4539-a6a7-7ced6dddb143"],
Cell[674815, 12647, 53736, 998, 399, "Output",ExpressionUUID->"4468abad-242b-4405-9b2c-58716012bf24"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[728600, 13651, 180, 3, 104, "Section",ExpressionUUID->"76ae8b4e-e4fc-4d67-91d4-1e141bb7087a"],
Cell[CellGroupData[{
Cell[728805, 13658, 180, 3, 51, "Subsection",ExpressionUUID->"6fa9b034-ed26-48ef-837b-de92d5d4dc40"],
Cell[CellGroupData[{
Cell[729010, 13665, 1402, 33, 250, "Input",ExpressionUUID->"8c1b54f9-e42c-44bd-959f-0c2c5ecc0949"],
Cell[730415, 13700, 101422, 1665, 542, "Output",ExpressionUUID->"24fe5063-d0ca-4355-8896-335074916c65"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[831886, 15371, 181, 3, 49, "Subsection",ExpressionUUID->"4e3e3e17-754c-4c95-8967-86eb6298b902"],
Cell[CellGroupData[{
Cell[832092, 15378, 1307, 34, 281, "Input",ExpressionUUID->"3bd5f0dd-d54f-440e-bbdd-a74e20b6579d"],
Cell[833402, 15414, 82143, 1355, 438, "Output",ExpressionUUID->"28e833e2-9314-48b4-ab30-fc126e736626"]
}, Open  ]],
Cell[915560, 16772, 195, 3, 52, "Text",ExpressionUUID->"32c1f4ea-bdaa-411f-8aff-22b0011841a9"],
Cell[CellGroupData[{
Cell[915780, 16779, 1655, 41, 188, "Input",ExpressionUUID->"a43d4dda-722c-4c4f-a373-26b0c1d25106"],
Cell[917438, 16822, 376, 7, 63, "Output",ExpressionUUID->"08356ede-357b-4ffd-a73d-6fa4182c73d6"]
}, Open  ]],
Cell[CellGroupData[{
Cell[917851, 16834, 1558, 41, 219, "Input",ExpressionUUID->"c247d8a3-5009-4d11-833c-a4c9f5caaafe"],
Cell[919412, 16877, 5651, 146, 190, "Output",ExpressionUUID->"eeead421-f566-43c4-8ebc-4e6c7266380a"]
}, Open  ]],
Cell[CellGroupData[{
Cell[925100, 17028, 1825, 46, 219, "Input",ExpressionUUID->"ac76ebda-4da4-4cfa-b4a6-7b8b32ec1826"],
Cell[926928, 17076, 5831, 154, 248, "Output",ExpressionUUID->"6533d30f-dfd1-44d2-b9e4-30d66bce565a"]
}, Open  ]],
Cell[CellGroupData[{
Cell[932796, 17235, 197, 3, 49, "Subsubsection",ExpressionUUID->"e4adc312-eea2-4c76-a3ad-330210a25043"],
Cell[CellGroupData[{
Cell[933018, 17242, 2708, 66, 498, "Input",ExpressionUUID->"5da59c09-9105-4fec-8faa-ae5233a1627e"],
Cell[935729, 17310, 24044, 469, 416, "Output",ExpressionUUID->"c9057bf3-93f1-4746-843d-94e51be3f9f8"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

